@@ -68,7 +68,7 @@ void KisBlurBenchmark::benchmarkFilter()
     KisFilterSP filter = KisFilterRegistry::instance()->value("blur");
     KisFilterConfigurationSP  kfc = filter->defaultConfiguration();
     // Get the predefined configuration from a file
-    QFile file(QString(FILES_DATA_DIR) + QDir::separator() + filter->id() + ".cfg");
+    QFile file(QString(FILES_DATA_DIR) + '/' + filter->id() + ".cfg");
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
         file.open(QIODevice::WriteOnly | QIODevice::Text);
         QTextStream out(&file);
