@@ -169,6 +169,19 @@ Java_org_krita_android_JNIWrappers_exitFullScreen(JNIEnv* /*env*/,
     mainWindow->viewFullscreen(false);
 }
 
+extern "C" JNIEXPORT void JNICALL
+Java_org_krita_android_JNIWrappers_openFileFromIntent(JNIEnv* /*env*/,
+                                                      jobject /*obj*/,
+                                                      jstring str)
+{
+    QAndroidJniObject jUri(str);
+    if (jUri.isValid()) {
+        QString uri = jUri.toString();
+        QMetaObject::invokeMethod(KisApplication::instance(), "fileOpenRequested",
+                                  Qt::QueuedConnection, Q_ARG(QString, uri));
+    }
+}
+
 __attribute__ ((visibility ("default")))
 #endif
 extern "C" int main(int argc, char **argv)
@@ -309,6 +322,29 @@ extern "C" int main(int argc, char **argv)
             originalXdgDataDirs = "/usr/local/share/:/usr/share/";
         }
         qputenv("XDG_DATA_DIRS", QFile::encodeName(root + "share") + ":" + originalXdgDataDirs);
+
+        // APPIMAGE SOUND ADDITIONS
+        // GStreamer needs a few environment variables to properly function in an appimage context.
+        // The following code should be configured to **only** run when we detect that Krita is being
+        // run within an appimage. Checking for the presence of an APPDIR path env variable seems to be
+        // enough to filter out this step for non-appimage krita builds.
+
+        const bool isInAppimage = qEnvironmentVariableIsSet("APPIMAGE");
+        if (isInAppimage) {
+            QByteArray appimageMountDir = qgetenv("APPDIR");
+
+            //We need to add new gstreamer plugin paths for the system to find the
+            //appropriate plugins.
+            const QByteArray gstPluginSystemPath = qgetenv("GST_PLUGIN_SYSTEM_PATH_1_0");
+            const QByteArray gstPluginScannerPath = qgetenv("GST_PLUGIN_SCANNER");
+
+            //Plugins Path is where libgstreamer-1.0 should expect to find plugin libraries.
+            qputenv("GST_PLUGIN_SYSTEM_PATH_1_0", appimageMountDir + QFile::encodeName("/usr/lib/gstreamer-1.0/") + ":" + gstPluginSystemPath);
+
+            //Plugin scanner is where gstreamer should expect to find the plugin scanner.
+            //Perhaps invoking the scanenr earlier in the code manually could allow ldd to quickly find all plugin dependencies?
+            qputenv("GST_PLUGIN_SCANNER", appimageMountDir + QFile::encodeName("/usr/lib/gstreamer-1.0/gst-plugin-scanner"));
+        }
     }
 #else
     qputenv("XDG_DATA_DIRS", QFile::encodeName(root + "share"));
@@ -346,6 +382,23 @@ extern "C" int main(int argc, char **argv)
 
         // And if there isn't one, check the one set by the system.
         QLocale locale = QLocale::system();
+
+#ifdef Q_OS_ANDROID
+        // QLocale::uiLanguages() fails on Android, so if the fallback locale is being
+        // used we, try to fetch the device's default locale.
+        if (locale.name() == QLocale::c().name()) {
+            QAndroidJniObject localeJniObj = QAndroidJniObject::callStaticObjectMethod(
+                "java/util/Locale", "getDefault", "()Ljava/util/Locale;");
+
+            if (localeJniObj.isValid()) {
+                QAndroidJniObject tag = localeJniObj.callObjectMethod("toLanguageTag",
+                                                                      "()Ljava/lang/String;");
+                if (tag.isValid()) {
+                    locale = QLocale(tag.toString());
+                }
+            }
+        }
+#endif
         if (locale.name() != QStringLiteral("en")) {
             QStringList uiLanguages = locale.uiLanguages();
             for (QString &uiLanguage : uiLanguages) {
@@ -408,6 +461,12 @@ extern "C" int main(int argc, char **argv)
 
     // first create the application so we can create a pixmap
     KisApplication app(key, argc, argv);
+
+    if (app.platformName() == "wayland") {
+        QMessageBox::critical(0, i18nc("@title:window", "Fatal Error"), i18n("Krita does not support the Wayland platform. Use XWayland to run Krita on Wayland. Krita will close now."));
+        return -1;
+    }
+
     KisUsageLogger::writeHeader();
     KisOpenGL::initialize();
 
@@ -426,6 +485,12 @@ extern "C" int main(int argc, char **argv)
             app.setLayoutDirection(Qt::LeftToRight);
         }
     }
+#ifdef Q_OS_ANDROID
+    // TODO: remove "share" - sh_zam
+    // points to /data/data/org.krita/files/share/locale
+    KLocalizedString::addDomainLocaleDir("krita", QStandardPaths::writableLocation(QStandardPaths::HomeLocation) + "/share/locale");
+#endif
+
     KLocalizedString::setApplicationDomain("krita");
 
     dbgKrita << "Available translations" << KLocalizedString::availableApplicationTranslations();
@@ -559,10 +624,13 @@ extern "C" int main(int argc, char **argv)
 #elif defined QT_HAS_WINTAB_SWITCH
 
     // Check if WinTab/WinInk has actually activated
-    const bool useWinTabAPI = app.testAttribute(Qt::AA_MSWindowsUseWinTabAPI);
+    const bool useWinInkAPI = !app.testAttribute(Qt::AA_MSWindowsUseWinTabAPI);
 
-    if (useWinTabAPI != !cfg.useWin8PointerInput()) {
-        cfg.setUseWin8PointerInput(useWinTabAPI);
+    if (useWinInkAPI != cfg.useWin8PointerInput()) {
+        KisUsageLogger::log("WARNING: WinTab tablet protocol is not supported on this device. Switching to WinInk...");
+
+        cfg.setUseWin8PointerInput(useWinInkAPI);
+        cfg.setUseRightMiddleTabletButtonWorkaround(true);
     }
 
 #endif
