@@ -54,6 +54,40 @@ KoCreatePathTool::~KoCreatePathTool()
 {
 }
 
+QRectF KoCreatePathTool::decorationsRect() const
+{
+    Q_D(const KoCreatePathTool);
+
+    QRectF dirtyRect;
+
+    if (pathStarted()) {
+        dirtyRect |= kisGrowRect(d->shape->boundingRect(), handleDocRadius());
+    }
+
+    if (d->hoveredPoint) {
+        dirtyRect |= kisGrowRect(d->hoveredPoint->boundingRect(false), handleDocRadius());
+    }
+
+    if (d->activePoint) {
+        dirtyRect |= kisGrowRect(d->activePoint->boundingRect(false), handleDocRadius());
+
+        if (d->pointIsDragged) {
+            // the path is not closed, therefore the point is not marked as
+            // active inside the path itself
+            dirtyRect |= handlePaintRect(
+                        d->activePoint->parent()->shapeToDocument(
+                            d->activePoint->controlPoint2()));
+        }
+
+    }
+
+    if (canvas()->snapGuide()->isSnapping()) {
+        dirtyRect |= canvas()->snapGuide()->boundingRect();
+    }
+
+    return dirtyRect;
+}
+
 void KoCreatePathTool::paint(QPainter &painter, const KoViewConverter &converter)
 {
     Q_D(KoCreatePathTool);
@@ -96,7 +130,7 @@ void KoCreatePathTool::paint(QPainter &painter, const KoViewConverter &converter
     }
 
     painter.save();
-    painter.setTransform(converter.documentToView());
+    painter.setTransform(converter.documentToView(), true);
     canvas()->snapGuide()->paint(painter, converter);
     painter.restore();
 }
@@ -161,10 +195,9 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
                 // the path shape will get closed when the user releases
                 // the mouse button
                 d->finishAfterThisPoint = true;
+                repaintDecorations();
             }
         } else {
-            canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
-
             QPointF point = canvas()->snapGuide()->snap(event->point, event->modifiers());
 
             // check whether we hit an start/end node of an existing path
@@ -176,8 +209,7 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
                 endPath();
             } else {
                 d->activePoint->setPoint(point);
-                canvas()->updateCanvas(d->shape->boundingRect());
-                canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
+                repaintDecorations();
             }
         }
     } else {
@@ -192,7 +224,6 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
         stroke->setColor(canvas()->resourceManager()->foregroundColor().toQColor());
 
         pathShape->setStroke(stroke);
-        canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
         QPointF point = canvas()->snapGuide()->snap(event->point, event->modifiers());
 
         // check whether we hit an start/end node of an existing path
@@ -204,13 +235,13 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
 
         d->activePoint = pathShape->moveTo(point);
         d->firstPoint = d->activePoint;
-        canvas()->updateCanvas(handlePaintRect(point));
-        canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
 
         canvas()->snapGuide()->setAdditionalEditedShape(pathShape);
 
         d->angleSnapStrategy = new AngleSnapStrategy(d->angleSnappingDelta, d->angleSnapStatus);
         canvas()->snapGuide()->addCustomSnapStrategy(d->angleSnapStrategy);
+
+        repaintDecorations();
     }
 
     d->dragStartPoint = event->point;
@@ -219,9 +250,9 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
         d->angleSnapStrategy->setStartPoint(d->activePoint->point());
 }
 
-bool KoCreatePathTool::pathStarted()
+bool KoCreatePathTool::pathStarted() const
 {
-    Q_D(KoCreatePathTool);
+    Q_D(const KoCreatePathTool);
     return ((bool) d->shape);
 }
 
@@ -248,27 +279,11 @@ void KoCreatePathTool::mouseMoveEvent(KoPointerEvent *event)
 {
     Q_D(KoCreatePathTool);
 
-    KoPathPoint *endPoint = d->endPointAtPosition(event->point);
-    if (d->hoveredPoint != endPoint) {
-
-        if (d->hoveredPoint) {
-            QPointF nodePos = d->hoveredPoint->parent()->shapeToDocument(d->hoveredPoint->point());
-            canvas()->updateCanvas(handlePaintRect(nodePos));
-        }
-
-        d->hoveredPoint = endPoint;
-
-        if (d->hoveredPoint) {
-            QPointF nodePos = d->hoveredPoint->parent()->shapeToDocument(d->hoveredPoint->point());
-            canvas()->updateCanvas(handlePaintRect(nodePos));
-        }
-    }
+    d->hoveredPoint = d->endPointAtPosition(event->point);
 
     if (!pathStarted()) {
-
-        canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
         canvas()->snapGuide()->snap(event->point, event->modifiers());
-        canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
+        repaintDecorations();
 
         d->mouseOverFirstPoint = false;
         return;
@@ -276,12 +291,8 @@ void KoCreatePathTool::mouseMoveEvent(KoPointerEvent *event)
 
     d->mouseOverFirstPoint = handleGrabRect(d->firstPoint->point()).contains(event->point);
 
-    canvas()->updateCanvas(d->shape->boundingRect());
-    canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
     QPointF snappedPosition = canvas()->snapGuide()->snap(event->point, event->modifiers());
 
-    d->repaintActivePoint();
-
     if (event->buttons() & Qt::LeftButton) {
         if (d->pointIsDragged ||
             !handleGrabRect(d->dragStartPoint).contains(event->point)) {
@@ -293,7 +304,6 @@ void KoCreatePathTool::mouseMoveEvent(KoPointerEvent *event)
             if ((event->modifiers() & Qt::AltModifier) == 0) {
                 d->activePoint->setControlPoint1(d->activePoint->point() - offset);
             }
-            d->repaintActivePoint();
         }
     } else {
         d->activePoint->setPoint(snappedPosition);
@@ -331,8 +341,7 @@ void KoCreatePathTool::mouseMoveEvent(KoPointerEvent *event)
 
     }
 
-    canvas()->updateCanvas(d->shape->boundingRect());
-    canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
+    repaintDecorations();
 }
 
 void KoCreatePathTool::mouseReleaseEvent(KoPointerEvent *event)
@@ -341,7 +350,6 @@ void KoCreatePathTool::mouseReleaseEvent(KoPointerEvent *event)
 
     if (! d->shape || (event->buttons() & Qt::RightButton)) return;
 
-    d->repaintActivePoint();
     d->prevPointWasDragged  = d->pointIsDragged;
     d->pointIsDragged = false;
     KoPathPoint *lastActivePoint = d->activePoint;
@@ -380,6 +388,8 @@ void KoCreatePathTool::mouseReleaseEvent(KoPointerEvent *event)
     if (d->angleSnapStrategy && lastActivePoint->activeControlPoint2()) {
         d->angleSnapStrategy->deactivate();
     }
+
+    repaintDecorations();
 }
 
 void KoCreatePathTool::keyPressEvent(QKeyEvent *event)
@@ -396,6 +406,7 @@ void KoCreatePathTool::endPath()
     Q_D(KoCreatePathTool);
 
     d->addPathShape();
+    repaintDecorations();
 }
 
 void KoCreatePathTool::endPathWithoutLastPoint()
@@ -403,11 +414,10 @@ void KoCreatePathTool::endPathWithoutLastPoint()
     Q_D(KoCreatePathTool);
 
     if (d->shape) {
-        QRectF dirtyRect = d->shape->boundingRect();
         delete d->shape->removePoint(d->shape->pathPointIndex(d->activePoint));
-        canvas()->updateCanvas(dirtyRect);
-
         d->addPathShape();
+
+        repaintDecorations();
     }
 }
 
@@ -416,12 +426,11 @@ void KoCreatePathTool::cancelPath()
     Q_D(KoCreatePathTool);
 
     if (d->shape) {
-        canvas()->updateCanvas(handlePaintRect(d->firstPoint->point()));
-        canvas()->updateCanvas(d->shape->boundingRect());
         d->firstPoint = 0;
         d->activePoint = 0;
     }
     d->cleanUp();
+    repaintDecorations();
 }
 
 void KoCreatePathTool::removeLastPoint()
@@ -437,8 +446,7 @@ void KoCreatePathTool::removeLastPoint()
 
             d->hoveredPoint = 0;
 
-            d->repaintActivePoint();
-            canvas()->updateCanvas(d->shape->boundingRect());
+            repaintDecorations();
         }
     }
 }
@@ -455,8 +463,8 @@ void KoCreatePathTool::activate(ToolActivation activation, const QSet<KoShape*>
     d->loadAutoSmoothValueFromConfig();
 
     // reset snap guide
-    canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
     canvas()->snapGuide()->reset();
+    repaintDecorations();
 }
 
 void KoCreatePathTool::deactivate()
