@@ -36,52 +36,62 @@ public:
 
     KisAutoBrush(KisMaskGenerator* as, qreal angle, qreal randomness, qreal density = 1.0);
     KisAutoBrush(const KisAutoBrush& rhs);
-    KisBrush* clone() const;
+    KisBrush* clone() const override;
 
-    virtual ~KisAutoBrush();
+    ~KisAutoBrush() override;
 
 public:
 
-    virtual KisFixedPaintDeviceSP paintDevice(const KoColorSpace*,
-            double, double,
+    qreal userEffectiveSize() const override;
+    void setUserEffectiveSize(qreal value) override;
+
+    qint32 maskWidth(KisDabShape const& shape, qreal subPixelX, qreal subPixelY,
+        const KisPaintInformation& info) const override;
+    qint32 maskHeight(KisDabShape const& shape, qreal subPixelX, qreal subPixelY,
+        const KisPaintInformation& info) const override;
+    QSizeF characteristicSize(KisDabShape const&) const override;
+
+    KisFixedPaintDeviceSP paintDevice(const KoColorSpace*,
+            KisDabShape const&,
             const KisPaintInformation&,
-            double = 0, double = 0) const {
+            double = 0, double = 0) const override {
         return 0; // The autobrush does NOT support images!
     }
 
-    virtual void generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
-            KisBrush::ColoringInformation* src,
-            double scaleX, double scaleY, double angle,
-            const KisPaintInformation& info,
-            double subPixelX = 0, double subPixelY = 0,
-            qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
+    void generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
+        KisBrush::ColoringInformation* src,
+        KisDabShape const&,
+        const KisPaintInformation& info,
+        double subPixelX = 0, double subPixelY = 0,
+        qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR,
+        qreal lightnessStrength = DEFAULT_LIGHTNESS_STRENGTH) const override;
 
-    virtual QPainterPath outline() const;
+    QPainterPath outline() const override;
 
 public:
 
-    bool load() {
+    bool load() override {
         return false;
     }
 
-    virtual bool loadFromDevice(QIODevice *) {
+    bool loadFromDevice(QIODevice *) override {
         return false;
     }
 
-    bool save()  {
+    bool save() override {
         return false;
     }
 
-    bool saveToDevice(QIODevice*) const {
+    bool saveToDevice(QIODevice*) const override {
         return false;
     }
 
-    void toXML(QDomDocument& , QDomElement&) const;
+    void toXML(QDomDocument& , QDomElement&) const override;
     const KisMaskGenerator* maskGenerator() const;
     qreal randomness() const;
     qreal density() const;
 
-    void lodLimitations(KisPaintopLodLimitations *l) const;
+    void lodLimitations(KisPaintopLodLimitations *l) const override;
 
 private:
 
