@@ -25,6 +25,7 @@
 
 #include <QDomElement>
 #include <QFile>
+#include <QPainterPath>
 #include <QPoint>
 #include <QFileInfo>
 #include <QBuffer>
@@ -46,7 +47,12 @@
 #include <brushengine/kis_paint_information.h>
 #include <kis_fixed_paint_device.h>
 #include <kis_qimage_pyramid.h>
+#include <KisSharedQImagePyramid.h>
 #include <brushengine/kis_paintop_lod_limitations.h>
+#include <resources/KoAbstractGradient.h>
+#include <resources/KoCachedGradient.h>
+#include <resources/KoResource.h>
+#include <KoResourceServerProvider.h>
 
 
 KisBrush::ColoringInformation::~ColoringInformation()
@@ -104,12 +110,14 @@ struct KisBrush::Private {
         : boundary(0)
         , angle(0)
         , scale(1.0)
-        , hasColor(false)
         , brushType(INVALID)
-        , brushPyramid(0)
+        , brushApplication(ALPHAMASK)
+        , gradient(0)
         , autoSpacingActive(false)
         , autoSpacingCoeff(1.0)
-    {}
+        , threadingAllowed(true)
+    {
+    }
 
     ~Private() {
         delete boundary;
@@ -118,24 +126,29 @@ struct KisBrush::Private {
     mutable KisBoundary* boundary;
     qreal angle;
     qreal scale;
-    bool hasColor;
     enumBrushType brushType;
+    enumBrushApplication brushApplication;
 
     qint32 width;
     qint32 height;
     double spacing;
     QPointF hotSpot;
 
-    mutable KisQImagePyramid *brushPyramid;
+    mutable QSharedPointer<KisSharedQImagePyramid> brushPyramid;
 
     QImage brushTipImage;
 
+    const KoAbstractGradient* gradient;
+    QScopedPointer<KoCachedGradient> cachedGradient;
+
     bool autoSpacingActive;
     qreal autoSpacingCoeff;
+
+    bool threadingAllowed;
 };
 
 KisBrush::KisBrush()
-    : KoResource("")
+    : KoResource(QString())
     , d(new Private)
 {
 }
@@ -147,29 +160,40 @@ KisBrush::KisBrush(const QString& filename)
 }
 
 KisBrush::KisBrush(const KisBrush& rhs)
-    : KoResource("")
+    : KoResource(QString())
     , KisShared()
     , d(new Private)
 {
     setBrushTipImage(rhs.brushTipImage());
     d->brushType = rhs.d->brushType;
+    d->brushApplication = rhs.d->brushApplication;
     d->width = rhs.d->width;
     d->height = rhs.d->height;
     d->spacing = rhs.d->spacing;
     d->hotSpot = rhs.d->hotSpot;
-    d->hasColor = rhs.d->hasColor;
     d->angle = rhs.d->angle;
     d->scale = rhs.d->scale;
     d->autoSpacingActive = rhs.d->autoSpacingActive;
     d->autoSpacingCoeff = rhs.d->autoSpacingCoeff;
+    d->threadingAllowed = rhs.d->threadingAllowed;
+    d->gradient = rhs.d->gradient;
+    if (rhs.d->cachedGradient) {
+        d->cachedGradient.reset(static_cast<KoCachedGradient*>(rhs.d->cachedGradient->clone()));
+    }
     setFilename(rhs.filename());
-    clearBrushPyramid();
+
+    /**
+     * Be careful! The pyramid is shared between two brush objects,
+     * therefore you cannot change it, only recreate! That is the
+     * reason why it is defined as const!
+     */
+    d->brushPyramid = rhs.d->brushPyramid;
+
     // don't copy the boundary, it will be regenerated -- see bug 291910
 }
 
 KisBrush::~KisBrush()
 {
-    clearBrushPyramid();
     delete d;
 }
 
@@ -219,11 +243,11 @@ void KisBrush::setHotSpot(QPointF pt)
     d->hotSpot = QPointF(x, y);
 }
 
-QPointF KisBrush::hotSpot(double scaleX, double scaleY, double rotation, const KisPaintInformation& info) const
+QPointF KisBrush::hotSpot(KisDabShape const& shape, const KisPaintInformation& info) const
 {
     Q_UNUSED(info);
 
-    QSizeF metric = characteristicSize(scaleX, scaleY, rotation);
+    QSizeF metric = characteristicSize(shape);
 
     qreal w = metric.width();
     qreal h = metric.height();
@@ -244,15 +268,37 @@ QPointF KisBrush::hotSpot(double scaleX, double scaleY, double rotation, const K
     return p;
 }
 
-
-bool KisBrush::hasColor() const
+void KisBrush::setBrushApplication(enumBrushApplication brushApplication)
 {
-    return d->hasColor;
+    d->brushApplication = brushApplication;
+    clearBrushPyramid();
 }
 
-void KisBrush::setHasColor(bool hasColor)
+enumBrushApplication KisBrush::brushApplication() const
 {
-    d->hasColor = hasColor;
+    return d->brushApplication;
+}
+
+bool KisBrush::preserveLightness() const
+{
+    return d->brushApplication == LIGHTNESSMAP;
+}
+
+bool KisBrush::applyingGradient() const
+{
+    return d->brushApplication == GRADIENTMAP;
+}
+
+void KisBrush::setGradient(const KoAbstractGradient* gradient) {
+    if (gradient && gradient->valid()) {
+        d->gradient = gradient;
+
+        if (!d->cachedGradient) {
+            d->cachedGradient.reset(new KoCachedGradient(d->gradient, 256, d->gradient->colorSpace()));
+        } else {
+            d->cachedGradient->setGradient(d->gradient, 256, d->gradient->colorSpace());
+        }
+    }
 }
 
 bool KisBrush::isPiercedApprox() const
@@ -296,7 +342,6 @@ bool KisBrush::canPaintFor(const KisPaintInformation& /*info*/)
 
 void KisBrush::setBrushTipImage(const QImage& image)
 {
-    //Q_ASSERT(!image.isNull());
     d->brushTipImage = image;
 
     if (!image.isNull()) {
@@ -332,6 +377,7 @@ void KisBrush::predefinedBrushToXML(const QString &type, QDomElement& e) const
     e.setAttribute("autoSpacingCoeff", QString::number(autoSpacingCoeff()));
     e.setAttribute("angle", QString::number(angle()));
     e.setAttribute("scale", QString::number(scale()));
+    e.setAttribute("brushApplication", QString::number((int)brushApplication()));
 }
 
 void KisBrush::toXML(QDomDocument& /*document*/ , QDomElement& element) const
@@ -341,45 +387,44 @@ void KisBrush::toXML(QDomDocument& /*document*/ , QDomElement& element) const
 
 KisBrushSP KisBrush::fromXML(const QDomElement& element)
 {
-    KisBrushSP brush = KisBrushRegistry::instance()->getOrCreateBrush(element);
+    KisBrushSP brush = KisBrushRegistry::instance()->createBrush(element);
     if (brush && element.attribute("BrushVersion", "1") == "1") {
         brush->setScale(brush->scale() * 2.0);
     }
     return brush;
 }
 
-QSizeF KisBrush::characteristicSize(double scaleX, double scaleY, double rotation) const
+QSizeF KisBrush::characteristicSize(KisDabShape const& shape) const
 {
-    Q_UNUSED(scaleY);
-
-    qreal angle = normalizeAngle(rotation + d->angle);
-    qreal scale = scaleX * d->scale;
-
-    return KisQImagePyramid::characteristicSize(QSize(width(), height()),
-                                                scale, angle);
+    KisDabShape normalizedShape(
+         shape.scale() * d->scale,
+         shape.ratio(),
+         normalizeAngle(shape.rotation() + d->angle));
+    return KisQImagePyramid::characteristicSize(
+        QSize(width(), height()), normalizedShape);
 }
 
-qint32 KisBrush::maskWidth(double scale, double angle, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const
+qint32 KisBrush::maskWidth(KisDabShape const& shape, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const
 {
     Q_UNUSED(info);
 
-    angle = normalizeAngle(angle + d->angle);
-    scale *= d->scale;
+    qreal angle = normalizeAngle(shape.rotation() + d->angle);
+    qreal scale = shape.scale() * d->scale;
 
     return KisQImagePyramid::imageSize(QSize(width(), height()),
-                                       scale, angle,
+                                       KisDabShape(scale, shape.ratio(), angle),
                                        subPixelX, subPixelY).width();
 }
 
-qint32 KisBrush::maskHeight(double scale, double angle, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const
+qint32 KisBrush::maskHeight(KisDabShape const& shape, qreal subPixelX, qreal subPixelY, const KisPaintInformation& info) const
 {
     Q_UNUSED(info);
 
-    angle = normalizeAngle(angle + d->angle);
-    scale *= d->scale;
+    qreal angle = normalizeAngle(shape.rotation() + d->angle);
+    qreal scale = shape.scale() * d->scale;
 
     return KisQImagePyramid::imageSize(QSize(width(), height()),
-                                       scale, angle,
+                                       KisDabShape(scale, shape.ratio(), angle),
                                        subPixelX, subPixelY).height();
 }
 
@@ -388,9 +433,8 @@ double KisBrush::maskAngle(double angle) const
     return normalizeAngle(angle + d->angle);
 }
 
-quint32 KisBrush::brushIndex(const KisPaintInformation& info) const
+quint32 KisBrush::brushIndex() const
 {
-    Q_UNUSED(info);
     return 0;
 }
 
@@ -425,144 +469,173 @@ void KisBrush::notifyStrokeStarted()
 {
 }
 
-void KisBrush::notifyCachedDabPainted(const KisPaintInformation& info)
+void KisBrush::prepareForSeqNo(const KisPaintInformation &info, int seqNo)
 {
     Q_UNUSED(info);
+    Q_UNUSED(seqNo);
 }
 
-void KisBrush::prepareBrushPyramid() const
+void KisBrush::setThreadingAllowed(bool value)
 {
-    if (!d->brushPyramid) {
-        d->brushPyramid = new KisQImagePyramid(brushTipImage());
-    }
+    d->threadingAllowed = value;
+}
+
+bool KisBrush::threadingAllowed() const
+{
+    return d->threadingAllowed;
 }
 
 void KisBrush::clearBrushPyramid()
 {
-    delete d->brushPyramid;
-    d->brushPyramid = 0;
+    d->brushPyramid.reset(new KisSharedQImagePyramid());
 }
 
-void KisBrush::mask(KisFixedPaintDeviceSP dst, double scaleX, double scaleY, double angle, const KisPaintInformation& info , double subPixelX, double subPixelY, qreal softnessFactor) const
-{
-    generateMaskAndApplyMaskOrCreateDab(dst, 0, scaleX, scaleY, angle, info, subPixelX, subPixelY, softnessFactor);
-}
-
-void KisBrush::mask(KisFixedPaintDeviceSP dst, const KoColor& color, double scaleX, double scaleY, double angle, const KisPaintInformation& info, double subPixelX, double subPixelY, qreal softnessFactor) const
+void KisBrush::mask(KisFixedPaintDeviceSP dst, const KoColor& color, KisDabShape const& shape, const KisPaintInformation& info, double subPixelX, double subPixelY, qreal softnessFactor, qreal lightnessStrength) const
 {
     PlainColoringInformation pci(color.data());
-    generateMaskAndApplyMaskOrCreateDab(dst, &pci, scaleX, scaleY, angle, info, subPixelX, subPixelY, softnessFactor);
+    generateMaskAndApplyMaskOrCreateDab(dst, &pci, shape, info, subPixelX, subPixelY, softnessFactor, lightnessStrength);
 }
 
-void KisBrush::mask(KisFixedPaintDeviceSP dst, const KisPaintDeviceSP src, double scaleX, double scaleY, double angle, const KisPaintInformation& info, double subPixelX, double subPixelY, qreal softnessFactor) const
+void KisBrush::mask(KisFixedPaintDeviceSP dst, const KisPaintDeviceSP src, KisDabShape const& shape, const KisPaintInformation& info, double subPixelX, double subPixelY, qreal softnessFactor, qreal lightnessStrength) const
 {
-    PaintDeviceColoringInformation pdci(src, maskWidth(scaleX, angle, subPixelX, subPixelY, info));
-    generateMaskAndApplyMaskOrCreateDab(dst, &pdci, scaleX, scaleY, angle, info, subPixelX, subPixelY, softnessFactor);
+    PaintDeviceColoringInformation pdci(src, maskWidth(shape, subPixelX, subPixelY, info));
+    generateMaskAndApplyMaskOrCreateDab(dst, &pdci, shape, info, subPixelX, subPixelY, softnessFactor, lightnessStrength);
 }
 
+namespace {
+void fetchPremultipliedRed(const QRgb* src, quint8 *dst, int maskWidth)
+{
+    for (int x = 0; x < maskWidth; x++) {
+        *dst = KoColorSpaceMaths<quint8>::multiply(255 - *src, qAlpha(*src));
+        src++;
+        dst++;
+    }
+}
+}
+
+void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
+    ColoringInformation* coloringInformation,
+    KisDabShape const& shape,
+    const KisPaintInformation& info_,
+    double subPixelX, double subPixelY, qreal softnessFactor) const
+{
+    generateMaskAndApplyMaskOrCreateDab(dst, coloringInformation, shape, info_, subPixelX, subPixelY, softnessFactor, DEFAULT_LIGHTNESS_STRENGTH);
+}
 
 void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
         ColoringInformation* coloringInformation,
-        double scaleX, double scaleY, double angle,
+        KisDabShape const& shape,
         const KisPaintInformation& info_,
-        double subPixelX, double subPixelY, qreal softnessFactor) const
+        double subPixelX, double subPixelY, qreal softnessFactor, qreal lightnessStrength) const
 {
-    Q_ASSERT(valid());
+    KIS_SAFE_ASSERT_RECOVER_RETURN(valid());
     Q_UNUSED(info_);
     Q_UNUSED(softnessFactor);
 
-    angle   = normalizeAngle(angle + d->angle);
-    scaleX *= d->scale;
-    scaleY *= d->scale;
-
-    double scale = 0.5 * (scaleX + scaleY);
-
-    prepareBrushPyramid();
-    QImage outputImage = d->brushPyramid->createImage(scale, -angle, subPixelX, subPixelY);
+    QImage outputImage = d->brushPyramid->pyramid(this)->createImage(KisDabShape(
+            shape.scale() * d->scale, shape.ratio(),
+            -normalizeAngle(shape.rotation() + d->angle)),
+        subPixelX, subPixelY);
 
     qint32 maskWidth = outputImage.width();
     qint32 maskHeight = outputImage.height();
 
     dst->setRect(QRect(0, 0, maskWidth, maskHeight));
-    dst->initialize();
+    dst->lazyGrowBufferWithoutInitialization();
+
+    KIS_SAFE_ASSERT_RECOVER_RETURN(coloringInformation);
 
     quint8* color = 0;
-
-    if (coloringInformation) {
-        if (dynamic_cast<PlainColoringInformation*>(coloringInformation)) {
-            color = const_cast<quint8*>(coloringInformation->color());
-        }
+    if (dynamic_cast<PlainColoringInformation*>(coloringInformation)) {
+        color = const_cast<quint8*>(coloringInformation->color());
     }
 
     const KoColorSpace *cs = dst->colorSpace();
-    qint32 pixelSize = cs->pixelSize();
-    quint8 *dabPointer = dst->data();
-    quint8 *rowPointer = dabPointer;
-    quint8 *alphaArray = new quint8[maskWidth];
-    bool hasColor = this->hasColor();
+    const quint32 pixelSize = cs->pixelSize();
+    const quint32 maskPixelSize = sizeof(QRgb);
+    quint8 *rowPointer = dst->data();
 
+    const bool preserveLightness = this->preserveLightness();
+    bool applyGradient = this->applyingGradient();
+    QScopedPointer<KoColor> fallbackColor;
+
+    if (applyGradient) {
+        if (d->cachedGradient) {
+            KIS_SAFE_ASSERT_RECOVER_RETURN(d->cachedGradient);
+            d->cachedGradient->setColorSpace(cs); //convert gradient to colorspace so we don't have to convert each pixel
+        } else {
+            fallbackColor.reset(new KoColor(Qt::red, cs));
+            color = fallbackColor->data();
+            applyGradient = false;
+        }
+    }
+
+    KoColor gradientcolor(Qt::blue, cs);
     for (int y = 0; y < maskHeight; y++) {
         const quint8* maskPointer = outputImage.constScanLine(y);
-        if (coloringInformation) {
-            for (int x = 0; x < maskWidth; x++) {
-                if (color) {
-                    memcpy(dabPointer, color, pixelSize);
-                }
-                else {
-                    memcpy(dabPointer, coloringInformation->color(), pixelSize);
-                    coloringInformation->nextColumn();
-                }
-                dabPointer += pixelSize;
+        if (color) {
+            if (preserveLightness) {
+                cs->fillGrayBrushWithColorAndLightnessWithStrength(rowPointer, reinterpret_cast<const QRgb*>(maskPointer), color, lightnessStrength, maskWidth);
             }
-        }
+            else if (applyGradient) {
+                quint8* pixel = rowPointer;
+                for (int x = 0; x < maskWidth; x++) {
+                    const QRgb* maskQRgb = reinterpret_cast<const QRgb*>(maskPointer);
+                    qreal maskOpacity = qreal(qAlpha(*maskQRgb)) / 255.0;
+                    if (maskOpacity > 0) {
+                        qreal gradientvalue = qreal(qGray(*maskQRgb)) / 255.0;
+                        gradientcolor.setColor(d->cachedGradient->cachedAt(gradientvalue), cs);
+                    }
+                    qreal gradientOpacity = gradientcolor.opacityF();
+                    qreal opacity = gradientOpacity * maskOpacity;
+                    gradientcolor.setOpacity(opacity);
+                    memcpy(pixel, gradientcolor.data(), pixelSize);
 
-        if (hasColor) {
-            const quint8 *src = maskPointer;
-            quint8 *dst = alphaArray;
-            for (int x = 0; x < maskWidth; x++) {
-                const QRgb *c = reinterpret_cast<const QRgb*>(src);
-
-                *dst = KoColorSpaceMaths<quint8>::multiply(255 - qGray(*c), qAlpha(*c));
-                src += 4;
-                dst++;
+                    maskPointer += maskPixelSize; 
+                    pixel += pixelSize;
+                }
+            }
+            else {
+                cs->fillGrayBrushWithColor(rowPointer, reinterpret_cast<const QRgb*>(maskPointer), color, maskWidth);
             }
         }
         else {
-            const quint8 *src = maskPointer;
-            quint8 *dst = alphaArray;
-            for (int x = 0; x < maskWidth; x++) {
-                const QRgb *c = reinterpret_cast<const QRgb*>(src);
-
-                *dst = KoColorSpaceMaths<quint8>::multiply(255 - *src, qAlpha(*c));
-                src += 4;
-                dst++;
+            {
+                quint8 *dst = rowPointer;
+                for (int x = 0; x < maskWidth; x++) {
+                    memcpy(dst, coloringInformation->color(), pixelSize);
+                    coloringInformation->nextColumn();
+                    dst += pixelSize;
+                }
             }
+
+            QScopedArrayPointer<quint8> alphaArray(new quint8[maskWidth]);
+            fetchPremultipliedRed(reinterpret_cast<const QRgb*>(maskPointer), alphaArray.data(), maskWidth);
+            cs->applyAlphaU8Mask(rowPointer, alphaArray.data(), maskWidth);
         }
 
-        cs->applyAlphaU8Mask(rowPointer, alphaArray, maskWidth);
         rowPointer += maskWidth * pixelSize;
-        dabPointer = rowPointer;
 
-        if (!color && coloringInformation) {
+        if (!color) {
             coloringInformation->nextRow();
         }
     }
 
-    delete[] alphaArray;
+
 }
 
 KisFixedPaintDeviceSP KisBrush::paintDevice(const KoColorSpace * colorSpace,
-        double scale, double angle,
+        KisDabShape const& shape,
         const KisPaintInformation& info,
         double subPixelX, double subPixelY) const
 {
     Q_ASSERT(valid());
     Q_UNUSED(info);
-    angle  = normalizeAngle(angle + d->angle);
-    scale *= d->scale;
+    double angle = normalizeAngle(shape.rotation() + d->angle);
+    double scale = shape.scale() * d->scale;
 
-    prepareBrushPyramid();
-    QImage outputImage = d->brushPyramid->createImage(scale, -angle, subPixelX, subPixelY);
+    QImage outputImage = d->brushPyramid->pyramid(this)->createImage(
+        KisDabShape(scale, shape.ratio(), -angle), subPixelX, subPixelY);
 
     KisFixedPaintDeviceSP dab = new KisFixedPaintDevice(colorSpace);
     Q_CHECK_PTR(dab);
@@ -580,14 +653,16 @@ void KisBrush::resetBoundary()
 void KisBrush::generateBoundary() const
 {
     KisFixedPaintDeviceSP dev;
+    KisDabShape inverseTransform(1.0 / scale(), 1.0, -angle());
 
-    if (brushType() == IMAGE || brushType() == PIPE_IMAGE) {
-        dev = paintDevice(KoColorSpaceRegistry::instance()->rgb8(), 1.0 / scale(), -angle(), KisPaintInformation());
+    if (brushApplication() == IMAGESTAMP) {
+        dev = paintDevice(KoColorSpaceRegistry::instance()->rgb8(),
+            inverseTransform, KisPaintInformation());
     }
     else {
         const KoColorSpace* cs = KoColorSpaceRegistry::instance()->rgb8();
         dev = new KisFixedPaintDevice(cs);
-        mask(dev, KoColor(Qt::black, cs) , 1.0 / scale(), 1.0 / scale(), -angle(), KisPaintInformation());
+        mask(dev, KoColor(Qt::black, cs), inverseTransform, KisPaintInformation());
     }
 
     d->boundary = new KisBoundary(dev);
