@@ -38,11 +38,11 @@ class BRUSH_EXPORT KisBrushRegistry : public QObject, public KoGenericRegistry<K
 
 public:
     KisBrushRegistry();
-    virtual ~KisBrushRegistry();
+    ~KisBrushRegistry() override;
 
     static KisBrushRegistry* instance();
 
-    KisBrushSP getOrCreateBrush(const QDomElement& element);
+    KisBrushSP createBrush(const QDomElement& element);
 
 private:
     KisBrushRegistry(const KisBrushRegistry&);
