@@ -57,21 +57,18 @@ public:
     }
 
     BrushType* currentBrush(const KisPaintInformation& info) {
-        return !m_brushes.isEmpty() ? m_brushes.at(chooseNextBrush(info)) : 0;
+        Q_UNUSED(info);
+        return !m_brushes.isEmpty() ? m_brushes.at(currentBrushIndex()) : 0;
     }
 
-    int brushIndex(const KisPaintInformation& info) {
-        return chooseNextBrush(info);
-    }
-
-    qint32 maskWidth(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) {
+    qint32 maskWidth(KisDabShape const& shape, double subPixelX, double subPixelY, const KisPaintInformation& info) {
         BrushType *brush = currentBrush(info);
-        return brush ? brush->maskWidth(scale, angle, subPixelX, subPixelY, info) : 0;
+        return brush ? brush->maskWidth(shape, subPixelX, subPixelY, info) : 0;
     }
 
-    qint32 maskHeight(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) {
+    qint32 maskHeight(KisDabShape const& shape, double subPixelX, double subPixelY, const KisPaintInformation& info) {
         BrushType *brush = currentBrush(info);
-        return brush ? brush->maskHeight(scale, angle, subPixelX, subPixelY, info) : 0;
+        return brush ? brush->maskHeight(shape, subPixelX, subPixelY, info) : 0;
     }
 
     void setAngle(qreal angle) {
@@ -99,34 +96,46 @@ public:
         return false;
     }
 
-    void notifyCachedDabPainted(const KisPaintInformation& info) {
-        updateBrushIndexes(info);
+
+    void setBrushApplication(enumBrushApplication brushApplication) const {
+        Q_FOREACH(BrushType * brush, m_brushes) {
+            brush->setBrushApplication(brushApplication);
+        }
+    }
+
+    void setGradient(const KoAbstractGradient* gradient) const {
+        Q_FOREACH(BrushType * brush, m_brushes) {
+            brush->setGradient(gradient);
+        }
+    }
+
+    void prepareForSeqNo(const KisPaintInformation& info, int seqNo) {
+        chooseNextBrush(info);
+        updateBrushIndexes(info, seqNo);
     }
 
     void generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst, KisBrush::ColoringInformation* coloringInformation,
-            double scaleX, double scaleY, double angle, const KisPaintInformation& info,
+            KisDabShape const& shape,
+            const KisPaintInformation& info,
             double subPixelX , double subPixelY,
-            qreal softnessFactor) {
+            qreal softnessFactor, qreal lightnessStrength = DEFAULT_LIGHTNESS_STRENGTH) {
 
         BrushType *brush = currentBrush(info);
         if (!brush) return;
 
 
-        brush->generateMaskAndApplyMaskOrCreateDab(dst, coloringInformation, scaleX, scaleY, angle, info, subPixelX, subPixelY, softnessFactor);
-        updateBrushIndexes(info);
+        brush->generateMaskAndApplyMaskOrCreateDab(dst, coloringInformation, shape, info, subPixelX, subPixelY, softnessFactor, lightnessStrength);
     }
 
     KisFixedPaintDeviceSP paintDevice(const KoColorSpace * colorSpace,
-                                      double scale, double angle,
+                                      KisDabShape const& shape,
                                       const KisPaintInformation& info,
                                       double subPixelX, double subPixelY) {
 
         BrushType *brush = currentBrush(info);
         if (!brush) return 0;
 
-
-        KisFixedPaintDeviceSP device = brush->paintDevice(colorSpace, scale, angle, info, subPixelX, subPixelY);
-        updateBrushIndexes(info);
+        KisFixedPaintDeviceSP device = brush->paintDevice(colorSpace, shape, info, subPixelX, subPixelY);
         return device;
     }
 
@@ -136,7 +145,7 @@ public:
 
     void testingSelectNextBrush(const KisPaintInformation& info) {
         (void) chooseNextBrush(info);
-        updateBrushIndexes(info);
+        updateBrushIndexes(info, -1);
     }
 
     /**
@@ -151,22 +160,37 @@ protected:
         m_brushes.append(brush);
     }
 
+    int sizeBrush() {
+        return m_brushes.size();
+    }
+
     /**
-     * Returns the index of the brush that corresponds to the current
+     * Returns the index of the next brush that corresponds to the current
      * values of \p info. This method is called *before* the dab is
      * actually painted.
      *
+     */
+    virtual int chooseNextBrush(const KisPaintInformation& info) = 0;
+
+    /**
+     * Returns the current index of the brush
+     * This method is called *before* the dab is
+     * actually painted.
+     *
      * The method is const, so no internal counters of the brush should
      * change during its execution
      */
-    virtual int chooseNextBrush(const KisPaintInformation& info) = 0;
+    virtual int currentBrushIndex() = 0;
 
     /**
      * Updates internal counters of the brush *after* a dab has been
      * painted on the canvas. Some incremental switching of the brushes
      * may me implemented in this method.
+     *
+     * If \p seqNo is equal or greater than zero, then incremental iteration is
+     * overridden by this seqNo value
      */
-    virtual void updateBrushIndexes(const KisPaintInformation& info) = 0;
+    virtual void updateBrushIndexes(const KisPaintInformation& info, int seqNo) = 0;
 
 protected:
     QVector<BrushType*> m_brushes;
