@@ -25,7 +25,8 @@ class KisImageBrushesPipe : public KisBrushesPipe<KisGbrBrush>
 {
 public:
     KisImageBrushesPipe()
-        : m_isInitialized(false)
+        : m_currentBrushIndex(0)
+        , m_isInitialized(false)
     {
     }
 
@@ -46,6 +47,8 @@ protected:
                          const KisPaintInformation& info) {
 
         qreal angle;
+        qreal velocity;
+        qreal capSpeed = 3;
 
         switch (mode) {
         case KisParasite::Constant:
@@ -56,8 +59,8 @@ protected:
             index = static_cast<int>(info.pressure() * (rank - 1) + 0.5);
             break;
         case KisParasite::Angular:
-            // + m_d->PI_2 to be compatible with the gimp
-            angle = info.drawingAngle() + M_PI_2;
+            // + M_PI_2 + M_PI_4 to be compatible with the gimp
+            angle = info.drawingAngle() + M_PI_2 + M_PI_4;
             angle = normalizeAngle(angle);
 
             index = static_cast<int>(angle / (2.0 * M_PI) * rank);
@@ -68,6 +71,16 @@ protected:
         case KisParasite::TiltY:
             index = qRound(info.yTilt() / 2.0 * rank) + rank / 2;
             break;
+        case KisParasite::Velocity:
+            // log is slow, but allows for nicer dab transition
+            velocity = log(info.drawingSpeed() + 1);
+            if (velocity > capSpeed) {
+                velocity = capSpeed;
+            }
+            velocity /= capSpeed;
+            velocity *= (rank - 1) + 0.5;
+            index = qRound(velocity);
+            break;
         default:
             warnImage << "Parasite" << mode << "is not implemented";
             index = 0;
@@ -78,21 +91,23 @@ protected:
 
     static int selectPost(KisParasite::SelectionMode mode,
                           int index, int rank,
-                          const KisPaintInformation& info) {
+                          const KisPaintInformation& info,
+                          int seqNo) {
 
         switch (mode) {
         case KisParasite::Constant: break;
         case KisParasite::Incremental:
-            index = (index + 1) % rank;
+            index = (seqNo >= 0 ? seqNo : (index + 1)) % rank;
             break;
         case KisParasite::Random:
-            index = info.randomSource()->generate(0, rank);
+            index = info.randomSource()->generate(0, rank-1);
             break;
         case KisParasite::Pressure:
         case KisParasite::Angular:
             break;
         case KisParasite::TiltX:
         case KisParasite::TiltY:
+        case KisParasite::Velocity:
             break;
         default:
             warnImage << "Parasite" << mode << "is not implemented";
@@ -102,7 +117,7 @@ protected:
         return index;
     }
 
-    int chooseNextBrush(const KisPaintInformation& info) {
+    int chooseNextBrush(const KisPaintInformation& info) override {
         quint32 brushIndex = 0;
 
         if (!m_isInitialized) {
@@ -113,7 +128,7 @@ protected:
             for (int i = 0; i < m_parasite.dim; i++) {
                 m_parasite.index[i] = 0;
             }
-            updateBrushIndexes(info);
+            updateBrushIndexes(info, 0);
             m_isInitialized = true;
         }
 
@@ -124,21 +139,28 @@ protected:
 
             brushIndex += m_parasite.brushesCount[i] * index;
         }
-        brushIndex %= m_brushes.size();
+        brushIndex %= (quint32)m_brushes.size();
+        m_currentBrushIndex = brushIndex;
         return brushIndex;
     }
 
-    void updateBrushIndexes(const KisPaintInformation& info) {
+    void updateBrushIndexes(const KisPaintInformation& info, int seqNo) override {
         for (int i = 0; i < m_parasite.dim; i++) {
             m_parasite.index[i] = selectPost(m_parasite.selection[i],
                                              m_parasite.index[i],
                                              m_parasite.rank[i],
-                                             info);
+                                             info,
+                                             seqNo);
         }
     }
 
 public:
     using KisBrushesPipe<KisGbrBrush>::addBrush;
+    using KisBrushesPipe<KisGbrBrush>::sizeBrush;
+
+    int currentBrushIndex() override {
+        return m_currentBrushIndex;
+    }
 
     void setParasite(const KisPipeBrushParasite& parasite) {
         m_parasite = parasite;
@@ -148,15 +170,27 @@ public:
         return m_parasite;
     }
 
-    void setUseColorAsMask(bool useColorAsMask) {
+    void setAdjustmentMidPoint(quint8 value) {
         Q_FOREACH (KisGbrBrush * brush, m_brushes) {
-            brush->setUseColorAsMask(useColorAsMask);
+            brush->setAdjustmentMidPoint(value);
         }
     }
 
-    void makeMaskImage() {
+    void setBrightnessAdjustment(qreal value) {
         Q_FOREACH (KisGbrBrush * brush, m_brushes) {
-            brush->makeMaskImage();
+            brush->setBrightnessAdjustment(value);
+        }
+    }
+
+    void setContrastAdjustment(qreal value) {
+        Q_FOREACH (KisGbrBrush * brush, m_brushes) {
+            brush->setContrastAdjustment(value);
+        }
+    }
+
+    void makeMaskImage(bool preserveAlpha) {
+        Q_FOREACH (KisGbrBrush * brush, m_brushes) {
+            brush->makeMaskImage(preserveAlpha);
         }
     }
 
@@ -169,12 +203,13 @@ public:
         return true;
     }
 
-    void notifyStrokeStarted() {
+    void notifyStrokeStarted() override {
         m_isInitialized = false;
     }
 
 private:
     KisPipeBrushParasite m_parasite;
+    int m_currentBrushIndex;
     bool m_isInitialized;
 };
 
@@ -193,7 +228,7 @@ KisImagePipeBrush::KisImagePipeBrush(const QString& filename)
 KisImagePipeBrush::KisImagePipeBrush(const QString& name, int w, int h,
                                      QVector< QVector<KisPaintDevice*> > devices,
                                      QVector<KisParasite::SelectionMode > modes)
-    : KisGbrBrush("")
+    : KisGbrBrush(QString())
     , m_d(new Private())
 {
     Q_ASSERT(devices.count() == modes.count());
@@ -209,6 +244,8 @@ KisImagePipeBrush::KisImagePipeBrush(const QString& name, int w, int h,
     parasite.ncells = devices.at(0).count();
     parasite.rank[0] = parasite.ncells; // ### This can masquerade some bugs, be careful here in the future
     parasite.selection[0] = modes.at(0);
+
+
     // XXX needsmovement!
 
     parasite.setBrushesCount();
@@ -256,7 +293,7 @@ bool KisImagePipeBrush::initFromData(const QByteArray &data)
 
     qint32 i = 0;
 
-    while (data[i] != '\n' && i < data.size()) {
+    while (i < data.size() && data[i] != '\n') {
         line1.append(data[i]);
         i++;
     }
@@ -268,7 +305,7 @@ bool KisImagePipeBrush::initFromData(const QByteArray &data)
 
     // XXX: This stuff is in utf-8, too.
     QByteArray line2;
-    while (data[i] != '\n' && i < data.size()) {
+    while (i < data.size() && data[i] != '\n') {
         line2.append(data[i]);
         i++;
     }
@@ -276,14 +313,16 @@ bool KisImagePipeBrush::initFromData(const QByteArray &data)
     QString paramline = QString::fromUtf8(line2, line2.size());
     qint32 numOfBrushes = paramline.left(paramline.indexOf(' ')).toUInt();
     QString parasiteString = paramline.mid(paramline.indexOf(' ') + 1);
+
     KisPipeBrushParasite parasite = KisPipeBrushParasite(parasiteString);
     parasite.sanitize();
 
+    parasiteSelectionString = parasite.selectionMode; // selection mode to return to UI
+
     m_d->brushesPipe.setParasite(parasite);
     i++; // Skip past the second newline
 
-
-    for (int brushIndex = 0;
+    for (int brushIndex = m_d->brushesPipe.sizeBrush();
             brushIndex < numOfBrushes && i < data.size(); brushIndex++) {
 
         KisGbrBrush* brush = new KisGbrBrush(name() + '_' + QString().setNum(brushIndex),
@@ -299,6 +338,8 @@ bool KisImagePipeBrush::initFromData(const QByteArray &data)
         setWidth(m_d->brushesPipe.firstBrush()->width());
         setHeight(m_d->brushesPipe.firstBrush()->height());
         setBrushTipImage(m_d->brushesPipe.firstBrush()->brushTipImage());
+        setBrushApplication(m_d->brushesPipe.firstBrush()->brushApplication());
+        setBrushType(m_d->brushesPipe.hasColor() ? PIPE_IMAGE : PIPE_MASK);
     }
 
     return true;
@@ -319,7 +360,7 @@ bool KisImagePipeBrush::saveToDevice(QIODevice* dev) const
     char const* name = utf8Name.data();
     int len = qstrlen(name);
 
-    if (m_d->brushesPipe.parasite().dim != 1) {
+    if (m_d->brushesPipe.parasite().dim >= KisPipeBrushParasite::MaxDim) {
         warnImage << "Save to file for pipe brushes with dim != not yet supported!";
         return false;
     }
@@ -354,17 +395,18 @@ void KisImagePipeBrush::notifyStrokeStarted()
     m_d->brushesPipe.notifyStrokeStarted();
 }
 
-void KisImagePipeBrush::notifyCachedDabPainted(const KisPaintInformation& info)
+void KisImagePipeBrush::prepareForSeqNo(const KisPaintInformation &info, int seqNo)
 {
-    m_d->brushesPipe.notifyCachedDabPainted(info);
+    m_d->brushesPipe.prepareForSeqNo(info, seqNo);
 }
 
 void KisImagePipeBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst, KisBrush::ColoringInformation* coloringInformation,
-        double scaleX, double scaleY, double angle, const KisPaintInformation& info,
+        KisDabShape const& shape,
+        const KisPaintInformation& info,
         double subPixelX , double subPixelY,
-        qreal softnessFactor) const
+        qreal softnessFactor, qreal lightnessStrength) const
 {
-    m_d->brushesPipe.generateMaskAndApplyMaskOrCreateDab(dst, coloringInformation, scaleX, scaleY, angle, info, subPixelX, subPixelY, softnessFactor);
+    m_d->brushesPipe.generateMaskAndApplyMaskOrCreateDab(dst, coloringInformation, shape, info, subPixelX, subPixelY, softnessFactor, lightnessStrength);
 }
 
 QVector<KisGbrBrush *> KisImagePipeBrush::brushes() const
@@ -372,31 +414,42 @@ QVector<KisGbrBrush *> KisImagePipeBrush::brushes() const
     return m_d->brushesPipe.brushes();
 }
 
-KisFixedPaintDeviceSP KisImagePipeBrush::paintDevice(const KoColorSpace * colorSpace, double scale, double angle, const KisPaintInformation& info, double subPixelX, double subPixelY) const
+KisFixedPaintDeviceSP KisImagePipeBrush::paintDevice(
+    const KoColorSpace * colorSpace,
+    KisDabShape const& shape,
+    const KisPaintInformation& info, double subPixelX, double subPixelY) const
 {
-    return m_d->brushesPipe.paintDevice(colorSpace, scale, angle, info, subPixelX, subPixelY);
+    return m_d->brushesPipe.paintDevice(colorSpace, shape, info, subPixelX, subPixelY);
 }
 
-enumBrushType KisImagePipeBrush::brushType() const
+QString KisImagePipeBrush::parasiteSelection()
 {
-    return !hasColor() || useColorAsMask() ? PIPE_MASK : PIPE_IMAGE;
+    return parasiteSelectionString;
 }
 
-bool KisImagePipeBrush::hasColor() const
+void KisImagePipeBrush::makeMaskImage(bool preserveAlpha)
 {
-    return m_d->brushesPipe.hasColor();
+    KisGbrBrush::makeMaskImage(preserveAlpha);
+    m_d->brushesPipe.makeMaskImage(preserveAlpha);
+    setBrushType(PIPE_MASK);
 }
 
-void KisImagePipeBrush::makeMaskImage()
+void KisImagePipeBrush::setAdjustmentMidPoint(quint8 value)
 {
-    m_d->brushesPipe.makeMaskImage();
-    setUseColorAsMask(false);
+    KisGbrBrush::setAdjustmentMidPoint(value);
+    m_d->brushesPipe.setAdjustmentMidPoint(value);
 }
 
-void KisImagePipeBrush::setUseColorAsMask(bool useColorAsMask)
+void KisImagePipeBrush::setBrightnessAdjustment(qreal value)
 {
-    KisGbrBrush::setUseColorAsMask(useColorAsMask);
-    m_d->brushesPipe.setUseColorAsMask(useColorAsMask);
+    KisGbrBrush::setBrightnessAdjustment(value);
+    m_d->brushesPipe.setBrightnessAdjustment(value);
+}
+
+void KisImagePipeBrush::setContrastAdjustment(qreal value)
+{
+    KisGbrBrush::setContrastAdjustment(value);
+    m_d->brushesPipe.setContrastAdjustment(value);
 }
 
 const KisBoundary* KisImagePipeBrush::boundary() const
@@ -422,19 +475,19 @@ QString KisImagePipeBrush::defaultFileExtension() const
     return QString(".gih");
 }
 
-quint32 KisImagePipeBrush::brushIndex(const KisPaintInformation& info) const
+quint32 KisImagePipeBrush::brushIndex() const
 {
-    return m_d->brushesPipe.brushIndex(info);
+    return m_d->brushesPipe.currentBrushIndex();
 }
 
-qint32 KisImagePipeBrush::maskWidth(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) const
+qint32 KisImagePipeBrush::maskWidth(KisDabShape const& shape, double subPixelX, double subPixelY, const KisPaintInformation& info) const
 {
-    return m_d->brushesPipe.maskWidth(scale, angle, subPixelX, subPixelY, info);
+    return m_d->brushesPipe.maskWidth(shape, subPixelX, subPixelY, info);
 }
 
-qint32 KisImagePipeBrush::maskHeight(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) const
+qint32 KisImagePipeBrush::maskHeight(KisDabShape const& shape, double subPixelX, double subPixelY, const KisPaintInformation& info) const
 {
-    return m_d->brushesPipe.maskHeight(scale, angle, subPixelX, subPixelY, info);
+    return m_d->brushesPipe.maskHeight(shape, subPixelX, subPixelY, info);
 }
 
 void KisImagePipeBrush::setAngle(qreal _angle)
@@ -455,18 +508,17 @@ void KisImagePipeBrush::setSpacing(double _spacing)
     m_d->brushesPipe.setSpacing(_spacing);
 }
 
-void KisImagePipeBrush::setBrushType(enumBrushType type)
+void KisImagePipeBrush::setBrushApplication(enumBrushApplication brushApplication)
 {
-    Q_UNUSED(type);
-    qFatal("FATAL: protected member setBrushType has no meaning for KisImagePipeBrush");
-    // brushType() is a function of hasColor() and useColorAsMask()
+    //Set all underlying brushes to use the same brush Application
+    KisGbrBrush::setBrushApplication(brushApplication);
+    m_d->brushesPipe.setBrushApplication(brushApplication);
 }
 
-void KisImagePipeBrush::setHasColor(bool hasColor)
-{
-    Q_UNUSED(hasColor);
-    qFatal("FATAL: protected member setHasColor has no meaning for KisImagePipeBrush");
-    // hasColor() is a function of the underlying brushes
+void KisImagePipeBrush::setGradient(const KoAbstractGradient* gradient) {
+    //Set all underlying brushes to use the same gradient
+    KisGbrBrush::setGradient(gradient);
+    m_d->brushesPipe.setGradient(gradient);
 }
 
 KisGbrBrush* KisImagePipeBrush::testingGetCurrentBrush(const KisPaintInformation& info) const
