@@ -60,67 +60,66 @@ public:
                       QVector< QVector<KisPaintDevice*> > devices,
                       QVector<KisParasite::SelectionMode> modes);
 
-    virtual ~KisImagePipeBrush();
+    ~KisImagePipeBrush() override;
 
-    virtual bool load();
-    virtual bool loadFromDevice(QIODevice *dev);
-    virtual bool save();
-    virtual bool saveToDevice(QIODevice* dev) const;
+    bool load() override;
+    bool loadFromDevice(QIODevice *dev) override;
+    bool save() override;
+    bool saveToDevice(QIODevice* dev) const override;
 
     /**
      * @return the next image in the pipe.
     */
-    virtual KisFixedPaintDeviceSP paintDevice(const KoColorSpace * colorSpace,
-            double scale, double angle,
+    KisFixedPaintDeviceSP paintDevice(const KoColorSpace * colorSpace,
+            KisDabShape const&,
             const KisPaintInformation& info,
-            double subPixelX = 0, double subPixelY = 0) const;
+            double subPixelX = 0, double subPixelY = 0) const override;
 
-    virtual void setUseColorAsMask(bool useColorAsMask);
-    virtual bool hasColor() const;
+    void setAdjustmentMidPoint(quint8 value) override;
+    void setBrightnessAdjustment(qreal value) override;
+    void setContrastAdjustment(qreal value) override;
 
-    virtual enumBrushType brushType() const;
+    QString parasiteSelection(); // returns random, constant, etc
 
-    virtual const KisBoundary* boundary() const;
+    const KisBoundary* boundary() const override;
 
-    virtual bool canPaintFor(const KisPaintInformation& info);
+    bool canPaintFor(const KisPaintInformation& info) override;
 
-    virtual void makeMaskImage();
+    void makeMaskImage(bool preserveAlpha) override;
 
-    virtual KisBrush* clone() const;
+    KisBrush* clone() const override;
 
-    virtual QString defaultFileExtension() const;
-    void setAngle(qreal _angle);
-    void setScale(qreal _scale);
-    void setSpacing(double _spacing);
+    QString defaultFileExtension() const override;
+    void setAngle(qreal _angle) override;
+    void setScale(qreal _scale) override;
+    void setSpacing(double _spacing) override;
 
-    quint32 brushIndex(const KisPaintInformation& info) const;
-    qint32 maskWidth(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) const;
-    qint32 maskHeight(double scale, double angle, double subPixelX, double subPixelY, const KisPaintInformation& info) const;
+    quint32 brushIndex() const override;
+    qint32 maskWidth(KisDabShape const&, double subPixelX, double subPixelY, const KisPaintInformation& info) const override;
+    qint32 maskHeight(KisDabShape const&, double subPixelX, double subPixelY, const KisPaintInformation& info) const override;
 
-    void notifyStrokeStarted();
-    void notifyCachedDabPainted(const KisPaintInformation& info);
+    void notifyStrokeStarted() override;
+    void prepareForSeqNo(const KisPaintInformation& info, int seqNo) override;
 
     void generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst, KisBrush::ColoringInformation* coloringInformation,
-            double scaleX, double scaleY, double angle,
+            KisDabShape const&,
             const KisPaintInformation& info,
-            double subPixelX = 0, double subPixelY = 0, qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR) const;
+            double subPixelX = 0, double subPixelY = 0, 
+            qreal softnessFactor = DEFAULT_SOFTNESS_FACTOR, qreal lightnessStrength = DEFAULT_LIGHTNESS_STRENGTH) const override;
 
 
     QVector<KisGbrBrush*> brushes() const;
 
-
     const KisPipeBrushParasite &parasite() const;
 
     void setParasite(const KisPipeBrushParasite& parasite);
     void setDevices(QVector< QVector<KisPaintDevice*> > devices, int w, int h);
 
 protected:
-    void setBrushType(enumBrushType type);
-    void setHasColor(bool hasColor);
+    virtual void setBrushApplication(enumBrushApplication brushApplication) override;
+    virtual void setGradient(const KoAbstractGradient* gradient) override;
     /// Will call KisBrush's saveToDevice as well
 
-
-
     KisImagePipeBrush(const KisImagePipeBrush& rhs);
 
 private:
@@ -131,6 +130,8 @@ private:
 
     bool initFromData(const QByteArray &data);
 
+    QString parasiteSelectionString; // incremental, random, etc.
+
 private:
     struct Private;
     Private * const m_d;
