@@ -131,10 +131,9 @@
     \sa ~KUndo2Command()
 */
 
-KUndo2Command::KUndo2Command(const KUndo2MagicString &text, KUndo2Command *parent):
-    m_hasParent(parent != 0),
-    m_timedID(-1),
-    m_endOfCommand(QTime::currentTime())
+KUndo2Command::KUndo2Command(const KUndo2MagicString &text, KUndo2Command *parent)
+    : m_hasParent(parent != 0)
+    , m_endOfCommand(QTime::currentTime())
 {
     d = new KUndo2CommandPrivate;
     if (parent != 0) {
@@ -154,9 +153,8 @@ KUndo2Command::KUndo2Command(const KUndo2MagicString &text, KUndo2Command *paren
     \sa ~KUndo2Command()
 */
 
-KUndo2Command::KUndo2Command(KUndo2Command *parent):
-    m_hasParent(parent != 0),
-    m_timedID(-1)
+KUndo2Command::KUndo2Command(KUndo2Command *parent)
+    : m_hasParent(parent != 0)
 {
     d = new KUndo2CommandPrivate;
     if (parent != 0)
