@@ -32,7 +32,7 @@
 
 class KoShape;
 class KoCanvasBase;
-
+class KoCanvasSupervisor;
 /**
  * KoCanvasController implementation for QWidget based canvases
  */
@@ -46,7 +46,7 @@ public:
      * @param actionCollection the action collection for this widget
      * @param parent the parent this widget will belong to
      */
-    explicit KoCanvasControllerWidget(KActionCollection * actionCollection, QWidget *parent = 0);
+    explicit KoCanvasControllerWidget(KActionCollection * actionCollection, KoCanvasSupervisor *observerProvider, QWidget *parent = 0);
     ~KoCanvasControllerWidget() override;
 
     /**
@@ -183,6 +183,8 @@ protected:
     void wheelEvent(QWheelEvent *event) override;
     /// reimplemented from QWidget
     bool focusNextPrevChild(bool next) override;
+    /// reimplemented from QAbstractScrollArea
+    bool viewportEvent(QEvent *event) override;
 
 private:
     Q_PRIVATE_SLOT(d, void activate())
