@@ -27,7 +27,6 @@
 #include "KoOdfWorkaround.h"
 #include "KoPathPoint.h"
 #include "KoShapeStrokeModel.h"
-#include "KoViewConverter.h"
 #include "KoPathShapeLoader.h"
 #include "KoShapeSavingContext.h"
 #include "KoShapeLoadingContext.h"
@@ -50,6 +49,7 @@
 
 #include <FlakeDebug.h>
 #include <QPainter>
+#include <QPainterPath>
 
 #include "kis_global.h"
 
@@ -58,36 +58,25 @@ static bool qIsNaNPoint(const QPointF &p) {
     return qIsNaN(p.x()) || qIsNaN(p.y());
 }
 
-KoPathShapePrivate::KoPathShapePrivate(KoPathShape *q)
-    : KoTosContainerPrivate(q),
-      fillRule(Qt::OddEvenFill),
-      autoFillMarkers(false)
+KoPathShape::Private::Private()
+    : fillRule(Qt::OddEvenFill)
+    , autoFillMarkers(false)
 {
 }
 
-KoPathShapePrivate::KoPathShapePrivate(const KoPathShapePrivate &rhs, KoPathShape *q)
-    : KoTosContainerPrivate(rhs, q),
-      fillRule(rhs.fillRule),
-      markersNew(rhs.markersNew),
-      autoFillMarkers(rhs.autoFillMarkers)
+KoPathShape::Private::Private(const Private &rhs)
+    : fillRule(rhs.fillRule)
+    , markersNew(rhs.markersNew)
+    , autoFillMarkers(rhs.autoFillMarkers)
 {
-    Q_FOREACH (KoSubpath *subPath, rhs.subpaths) {
-        KoSubpath *clonedSubPath = new KoSubpath();
-
-        Q_FOREACH (KoPathPoint *point, *subPath) {
-            *clonedSubPath << new KoPathPoint(*point, q);
-        }
-
-        subpaths << clonedSubPath;
-    }
 }
 
-QRectF KoPathShapePrivate::handleRect(const QPointF &p, qreal radius) const
+QRectF KoPathShape::Private::handleRect(const QPointF &p, qreal radius) const
 {
     return QRectF(p.x() - radius, p.y() - radius, 2*radius, 2*radius);
 }
 
-void KoPathShapePrivate::applyViewboxTransformation(const KoXmlElement &element)
+void KoPathShape::Private::applyViewboxTransformation(const KoXmlElement &element)
 {
     // apply viewbox transformation
     const QRect viewBox = KoPathShape::loadOdfViewbox(element);
@@ -114,18 +103,28 @@ void KoPathShapePrivate::applyViewboxTransformation(const KoXmlElement &element)
 }
 
 KoPathShape::KoPathShape()
-    :KoTosContainer(new KoPathShapePrivate(this))
-{
-}
-
-KoPathShape::KoPathShape(KoPathShapePrivate *dd)
-    : KoTosContainer(dd)
+    : KoTosContainer()
+    , d(new Private)
 {
 }
 
 KoPathShape::KoPathShape(const KoPathShape &rhs)
-    : KoTosContainer(new KoPathShapePrivate(*rhs.d_func(), this))
+    : KoTosContainer(rhs)
+    , d(new Private(*rhs.d))
 {
+    // local data cannot be shared via QSharedData because
+    // every path point holds a pointer to the parent shape
+    KoSubpathList subpaths;
+    Q_FOREACH (KoSubpath *subPath, rhs.d->subpaths) {
+        KoSubpath *clonedSubPath = new KoSubpath();
+
+        Q_FOREACH (KoPathPoint *point, *subPath) {
+            *clonedSubPath << new KoPathPoint(*point, this);
+        }
+
+        subpaths << clonedSubPath;
+    }
+    d->subpaths = subpaths;
 }
 
 KoPathShape::~KoPathShape()
@@ -140,8 +139,6 @@ KoShape *KoPathShape::cloneShape() const
 
 void KoPathShape::saveContourOdf(KoShapeSavingContext &context, const QSizeF &scaleFactor) const
 {
-    Q_D(const KoPathShape);
-
     if (d->subpaths.length() <= 1) {
         QTransform matrix;
         matrix.scale(scaleFactor.width(), scaleFactor.height());
@@ -191,7 +188,6 @@ void KoPathShape::saveContourOdf(KoShapeSavingContext &context, const QSizeF &sc
 
 void KoPathShape::saveOdf(KoShapeSavingContext & context) const
 {
-    Q_D(const KoPathShape);
     context.xmlWriter().startElement("draw:path");
     saveOdfAttributes(context, OdfAllAttributes | OdfViewbox);
 
@@ -205,8 +201,6 @@ void KoPathShape::saveOdf(KoShapeSavingContext & context) const
 
 bool KoPathShape::loadContourOdf(const KoXmlElement &element, KoShapeLoadingContext &, const QSizeF &scaleFactor)
 {
-    Q_D(KoPathShape);
-
     // first clear the path data from the default path
     clear();
 
@@ -258,7 +252,6 @@ bool KoPathShape::loadContourOdf(const KoXmlElement &element, KoShapeLoadingCont
 
 bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
-    Q_D(KoPathShape);
     loadOdfAttributes(element, context, OdfMandatories | OdfAdditionalAttributes | OdfCommonChildElements);
 
     // first clear the path data from the default path
@@ -326,8 +319,6 @@ bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &c
 
 QString KoPathShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const
 {
-    Q_D(const KoPathShape);
-
     style.addProperty("svg:fill-rule", d->fillRule == Qt::OddEvenFill ? "evenodd" : "nonzero");
 
     QSharedPointer<KoShapeStroke> lineBorder = qSharedPointerDynamicCast<KoShapeStroke>(stroke());
@@ -343,7 +334,6 @@ QString KoPathShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context)
 
 void KoPathShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
-    Q_D(KoPathShape);
     KoTosContainer::loadStyle(element, context);
 
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
@@ -387,8 +377,6 @@ QRect KoPathShape::loadOdfViewbox(const KoXmlElement & element)
 
 void KoPathShape::clear()
 {
-    Q_D(KoPathShape);
-
     Q_FOREACH (KoSubpath *subpath, d->subpaths) {
         Q_FOREACH (KoPathPoint *point, *subpath)
             delete point;
@@ -399,27 +387,23 @@ void KoPathShape::clear()
     notifyPointsChanged();
 }
 
-void KoPathShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+void KoPathShape::paint(QPainter &painter, KoShapePaintingContext &paintContext) const
 {
-    Q_D(KoPathShape);
-
     KisQPainterStateSaver saver(&painter);
 
-    applyConversion(painter, converter);
     QPainterPath path(outline());
     path.setFillRule(d->fillRule);
 
     if (background()) {
-        background()->paint(painter, converter, paintContext, path);
+        background()->paint(painter, paintContext, path);
     }
     //d->paintDebug(painter);
 }
 
 
 #ifndef NDEBUG
-void KoPathShapePrivate::paintDebug(QPainter &painter)
+void KoPathShape::Private::paintDebug(QPainter &painter)
 {
-    Q_Q(KoPathShape);
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
     int i = 0;
 
@@ -452,9 +436,8 @@ void KoPathShapePrivate::paintDebug(QPainter &painter)
     debugFlake << "nop =" << i;
 }
 
-void KoPathShapePrivate::debugPath() const
+void KoPathShape::Private::debugPath() const
 {
-    Q_Q(const KoPathShape);
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
     for (; pathIt != subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
@@ -467,8 +450,6 @@ void KoPathShapePrivate::debugPath() const
 
 void KoPathShape::paintPoints(KisHandlePainterHelper &handlesHelper)
 {
-    Q_D(KoPathShape);
-
     KoSubpathList::const_iterator pathIt(d->subpaths.constBegin());
 
     for (; pathIt != d->subpaths.constEnd(); ++pathIt) {
@@ -485,8 +466,6 @@ QRectF KoPathShape::outlineRect() const
 
 QPainterPath KoPathShape::outline() const
 {
-    Q_D(const KoPathShape);
-
     QPainterPath path;
     for (auto subpathIt = d->subpaths.constBegin(); subpathIt != d->subpaths.constEnd(); ++subpathIt) {
         const KoSubpath * subpath = *subpathIt;
@@ -552,7 +531,7 @@ QPainterPath KoPathShape::outline() const
 
 QRectF KoPathShape::boundingRect() const
 {
-    const QTransform transform = absoluteTransformation(0);
+    const QTransform transform = absoluteTransformation();
 
     /**
      * First we approximate the insets of the stroke by rendering a fat bezier curve
@@ -578,14 +557,23 @@ QRectF KoPathShape::boundingRect() const
                                       2.0 * stroke()->strokeMaxMarkersInset(this)});
     }
 
+
+
+    /// NOTE: stroking the entire shape might be too expensive, so try to
+    ///       estimate the bounds using insets only...
+
+#if 0
     QPen pen(Qt::black, outlineSweepWidth);
 
     // select round joins and caps to ensure it sweeps exactly
     // 'outlineSweepWidth' pixels in every possible
     pen.setJoinStyle(Qt::RoundJoin);
     pen.setCapStyle(Qt::RoundCap);
-
     QRectF bb = transform.map(pathStroke(pen)).boundingRect();
+#endif
+
+    // add 10% extra update area around the doubled insets
+    QRectF bb = transform.mapRect(kisGrowRect(outline().boundingRect(), 1.1 * 0.5 * outlineSweepWidth));
 
     if (shadow()) {
         KoInsets insets;
@@ -608,7 +596,6 @@ QSizeF KoPathShape::size() const
 
 void KoPathShape::setSize(const QSizeF &newSize)
 {
-    Q_D(KoPathShape);
     QTransform matrix(resizeMatrix(newSize));
 
     KoShape::setSize(newSize);
@@ -638,8 +625,6 @@ QTransform KoPathShape::resizeMatrix(const QSizeF & newSize) const
 
 KoPathPoint * KoPathShape::moveTo(const QPointF &p)
 {
-    Q_D(KoPathShape);
-
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StartSubpath | KoPathPoint::StopSubpath);
     KoSubpath * path = new KoSubpath;
     path->push_back(point);
@@ -650,13 +635,12 @@ KoPathPoint * KoPathShape::moveTo(const QPointF &p)
 
 KoPathPoint * KoPathShape::lineTo(const QPointF &p)
 {
-    Q_D(KoPathShape);
     if (d->subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     KoPathPoint * lastPoint = d->subpaths.last()->last();
-    d->updateLast(&lastPoint);
+    updateLastPriv(&lastPoint);
     d->subpaths.last()->push_back(point);
     notifyPointsChanged();
     return point;
@@ -664,12 +648,11 @@ KoPathPoint * KoPathShape::lineTo(const QPointF &p)
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const QPointF &p)
 {
-    Q_D(KoPathShape);
     if (d->subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * lastPoint = d->subpaths.last()->last();
-    d->updateLast(&lastPoint);
+    updateLastPriv(&lastPoint);
     lastPoint->setControlPoint2(c1);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     point->setControlPoint1(c2);
@@ -680,12 +663,11 @@ KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const Q
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c, const QPointF &p)
 {
-    Q_D(KoPathShape);
     if (d->subpaths.empty())
         moveTo(QPointF(0, 0));
 
     KoPathPoint * lastPoint = d->subpaths.last()->last();
-    d->updateLast(&lastPoint);
+    updateLastPriv(&lastPoint);
     lastPoint->setControlPoint2(c);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     d->subpaths.last()->push_back(point);
@@ -695,8 +677,6 @@ KoPathPoint * KoPathShape::curveTo(const QPointF &c, const QPointF &p)
 
 KoPathPoint * KoPathShape::arcTo(qreal rx, qreal ry, qreal startAngle, qreal sweepAngle)
 {
-    Q_D(KoPathShape);
-
     if (d->subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
@@ -777,25 +757,22 @@ int KoPathShape::arcToCurve(qreal rx, qreal ry, qreal startAngle, qreal sweepAng
 
 void KoPathShape::close()
 {
-    Q_D(KoPathShape);
     if (d->subpaths.empty()) {
         return;
     }
-    d->closeSubpath(d->subpaths.last());
+    closeSubpathPriv(d->subpaths.last());
 }
 
 void KoPathShape::closeMerge()
 {
-    Q_D(KoPathShape);
     if (d->subpaths.empty()) {
         return;
     }
-    d->closeMergeSubpath(d->subpaths.last());
+    closeMergeSubpathPriv(d->subpaths.last());
 }
 
 QPointF KoPathShape::normalize()
 {
-    Q_D(KoPathShape);
     QPointF tl(outline().boundingRect().topLeft());
     QTransform matrix;
     matrix.translate(-tl.x(), -tl.y());
@@ -803,11 +780,11 @@ QPointF KoPathShape::normalize()
 
     // keep the top left point of the object
     applyTransformation(matrix.inverted());
-    d->shapeChanged(ContentChanged);
+    shapeChangedPriv(ContentChanged);
     return tl;
 }
 
-void KoPathShapePrivate::map(const QTransform &matrix)
+void KoPathShape::Private::map(const QTransform &matrix)
 {
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
     for (; pathIt != subpaths.constEnd(); ++pathIt) {
@@ -821,22 +798,21 @@ void KoPathShapePrivate::map(const QTransform &matrix)
     }
 }
 
-void KoPathShapePrivate::updateLast(KoPathPoint **lastPoint)
+void KoPathShape::updateLastPriv(KoPathPoint **lastPoint)
 {
-    Q_Q(KoPathShape);
     // check if we are about to add a new point to a closed subpath
     if ((*lastPoint)->properties() & KoPathPoint::StopSubpath
             && (*lastPoint)->properties() & KoPathPoint::CloseSubpath) {
         // get the first point of the subpath
-        KoPathPoint *subpathStart = subpaths.last()->first();
+        KoPathPoint *subpathStart = d->subpaths.last()->first();
         // clone the first point of the subpath...
-        KoPathPoint * newLastPoint = new KoPathPoint(*subpathStart, q);
+        KoPathPoint * newLastPoint = new KoPathPoint(*subpathStart, this);
         // ... and make it a normal point
         newLastPoint->setProperties(KoPathPoint::Normal);
         // now start a new subpath with the cloned start point
         KoSubpath *path = new KoSubpath;
         path->push_back(newLastPoint);
-        subpaths.push_back(path);
+        d->subpaths.push_back(path);
         *lastPoint = newLastPoint;
     } else {
         // the subpath was not closed so the formerly last point
@@ -848,8 +824,6 @@ void KoPathShapePrivate::updateLast(KoPathPoint **lastPoint)
 
 QList<KoPathPoint*> KoPathShape::pointsAt(const QRectF &r) const
 {
-    Q_D(const KoPathShape);
-
     QList<KoPathPoint*> result;
 
     KoSubpathList::const_iterator pathIt(d->subpaths.constBegin());
@@ -869,8 +843,6 @@ QList<KoPathPoint*> KoPathShape::pointsAt(const QRectF &r) const
 
 QList<KoPathSegment> KoPathShape::segmentsAt(const QRectF &r) const
 {
-    Q_D(const KoPathShape);
-
     QList<KoPathSegment> segments;
     int subpathCount = d->subpaths.count();
     for (int subpathIndex = 0; subpathIndex < subpathCount; ++subpathIndex) {
@@ -896,8 +868,6 @@ QList<KoPathSegment> KoPathShape::segmentsAt(const QRectF &r) const
 
 KoPathPointIndex KoPathShape::pathPointIndex(const KoPathPoint *point) const
 {
-    Q_D(const KoPathShape);
-
     for (int subpathIndex = 0; subpathIndex < d->subpaths.size(); ++subpathIndex) {
         KoSubpath * subpath = d->subpaths.at(subpathIndex);
         for (int pointPos = 0; pointPos < subpath->size(); ++pointPos) {
@@ -911,7 +881,6 @@ KoPathPointIndex KoPathShape::pathPointIndex(const KoPathPoint *point) const
 
 KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) const
 {
-    Q_D(const KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
@@ -922,7 +891,6 @@ KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) cons
 
 KoPathSegment KoPathShape::segmentByIndex(const KoPathPointIndex &pointIndex) const
 {
-    Q_D(const KoPathShape);
     KoPathSegment segment(0, 0);
 
     KoSubpath *subpath = d->subPath(pointIndex.first);
@@ -945,8 +913,6 @@ KoPathSegment KoPathShape::segmentByIndex(const KoPathPointIndex &pointIndex) co
 
 int KoPathShape::pointCount() const
 {
-    Q_D(const KoPathShape);
-
     int i = 0;
     KoSubpathList::const_iterator pathIt(d->subpaths.constBegin());
     for (; pathIt != d->subpaths.constEnd(); ++pathIt) {
@@ -958,14 +924,11 @@ int KoPathShape::pointCount() const
 
 int KoPathShape::subpathCount() const
 {
-    Q_D(const KoPathShape);
-
     return d->subpaths.count();
 }
 
 int KoPathShape::subpathPointCount(int subpathIndex) const
 {
-    Q_D(const KoPathShape);
     KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
@@ -976,7 +939,6 @@ int KoPathShape::subpathPointCount(int subpathIndex) const
 
 bool KoPathShape::isClosedSubpath(int subpathIndex) const
 {
-    Q_D(const KoPathShape);
     KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
@@ -990,7 +952,6 @@ bool KoPathShape::isClosedSubpath(int subpathIndex) const
 
 bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second > subpath->size())
@@ -1033,7 +994,6 @@ bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointI
 
 KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
@@ -1074,7 +1034,6 @@ KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second > subpath->size() - 2
@@ -1101,7 +1060,6 @@ bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::join(int subpathIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(subpathIndex);
     KoSubpath *nextSubpath = d->subPath(subpathIndex + 1);
 
@@ -1131,7 +1089,6 @@ bool KoPathShape::join(int subpathIndex)
 
 bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(oldSubpathIndex);
 
     if (subpath == 0 || newSubpathIndex >= d->subpaths.size())
@@ -1150,7 +1107,6 @@ bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 
 KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
@@ -1179,7 +1135,6 @@ KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
 
 KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
@@ -1199,7 +1154,7 @@ KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
     subpath->first()->setProperty(KoPathPoint::StartSubpath);
     subpath->last()->setProperty(KoPathPoint::StopSubpath);
 
-    d->closeSubpath(subpath);
+    closeSubpathPriv(subpath);
 
     notifyPointsChanged();
 
@@ -1208,7 +1163,6 @@ KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::reverseSubpath(int subpathIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
@@ -1246,7 +1200,6 @@ bool KoPathShape::reverseSubpath(int subpathIndex)
 
 KoSubpath * KoPathShape::removeSubpath(int subpathIndex)
 {
-    Q_D(KoPathShape);
     KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath != 0) {
@@ -1263,8 +1216,6 @@ KoSubpath * KoPathShape::removeSubpath(int subpathIndex)
 
 bool KoPathShape::addSubpath(KoSubpath * subpath, int subpathIndex)
 {
-    Q_D(KoPathShape);
-
     if (subpathIndex < 0 || subpathIndex > d->subpaths.size())
         return false;
 
@@ -1280,14 +1231,13 @@ bool KoPathShape::addSubpath(KoSubpath * subpath, int subpathIndex)
 }
 int KoPathShape::combine(KoPathShape *path)
 {
-    Q_D(KoPathShape);
     int insertSegmentPosition = -1;
     if (!path) return insertSegmentPosition;
 
-    QTransform pathMatrix = path->absoluteTransformation(0);
-    QTransform myMatrix = absoluteTransformation(0).inverted();
+    QTransform pathMatrix = path->absoluteTransformation();
+    QTransform myMatrix = absoluteTransformation().inverted();
 
-    Q_FOREACH (KoSubpath* subpath, path->d_func()->subpaths) {
+    Q_FOREACH (KoSubpath* subpath, path->d->subpaths) {
         KoSubpath *newSubpath = new KoSubpath();
 
         Q_FOREACH (KoPathPoint* point, *subpath) {
@@ -1310,12 +1260,10 @@ int KoPathShape::combine(KoPathShape *path)
 
 bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
 {
-    Q_D(KoPathShape);
-
     if (! d->subpaths.size())
         return false;
 
-    QTransform myMatrix = absoluteTransformation(0);
+    QTransform myMatrix = absoluteTransformation();
 
     Q_FOREACH (KoSubpath* subpath, d->subpaths) {
         KoPathShape *shape = new KoPathShape();
@@ -1332,7 +1280,7 @@ bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
             newPoint->map(myMatrix);
             newSubpath->append(newPoint);
         }
-        shape->d_func()->subpaths.append(newSubpath);
+        shape->d->subpaths.append(newSubpath);
         shape->normalize();
 
         // NOTE: shape cannot have any listeners yet, so no notification about
@@ -1343,23 +1291,19 @@ bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
     return true;
 }
 
-void KoPathShapePrivate::closeSubpath(KoSubpath *subpath)
+void KoPathShape::closeSubpathPriv(KoSubpath *subpath)
 {
-    Q_Q(KoPathShape);
-
     if (! subpath)
         return;
 
     subpath->last()->setProperty(KoPathPoint::CloseSubpath);
     subpath->first()->setProperty(KoPathPoint::CloseSubpath);
 
-    q->notifyPointsChanged();
+    notifyPointsChanged();
 }
 
-void KoPathShapePrivate::closeMergeSubpath(KoSubpath *subpath)
+void KoPathShape::closeMergeSubpathPriv(KoSubpath *subpath)
 {
-    Q_Q(KoPathShape);
-
     if (! subpath || subpath->size() < 2)
         return;
 
@@ -1381,13 +1325,28 @@ void KoPathShapePrivate::closeMergeSubpath(KoSubpath *subpath)
         lastPoint->setProperty(KoPathPoint::StopSubpath);
         lastPoint->setProperty(KoPathPoint::CloseSubpath);
 
-        q->notifyPointsChanged();
+        notifyPointsChanged();
     } else {
-        closeSubpath(subpath);
+        closeSubpathPriv(subpath);
     }
 }
 
-KoSubpath *KoPathShapePrivate::subPath(int subpathIndex) const
+const KoSubpathList &KoPathShape::subpaths() const
+{
+    return d->subpaths;
+}
+
+KoSubpathList &KoPathShape::subpaths()
+{
+    return d->subpaths;
+}
+
+void KoPathShape::map(const QTransform &matrix)
+{
+    return d->map(matrix);
+}
+
+KoSubpath *KoPathShape::Private::subPath(int subpathIndex) const
 {
     if (subpathIndex < 0 || subpathIndex >= subpaths.size())
         return 0;
@@ -1402,8 +1361,6 @@ QString KoPathShape::pathShapeId() const
 
 QString KoPathShape::toString(const QTransform &matrix) const
 {
-    Q_D(const KoPathShape);
-
     QString pathString;
 
     // iterate over all subpaths
@@ -1497,7 +1454,7 @@ char nodeType(const KoPathPoint * point)
     }
 }
 
-QString KoPathShapePrivate::nodeTypes() const
+QString KoPathShape::Private::nodeTypes() const
 {
     QString types;
     KoSubpathList::const_iterator pathIt(subpaths.constBegin());
@@ -1531,7 +1488,7 @@ void updateNodeType(KoPathPoint * point, const QChar & nodeType)
     }
 }
 
-void KoPathShapePrivate::loadNodeTypes(const KoXmlElement &element)
+void KoPathShape::Private::loadNodeTypes(const KoXmlElement &element)
 {
     if (element.hasAttributeNS(KoXmlNS::calligra, "nodeTypes")) {
         QString nodeTypes = element.attributeNS(KoXmlNS::calligra, "nodeTypes");
@@ -1562,13 +1519,11 @@ void KoPathShapePrivate::loadNodeTypes(const KoXmlElement &element)
 
 Qt::FillRule KoPathShape::fillRule() const
 {
-    Q_D(const KoPathShape);
     return d->fillRule;
 }
 
 void KoPathShape::setFillRule(Qt::FillRule fillRule)
 {
-    Q_D(KoPathShape);
     d->fillRule = fillRule;
 }
 
@@ -1607,7 +1562,7 @@ bool KoPathShape::hitTest(const QPointF &position) const
     if (parent() && parent()->isClipped(this) && ! parent()->hitTest(position))
         return false;
 
-    QPointF point = absoluteTransformation(0).inverted().map(position);
+    QPointF point = absoluteTransformation().inverted().map(position);
     const QPainterPath outlinePath = outline();
     if (stroke()) {
         KoInsets insets;
@@ -1628,51 +1583,46 @@ bool KoPathShape::hitTest(const QPointF &position) const
 
     // the shadow has an offset to the shape, so we simply
     // check if the position minus the shadow offset hits the shape
-    point = absoluteTransformation(0).inverted().map(position - shadow()->offset());
+    point = absoluteTransformation().inverted().map(position - shadow()->offset());
 
     return outlinePath.contains(point);
 }
 
 void KoPathShape::setMarker(KoMarker *marker, KoFlake::MarkerPosition pos)
 {
-    Q_D(KoPathShape);
-
     if (!marker && d->markersNew.contains(pos)) {
         d->markersNew.remove(pos);
     } else {
         d->markersNew[pos] = marker;
     }
+
+    notifyChanged();
+    shapeChangedPriv(StrokeChanged);
 }
 
 KoMarker *KoPathShape::marker(KoFlake::MarkerPosition pos) const
 {
-    Q_D(const KoPathShape);
     return d->markersNew[pos].data();
 }
 
 bool KoPathShape::hasMarkers() const
 {
-    Q_D(const KoPathShape);
     return !d->markersNew.isEmpty();
 }
 
 bool KoPathShape::autoFillMarkers() const
 {
-    Q_D(const KoPathShape);
     return d->autoFillMarkers;
 }
 
 void KoPathShape::setAutoFillMarkers(bool value)
 {
-    Q_D(KoPathShape);
     d->autoFillMarkers = value;
 }
 
 void KoPathShape::recommendPointSelectionChange(const QList<KoPathPointIndex> &newSelection)
 {
-    Q_D(KoShape);
-
-    Q_FOREACH (KoShape::ShapeChangeListener *listener, d->listeners) {
+    Q_FOREACH (KoShape::ShapeChangeListener *listener, listeners()) {
         PointSelectionChangeListener *pointListener = dynamic_cast<PointSelectionChangeListener*>(listener);
         if (pointListener) {
             pointListener->recommendPointSelectionChange(this, newSelection);
@@ -1682,9 +1632,7 @@ void KoPathShape::recommendPointSelectionChange(const QList<KoPathPointIndex> &n
 
 void KoPathShape::notifyPointsChanged()
 {
-    Q_D(KoShape);
-
-    Q_FOREACH (KoShape::ShapeChangeListener *listener, d->listeners) {
+    Q_FOREACH (KoShape::ShapeChangeListener *listener, listeners()) {
         PointSelectionChangeListener *pointListener = dynamic_cast<PointSelectionChangeListener*>(listener);
         if (pointListener) {
             pointListener->notifyPathPointsChanged(this);
@@ -1694,8 +1642,6 @@ void KoPathShape::notifyPointsChanged()
 
 QPainterPath KoPathShape::pathStroke(const QPen &pen) const
 {
-    Q_D(const KoPathShape);
-
     if (d->subpaths.isEmpty()) {
         return QPainterPath();
     }
