@@ -3,7 +3,7 @@
    Copyright (C) 2006 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2006 C. Boemann Rasmussen <cbo@boemann.dk>
    Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
-   Copyright (C) 2012 Boudewijn Rempt <boud@kogmbh.com>
+   Copyright (C) 2012 Boudewijn Rempt <boud@valdyas.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -30,8 +30,6 @@ class QTabletEvent;
 class QMouseEvent;
 class QWheelEvent;
 class KoInputDeviceHandlerEvent;
-class QGraphicsSceneMouseEvent;
-class QGraphicsSceneWheelEvent;
 
 #include "kritaflake_export.h"
 
@@ -62,22 +60,6 @@ public:
      */
     KoPointerEvent(QMouseEvent *event, const QPointF &point);
 
-    /**
-     * Constructor.
-     *
-     * @param event the mouse event that is the base of this event.
-     * @param point the zoomed point in the normal coordinate system.
-     */
-    KoPointerEvent(QGraphicsSceneMouseEvent *event, const QPointF &point);
-
-    /**
-     * Constructor.
-     *
-     * @param event the mouse event that is the base of this event.
-     * @param point the zoomed point in the normal coordinate system.
-     */
-    KoPointerEvent(QGraphicsSceneWheelEvent *event, const QPointF &point);
-
     /**
      * Constructor.
      *
@@ -86,22 +68,7 @@ public:
      */
     KoPointerEvent(QTabletEvent *event, const QPointF &point);
 
-    /**
-     * Constructor.
-     *
-     * @param event the touch event that is the base of this event.
-     * @param point the zoomed point of the primary touch event in the normal coordinate system.
-     * @param touchpoints the zoomed points of the touch event in the normal coordinate system.
-     */
-    KoPointerEvent(QTouchEvent *event, const QPointF &point, QList<KoTouchPoint> _touchPoints);
-
-    /**
-     * Constructor.
-     *
-     * @param event the tablet event that is the base of this event.
-     * @param point the zoomed point in the normal coordinate system.
-     */
-    KoPointerEvent(QWheelEvent *event, const QPointF &point);
+    KoPointerEvent(QTouchEvent* ev, const QPointF& pnt);
 
     KoPointerEvent(KoInputDeviceHandlerEvent *event, int x, int y, int z = 0, int rx = 0, int ry = 0, int rz = 0);
 
@@ -109,7 +76,6 @@ public:
 
     KoPointerEvent(const KoPointerEvent &rhs);
 
-
     ~KoPointerEvent();
 
     /**
@@ -211,14 +177,6 @@ public:
      * this value is always zero. This is <em>not</em> the same as pressure.
      */
     int z() const;
-
-    /**
-     * Returns the distance that the wheel is rotated, in eights of a degree, or 0 otherwise.
-     * @return the distance of rotation.
-     * @see orientation()
-     */
-    int delta() const;
-
     /**
      * Returns the rotation around the X-axis. If the device does not support
      * this, the value is always zero.
@@ -237,10 +195,6 @@ public:
      */
     int rotationZ() const;
 
-    /**
-     * Returns the orientation of the delta.
-     */
-    Qt::Orientation orientation() const;
 
     /// The point in document coordinates.
     const QPointF point;
