@@ -34,21 +34,27 @@
 
 #include "kis_debug.h"
 KoSelection::KoSelection(QObject *parent)
-    : QObject(parent),
-      KoShape(new KoSelectionPrivate(this))
+    : QObject(parent)
+    , KoShape()
+    , d(new Private)
 {
-    Q_D(KoSelection);
     connect(&d->selectionChangedCompressor, SIGNAL(timeout()), SIGNAL(selectionChanged()));
 }
 
+KoSelection::KoSelection(const KoSelection &rhs)
+    : QObject()
+    , KoShape(rhs)
+    , d(rhs.d)
+{
+}
+
 KoSelection::~KoSelection()
 {
 }
 
-void KoSelection::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintcontext)
+void KoSelection::paint(QPainter &painter, KoShapePaintingContext &paintcontext) const
 {
     Q_UNUSED(painter);
-    Q_UNUSED(converter);
     Q_UNUSED(paintcontext);
 }
 
@@ -65,14 +71,19 @@ QSizeF KoSelection::size() const
 
 QRectF KoSelection::outlineRect() const
 {
-    QPolygonF globalPolygon;
-    Q_FOREACH (KoShape *shape, selectedVisibleShapes()) {
-        globalPolygon = globalPolygon.united(
-            shape->absoluteTransformation(0).map(QPolygonF(shape->outlineRect())));
-    }
-    const QPolygonF localPolygon = transformation().inverted().map(globalPolygon);
+    const QTransform invertedTransform = transformation().inverted();
+    QRectF boundingRect;
 
-    return localPolygon.boundingRect();
+    Q_FOREACH (KoShape *shape, selectedVisibleShapes()) {
+        // it is cheaper to invert-transform each outline, than
+        // to group 300+ rotated rectangles into a polygon
+        boundingRect |=
+            invertedTransform.map(
+                shape->absoluteTransformation().map(
+                        QPolygonF(shape->outlineRect()))).boundingRect();
+    }
+
+    return boundingRect;
 }
 
 QRectF KoSelection::boundingRect() const
@@ -82,7 +93,6 @@ QRectF KoSelection::boundingRect() const
 
 void KoSelection::select(KoShape *shape)
 {
-    Q_D(KoSelection);
     KIS_SAFE_ASSERT_RECOVER_RETURN(shape != this);
     KIS_SAFE_ASSERT_RECOVER_RETURN(shape);
 
@@ -104,7 +114,7 @@ void KoSelection::select(KoShape *shape)
     shape->addShapeChangeListener(this);
 
     if (d->selectedShapes.size() == 1) {
-        setTransformation(shape->absoluteTransformation(0));
+        setTransformation(shape->absoluteTransformation());
     } else {
         setTransformation(QTransform());
     }
@@ -114,7 +124,6 @@ void KoSelection::select(KoShape *shape)
 
 void KoSelection::deselect(KoShape *shape)
 {
-    Q_D(KoSelection);
     if (!d->selectedShapes.contains(shape))
         return;
 
@@ -122,7 +131,7 @@ void KoSelection::deselect(KoShape *shape)
     shape->removeShapeChangeListener(this);
 
     if (d->selectedShapes.size() == 1) {
-        setTransformation(d->selectedShapes.first()->absoluteTransformation(0));
+        setTransformation(d->selectedShapes.first()->absoluteTransformation());
     }
 
     d->selectionChangedCompressor.start();
@@ -130,7 +139,6 @@ void KoSelection::deselect(KoShape *shape)
 
 void KoSelection::deselectAll()
 {
-    Q_D(KoSelection);
 
     if (d->selectedShapes.isEmpty())
         return;
@@ -148,13 +156,11 @@ void KoSelection::deselectAll()
 
 int KoSelection::count() const
 {
-    Q_D(const KoSelection);
     return d->selectedShapes.size();
 }
 
 bool KoSelection::hitTest(const QPointF &position) const
 {
-    Q_D(const KoSelection);
 
     Q_FOREACH (KoShape *shape, d->selectedShapes) {
         if (shape->isVisible()) continue;
@@ -166,7 +172,6 @@ bool KoSelection::hitTest(const QPointF &position) const
 
 const QList<KoShape*> KoSelection::selectedShapes() const
 {
-    Q_D(const KoSelection);
     return d->selectedShapes;
 }
 
@@ -210,7 +215,6 @@ const QList<KoShape *> KoSelection::selectedEditableShapesAndDelegates() const
 
 bool KoSelection::isSelected(const KoShape *shape) const
 {
-    Q_D(const KoSelection);
     if (shape == this)
         return true;
 
@@ -224,20 +228,17 @@ bool KoSelection::isSelected(const KoShape *shape) const
 
 KoShape *KoSelection::firstSelectedShape() const
 {
-    Q_D(const KoSelection);
     return !d->selectedShapes.isEmpty() ? d->selectedShapes.first() : 0;
 }
 
 void KoSelection::setActiveLayer(KoShapeLayer *layer)
 {
-    Q_D(KoSelection);
     d->activeLayer = layer;
     emit currentLayerChanged(layer);
 }
 
 KoShapeLayer* KoSelection::activeLayer() const
 {
-    Q_D(const KoSelection);
     return d->activeLayer;
 }
 
