@@ -29,6 +29,7 @@ class KoShape;
 class QColor;
 class QTransform;
 class QGradient;
+class SvgMeshGradient;
 
 class KRITAFLAKE_EXPORT KoShapeFillWrapper
 {
@@ -45,6 +46,7 @@ public:
     const QGradient* gradient() const;
     QTransform gradientTransform() const;
     bool hasZeroLineWidth() const;
+    const SvgMeshGradient* meshgradient() const;
 
     KUndo2Command* setColor(const QColor &color);
     KUndo2Command* setLineWidth(const float &lineWidth);
@@ -53,6 +55,8 @@ public:
     KUndo2Command* applyGradient(const QGradient *gradient);
     KUndo2Command* applyGradientStopsOnly(const QGradient *gradient);
 
+    KUndo2Command* setMeshGradient(const SvgMeshGradient *gradient, const QTransform &transform);
+
 private:
     struct Private;
     const QScopedPointer<Private> m_d;
