@@ -78,11 +78,7 @@ public:
     explicit KoToolBase(KoCanvasBase *canvas);
     ~KoToolBase() override;
 
-    /**
-     * request a repaint of the decorations to be made. This triggers
-     * an update call on the canvas, but does not paint directly.
-     */
-    virtual void repaintDecorations();
+    virtual QRectF decorationsRect() const;
 
     /**
      * Return if dragging (moving with the mouse down) to the edge of a canvas should scroll the
@@ -109,17 +105,11 @@ public:
      */
     QList<QPointer<QWidget> > optionWidgets();
 
-    /**
-     * Retrieves the entire collection of actions for the tool.
-     */
-    QHash<QString, QAction *> actions() const;
-
     /**
      * Retrieve an action by name.
      */
     QAction *action(const QString &name) const;
 
-
     /**
      * Called when (one of) the mouse or stylus buttons is pressed.
      * Implementors should call event->ignore() if they do not actually use the event.
@@ -185,7 +175,7 @@ public:
      * able to support complex input method operations as support for surrounding
      * text and reconversions.
      * Default implementation returns simple defaults, for tools that want to provide
-     * a more responsive text entry experience for CJK languages it would be good to reimplemnt.
+     * a more responsive text entry experience for CJK languages it would be good to reimplement.
      * @param query specifies which property is queried.
      * @param converter the view converter for the current canvas.
      */
@@ -401,6 +391,12 @@ public Q_SLOTS:
      */
     void setStatusText(const QString &statusText);
 
+    /**
+     * request a repaint of the decorations to be made. This triggers
+     * an update call on the canvas, but does not paint directly.
+     */
+    virtual void repaintDecorations();
+
 Q_SIGNALS:
 
     /**
@@ -460,24 +456,15 @@ protected:
     virtual QWidget *createOptionWidget();
     virtual QList<QPointer<QWidget> > createOptionWidgets();
 
-    /**
-     * Add an action under the given name to the collection.
-     *
-     * Inserting an action under a name that is already used for another action will replace
-     * the other action in the collection.
-     *
-     * @param name The name by which the action be retrieved again from the collection.
-     * @param action The action to add.
-     * @param readWrite set this to ReadOnlyAction to keep the action available on
-     *      read-only documents
-     */
-    void addAction(const QString &name, QAction *action);
-
     /// Convenience function to get the current handle radius
-    uint handleRadius() const;
+    int handleRadius() const;
+
+    /// Convenience function to get the current handle radius measured in document
+    /// coordinates (points)
+    qreal handleDocRadius() const;
 
     /// Convencience function to get the current grab sensitivity
-    uint grabSensitivity() const;
+    int grabSensitivity() const;
 
     /**
     * Returns a handle grab rect at the given position.
