@@ -27,10 +27,10 @@
 #include <FlakeDebug.h>
 #include <resources/KoResource.h>
 #include <KoShape.h>
-#include <KisGamutMaskViewConverter.h>
 #include <KoShapePaintingContext.h>
 
-class KoViewConverter;
+//class KoViewConverter;
+class QTransform;
 
 class KoGamutMaskShape
 {
@@ -39,10 +39,10 @@ public:
     KoGamutMaskShape();
     ~KoGamutMaskShape();
 
-    bool coordIsClear(const QPointF& coord, const KoViewConverter& viewConverter, int maskRotation) const;
+    bool coordIsClear(const QPointF& coord) const;
     QPainterPath outline();
-    void paint(QPainter &painter, const KoViewConverter& viewConverter, int maskRotation);
-    void paintStroke(QPainter &painter, const KoViewConverter& viewConverter, int maskRotation);
+    void paint(QPainter &painter);
+    void paintStroke(QPainter &painter);
     KoShape* koShape();
 
 private:
@@ -64,14 +64,17 @@ public:
     KoGamutMask(KoGamutMask *rhs);
     ~KoGamutMask() override;
 
-    bool coordIsClear(const QPointF& coord, KoViewConverter& viewConverter, bool preview);
+    bool coordIsClear(const QPointF& coord, bool preview);
     bool load() override;
     bool loadFromDevice(QIODevice *dev) override;
     bool save() override;
     bool saveToDevice(QIODevice* dev) const override;
 
-    void paint(QPainter &painter, KoViewConverter& viewConverter, bool preview);
-    void paintStroke(QPainter &painter, KoViewConverter& viewConverter, bool preview);
+    void paint(QPainter &painter, bool preview);
+    void paintStroke(QPainter &painter, bool preview);
+
+    QTransform maskToViewTransform(qreal viewSize);
+    QTransform viewToMaskTransform(qreal viewSize);
 
     QString title();
     void setTitle(QString title);
