@@ -75,12 +75,12 @@ QString SvgUtil::toPercentage(qreal value)
     return KisDomUtils::toString(value * 100.0) + "%";
 }
 
-double SvgUtil::fromPercentage(QString s)
+double SvgUtil::fromPercentage(QString s, bool *ok)
 {
     if (s.endsWith('%'))
-        return KisDomUtils::toDouble(s.remove('%')) / 100.0;
+        return KisDomUtils::toDouble(s.remove('%'), ok) / 100.0;
     else
-        return KisDomUtils::toDouble(s);
+        return KisDomUtils::toDouble(s, ok);
 }
 
 QPointF SvgUtil::objectToUserSpace(const QPointF &position, const QRectF &objectBound)
@@ -159,7 +159,7 @@ bool SvgUtil::parseViewBox(const KoXmlElement &e,
     QRectF viewBoxRect;
     // this is a workaround for bug 260429 for a file generated by blender
     // who has px in the viewbox which is wrong.
-    // reported as bug http://projects.blender.org/tracker/?group_id=9&atid=498&func=detail&aid=30971
+    // reported as bug https://developer.blender.org/T30971
     viewBoxStr.remove("px");
 
     QStringList points = viewBoxStr.replace(',', ' ').simplified().split(' ');
