@@ -30,6 +30,9 @@ private Q_SLOTS:
     void testUnitChanged();
     void testConverters();
     void testDerivedChanged();
+    void testComplexResource();
+
+    void testNeverChangingConverters();
 };
 
 #endif // TESTRESOURCEMANAGER_H
