@@ -23,6 +23,7 @@
 
 #include <KoShapeContainer.h>
 #include <SvgShape.h>
+#include <KoSvgText.h>
 
 class HtmlSavingContext;
 class KoSvgTextProperties;
@@ -66,7 +67,7 @@ public:
     QRectF outlineRect() const override;
     QPainterPath outline() const override;
 
-    void paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext) override;
+    void paintComponent(QPainter &painter, KoShapePaintingContext &paintContext) const override;
     void saveOdf(KoShapeSavingContext &Context) const override;
     bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &Context) override;
 
@@ -141,7 +142,7 @@ public:
      * A special interface for KoSvgTextShape's layout code. Don't use it
      * unless you are KoSvgTextShape.
      */
-    KoSvgTextChunkShapeLayoutInterface* layoutInterface();
+    KoSvgTextChunkShapeLayoutInterface* layoutInterface() const;
 
     /**
      * WARNING: this propperty is available only if isRootTextNode() is true
@@ -168,7 +169,16 @@ protected:
     KoSvgTextChunkShape(KoSvgTextChunkShapePrivate *dd);
 
 private:
-    Q_DECLARE_PRIVATE(KoSvgTextChunkShape)
+    KoSvgText::KoSvgCharChunkFormat fetchCharFormat() const;
+
+    void applyParentCharTransformations(const QVector<KoSvgText::CharTransformation> transformations);
+
+private:
+    class Private;
+    QScopedPointer<Private> d;
+
+    class SharedData;
+    QSharedDataPointer<SharedData> s;
 };
 
 #endif // KOSVGTEXTCHUNKSHAPE_H
