@@ -20,18 +20,21 @@
 
 #include "KoSvgText.h"
 #include "KoSvgTextProperties.h"
-#include <KoShapeContainer_p.h>
+#include <QSharedData>
+#include <QPainterPath>
 #include <QTextCharFormat>
 
+#include <KoShapeContainer_p.h>
+
 class SvgGraphicsContext;
 
-class KoSvgTextChunkShapePrivate : public KoShapeContainerPrivate
+class KoSvgTextChunkShape::SharedData : public QSharedData
 {
 public:
 
-    KoSvgTextChunkShapePrivate(KoSvgTextChunkShape *_q);
-    KoSvgTextChunkShapePrivate(const KoSvgTextChunkShapePrivate &rhs, KoSvgTextChunkShape *q);
-    ~KoSvgTextChunkShapePrivate();
+    SharedData();
+    SharedData(const SharedData &rhs);
+    ~SharedData();
 
     KoSvgTextProperties properties;
     QFont font;
@@ -44,17 +47,16 @@ public:
 
     QString text;
 
-    struct LayoutInterface;
-    QScopedPointer<LayoutInterface> layoutInterface;
-
     QPainterPath associatedOutline;
 
-    KoSvgText::KoSvgCharChunkFormat fetchCharFormat() const;
-
-    void applyParentCharTransformations(const QVector<KoSvgText::CharTransformation> transformations);
     void loadContextBasedProperties(SvgGraphicsContext *gc);
     bool isRichTextPreferred = true;
+};
 
-    Q_DECLARE_PUBLIC(KoSvgTextChunkShape)
+class KoSvgTextChunkShape::Private
+{
+public:
+    struct LayoutInterface;
+    QScopedPointer<LayoutInterface> layoutInterface;
 };
 
