@@ -48,43 +48,41 @@
 #include <vector>
 #include <memory>
 #include <QPainter>
+#include <QPainterPath>
 #include <boost/optional.hpp>
 
 #include <text/KoSvgTextChunkShapeLayoutInterface.h>
 
 #include <FlakeDebug.h>
 
-class KoSvgTextShapePrivate : public KoSvgTextChunkShapePrivate
+#include <QSharedData>
+
+class KoSvgTextShape::Private
 {
-    KoSvgTextShapePrivate(KoSvgTextShape *_q)
-        : KoSvgTextChunkShapePrivate(_q)
-    {
-    }
+public:
 
-    KoSvgTextShapePrivate(const KoSvgTextShapePrivate &rhs, KoSvgTextShape *q)
-        : KoSvgTextChunkShapePrivate(rhs, q)
-    {
-    }
-
-    std::vector<std::unique_ptr<QTextLayout>> cachedLayouts;
+    // NOTE: the cache data is shared between all the instances of
+    //       the shape, though it will be reset locally if the
+    //       accessing thread changes
+    std::vector<std::shared_ptr<QTextLayout>> cachedLayouts;
     std::vector<QPointF> cachedLayoutsOffsets;
     QThread *cachedLayoutsWorkingThread = 0;
 
 
-    void clearAssociatedOutlines(KoShape *rootShape);
+    void clearAssociatedOutlines(const KoShape *rootShape);
 
-
-    Q_DECLARE_PUBLIC(KoSvgTextShape)
 };
 
 KoSvgTextShape::KoSvgTextShape()
-    : KoSvgTextChunkShape(new KoSvgTextShapePrivate(this))
+    : KoSvgTextChunkShape()
+    , d(new Private)
 {
     setShapeId(KoSvgTextShape_SHAPEID);
 }
 
 KoSvgTextShape::KoSvgTextShape(const KoSvgTextShape &rhs)
-    : KoSvgTextChunkShape(new KoSvgTextShapePrivate(*rhs.d_func(), this))
+    : KoSvgTextChunkShape(rhs)
+    , d(new Private)
 {
     setShapeId(KoSvgTextShape_SHAPEID);
     // QTextLayout has no copy-ctor, so just relayout everything!
@@ -109,9 +107,8 @@ void KoSvgTextShape::shapeChanged(ChangeType type, KoShape *shape)
     }
 }
 
-void KoSvgTextShape::paintComponent(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+void KoSvgTextShape::paintComponent(QPainter &painter, KoShapePaintingContext &paintContext) const
 {
-    Q_D(KoSvgTextShape);
 
     Q_UNUSED(paintContext);
 
@@ -126,7 +123,6 @@ void KoSvgTextShape::paintComponent(QPainter &painter, const KoViewConverter &co
         relayout();
     }
 
-    applyConversion(painter, converter);
     for (int i = 0; i < (int)d->cachedLayouts.size(); i++) {
         d->cachedLayouts[i]->draw(&painter, d->cachedLayoutsOffsets[i]);
     }
@@ -145,10 +141,9 @@ void KoSvgTextShape::paintComponent(QPainter &painter, const KoViewConverter &co
     }
 }
 
-void KoSvgTextShape::paintStroke(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+void KoSvgTextShape::paintStroke(QPainter &painter, KoShapePaintingContext &paintContext) const
 {
     Q_UNUSED(painter);
-    Q_UNUSED(converter);
     Q_UNUSED(paintContext);
 
     // do nothing! everything is painted in paintComponent()
@@ -156,7 +151,6 @@ void KoSvgTextShape::paintStroke(QPainter &painter, const KoViewConverter &conve
 
 QPainterPath KoSvgTextShape::textOutline()
 {
-    Q_D(KoSvgTextShape);
 
     QPainterPath result;
     result.setFillRule(Qt::WindingFill);
@@ -418,9 +412,8 @@ private:
     QTextLine m_danglingLine;
 };
 
-void KoSvgTextShape::relayout()
+void KoSvgTextShape::relayout() const
 {
-    Q_D(KoSvgTextShape);
 
     d->cachedLayouts.clear();
     d->cachedLayoutsOffsets.clear();
@@ -431,7 +424,7 @@ void KoSvgTextShape::relayout()
     QVector<TextChunk> textChunks = mergeIntoChunks(layoutInterface()->collectSubChunks());
 
     Q_FOREACH (const TextChunk &chunk, textChunks) {
-        std::unique_ptr<QTextLayout> layout(new QTextLayout());
+        std::shared_ptr<QTextLayout> layout(new QTextLayout());
 
         QTextOption option;
 
@@ -493,7 +486,7 @@ void KoSvgTextShape::relayout()
             diff.ry() = 0;
         }
 
-        d->cachedLayouts.push_back(std::move(layout));
+        d->cachedLayouts.push_back(layout);
         d->cachedLayoutsOffsets.push_back(-diff);
 
     }
@@ -557,10 +550,13 @@ void KoSvgTextShape::relayout()
                      *
                      * BUG: 389528
                      * BUG: 392068
+                     * BUG: 420408 (vertically)
                      */
                     rect.setLeft(qMin(rect.left(), lastGlyphRect.left()) - 0.5 * firstGlyphRect.width());
                     rect.setRight(qMax(rect.right(), lastGlyphRect.right()) + 0.5 * lastGlyphRect.width());
 
+                    rect.adjust(0, -0.5*rect.height(), 0, 0.5*rect.height()); // add some vertical margin too
+
                     wrapper.addCharacterRect(rect.translated(layoutOffset));
                 }
             }
@@ -568,9 +564,9 @@ void KoSvgTextShape::relayout()
     }
 }
 
-void KoSvgTextShapePrivate::clearAssociatedOutlines(KoShape *rootShape)
+void KoSvgTextShape::Private::clearAssociatedOutlines(const KoShape *rootShape)
 {
-    KoSvgTextChunkShape *chunkShape = dynamic_cast<KoSvgTextChunkShape*>(rootShape);
+    const KoSvgTextChunkShape *chunkShape = dynamic_cast<const KoSvgTextChunkShape*>(rootShape);
     KIS_SAFE_ASSERT_RECOVER_RETURN(chunkShape);
 
     chunkShape->layoutInterface()->clearAssociatedOutline();
@@ -608,7 +604,7 @@ KoShape *KoSvgTextShapeFactory::createDefaultShape(KoDocumentResourceManager *do
     shape->setShapeId(KoSvgTextShape_SHAPEID);
 
     KoSvgTextShapeMarkupConverter converter(shape);
-    converter.convertFromSvg("<text>Lorem ipsum dolor sit amet, consectetur adipiscing elit.</text>",
+    converter.convertFromSvg(i18nc("Default text for the text shape", "<text>Placeholder Text</text>"),
                              "<defs/>",
                              QRectF(0, 0, 200, 60),
                              documentResources->documentResolution());
@@ -623,7 +619,7 @@ KoShape *KoSvgTextShapeFactory::createShape(const KoProperties *params, KoDocume
     KoSvgTextShape *shape = new KoSvgTextShape();
     shape->setShapeId(KoSvgTextShape_SHAPEID);
 
-    QString svgText = params->stringProperty("svgText", "<text>Lorem ipsum dolor sit amet, consectetur adipiscing elit.</text>");
+    QString svgText = params->stringProperty("svgText", i18nc("Default text for the text shape", "<text>Placeholder Text</text>"));
     QString defs = params->stringProperty("defs" , "<defs/>");
     QRectF shapeRect = QRectF(0, 0, 200, 60);
     QVariant rect = params->property("shapeRect");
