@@ -28,6 +28,9 @@
 #include <klocalizedstring.h>
 #include <limits>
 #include <math.h>
+#include "kis_command_utils.h"
+#include <KisBezierUtils.h>
+
 
 KoPathSegmentChangeStrategy::KoPathSegmentChangeStrategy(KoPathTool *tool, const QPointF &pos, const KoPathPointData &segment, qreal segmentParam)
 : KoInteractionStrategy(tool)
@@ -61,10 +64,7 @@ KoPathSegmentChangeStrategy::~KoPathSegmentChangeStrategy()
 
 void KoPathSegmentChangeStrategy::handleMouseMove(const QPointF &mouseLocation, Qt::KeyboardModifiers modifiers)
 {
-    m_tool->canvas()->updateCanvas(m_tool->canvas()->snapGuide()->boundingRect());
     QPointF snappedPosition = m_tool->canvas()->snapGuide()->snap(mouseLocation, modifiers);
-    m_tool->canvas()->updateCanvas(m_tool->canvas()->snapGuide()->boundingRect());
-
     QPointF localPos = m_path->documentToShape(snappedPosition);
 
     if (m_segment.degree() == 1) {
@@ -86,33 +86,13 @@ void KoPathSegmentChangeStrategy::handleMouseMove(const QPointF &mouseLocation,
         }
     }
     else if (m_segment.degree() == 3) {
-        /*
-        * method from inkscape, original method and idea borrowed from Simon Budig
-        * <simon@gimp.org> and the GIMP
-        * cf. app/vectors/gimpbezierstroke.c, gimp_bezier_stroke_point_move_relative()
-        *
-        * feel good is an arbitrary parameter that distributes the delta between handles
-        * if t of the drag point is less than 1/6 distance form the endpoint only
-        * the corresponding handle is adjusted. This matches the behavior in GIMP
-        */
-        const qreal t = m_segmentParam;
-        qreal feel_good;
-        if (t <= 1.0 / 6.0)
-            feel_good = 0;
-        else if (t <= 0.5)
-            feel_good = (pow((6 * t - 1) / 2.0, 3)) / 2;
-        else if (t <= 5.0 / 6.0)
-            feel_good = (1 - pow((6 * (1-t) - 1) / 2.0, 3)) / 2 + 0.5;
-        else
-            feel_good = 1;
-
         QPointF lastLocalPos = m_path->documentToShape(m_lastPosition);
         QPointF delta = localPos - lastLocalPos;
-        move2 = ((1-feel_good)/(3*t*(1-t)*(1-t))) * delta;
-        move1 = (feel_good/(3*t*t*(1-t))) * delta;
+
+        std::tie(move2, move1) =
+            KisBezierUtils::offsetSegment(m_segmentParam, delta);
     }
 
-    m_path->update();
     if(m_segment.first()->activeControlPoint2()) {
         KoPathControlPointMoveCommand cmd(m_pointData1, move2, KoPathPoint::ControlPoint2);
         cmd.redo();
@@ -122,7 +102,6 @@ void KoPathSegmentChangeStrategy::handleMouseMove(const QPointF &mouseLocation,
         cmd.redo();
     }
     m_path->normalize();
-    m_path->update();
 
     m_ctrlPoint1Move += move1;
     m_ctrlPoint2Move += move2;
@@ -138,26 +117,18 @@ void KoPathSegmentChangeStrategy::finishInteraction(Qt::KeyboardModifiers modifi
 
 KUndo2Command* KoPathSegmentChangeStrategy::createCommand()
 {
-    m_tool->canvas()->updateCanvas(m_tool->canvas()->snapGuide()->boundingRect());
-
     bool hasControlPoint1 = m_segment.second()->activeControlPoint1();
     bool hasControlPoint2 = m_segment.first()->activeControlPoint2();
 
-    KUndo2Command * cmd = new KUndo2Command(kundo2_i18n("Change Segment"));
+    KUndo2Command * cmd = new KisCommandUtils::SkipFirstRedoWrapper(new KUndo2Command(kundo2_i18n("Change Segment")));
     if (m_originalSegmentDegree == 1) {
-        m_segment.first()->removeControlPoint2();
-        m_segment.second()->removeControlPoint1();
         new KoPathSegmentTypeCommand(m_pointData1, KoPathSegmentTypeCommand::Curve, cmd);
     }
 
     if (hasControlPoint2) {
-        QPointF oldCtrlPointPos = m_segment.first()->controlPoint2()-m_ctrlPoint2Move;
-        m_segment.first()->setControlPoint2(oldCtrlPointPos);
         new KoPathControlPointMoveCommand(m_pointData1, m_ctrlPoint2Move, KoPathPoint::ControlPoint2, cmd);
     }
     if (hasControlPoint1) {
-        QPointF oldCtrlPointPos = m_segment.second()->controlPoint1()-m_ctrlPoint1Move;
-        m_segment.second()->setControlPoint1(oldCtrlPointPos);
         new KoPathControlPointMoveCommand(m_pointData2, m_ctrlPoint1Move, KoPathPoint::ControlPoint1, cmd);
     }
 
