@@ -28,9 +28,9 @@ KoZoomToolFactory::KoZoomToolFactory()
         : KoToolFactoryBase("ZoomTool")
 {
     setToolTip(i18n("Zoom"));
-    setToolType(navigationToolType());
-    setPriority(5);
-    setIconName(koIconNameCStr("zoom-original"));
+    setSection(navigationToolType());
+    setPriority(0);
+    setIconName(koIconNameCStr("tool_zoom"));
     setActivationShapeId("flake/always");
 }
 
