@@ -81,6 +81,12 @@ Point normalize(const Point &a)
     return (1.0 / length) * a;
 }
 
+template <typename Point>
+Point lerp(const Point &pt1, const Point &pt2, qreal t)
+{
+    return pt1 + (pt2 - pt1) * t;
+}
+
 /**
  * Usual sign() function with positive zero
  */
@@ -364,6 +370,30 @@ inline Point abs(const Point &pt) {
     return Point(qAbs(pt.x()), qAbs(pt.y()));
 }
 
+template<typename T, typename std::enable_if<std::is_integral<T>::value, T>::type* = nullptr>
+inline T wrapValue(T value, T wrapBounds) {
+    value %= wrapBounds;
+    if (value < 0) {
+        value += wrapBounds;
+    }
+    return value;
+}
+
+template<typename T, typename std::enable_if<std::is_floating_point<T>::value, T>::type* = nullptr>
+inline T wrapValue(T value, T wrapBounds) {
+    value = std::fmod(value, wrapBounds);
+    if (value < 0) {
+        value += wrapBounds;
+    }
+    return value;
+}
+
+template<typename T, typename std::enable_if<std::is_same<decltype(T().x()), decltype(T().y())>::value, T>::type* = nullptr>
+inline T wrapValue(T value, T wrapBounds) {
+    value.rx() = wrapValue(value.x(), wrapBounds.x());
+    value.ry() = wrapValue(value.y(), wrapBounds.y());
+    return value;
+}
 
 class RightHalfPlane {
 public:
@@ -485,6 +515,9 @@ QVector<QPointF> intersectTwoCircles(const QPointF &c1, qreal r1,
 KRITAGLOBAL_EXPORT
 QTransform mapToRect(const QRectF &rect);
 
+KRITAGLOBAL_EXPORT
+QTransform mapToRectInverse(const QRectF &rect);
+
 /**
  * Scale the relative point \pt into the bounds of \p rc. The point might be
  * outside the rectangle.
@@ -498,10 +531,9 @@ inline QPointF relativeToAbsolute(const QPointF &pt, const QRectF &rc) {
  * outside the rectangle.
  */
 inline QPointF absoluteToRelative(const QPointF &pt, const QRectF &rc) {
-    if (!rc.isValid()) return QPointF();
-
     const QPointF rel = pt - rc.topLeft();
-    return QPointF(rel.x() / rc.width(), rel.y() / rc.height());
+    return QPointF(rc.width() > 0 ? rel.x() / rc.width() : 0,
+                   rc.height() > 0 ? rel.y() / rc.height() : 0);
 
 }
 
@@ -523,6 +555,22 @@ inline qreal absoluteToRelative(const qreal value, const QRectF &rc) {
     return coeff != 0 ? value / coeff : 0;
 }
 
+/**
+ * Scales relative isotropic value from relative to absolute coordinate system
+ * using SVG 1.1 rules (see chapter 7.10)
+ */
+inline QRectF relativeToAbsolute(const QRectF &rel, const QRectF &rc) {
+    return QRectF(relativeToAbsolute(rel.topLeft(), rc), relativeToAbsolute(rel.bottomRight(), rc));
+}
+
+/**
+ * Scales absolute isotropic value from absolute to relative coordinate system
+ * using SVG 1.1 rules (see chapter 7.10)
+ */
+inline QRectF absoluteToRelative(const QRectF &rel, const QRectF &rc) {
+    return QRectF(absoluteToRelative(rel.topLeft(), rc), absoluteToRelative(rel.bottomRight(), rc));
+}
+
 /**
  * Compare the matrices with tolerance \p delta
  */
@@ -539,6 +587,25 @@ bool KRITAGLOBAL_EXPORT fuzzyPointCompare(const QPointF &p1, const QPointF &p2);
  */
 bool KRITAGLOBAL_EXPORT fuzzyPointCompare(const QPointF &p1, const QPointF &p2, qreal delta);
 
+/**
+ * Returns true if points in two containers are equal with specified tolerance
+ */
+template <template<typename> class Cont, class Point>
+bool fuzzyPointCompare(const Cont<Point> &c1, const Cont<Point> &c2, qreal delta)
+{
+    if (c1.size() != c2.size()) return false;
+
+    const qreal eps = delta;
+
+    return std::mismatch(c1.cbegin(),
+                         c1.cend(),
+                         c2.cbegin(),
+                         [eps] (const QPointF &pt1, const QPointF &pt2) {
+                               return fuzzyPointCompare(pt1, pt2, eps);
+                         })
+            .first == c1.cend();
+}
+
 /**
  * Compare two rectangles with tolerance \p tolerance. The tolerance means that the
  * coordinates of top left and bottom right corners should not differ more than \p tolerance
@@ -618,7 +685,21 @@ private:
     bool valid = true;
 };
 
+std::pair<QPointF, QTransform> KRITAGLOBAL_EXPORT transformEllipse(const QPointF &axes, const QTransform &fullLocalToGlobal);
+
 QPointF KRITAGLOBAL_EXPORT alignForZoom(const QPointF &pt, qreal zoom);
+
+
+/**
+ * Linearly reshape function \p x so that in range [x0, x1]
+ * it would cross points (x0, y0) and (x1, y1).
+ */
+template <typename T>
+inline T linearReshapeFunc(T x, T x0, T x1, T y0, T y1)
+{
+    return y0 + (y1 - y0) * (x - x0) / (x1 - x0);
+}
+
 }
 
 
