@@ -27,7 +27,7 @@
 #include <kis_assert_exception.h>
 #include <KisUsageLogger.h>
 #include <string>
-#include "config-hide-safe-asserts.h"
+#include "config-safe-asserts.h"
 
 /**
  * TODO: Add automatic saving of the documents
@@ -70,16 +70,22 @@ void kis_assert_common(const char *assertion, const char *file, int line, bool t
         disableAssertMsg = true;
     }
 
+    bool shouldIgnoreAsserts = false;
+    bool forceCrashOnSafeAsserts = false;
+
 #ifdef HIDE_SAFE_ASSERTS
-    const bool shouldIgnoreAsserts = HIDE_SAFE_ASSERTS;
-#else
-    const bool shouldIgnoreAsserts = false;
+    shouldIgnoreAsserts |= HIDE_SAFE_ASSERTS;
 #endif
 
-    disableAssertMsg |= shouldIgnoreAsserts;
+#ifdef CRASH_ON_SAFE_ASSERTS
+    forceCrashOnSafeAsserts |= CRASH_ON_SAFE_ASSERTS;
+#endif
+
+    disableAssertMsg |= shouldIgnoreAsserts || forceCrashOnSafeAsserts;
 
     QMessageBox::StandardButton button =
-        isIgnorable ? QMessageBox::Ignore : QMessageBox::Abort;
+        isIgnorable && !forceCrashOnSafeAsserts ?
+            QMessageBox::Ignore : QMessageBox::Abort;
 
     if (!disableAssertMsg) {
         button =
