@@ -83,25 +83,30 @@ public:
         return false;
     }
 
-    qreal getRadius(){
-        return m_radius;
+#if defined HAVE_VC
+    Vc::float_m needFade(Vc::float_v &dist) {
+        const Vc::float_v vOne(Vc::One);
+        const Vc::float_v vValMax(255.f);
+
+        Vc::float_v vRadius(m_radius);
+        Vc::float_v vFadeStartValue(m_fadeStartValue);
+        Vc::float_v vAntialiasingFadeStart(m_antialiasingFadeStart);
+        Vc::float_v vAntialiasingFadeCoeff(m_antialiasingFadeCoeff);
+
+        Vc::float_m outsideMask = dist > vRadius;
+        dist(outsideMask) = vOne;
+
+        Vc::float_m fadeStartMask(false);
+
+        if(m_enableAntialiasing){
+            fadeStartMask = dist > vAntialiasingFadeStart;
+            dist((outsideMask ^ fadeStartMask) & fadeStartMask) = (vFadeStartValue +
+                                                                (dist - vAntialiasingFadeStart) * vAntialiasingFadeCoeff) / vValMax;
+        }
+        return (outsideMask | fadeStartMask);
     }
 
-    qreal getAntialiasingFadeStart(){
-        return m_antialiasingFadeStart;
-    }
-
-    qreal getFadeStartValue() {
-        return m_fadeStartValue;
-    }
-
-    qreal getAntialiasingFadeCoeff(){
-        return m_antialiasingFadeCoeff;
-    }
-
-    bool getAliasingEnabled(){
-        return m_enableAntialiasing;
-    }
+#endif /* defined HAVE_VC */
 
 private:
     qreal m_radius;
@@ -194,6 +199,58 @@ public:
         return false;
     }
 
+#if defined HAVE_VC
+    Vc::float_m needFade(Vc::float_v &xr, Vc::float_v &yr) const {
+
+        Vc::float_v vXLimit(m_xLimit);
+        Vc::float_v vYLimit(m_yLimit);
+
+        Vc::float_m outXMask = Vc::abs(xr) > vXLimit;
+        Vc::float_m outYMask = Vc::abs(yr) > vYLimit;
+
+        return (outXMask | outYMask);
+    }
+
+    // Apply fader separately to avoid calculating vValue twice.
+    void apply2DFader(Vc::float_v &vValue, Vc::float_m &excludeMask, Vc::float_v &xr, Vc::float_v &yr) const {
+        const Vc::float_v vValMax(255.f);
+
+        if(m_enableAntialiasing){
+            Vc::float_v vXFadeLimitStart(m_xFadeLimitStart);
+            Vc::float_v vYFadeLimitStart(m_yFadeLimitStart);
+            Vc::float_v vXFadeCoeff(m_xFadeCoeff);
+            Vc::float_v vYFadeCoeff(m_yFadeCoeff);
+
+            Vc::float_v xra = abs(xr);
+            Vc::float_m fadeXStartMask(false);
+            Vc::float_m fadeYStartMask(false);
+
+            Vc::float_v fadeValue;
+            Vc::SimdArray<quint16,Vc::float_v::size()> vBaseValue(vValue);
+
+            fadeXStartMask = xra > vXFadeLimitStart;
+            fadeXStartMask = (fadeXStartMask ^ excludeMask) & fadeXStartMask;
+            if (!fadeXStartMask.isFull()) {
+                fadeValue = vBaseValue + (vValMax - vBaseValue) * (xra - vXFadeLimitStart) * vXFadeCoeff;
+                fadeValue(fadeXStartMask & ((yr > vYFadeLimitStart) & (fadeValue < vValMax)) ) =
+                        fadeValue + (vValMax - fadeValue) * (yr - vYFadeLimitStart) * vYFadeCoeff;
+                vValue(fadeXStartMask) = fadeValue;
+            }
+
+            fadeYStartMask = yr > vYFadeLimitStart;
+            fadeYStartMask = (fadeYStartMask ^ fadeXStartMask) & fadeYStartMask;
+            if (!fadeYStartMask.isFull()) {
+                fadeValue = vBaseValue + (vValMax - vBaseValue) * (yr - vYFadeLimitStart) * vYFadeCoeff;
+                fadeValue(fadeYStartMask & ((xra > vXFadeLimitStart) & (fadeValue < vValMax)) ) =
+                        fadeValue + (vValMax - fadeValue) * (xra - vXFadeLimitStart) * vXFadeCoeff;
+                vValue(fadeYStartMask) = fadeValue;
+            }
+        }
+        return;
+    }
+
+#endif /* defined HAVE_VC */
+
 private:
     qreal m_xLimit;
     qreal m_yLimit;
