@@ -52,11 +52,15 @@ public:
 public:
 
     /**
-     * This function creates an auto brush shape with the following value :
-     * @param w width
-     * @param h height
-     * @param fh horizontal fade (fh \< w / 2 )
-     * @param fv vertical fade (fv \< h / 2 )
+     * This function creates an auto brush shape with the following values:
+     * @param radius radius
+     * @param ratio aspect ratio
+     * @param fh horizontal fade
+     * @param fv vertical fade
+     * @param spikes number of spikes
+     * @param antialiasEdges whether to antialias edges
+     * @param type type
+     * @param id the brush identifier
      */
     KisMaskGenerator(qreal radius, qreal ratio, qreal fh, qreal fv, int spikes, bool antialiasEdges, Type type, const KoID& id = DefaultId);
     KisMaskGenerator(const KisMaskGenerator &rhs);
@@ -92,7 +96,9 @@ public:
 
     qreal height() const;
 
-    qreal diameter() const;
+    qreal diameter() const;    
+    void setDiameter(qreal value);
+
     qreal ratio() const;
     qreal horizontalFade() const;
     qreal verticalFade() const;
