@@ -26,9 +26,11 @@
 class KisDefaultBounds;
 class KisSelectionDefaultBounds;
 class KisSelectionEmptyBounds;
+class KisWrapAroundBoundsWrapper;
 typedef KisSharedPtr<KisDefaultBounds> KisDefaultBoundsSP;
 typedef KisSharedPtr<KisSelectionDefaultBounds> KisSelectionDefaultBoundsSP;
 typedef KisSharedPtr<KisSelectionEmptyBounds> KisSelectionEmptyBoundsSP;
+typedef KisSharedPtr<KisWrapAroundBoundsWrapper> KisWrapAroundBoundsWrapperSP;
 
 class KRITAIMAGE_EXPORT KisDefaultBounds :  public KisDefaultBoundsBase
 {
@@ -61,6 +63,7 @@ public:
     ~KisSelectionDefaultBounds() override;
 
     QRect bounds() const override;
+    QRect imageBorderRect() const override;
     bool wrapAroundMode() const override;
     int currentLevelOfDetail() const override;
     int currentTime() const override;
@@ -82,4 +85,32 @@ public:
     QRect bounds() const override;
 };
 
+/**
+ * @brief The KisWrapAroundBoundsWrapper class
+ * wrapper around a KisDefaultBoundsBaseSP to enable
+ * wraparound. Used for patterns.
+ */
+class KRITAIMAGE_EXPORT KisWrapAroundBoundsWrapper :  public KisDefaultBoundsBase
+{
+public:
+    KisWrapAroundBoundsWrapper(KisDefaultBoundsBaseSP base, QRect bounds);
+    ~KisWrapAroundBoundsWrapper() override;
+
+    QRect bounds() const override;
+    bool wrapAroundMode() const override;
+    int currentLevelOfDetail() const override;
+    int currentTime() const override;
+    bool externalFrameActive() const override;
+    void * sourceCookie() const override;
+
+protected:
+    friend class KisPaintDeviceTest;
+
+private:
+    Q_DISABLE_COPY(KisWrapAroundBoundsWrapper)
+
+    struct Private;
+    const QScopedPointer<Private> m_d;
+};
+
 #endif // KIS_DEFAULT_BOUNDS_H
