@@ -21,10 +21,10 @@
 
 #include <KoIcon.h>
 #include <kis_icon.h>
+#include "kis_image.h"
 
-
-KisEffectMask::KisEffectMask()
-        : KisMask("effect")
+KisEffectMask::KisEffectMask(KisImageWSP image, const QString &name)
+        : KisMask(image, name)
 {
 }
 
