@@ -22,14 +22,15 @@
 
 #include "kritaimage_export.h"
 
-
+#include "kis_mask_generator.h"
 
 /**
  * This mask generator uses a Gaussian-blurred rectangle
  */
 class KRITAIMAGE_EXPORT KisGaussRectangleMaskGenerator : public KisMaskGenerator
 {
-
+public:
+    struct FastRowProcessor;
 public:
 
     KisGaussRectangleMaskGenerator(qreal diameter, qreal ratio, qreal fh, qreal fv, int spikes, bool antialiasEdges);
@@ -40,6 +41,10 @@ public:
     quint8 valueAt(qreal x, qreal y) const override;
     void setScale(qreal scaleX, qreal scaleY) override;
 
+    bool shouldVectorize() const override;
+    KisBrushMaskApplicatorBase* applicator() override;
+    void resetMaskApplicator(bool forceScalar);
+
 private:
     struct Private;
     const QScopedPointer<Private> d;
