@@ -56,7 +56,7 @@ public:
                               bool createTransaction = false,
                               KisConvolutionBorderOp borderOp = BORDER_REPEAT);
 
-    static Eigen::Matrix<qreal, Eigen::Dynamic, Eigen::Dynamic> createLoGMatrix(qreal radius, qreal coeff = 1.0);
+    static Eigen::Matrix<qreal, Eigen::Dynamic, Eigen::Dynamic> createLoGMatrix(qreal radius, qreal coeff, bool zeroCentered, bool includeWrappedArea);
 
     static void applyLoG(KisPaintDeviceSP device,
                          const QRect& rect,
@@ -65,6 +65,13 @@ public:
                          const QBitArray &channelFlags,
                          KoUpdater *progressUpdater);
 
+    static void applyTightLoG(KisPaintDeviceSP device,
+                              const QRect& rect,
+                              qreal radius, qreal coeff,
+                              const QBitArray &channelFlags,
+                              KoUpdater *progressUpdater);
+
+
     static Eigen::Matrix<qreal, Eigen::Dynamic, Eigen::Dynamic> createDilateMatrix(qreal radius);
 
     static void applyDilate(KisPaintDeviceSP device,
