@@ -29,7 +29,6 @@
 #include <QSize>
 #include <QDateTime>
 #include <QRect>
-#include <QRegion>
 #include <QtConcurrent>
 
 #include <klocalizedstring.h>
@@ -48,7 +47,6 @@
 #include "commands/kis_image_commands.h"
 #include "kis_layer.h"
 #include "kis_meta_data_merge_strategy_registry.h"
-#include "kis_name_server.h"
 #include "kis_paint_layer.h"
 #include "kis_projection_leaf.h"
 #include "kis_painter.h"
@@ -56,6 +54,8 @@
 #include "kis_transaction.h"
 #include "kis_meta_data_merge_strategy.h"
 #include "kis_memory_statistics_server.h"
+#include "kis_node.h"
+#include "kis_types.h"
 
 #include "kis_image_config.h"
 #include "kis_update_scheduler.h"
@@ -97,7 +97,7 @@
 #include "kis_layer_projection_plane.h"
 
 #include "kis_update_time_monitor.h"
-#include "tiles3/kis_lockless_stack.h"
+#include "kis_lockless_stack.h"
 
 #include <QtCore>
 
@@ -110,6 +110,9 @@
 #include "KisRunnableStrokeJobUtils.h"
 #include "KisRunnableStrokeJobsInterface.h"
 
+#include "KisBusyWaitBroker.h"
+
+
 // #define SANITY_CHECKS
 
 #ifdef SANITY_CHECKS
@@ -141,7 +144,6 @@ public:
         , width(w)
         , height(h)
         , colorSpace(c ? c : KoColorSpaceRegistry::instance()->rgb8())
-        , nserver(1)
         , undoStore(undo ? undo : new KisDumbUndoStore())
         , legacyUndoAdapter(undoStore.data(), _q)
         , postExecutionUndoAdapter(undoStore.data(), _q)
@@ -215,8 +217,6 @@ public:
     KisNodeSP isolatedRootNode;
     bool wrapAroundModePermitted = false;
 
-    KisNameServer nserver;
-
     QScopedPointer<KisUndoStore> undoStore;
     KisLegacyUndoAdapter legacyUndoAdapter;
     KisPostExecutionUndoAdapter postExecutionUndoAdapter;
@@ -254,7 +254,7 @@ public:
     struct SetImageProjectionColorSpace;
 };
 
-KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
+KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace *colorSpace, const QString& name)
         : QObject(0)
         , KisShared()
         , m_d(new KisImagePrivate(this, width, height,
@@ -271,8 +271,6 @@ KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const K
 
 KisImage::~KisImage()
 {
-    dbgImage << "deleting kisimage" << objectName();
-
     /**
      * Request the tools to end currently running strokes
      */
@@ -282,11 +280,188 @@ KisImage::~KisImage()
     disconnect(); // in case Qt gets confused
 }
 
+KisImageSP KisImage::fromQImage(const QImage &image, KisUndoStore *undoStore)
+{
+    const KoColorSpace *colorSpace = 0;
+
+    switch (image.format()) {
+    case QImage::Format_Invalid:
+    case QImage::Format_Mono:
+    case QImage::Format_MonoLSB:
+        colorSpace = KoColorSpaceRegistry::instance()->graya8();
+        break;
+    case QImage::Format_Indexed8:
+    case QImage::Format_RGB32:
+    case QImage::Format_ARGB32:
+    case QImage::Format_ARGB32_Premultiplied:
+        colorSpace = KoColorSpaceRegistry::instance()->rgb8();
+        break;
+    case QImage::Format_RGB16:
+        colorSpace = KoColorSpaceRegistry::instance()->rgb16();
+        break;
+    case QImage::Format_ARGB8565_Premultiplied:
+    case QImage::Format_RGB666:
+    case QImage::Format_ARGB6666_Premultiplied:
+    case QImage::Format_RGB555:
+    case QImage::Format_ARGB8555_Premultiplied:
+    case QImage::Format_RGB888:
+    case QImage::Format_RGB444:
+    case QImage::Format_ARGB4444_Premultiplied:
+    case QImage::Format_RGBX8888:
+    case QImage::Format_RGBA8888:
+    case QImage::Format_RGBA8888_Premultiplied:
+        colorSpace = KoColorSpaceRegistry::instance()->rgb8();
+        break;
+    case QImage::Format_BGR30:
+    case QImage::Format_A2BGR30_Premultiplied:
+    case QImage::Format_RGB30:
+    case QImage::Format_A2RGB30_Premultiplied:
+        colorSpace = KoColorSpaceRegistry::instance()->rgb8();
+        break;
+    case QImage::Format_Alpha8:
+        colorSpace = KoColorSpaceRegistry::instance()->alpha8();
+        break;
+    case QImage::Format_Grayscale8:
+        colorSpace = KoColorSpaceRegistry::instance()->graya8();
+        break;
+#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
+    case QImage::Format_Grayscale16:
+        colorSpace = KoColorSpaceRegistry::instance()->graya16();
+        break;
+#endif
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+    case QImage::Format_RGBX64:
+    case QImage::Format_RGBA64:
+    case QImage::Format_RGBA64_Premultiplied:
+        colorSpace = KoColorSpaceRegistry::instance()->colorSpace(RGBAColorModelID.id(), Float32BitsColorDepthID.id(), 0);
+        break;
+#endif
+    default:
+        colorSpace = 0;
+    }
+
+    KisImageSP img = new KisImage(undoStore, image.width(), image.height(), colorSpace, i18n("Imported Image"));
+    KisPaintLayerSP layer = new KisPaintLayer(img, img->nextLayerName(), 255);
+    layer->paintDevice()->convertFromQImage(image, 0, 0, 0);
+    img->addNode(layer.data(), img->rootLayer().data());
+
+    return img;
+}
+
 KisImage *KisImage::clone(bool exactCopy)
 {
     return new KisImage(*this, 0, exactCopy);
 }
 
+void KisImage::copyFromImage(const KisImage &rhs)
+{
+    copyFromImageImpl(rhs, REPLACE);
+}
+
+void KisImage::copyFromImageImpl(const KisImage &rhs, int policy)
+{
+    // make sure we choose exactly one from REPLACE and CONSTRUCT
+    KIS_ASSERT_RECOVER_RETURN((policy & REPLACE) != (policy & CONSTRUCT));
+
+    // only when replacing do we need to emit signals
+#define EMIT_IF_NEEDED if (!(policy & REPLACE)) {} else emit
+
+    if (policy & REPLACE) { // if we are constructing the image, these are already set
+        if (m_d->width != rhs.width() || m_d->height != rhs.height()) {
+            m_d->width = rhs.width();
+            m_d->height = rhs.height();
+            emit sigSizeChanged(QPointF(), QPointF());
+        }
+        if (m_d->colorSpace != rhs.colorSpace()) {
+            m_d->colorSpace = rhs.colorSpace();
+            emit sigColorSpaceChanged(m_d->colorSpace);
+        }
+    }
+
+    // from KisImage::KisImage(const KisImage &, KisUndoStore *, bool)
+    setObjectName(rhs.objectName());
+
+    if (m_d->xres != rhs.m_d->xres || m_d->yres != rhs.m_d->yres) {
+        m_d->xres = rhs.m_d->xres;
+        m_d->yres = rhs.m_d->yres;
+        EMIT_IF_NEEDED sigResolutionChanged(m_d->xres, m_d->yres);
+    }
+
+    m_d->allowMasksOnRootNode = rhs.m_d->allowMasksOnRootNode;
+
+    if (rhs.m_d->proofingConfig) {
+        KisProofingConfigurationSP proofingConfig(new KisProofingConfiguration(*rhs.m_d->proofingConfig));
+        if (policy & REPLACE) {
+            setProofingConfiguration(proofingConfig);
+        } else {
+            m_d->proofingConfig = proofingConfig;
+        }
+    }
+
+    KisNodeSP newRoot = rhs.root()->clone();
+    newRoot->setGraphListener(this);
+    newRoot->setImage(this);
+    m_d->rootLayer = dynamic_cast<KisGroupLayer*>(newRoot.data());
+    setRoot(newRoot);
+
+    bool exactCopy = policy & EXACT_COPY;
+
+    if (exactCopy || rhs.m_d->isolatedRootNode || rhs.m_d->overlaySelectionMask) {
+        QQueue<KisNodeSP> linearizedNodes;
+        KisLayerUtils::recursiveApplyNodes(rhs.root(),
+                                           [&linearizedNodes](KisNodeSP node) {
+                                               linearizedNodes.enqueue(node);
+                                           });
+        KisLayerUtils::recursiveApplyNodes(newRoot,
+                                           [&linearizedNodes, exactCopy, &rhs, this](KisNodeSP node) {
+                                               KisNodeSP refNode = linearizedNodes.dequeue();
+
+                                               if (exactCopy) {
+                                                   node->setUuid(refNode->uuid());
+                                               }
+
+                                               if (rhs.m_d->isolatedRootNode &&
+                                                   rhs.m_d->isolatedRootNode == refNode) {
+                                                   m_d->isolatedRootNode = node;
+                                               }
+
+                                               if (rhs.m_d->overlaySelectionMask &&
+                                                   KisNodeSP(rhs.m_d->overlaySelectionMask) == refNode) {
+                                                   m_d->targetOverlaySelectionMask = dynamic_cast<KisSelectionMask*>(node.data());
+                                                   m_d->overlaySelectionMask = m_d->targetOverlaySelectionMask;
+                                                   m_d->rootLayer->notifyChildMaskChanged();
+                                               }
+                                           });
+    }
+
+    KisLayerUtils::recursiveApplyNodes(newRoot,
+                                       [](KisNodeSP node) {
+                                           dbgImage << "Node: " << (void *)node.data();
+                                       });
+
+    m_d->compositions.clear();
+
+    Q_FOREACH (KisLayerCompositionSP comp, rhs.m_d->compositions) {
+        m_d->compositions << toQShared(new KisLayerComposition(*comp, this));
+    }
+
+    EMIT_IF_NEEDED sigLayersChangedAsync();
+
+    vKisAnnotationSP newAnnotations;
+    Q_FOREACH (KisAnnotationSP annotation, rhs.m_d->annotations) {
+        newAnnotations << annotation->clone();
+    }
+    m_d->annotations = newAnnotations;
+
+    KIS_ASSERT_RECOVER_NOOP(rhs.m_d->projectionUpdatesFilters.isEmpty());
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableUIUpdateSignals);
+    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableDirtyRequests);
+
+    m_d->blockLevelOfDetail = rhs.m_d->blockLevelOfDetail;
+
+#undef EMIT_IF_NEEDED
+}
+
 KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
     : KisNodeFacade(),
       KisNodeGraphListener(),
@@ -301,79 +476,7 @@ KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
     moveToThread(qApp->thread());
     connect(this, SIGNAL(sigInternalStopIsolatedModeRequested()), SLOT(stopIsolatedMode()));
 
-    setObjectName(rhs.objectName());
-
-    m_d->xres = rhs.m_d->xres;
-    m_d->yres = rhs.m_d->yres;
-
-    m_d->allowMasksOnRootNode = rhs.m_d->allowMasksOnRootNode;
-
-    if (rhs.m_d->proofingConfig) {
-        m_d->proofingConfig = toQShared(new KisProofingConfiguration(*rhs.m_d->proofingConfig));
-    }
-
-    KisNodeSP newRoot = rhs.root()->clone();
-    newRoot->setGraphListener(this);
-    newRoot->setImage(this);
-    m_d->rootLayer = dynamic_cast<KisGroupLayer*>(newRoot.data());
-    setRoot(newRoot);
-
-    if (exactCopy || rhs.m_d->isolatedRootNode || rhs.m_d->overlaySelectionMask) {
-        QQueue<KisNodeSP> linearizedNodes;
-        KisLayerUtils::recursiveApplyNodes(rhs.root(),
-            [&linearizedNodes](KisNodeSP node) {
-                linearizedNodes.enqueue(node);
-            });
-        KisLayerUtils::recursiveApplyNodes(newRoot,
-            [&linearizedNodes, exactCopy, &rhs, this](KisNodeSP node) {
-                KisNodeSP refNode = linearizedNodes.dequeue();
-
-                if (exactCopy) {
-                    node->setUuid(refNode->uuid());
-                }
-
-                if (rhs.m_d->isolatedRootNode &&
-                    rhs.m_d->isolatedRootNode == refNode) {
-
-                    m_d->isolatedRootNode = node;
-                }
-
-                if (rhs.m_d->overlaySelectionMask &&
-                    KisNodeSP(rhs.m_d->overlaySelectionMask) == refNode) {
-
-                    m_d->targetOverlaySelectionMask = dynamic_cast<KisSelectionMask*>(node.data());
-                    m_d->overlaySelectionMask = m_d->targetOverlaySelectionMask;
-                    m_d->rootLayer->notifyChildMaskChanged();
-                }
-        });
-    }
-
-    m_d->compositions.clear();
-    Q_FOREACH (KisLayerCompositionSP comp, rhs.m_d->compositions) {
-        m_d->compositions << toQShared(new KisLayerComposition(*comp, this));
-    }
-
-    rhs.m_d->nserver = KisNameServer(rhs.m_d->nserver);
-
-    vKisAnnotationSP newAnnotations;
-    Q_FOREACH (KisAnnotationSP annotation, rhs.m_d->annotations) {
-        newAnnotations << annotation->clone();
-    }
-    m_d->annotations = newAnnotations;
-
-    KIS_ASSERT_RECOVER_NOOP(rhs.m_d->projectionUpdatesFilters.isEmpty());
-    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableUIUpdateSignals);
-    KIS_ASSERT_RECOVER_NOOP(!rhs.m_d->disableDirtyRequests);
-
-    m_d->blockLevelOfDetail = rhs.m_d->blockLevelOfDetail;
-
-    /**
-     * The overlay device is not inherited when cloning the image!
-     */
-    if (rhs.m_d->overlaySelectionMask) {
-        const QRect dirtyRect = rhs.m_d->overlaySelectionMask->extent();
-        m_d->rootLayer->setDirty(dirtyRect);
-    }
+    copyFromImageImpl(rhs, CONSTRUCT | (exactCopy ? EXACT_COPY : 0));
 }
 
 void KisImage::aboutToAddANode(KisNode *parent, int index)
@@ -425,7 +528,7 @@ void KisImage::setOverlaySelectionMask(KisSelectionMaskSP mask)
 
     struct UpdateOverlaySelectionStroke : public KisSimpleStrokeStrategy {
         UpdateOverlaySelectionStroke(KisImageSP image)
-            : KisSimpleStrokeStrategy("update-overlay-selection-mask", kundo2_noi18n("update-overlay-selection-mask")),
+            : KisSimpleStrokeStrategy(QLatin1String("update-overlay-selection-mask"), kundo2_noi18n("update-overlay-selection-mask")),
               m_image(image)
         {
             this->enableJob(JOB_INIT, true, KisStrokeJobData::BARRIER, KisStrokeJobData::EXCLUSIVE);
@@ -495,7 +598,7 @@ void KisImage::setGlobalSelection(KisSelectionSP globalSelection)
     }
     else {
         if (!selectionMask) {
-            selectionMask = new KisSelectionMask(this);
+            selectionMask = new KisSelectionMask(this, i18n("Selection Mask"));
             selectionMask->initSelection(m_d->rootLayer);
             addNode(selectionMask);
             // If we do not set the selection now, the setActive call coming next
@@ -538,21 +641,31 @@ QString KisImage::nextLayerName(const QString &_baseName) const
 {
     QString baseName = _baseName;
 
-    if (m_d->nserver.currentSeed() == 0) {
-        m_d->nserver.number();
+    int numLayers = 0;
+    int maxLayerIndex = 0;
+    QRegularExpression numberedLayerRegexp(".* (\\d+)$");
+    KisLayerUtils::recursiveApplyNodes(root(),
+        [&numLayers, &maxLayerIndex, &numberedLayerRegexp] (KisNodeSP node) {
+            if (node->inherits("KisLayer")) {
+                QRegularExpressionMatch match = numberedLayerRegexp.match(node->name());
+
+                if (match.hasMatch()) {
+                    maxLayerIndex = qMax(maxLayerIndex, match.captured(1).toInt());
+                }
+                numLayers++;
+            }
+        });
+
+    // special case if there is only root node
+    if (numLayers == 1) {
         return i18n("background");
     }
 
     if (baseName.isEmpty()) {
-        baseName = i18n("Layer");
+        baseName = i18n("Paint Layer");
     }
 
-    return QString("%1 %2").arg(baseName).arg(m_d->nserver.number());
-}
-
-void KisImage::rollBackLayerName()
-{
-    m_d->nserver.rollback();
+    return QString("%1 %2").arg(baseName).arg(maxLayerIndex + 1);
 }
 
 KisCompositeProgressProxy* KisImage::compositeProgressProxy()
@@ -569,7 +682,9 @@ void KisImage::barrierLock(bool readOnly)
 {
     if (!locked()) {
         requestStrokeEnd();
+        KisBusyWaitBroker::instance()->notifyWaitOnImageStarted(this);
         m_d->scheduler.barrierLock();
+        KisBusyWaitBroker::instance()->notifyWaitOnImageEnded(this);
         m_d->lockedForReadOnly = readOnly;
     } else {
         m_d->lockedForReadOnly &= readOnly;
@@ -604,7 +719,9 @@ void KisImage::lock()
 {
     if (!locked()) {
         requestStrokeEnd();
+        KisBusyWaitBroker::instance()->notifyWaitOnImageStarted(this);
         m_d->scheduler.lock();
+        KisBusyWaitBroker::instance()->notifyWaitOnImageEnded(this);
     }
     m_d->lockCount++;
     m_d->lockedForReadOnly = false;
@@ -681,6 +798,56 @@ void KisImage::cropImage(const QRect& newRect)
     resizeImageImpl(newRect, true);
 }
 
+void KisImage::purgeUnusedData(bool isCancellable)
+{
+    /**
+     * WARNING: don't use this function unless you know what you are doing!
+     *
+     * It breaks undo on layers! Therefore, after calling it, KisImage is not
+     * undo-capable anymore!
+     */
+
+    struct PurgeUnusedDataStroke : public KisRunnableBasedStrokeStrategy {
+        PurgeUnusedDataStroke(KisImageSP image, bool isCancellable)
+            : KisRunnableBasedStrokeStrategy(QLatin1String("purge-unused-data"),
+                                             kundo2_noi18n("purge-unused-data")),
+              m_image(image)
+        {
+            this->enableJob(JOB_INIT, true, KisStrokeJobData::BARRIER, KisStrokeJobData::EXCLUSIVE);
+            this->enableJob(JOB_DOSTROKE, true);
+            setClearsRedoOnStart(false);
+            setRequestsOtherStrokesToEnd(!isCancellable);
+            setCanForgetAboutMe(isCancellable);
+        }
+
+        void initStrokeCallback() {
+            KisPaintDeviceList deviceList;
+            QVector<KisStrokeJobData*> jobsData;
+
+            KisLayerUtils::recursiveApplyNodes(m_image->root(),
+                [&deviceList](KisNodeSP node) {
+                   deviceList << node->getLodCapableDevices();
+                 });
+
+            Q_FOREACH (KisPaintDeviceSP device, deviceList) {
+                if (!device) continue;
+
+                KritaUtils::addJobConcurrent(jobsData,
+                    [device] () {
+                        const_cast<KisPaintDevice*>(device.data())->purgeDefaultPixels();
+                    });
+            }
+
+            addMutatedJobs(jobsData);
+        }
+
+    private:
+        KisImageSP m_image;
+    };
+
+    KisStrokeId id = startStroke(new PurgeUnusedDataStroke(this, isCancellable));
+    endStroke(id);
+}
 
 void KisImage::cropNode(KisNodeSP node, const QRect& newRect)
 {
@@ -1005,24 +1172,29 @@ void KisImage::convertLayerColorSpace(KisNodeSP node,
 {
     if (!node->projectionLeaf()->isLayer()) return;
 
-    const KoColorSpace *srcColorSpace = node->colorSpace();
+    if (!dstColorSpace) return;
 
-    if (!dstColorSpace || *srcColorSpace == *dstColorSpace) return;
+    const bool hasNodesToConvert =
+        KisLayerUtils::recursiveFindNode(node,
+            [dstColorSpace](KisNodeSP node) {
+                return *node->colorSpace() != *dstColorSpace;
+            });
+
+    if (!hasNodesToConvert) return;
 
     KUndo2MagicString actionName =
-        kundo2_i18n("Convert Layer Type"); // restored old name for 4.2 branch
+        kundo2_i18n("Convert Layer Color Space");
 
     KisImageSignalVector emitSignals;
     emitSignals << ModifiedSignal;
 
     KisProcessingApplicator applicator(this, node,
-                                       KisProcessingApplicator::RECURSIVE |
-                                       KisProcessingApplicator::NO_UI_UPDATES,
+                                       KisProcessingApplicator::RECURSIVE,
                                        emitSignals, actionName);
 
     applicator.applyVisitor(
         new KisConvertColorSpaceProcessingVisitor(
-            srcColorSpace, dstColorSpace,
+            dstColorSpace,
             renderingIntent, conversionFlags),
         KisStrokeJobData::CONCURRENT);
 
@@ -1059,7 +1231,19 @@ void KisImage::KisImagePrivate::convertImageColorSpaceImpl(const KoColorSpace *d
 {
     const KoColorSpace *srcColorSpace = this->colorSpace;
 
-    if (!dstColorSpace || *srcColorSpace == *dstColorSpace) return;
+    if (!dstColorSpace) return;
+
+    if (convertLayers) {
+        const bool hasNodesToConvert =
+            KisLayerUtils::recursiveFindNode(this->rootLayer,
+                [dstColorSpace](KisNodeSP node) {
+                    return *node->colorSpace() != *dstColorSpace;
+                });
+
+        if (!hasNodesToConvert && *this->colorSpace == *dstColorSpace) return;
+    } else {
+        if (*this->colorSpace == *dstColorSpace) return;
+    }
 
     const KUndo2MagicString actionName =
         convertLayers ?
@@ -1084,7 +1268,7 @@ void KisImage::KisImagePrivate::convertImageColorSpaceImpl(const KoColorSpace *d
     if (convertLayers) {
         applicator.applyVisitor(
                     new KisConvertColorSpaceProcessingVisitor(
-                        srcColorSpace, dstColorSpace,
+                        dstColorSpace,
                         renderingIntent, conversionFlags),
                     KisStrokeJobData::CONCURRENT);
     } else {
@@ -1240,7 +1424,6 @@ void KisImage::setResolution(double xres, double yres)
 {
     m_d->xres = xres;
     m_d->yres = yres;
-    m_d->signalRouter.emitNotification(ResolutionChangedSignal);
 }
 
 QPointF KisImage::documentToPixel(const QPointF &documentCoord) const
@@ -1556,7 +1739,9 @@ KisImageSignalRouter* KisImage::signalRouter()
 void KisImage::waitForDone()
 {
     requestStrokeEnd();
+    KisBusyWaitBroker::instance()->notifyWaitOnImageStarted(this);
     m_d->scheduler.waitForDone();
+    KisBusyWaitBroker::instance()->notifyWaitOnImageEnded(this);
 }
 
 KisStrokeId KisImage::startStroke(KisStrokeStrategy *strokeStrategy)
@@ -1605,29 +1790,42 @@ bool KisImage::startIsolatedMode(KisNodeSP node)
 {
     struct StartIsolatedModeStroke : public KisRunnableBasedStrokeStrategy {
         StartIsolatedModeStroke(KisNodeSP node, KisImageSP image)
-            : KisRunnableBasedStrokeStrategy("start-isolated-mode", kundo2_noi18n("start-isolated-mode")),
+            : KisRunnableBasedStrokeStrategy(QLatin1String("start-isolated-mode"),
+                                             kundo2_noi18n("start-isolated-mode")),
               m_node(node),
-              m_image(image)
+              m_image(image),
+              m_needsFullRefresh(false)
         {
             this->enableJob(JOB_INIT, true, KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
             this->enableJob(JOB_DOSTROKE, true);
+            this->enableJob(JOB_FINISH, true, KisStrokeJobData::BARRIER);
             setClearsRedoOnStart(false);
         }
 
-        void initStrokeCallback() {
+        void initStrokeCallback() override {
             // pass-though node don't have any projection prepared, so we should
             // explicitly regenerate it before activating isolated mode.
             m_node->projectionLeaf()->explicitlyRegeneratePassThroughProjection();
 
+            const bool beforeVisibility = m_node->projectionLeaf()->visible();
             m_image->m_d->isolatedRootNode = m_node;
             emit m_image->sigIsolatedModeChanged();
+            const bool afterVisibility = m_node->projectionLeaf()->visible();
 
+            m_needsFullRefresh = (beforeVisibility != afterVisibility);
+        }
+
+        void finishStrokeCallback() override {
             // the GUI uses our thread to do the color space conversion so we
             // need to emit this signal in multiple threads
-            QVector<KisRunnableStrokeJobData*> jobs;
-            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
-            this->runnableJobsInterface()->addRunnableJobs(jobs);
 
+            if (m_needsFullRefresh) {
+                m_image->refreshGraphAsync(m_node);
+            } else {
+                QVector<KisRunnableStrokeJobData*> jobs;
+                m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
+                this->runnableJobsInterface()->addRunnableJobs(jobs);
+            }
 
             m_image->invalidateAllFrames();
         }
@@ -1635,6 +1833,7 @@ bool KisImage::startIsolatedMode(KisNodeSP node)
     private:
         KisNodeSP m_node;
         KisImageSP m_image;
+        bool m_needsFullRefresh;
     };
 
     KisStrokeId id = startStroke(new StartIsolatedModeStroke(node, this));
@@ -1649,39 +1848,52 @@ void KisImage::stopIsolatedMode()
 
     struct StopIsolatedModeStroke : public KisRunnableBasedStrokeStrategy {
         StopIsolatedModeStroke(KisImageSP image)
-            : KisRunnableBasedStrokeStrategy("stop-isolated-mode", kundo2_noi18n("stop-isolated-mode")),
-              m_image(image)
+            : KisRunnableBasedStrokeStrategy(QLatin1String("stop-isolated-mode"), kundo2_noi18n("stop-isolated-mode")),
+              m_image(image),
+              m_oldRootNode(nullptr),
+              m_oldNodeNeedsRefresh(false)
         {
             this->enableJob(JOB_INIT);
             this->enableJob(JOB_DOSTROKE, true);
+            this->enableJob(JOB_FINISH, true, KisStrokeJobData::BARRIER);
             setClearsRedoOnStart(false);
         }
 
         void initStrokeCallback() {
             if (!m_image->m_d->isolatedRootNode)  return;
 
-            //KisNodeSP oldRootNode = m_image->m_d->isolatedRootNode;
+            m_oldRootNode = m_image->m_d->isolatedRootNode;
+            const bool beforeVisibility = m_oldRootNode->projectionLeaf()->visible();
             m_image->m_d->isolatedRootNode = 0;
-
             emit m_image->sigIsolatedModeChanged();
+            const bool afterVisibility = m_oldRootNode->projectionLeaf()->visible();
+
+            m_oldNodeNeedsRefresh = (beforeVisibility != afterVisibility);
+        }
+
+        void finishStrokeCallback() override {
 
             m_image->invalidateAllFrames();
 
-            // the GUI uses our thread to do the color space conversion so we
-            // need to emit this signal in multiple threads
-            QVector<KisRunnableStrokeJobData*> jobs;
-            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
-            this->runnableJobsInterface()->addRunnableJobs(jobs);
+            if (m_oldNodeNeedsRefresh){
+                m_oldRootNode->setDirty(m_image->bounds());
+            } else {
+                // TODO: Substitute notifyProjectionUpdated() with this code
+                // when update optimization is implemented
+                //
+                // QRect updateRect = bounds() | oldRootNode->extent();
+                //oldRootNode->setDirty(updateRect);
 
-            // TODO: Substitute notifyProjectionUpdated() with this code
-            // when update optimization is implemented
-            //
-            // QRect updateRect = bounds() | oldRootNode->extent();
-            // oldRootNode->setDirty(updateRect);
+                QVector<KisRunnableStrokeJobData*> jobs;
+                m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
+                this->runnableJobsInterface()->addRunnableJobs(jobs);
+            }
         }
 
     private:
         KisImageSP m_image;
+        KisNodeSP m_oldRootNode;
+        bool m_oldNodeNeedsRefresh;
     };
 
     KisStrokeId id = startStroke(new StopIsolatedModeStroke(this));
@@ -1777,18 +1989,47 @@ void KisImage::refreshGraphAsync(KisNodeSP root, const QRect &rc)
 }
 
 void KisImage::refreshGraphAsync(KisNodeSP root, const QRect &rc, const QRect &cropRect)
+{
+    refreshGraphAsync(root, QVector<QRect>({rc}), cropRect);
+}
+
+void KisImage::refreshGraphAsync(KisNodeSP root, const QVector<QRect> &rects, const QRect &cropRect)
 {
     if (!root) root = m_d->rootLayer;
 
-    m_d->animationInterface->notifyNodeChanged(root.data(), rc, true);
-    m_d->scheduler.fullRefreshAsync(root, rc, cropRect);
+    /**
+     * We iterate through the filters in a reversed way. It makes the most nested filters
+     * to execute first.
+     */
+    for (auto it = m_d->projectionUpdatesFilters.rbegin();
+         it != m_d->projectionUpdatesFilters.rend();
+         ++it) {
+
+        KIS_SAFE_ASSERT_RECOVER(*it) { continue; }
+
+        if ((*it)->filterRefreshGraph(this, root.data(), rects, cropRect)) {
+            return;
+        }
+    }
+
+
+    m_d->animationInterface->notifyNodeChanged(root.data(), rects, true);
+    m_d->scheduler.fullRefreshAsync(root, rects, cropRect);
 }
 
+
 void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QRect &rc, const QRect &cropRect)
+{
+    requestProjectionUpdateNoFilthy(pseudoFilthy, rc, cropRect, true);
+}
+
+void KisImage::requestProjectionUpdateNoFilthy(KisNodeSP pseudoFilthy, const QRect &rc, const QRect &cropRect, const bool resetAnimationCache)
 {
     KIS_ASSERT_RECOVER_RETURN(pseudoFilthy);
 
-    m_d->animationInterface->notifyNodeChanged(pseudoFilthy.data(), rc, false);
+    if (resetAnimationCache) {
+        m_d->animationInterface->notifyNodeChanged(pseudoFilthy.data(), rc, false);
+    }
     m_d->scheduler.updateProjectionNoFilthy(pseudoFilthy, rc, cropRect);
 }
 
@@ -1908,7 +2149,7 @@ int KisImage::workingThreadsLimit() const
 void KisImage::notifySelectionChanged()
 {
     /**
-     * The selection is calculated asynchromously, so it is not
+     * The selection is calculated asynchronously, so it is not
      * handled by disableUIUpdates() and other special signals of
      * KisImageSignalRouter
      */
@@ -2010,6 +2251,24 @@ void KisImage::removeComposition(KisLayerCompositionSP composition)
     m_d->compositions.removeAll(composition);
 }
 
+void KisImage::moveCompositionUp(KisLayerCompositionSP composition)
+{
+    int index = m_d->compositions.indexOf(composition);
+    if (index <= 0) {
+        return;
+    }
+    m_d->compositions.move(index, index - 1);
+}
+
+void KisImage::moveCompositionDown(KisLayerCompositionSP composition)
+{
+    int index = m_d->compositions.indexOf(composition);
+    if (index >= m_d->compositions.size() -1) {
+        return;
+    }
+    m_d->compositions.move(index, index + 1);
+}
+
 bool checkMasksNeedConversion(KisNodeSP root, const QRect &bounds)
 {
     KisSelectionMask *mask = dynamic_cast<KisSelectionMask*>(root.data());
