@@ -26,6 +26,10 @@
 #define CONNECT_TO_IMAGE(signal)                                        \
     connect(this, SIGNAL(signal), m_image, SIGNAL(signal), Qt::DirectConnection)
 
+#define CONNECT_TO_IMAGE_QUEUED(signal)                                 \
+    connect(this, SIGNAL(signal), m_image, SIGNAL(signal), Qt::QueuedConnection)
+
+
 struct ImageSignalsStaticRegistrar {
     ImageSignalsStaticRegistrar() {
         qRegisterMetaType<KisImageSignalType>("KisImageSignalType");
@@ -35,15 +39,14 @@ static ImageSignalsStaticRegistrar __registrar;
 
 
 KisImageSignalRouter::KisImageSignalRouter(KisImageWSP image)
-    : m_image(image)
+    : QObject(image.data()),
+      m_image(image)
 {
     connect(this, SIGNAL(sigNotification(KisImageSignalType)),
             SLOT(slotNotification(KisImageSignalType)));
 
     CONNECT_TO_IMAGE(sigImageModified());
     CONNECT_TO_IMAGE(sigSizeChanged(const QPointF&, const QPointF&));
-    CONNECT_TO_IMAGE(sigProfileChanged(const KoColorProfile*));
-    CONNECT_TO_IMAGE(sigColorSpaceChanged(const KoColorSpace*));
     CONNECT_TO_IMAGE(sigResolutionChanged(double, double));
     CONNECT_TO_IMAGE(sigRequestNodeReselection(KisNodeSP, const KisNodeList&));
 
@@ -51,6 +54,17 @@ KisImageSignalRouter::KisImageSignalRouter(KisImageWSP image)
     CONNECT_TO_IMAGE(sigNodeAddedAsync(KisNodeSP));
     CONNECT_TO_IMAGE(sigRemoveNodeAsync(KisNodeSP));
     CONNECT_TO_IMAGE(sigLayersChangedAsync());
+
+    /**
+     * Color space and profile conversion functions run without storkes,
+     * therefore they are executed in GUI hread under the global lock held.
+     *
+     * To ensure that the receiver of the signal will not deadlock by
+     * barrier-locking the image, we should make these signals queued.
+     */
+
+    CONNECT_TO_IMAGE_QUEUED(sigProfileChanged(const KoColorProfile*));
+    CONNECT_TO_IMAGE_QUEUED(sigColorSpaceChanged(const KoColorSpace*));
 }
 
 KisImageSignalRouter::~KisImageSignalRouter()
@@ -86,43 +100,80 @@ void KisImageSignalRouter::emitNodeChanged(KisNodeSP node)
 
 void KisImageSignalRouter::emitNodeHasBeenAdded(KisNode *parent, int index)
 {
-    m_image->invalidateAllFrames();
-    emit sigNodeAddedAsync(parent->at(index));
+    KisNodeSP newNode = parent->at(index);
+
+    // overlay selection masks reset frames themselves
+    if (!newNode->inherits("KisSelectionMask")) {
+        KisImageSP image = m_image.toStrongRef();
+        if (image) {
+            image->invalidateAllFrames();
+        }
+    }
+
+    emit sigNodeAddedAsync(newNode);
 }
 
 void KisImageSignalRouter::emitAboutToRemoveANode(KisNode *parent, int index)
 {
-    m_image->invalidateAllFrames();
-    emit sigRemoveNodeAsync(parent->at(index));
+    KisNodeSP removedNode = parent->at(index);
+
+    // overlay selection masks reset frames themselves
+    if (!removedNode->inherits("KisSelectionMask")) {
+        KisImageSP image = m_image.toStrongRef();
+        if (image) {
+            image->invalidateAllFrames();
+        }
+    }
+
+    emit sigRemoveNodeAsync(removedNode);
 }
 
+void KisImageSignalRouter::emitRequestLodPlanesSyncBlocked(bool value)
+{
+    emit sigRequestLodPlanesSyncBlocked(value);
+}
+
+void KisImageSignalRouter::emitNotifyBatchUpdateStarted()
+{
+    emit sigNotifyBatchUpdateStarted();
+}
+
+void KisImageSignalRouter::emitNotifyBatchUpdateEnded()
+{
+    emit sigNotifyBatchUpdateEnded();
+}
 
 void KisImageSignalRouter::slotNotification(KisImageSignalType type)
 {
+    KisImageSP image = m_image.toStrongRef();
+    if (!image) {
+        return;
+    }
+
     switch(type.id) {
     case LayersChangedSignal:
-        m_image->invalidateAllFrames();
+        image->invalidateAllFrames();
         emit sigLayersChangedAsync();
         break;
     case ModifiedSignal:
         emit sigImageModified();
         break;
     case SizeChangedSignal:
-        m_image->invalidateAllFrames();
+        image->invalidateAllFrames();
         emit sigSizeChanged(type.sizeChangedSignal.oldStillPoint,
                             type.sizeChangedSignal.newStillPoint);
         break;
     case ProfileChangedSignal:
-        m_image->invalidateAllFrames();
-        emit sigProfileChanged(m_image->profile());
+        image->invalidateAllFrames();
+        emit sigProfileChanged(image->profile());
         break;
     case ColorSpaceChangedSignal:
-        m_image->invalidateAllFrames();
-        emit sigColorSpaceChanged(m_image->colorSpace());
+        image->invalidateAllFrames();
+        emit sigColorSpaceChanged(image->colorSpace());
         break;
     case ResolutionChangedSignal:
-        m_image->invalidateAllFrames();
-        emit sigResolutionChanged(m_image->xRes(), m_image->yRes());
+        image->invalidateAllFrames();
+        emit sigResolutionChanged(image->xRes(), image->yRes());
         break;
     case NodeReselectionRequestSignal:
         if (type.nodeReselectionSignal.newActiveNode ||
