@@ -28,7 +28,7 @@
 #include "kis_image_config.h"
 #include "kis_raster_keyframe_channel.h"
 
-Q_GLOBAL_STATIC(KisOnionSkinCompositor, s_instance);
+Q_GLOBAL_STATIC(KisOnionSkinCompositor, s_instance)
 
 struct KisOnionSkinCompositor::Private
 {
@@ -39,6 +39,7 @@ struct KisOnionSkinCompositor::Private
     QVector<int> backwardOpacities;
     QVector<int> forwardOpacities;
     int configSeqNo = 0;
+    QList<int> colorLabelFilter;
 
     int skinOpacity(int offset)
     {
@@ -52,10 +53,26 @@ struct KisOnionSkinCompositor::Private
     {
         KisPaintDeviceSP tintDevice = new KisPaintDevice(colorSpace);
         KoColor color = KoColor(tintColor, colorSpace);
-        tintDevice->setDefaultPixel(color.data());
+        tintDevice->setDefaultPixel(color);
         return tintDevice;
     }
 
+
+    KisKeyframeSP getNextFrameToComposite(KisKeyframeChannel *channel, KisKeyframeSP keyframe, bool backwards)
+    {
+        while (!keyframe.isNull()) {
+            keyframe = backwards ? channel->previousKeyframe(keyframe) : channel->nextKeyframe(keyframe);
+            if (colorLabelFilter.isEmpty()) {
+                return keyframe;
+            } else if (!keyframe.isNull()) {
+                if (colorLabelFilter.contains(keyframe->colorLabel())) {
+                    return keyframe;
+                }
+            }
+        }
+        return keyframe;
+    }
+
     void tryCompositeFrame(KisRasterKeyframeChannel *keyframes, KisKeyframeSP keyframe, KisPainter &gcFrame, KisPainter &gcDest, KisPaintDeviceSP tintSource, int opacity, const QRect &rect)
     {
         if (keyframe.isNull() || opacity == OPACITY_TRANSPARENT_U8) return;
@@ -70,7 +87,7 @@ struct KisOnionSkinCompositor::Private
 
     void refreshConfig()
     {
-        KisImageConfig config;
+        KisImageConfig config(true);
 
         numberOfSkins = config.numberOfOnionSkins();
         tintFactor = config.onionSkinTintFactor();
@@ -114,6 +131,11 @@ int KisOnionSkinCompositor::configSeqNo() const
     return m_d->configSeqNo;
 }
 
+void KisOnionSkinCompositor::setColorLabelFilter(QList<int> colors)
+{
+    m_d->colorLabelFilter = colors;
+}
+
 void KisOnionSkinCompositor::composite(const KisPaintDeviceSP sourceDevice, KisPaintDeviceSP targetDevice, const QRect& rect)
 {
     KisRasterKeyframeChannel *keyframes = sourceDevice->keyframeChannel();
@@ -134,16 +156,22 @@ void KisOnionSkinCompositor::composite(const KisPaintDeviceSP sourceDevice, KisP
     KisKeyframeSP keyframeFwd;
 
     int time = sourceDevice->defaultBounds()->currentTime();
+
+    if (!keyframes) { // it happens when you try to show onion skins on non-animated layer with opacity keyframes
+        return;
+    }
+
     keyframeBck = keyframeFwd = keyframes->activeKeyframeAt(time);
 
     for (int offset = 1; offset <= m_d->numberOfSkins; offset++) {
+        keyframeBck = m_d->getNextFrameToComposite(keyframes, keyframeBck, true);
+        keyframeFwd = m_d->getNextFrameToComposite(keyframes, keyframeFwd, false);
+
         if (!keyframeBck.isNull()) {
-            keyframeBck = keyframes->previousKeyframe(keyframeBck);
             m_d->tryCompositeFrame(keyframes, keyframeBck, gcFrame, gcDest, backwardTintDevice, m_d->skinOpacity(-offset), rect);
         }
 
         if (!keyframeFwd.isNull()) {
-            keyframeFwd = keyframes->nextKeyframe(keyframeFwd);
             m_d->tryCompositeFrame(keyframes, keyframeFwd, gcFrame, gcDest, forwardTintDevice, m_d->skinOpacity(offset), rect);
         }
     }
@@ -174,6 +202,11 @@ QRect KisOnionSkinCompositor::calculateExtent(const KisPaintDeviceSP device)
     KisKeyframeSP keyframeFwd;
 
     KisRasterKeyframeChannel *channel = device->keyframeChannel();
+
+    if (!channel) { // it happens when you try to show onion skins on non-animated layer with opacity keyframes
+        return rect;
+    }
+
     keyframeBck = keyframeFwd = channel->activeKeyframeAt(device->defaultBounds()->currentTime());
 
     for (int offset = 1; offset <= m_d->numberOfSkins; offset++) {
