@@ -27,9 +27,13 @@
 #include "kis_stroke_strategy_undo_command_based.h"
 #include "kis_layer_utils.h"
 #include "kis_command_utils.h"
+#include "kis_time_range.h"
+#include "kis_node.h"
 #include "kis_image_signal_router.h"
+#include "KisAsynchronouslyMergeableCommandInterface.h"
+#include "kis_command_ids.h"
 
-class DisableUIUpdatesCommand : public KisCommandUtils::FlipFlopCommand
+class DisableUIUpdatesCommand : public KisCommandUtils::FlipFlopCommand, public KisAsynchronouslyMergeableCommandInterface
 {
 public:
     DisableUIUpdatesCommand(KisImageWSP image,
@@ -47,22 +51,37 @@ public:
         m_image->enableUIUpdates();
     }
 
+    int id() const override {
+        return KisCommandUtils::DisableUIUpdatesCommandId;
+    }
+
+    bool mergeWith(const KUndo2Command *command) override {
+        return canMergeWith(command);
+    }
+
+    bool canMergeWith(const KUndo2Command *command) const override {
+        const DisableUIUpdatesCommand *other =
+            dynamic_cast<const DisableUIUpdatesCommand*>(command);
+
+        return other && other->m_image == m_image;
+    }
+
 private:
     KisImageWSP m_image;
 };
 
 
-class UpdateCommand : public KisCommandUtils::FlipFlopCommand
+class UpdateCommand : public KisCommandUtils::FlipFlopCommand, public KisAsynchronouslyMergeableCommandInterface
 {
 public:
     UpdateCommand(KisImageWSP image, KisNodeSP node,
                   KisProcessingApplicator::ProcessingFlags flags,
-                  bool finalUpdate, bool multiframeApplication)
-        : FlipFlopCommand(finalUpdate),
+                  State initialState, QSharedPointer<bool> sharedAllFramesToken)
+        : FlipFlopCommand(initialState),
           m_image(image),
           m_node(node),
           m_flags(flags),
-          m_multiframeApplication(multiframeApplication)
+          m_sharedAllFramesToken(sharedAllFramesToken)
     {
     }
 
@@ -81,7 +100,7 @@ private:
     void partB() override {
         m_image->enableDirtyRequests();
 
-        if (m_multiframeApplication) {
+        if (*m_sharedAllFramesToken) {
             KisLayerUtils::recursiveApplyNodes(m_image->root(), [](KisNodeSP node){
                 KisPaintLayer* paintLayer = qobject_cast<KisPaintLayer*>(node.data());
                 if (paintLayer && paintLayer->onionSkinEnabled()) {
@@ -90,6 +109,8 @@ private:
             });
         }
 
+        m_image->root()->graphListener()->invalidateFrames(KisTimeRange::infinite(0), m_node->exactBounds());
+
         if (!m_flags.testFlag(KisProcessingApplicator::NO_IMAGE_UPDATES)) {
             if(m_flags.testFlag(KisProcessingApplicator::RECURSIVE)) {
                 m_image->refreshGraphAsync(m_node);
@@ -124,14 +145,34 @@ private:
         }
     }
 
+    int id() const override {
+        return KisCommandUtils::UpdateCommandId;
+    }
+
+    bool mergeWith(const KUndo2Command *command) override {
+        return canMergeWith(command);
+    }
+
+    bool canMergeWith(const KUndo2Command *command) const override {
+        const UpdateCommand *other =
+            dynamic_cast<const UpdateCommand*>(command);
+
+        return other &&
+            other->m_image == m_image &&
+            other->m_node == m_node &&
+            other->m_flags == m_flags &&
+            bool(other->m_sharedAllFramesToken) == bool(m_sharedAllFramesToken) &&
+            (!m_sharedAllFramesToken || *m_sharedAllFramesToken == *other->m_sharedAllFramesToken);
+    }
+
 private:
     KisImageWSP m_image;
     KisNodeSP m_node;
     KisProcessingApplicator::ProcessingFlags m_flags;
-    bool m_multiframeApplication;
+    QSharedPointer<bool> m_sharedAllFramesToken;
 };
 
-class EmitImageSignalsCommand : public KisCommandUtils::FlipFlopCommand
+class EmitImageSignalsCommand : public KisCommandUtils::FlipFlopCommand, public KisAsynchronouslyMergeableCommandInterface
 {
 public:
     EmitImageSignalsCommand(KisImageWSP image,
@@ -159,6 +200,25 @@ public:
         }
     }
 
+    int id() const override {
+        return KisCommandUtils::EmitImageSignalsCommandId;
+    }
+
+    bool mergeWith(const KUndo2Command *command) override {
+        return canMergeWith(command);
+    }
+
+    bool canMergeWith(const KUndo2Command *command) const override {
+        const EmitImageSignalsCommand *other =
+            dynamic_cast<const EmitImageSignalsCommand*>(command);
+
+        return other &&
+            other->m_image == m_image;
+
+            // TODO: implement proper comparison for emitted signals
+            // other->m_emitSignals == m_emitSignals;
+    }
+
 private:
     void doUpdate(KisImageSignalVector emitSignals) {
         Q_FOREACH (KisImageSignalType type, emitSignals) {
@@ -184,7 +244,7 @@ KisProcessingApplicator::KisProcessingApplicator(KisImageWSP image,
       m_flags(flags),
       m_emitSignals(emitSignals),
       m_finalSignalsEmitted(false),
-      m_appliedVisitorToAllFrames(false)
+      m_sharedAllFramesToken(new bool(false))
 {
     KisStrokeStrategyUndoCommandBased *strategy =
             new KisStrokeStrategyUndoCommandBased(name, false, m_image.data());
@@ -209,7 +269,9 @@ KisProcessingApplicator::KisProcessingApplicator(KisImageWSP image,
     }
 
     if (m_node) {
-        applyCommand(new UpdateCommand(m_image, m_node, m_flags, false, true));
+        applyCommand(new UpdateCommand(m_image, m_node, m_flags,
+                                       UpdateCommand::INITIALIZING,
+                                       m_sharedAllFramesToken));
     }
 }
 
@@ -217,6 +279,10 @@ KisProcessingApplicator::~KisProcessingApplicator()
 {
 }
 
+const KisStrokeId KisProcessingApplicator::getStroke() const
+{
+    return m_strokeId;
+}
 
 void KisProcessingApplicator::applyVisitor(KisProcessingVisitorSP visitor,
                                            KisStrokeJobData::Sequentiality sequentiality,
@@ -241,7 +307,7 @@ void KisProcessingApplicator::applyVisitorAllFrames(KisProcessingVisitorSP visit
                                                     KisStrokeJobData::Sequentiality sequentiality,
                                                     KisStrokeJobData::Exclusivity exclusivity)
 {
-    m_appliedVisitorToAllFrames = true;
+    *m_sharedAllFramesToken = true;
 
     KUndo2Command *initCommand = visitor->createInitCommand();
     if (initCommand) {
@@ -322,7 +388,9 @@ void KisProcessingApplicator::explicitlyEmitFinalSignals()
     KIS_ASSERT_RECOVER_RETURN(!m_finalSignalsEmitted);
 
     if (m_node) {
-        applyCommand(new UpdateCommand(m_image, m_node, m_flags, true, m_appliedVisitorToAllFrames));
+        applyCommand(new UpdateCommand(m_image, m_node, m_flags,
+                                       UpdateCommand::FINALIZING,
+                                       m_sharedAllFramesToken));
     }
 
     if(m_flags.testFlag(NO_UI_UPDATES)) {
