@@ -26,10 +26,20 @@ KisProjectionUpdatesFilter::~KisProjectionUpdatesFilter()
 {
 }
 
-bool KisDropAllProjectionUpdatesFilter::filter(KisImage *image, KisNode *node, const QRect& rect)
+bool KisDropAllProjectionUpdatesFilter::filter(KisImage *image, KisNode *node, const QVector<QRect> &rects, bool resetAnimationCache)
 {
     Q_UNUSED(image);
     Q_UNUSED(node);
-    Q_UNUSED(rect);
+    Q_UNUSED(rects);
+    Q_UNUSED(resetAnimationCache);
+    return true;
+}
+
+bool KisDropAllProjectionUpdatesFilter::filterRefreshGraph(KisImage *image, KisNode *node, const QVector<QRect> &rects, const QRect &cropRect)
+{
+    Q_UNUSED(image);
+    Q_UNUSED(node);
+    Q_UNUSED(rects);
+    Q_UNUSED(cropRect);
     return true;
 }
