@@ -22,10 +22,12 @@
 #include <functional>
 using KisStrokeStrategyFactory = std::function<KisStrokeStrategy*()>;
 
-using KisLodSyncPair = QPair<KisStrokeStrategy*, QList<KisStrokeJobData*>>;
+using KisLodSyncPair = std::pair<KisStrokeStrategy*, QList<KisStrokeJobData*>>;
 using KisLodSyncStrokeStrategyFactory = std::function<KisLodSyncPair(bool /*forgettable*/)>;
 
-using KisSuspendResumePair = QPair<KisStrokeStrategy*, QList<KisStrokeJobData*>>;
+using KisSuspendResumePair = std::pair<KisStrokeStrategy*, QList<KisStrokeJobData*>>;
 using KisSuspendResumeStrategyFactory = std::function<KisSuspendResumePair()>;
+using KisSuspendResumeStrategyPairFactory = std::function<std::pair<KisSuspendResumePair, KisSuspendResumePair>()>;
+
 
 #endif /* __KIS_STROKE_STRATEGY_FACTORY_H */
