@@ -81,7 +81,7 @@ public:
     void debugDumpAllStrokes();
 
     // interface for KisStrokeStrategy only!
-    void addMutatedJobs(KisStrokeId id, const QVector<KisStrokeJobData*> list) final;
+    void addMutatedJobs(KisStrokeId id, const QVector<KisStrokeJobData*> list) final override;
 
 private:
     bool processOneJob(KisUpdaterContext &updaterContext,
