@@ -23,6 +23,7 @@
 #include "kis_paint_device_frames_interface.h"
 #include "kis_datamanager.h"
 #include "kis_image.h"
+#include "KoColor.h"
 
 //#define DEBUG_TRANSACTIONS
 
@@ -43,9 +44,12 @@ public:
     QPoint oldOffset;
     QPoint newOffset;
 
+    KoColor oldDefaultPixel;
+    bool defaultPixelChanged = false;
+
     bool savedOutlineCacheValid;
     QPainterPath savedOutlineCache;
-    KUndo2Command *flattenUndoCommand;
+    QScopedPointer<KUndo2Command> flattenUndoCommand;
     bool resetSelectionOutlineCache;
 
     int transactionTime;
@@ -69,6 +73,8 @@ KisTransactionData::KisTransactionData(const KUndo2MagicString& name, KisPaintDe
 {
     m_d->resetSelectionOutlineCache = resetSelectionOutlineCache;
     setTimedID(-1);
+
+    possiblyFlattenSelection(device);
     init(device);
     saveSelectionOutlineCache();
 }
@@ -101,9 +107,9 @@ void KisTransactionData::init(KisPaintDeviceSP device)
     DEBUG_ACTION("Transaction started");
 
     m_d->oldOffset = QPoint(device->x(), device->y());
+    m_d->oldDefaultPixel = device->defaultPixel();
     m_d->firstRedo = true;
     m_d->transactionFinished = false;
-    m_d->flattenUndoCommand = 0;
 
     m_d->transactionTime = device->defaultBounds()->currentTime();
 
@@ -144,6 +150,7 @@ void KisTransactionData::endTransaction()
         m_d->transactionFinished = true;
         m_d->savedDataManager->commit();
         m_d->newOffset = QPoint(m_d->device->x(), m_d->device->y());
+        m_d->defaultPixelChanged = m_d->oldDefaultPixel != m_d->device->defaultPixel();
     }
 }
 
@@ -166,6 +173,10 @@ void KisTransactionData::startUpdates()
                 totalExtent.translated(m_d->newOffset);
         }
 
+        if (m_d->defaultPixelChanged) {
+            rc |= m_d->device->defaultBounds()->bounds();
+        }
+
         m_d->device->setDirty(rc);
     } else {
         m_d->device->framesInterface()->invalidateFrameCache(m_d->transactionFrameId);
@@ -195,6 +206,39 @@ void KisTransactionData::possiblyResetOutlineCache()
     }
 }
 
+void KisTransactionData::possiblyFlattenSelection(KisPaintDeviceSP device)
+{
+    KisPixelSelectionSP pixelSelection =
+        dynamic_cast<KisPixelSelection*>(device.data());
+
+    if (pixelSelection) {
+        KisSelection *selection = pixelSelection->parentSelection().data();
+        if (selection) {
+            m_d->flattenUndoCommand.reset(selection->flatten());
+
+            if (m_d->flattenUndoCommand) {
+                m_d->flattenUndoCommand->redo();
+            }
+        }
+    }
+}
+
+void KisTransactionData::doFlattenUndoRedo(bool undo)
+{
+    KisPixelSelectionSP pixelSelection =
+        dynamic_cast<KisPixelSelection*>(m_d->device.data());
+
+    if (pixelSelection) {
+        if (m_d->flattenUndoCommand) {
+            if (undo) {
+                m_d->flattenUndoCommand->undo();
+            } else {
+                m_d->flattenUndoCommand->redo();
+            }
+        }
+    }
+}
+
 void KisTransactionData::Private::possiblySwitchCurrentTime()
 {
     if (device->defaultBounds()->currentTime() == transactionTime) return;
@@ -215,7 +259,7 @@ void KisTransactionData::redo()
         return;
     }
 
-
+    doFlattenUndoRedo(false);
     restoreSelectionOutlineCache(false);
 
     m_d->newFrameCommand.redo();
@@ -245,6 +289,7 @@ void KisTransactionData::undo()
     }
 
     restoreSelectionOutlineCache(true);
+    doFlattenUndoRedo(true);
 
     m_d->possiblySwitchCurrentTime();
     startUpdates();
@@ -267,14 +312,6 @@ void KisTransactionData::saveSelectionOutlineCache()
 
             possiblyResetOutlineCache();
         }
-
-        KisSelectionSP selection = pixelSelection->parentSelection();
-        if (selection) {
-            m_d->flattenUndoCommand = selection->flatten();
-            if (m_d->flattenUndoCommand) {
-                m_d->flattenUndoCommand->redo();
-            }
-        }
     }
 }
 
@@ -302,13 +339,5 @@ void KisTransactionData::restoreSelectionOutlineCache(bool undo)
         if (m_d->savedOutlineCacheValid) {
             m_d->savedOutlineCache = savedOutlineCache;
         }
-
-        if (m_d->flattenUndoCommand) {
-            if (undo) {
-                m_d->flattenUndoCommand->undo();
-            } else {
-                m_d->flattenUndoCommand->redo();
-            }
-        }
     }
 }
