@@ -168,9 +168,9 @@ void KisUpdateScheduler::updateProjectionNoFilthy(KisNodeSP node, const QRect& r
     processQueues();
 }
 
-void KisUpdateScheduler::fullRefreshAsync(KisNodeSP root, const QRect& rc, const QRect &cropRect)
+void KisUpdateScheduler::fullRefreshAsync(KisNodeSP root, const QVector<QRect>& rects, const QRect &cropRect)
 {
-    m_d->updatesQueue.addFullRefreshJob(root, rc, cropRect, currentLevelOfDetail());
+    m_d->updatesQueue.addFullRefreshJob(root, rects, cropRect, currentLevelOfDetail());
     processQueues();
 }
 
@@ -470,13 +470,11 @@ void KisUpdateScheduler::spareThreadAppeared()
 KisTestableUpdateScheduler::KisTestableUpdateScheduler(KisProjectionUpdateListener *projectionUpdateListener,
                                                        qint32 threadCount)
 {
-    Q_UNUSED(threadCount);
     updateSettings();
     m_d->projectionUpdateListener = projectionUpdateListener;
 
-    // The queue will update settings in a constructor itself
-    // m_d->updatesQueue = new KisTestableSimpleUpdateQueue();
-    // m_d->strokesQueue = new KisStrokesQueue();
+    setThreadsLimit(threadCount);
+    m_d->updaterContext.setTestingMode(true);
 
     connectSignals();
 }
@@ -485,8 +483,3 @@ KisUpdaterContext *KisTestableUpdateScheduler::updaterContext()
 {
     return &m_d->updaterContext;
 }
-
-KisTestableSimpleUpdateQueue* KisTestableUpdateScheduler::updateQueue()
-{
-    return dynamic_cast<KisTestableSimpleUpdateQueue*>(&m_d->updatesQueue);
-}
