@@ -39,7 +39,8 @@
 #include "kis_layer_style_filter_environment.h"
 #include "kis_selection_filters.h"
 #include "kis_multiple_projection.h"
-
+#include "kis_default_bounds_base.h"
+#include "kis_cached_paint_device.h"
 
 namespace KisLsUtils
 {
@@ -50,26 +51,27 @@ namespace KisLsUtils
 
         if (growSize > 0) {
             KisGrowSelectionFilter filter(growSize, growSize);
-            changeRect = filter.changeRect(applyRect);
+            changeRect = filter.changeRect(applyRect, selection->defaultBounds());
             filter.process(selection, applyRect);
         } else if (growSize < 0) {
             KisShrinkSelectionFilter filter(qAbs(growSize), qAbs(growSize), false);
-            changeRect = filter.changeRect(applyRect);
+            changeRect = filter.changeRect(applyRect, selection->defaultBounds());
             filter.process(selection, applyRect);
         }
 
         return changeRect;
     }
 
-    KisSelectionSP selectionFromAlphaChannel(KisPaintDeviceSP device,
-                                             const QRect &srcRect)
+    void selectionFromAlphaChannel(KisPaintDeviceSP srcDevice,
+                                      KisSelectionSP dstSelection,
+                                      const QRect &srcRect)
     {
-        const KoColorSpace *cs = device->colorSpace();
+        const KoColorSpace *cs = srcDevice->colorSpace();
 
-        KisSelectionSP baseSelection = new KisSelection(new KisSelectionEmptyBounds(0));
-        KisPixelSelectionSP selection = baseSelection->pixelSelection();
 
-        KisSequentialConstIterator srcIt(device, srcRect);
+        KisPixelSelectionSP selection = dstSelection->pixelSelection();
+
+        KisSequentialConstIterator srcIt(srcDevice, srcRect);
         KisSequentialIterator dstIt(selection, srcRect);
 
         while (srcIt.nextPixel() && dstIt.nextPixel()) {
@@ -78,7 +80,6 @@ namespace KisLsUtils
             *dstPtr = cs->opacityU8(srcPtr);
         }
 
-        return baseSelection;
     }
 
     void findEdge(KisPixelSelectionSP selection, const QRect &applyRect, const bool edgeHidden)
@@ -246,18 +247,21 @@ namespace KisLsUtils
                     const QRect &applyRect,
                     int noise,
                     const psd_layer_effects_context *context,
-                    const KisLayerStyleFilterEnvironment *env)
+                    KisLayerStyleFilterEnvironment *env)
     {
         Q_UNUSED(context);
 
         const QRect overlayRect = kisGrowRect(applyRect, noiseNeedBorder);
 
         KisPixelSelectionSP randomSelection = env->cachedRandomSelection(overlayRect);
-        KisPixelSelectionSP randomOverlay = new KisPixelSelection();
+
+
+        KisCachedSelection::Guard s1(*env->cachedSelection());
+        KisPixelSelectionSP randomOverlay = s1.selection()->pixelSelection();
 
         KisSequentialConstIterator noiseIt(randomSelection, overlayRect);
         KisSequentialConstIterator srcIt(selection, overlayRect);
-        KisRandomAccessorSP dstIt = randomOverlay->createRandomAccessorNG(overlayRect.x(), overlayRect.y());
+        KisRandomAccessorSP dstIt = randomOverlay->createRandomAccessorNG();
 
         while (noiseIt.nextPixel() && srcIt.nextPixel()) {
             int itX = noiseIt.x();
@@ -385,9 +389,6 @@ namespace KisLsUtils
                      int verticalPhase,
                      bool alignWithLayer)
     {
-        if (scale != 100) {
-            warnKrita << "KisLsOverlayFilter::applyOverlay(): Pattern scaling is NOT implemented!";
-        }
         KIS_SAFE_ASSERT_RECOVER_RETURN(pattern);
 
         QSize psize(pattern->width(), pattern->height());
@@ -406,8 +407,11 @@ namespace KisLsUtils
         QRect fillRect = applyRect | applyRect.translated(patternOffset);
 
         KisFillPainter gc(fillDevice);
-        gc.fillRect(fillRect.x(), fillRect.y(),
-                    fillRect.width(), fillRect.height(), pattern, -patternOffset);
+        QTransform transform;
+        transform.translate(-patternOffset.x(), -patternOffset.y());
+        qreal scaleNorm = qreal(scale*0.01);
+        transform.scale(scaleNorm, scaleNorm);
+        gc.fillRectNoCompose(fillRect, pattern, transform);
         gc.end();
     }
 
@@ -548,7 +552,7 @@ namespace KisLsUtils
 
         const QRect effectRect(dstRect);
         const QString compositeOp = config->blendMode();
-        const quint8 opacityU8 = 255.0 / 100.0 * config->opacity();
+        const quint8 opacityU8 = quint8(qRound(255.0 / 100.0 * config->opacity()));
         KisPaintDeviceSP dstDevice = dst->getProjection(projectionId, compositeOp, opacityU8, QBitArray(), srcDevice);
 
         if (config->fillType() == psd_fill_solid_color) {
