@@ -89,7 +89,8 @@ class ForbiddenLodStrokeStrategy : public KisStrokeStrategy
 {
 public:
     ForbiddenLodStrokeStrategy(std::function<void()> lodCallback)
-        : m_lodCallback(lodCallback)
+        : KisStrokeStrategy(QLatin1String("ForbiddenLodStrokeStrategy")),
+          m_lodCallback(lodCallback)
     {
     }
 
@@ -455,7 +456,7 @@ struct FlattenTestImage
         layer5->disableAlphaChannel(true);
 
         layer2 = new KisPaintLayer(p.image, "paint2", OPACITY_OPAQUE_U8);
-        tmask = new KisTransparencyMask();
+        tmask = new KisTransparencyMask(p.image, "tmask");
 
         // check channel flags
         // make addition composite op
@@ -927,7 +928,6 @@ void KisImageTest::testMergeMultiple()
 
 void testMergeCrossColorSpaceImpl(bool useProjectionColorSpace, bool swapSpaces)
 {
-    QRect refRect;
     TestUtil::MaskParent p;
 
     KisPaintLayerSP layer1;
@@ -995,7 +995,6 @@ void KisImageTest::testMergeCrossColorSpace()
 
 void KisImageTest::testMergeSelectionMasks()
 {
-    QRect refRect;
     TestUtil::MaskParent p;
 
     QRect rect1(100, 100, 100, 100);
