@@ -27,6 +27,7 @@
 #include "kis_transform_mask_params_interface.h"
 
 #include "testutil.h"
+#include "kistest.h"
 
 #include "kis_algebra_2d.h"
 #include "kis_safe_transform.h"
@@ -200,7 +201,7 @@ void KisTransformMaskTest::testSafeTransformSingleVanishingPoint()
 bool doPartialTests(const QString &prefix, KisImageSP image, KisLayerSP paintLayer,
                     KisLayerSP visibilityToggleLayer, KisTransformMaskSP mask)
 {
-    TestUtil::ExternalImageChecker chk(prefix, "transform_mask_updates");
+    TestUtil::ReferenceImageChecker chk(prefix, "transform_mask_updates");
 
     bool result = true;
 
@@ -377,7 +378,7 @@ void KisTransformMaskTest::testMaskOnPaintLayer()
     KisPaintLayerSP player = new KisPaintLayer(p.image, "bg", OPACITY_OPAQUE_U8, p.image->colorSpace());
     p.image->addNode(player, p.image->root(), KisNodeSP());
 
-    KisTransformMaskSP mask = new KisTransformMask();
+    KisTransformMaskSP mask = new KisTransformMask(p.image, "mask");
     p.image->addNode(mask, p.layer);
 
     QTransform transform(-0.177454, -0.805953, -0.00213713,
@@ -404,7 +405,7 @@ void KisTransformMaskTest::testMaskOnCloneLayer()
     KisCloneLayerSP clone = new KisCloneLayer(p.layer, p.image, "clone", OPACITY_OPAQUE_U8);
     p.image->addNode(clone, p.image->root());
 
-    KisTransformMaskSP mask = new KisTransformMask();
+    KisTransformMaskSP mask = new KisTransformMask(p.image, "mask");
     p.image->addNode(mask, clone);
 
     QTransform transform(-0.177454, -0.805953, -0.00213713,
@@ -419,7 +420,7 @@ void KisTransformMaskTest::testMaskOnCloneLayer()
 
 void KisTransformMaskTest::testMaskOnCloneLayerWithOffset()
 {
-    TestUtil::ExternalImageChecker chk("clone_offset_simple", "transform_mask_updates");
+    TestUtil::ReferenceImageChecker chk("clone_offset_simple", "transform_mask_updates");
 
     QRect refRect(0,0,512,512);
     QRect fillRect(400,400,100,100);
@@ -433,7 +434,7 @@ void KisTransformMaskTest::testMaskOnCloneLayerWithOffset()
     KisCloneLayerSP clone = new KisCloneLayer(p.layer, p.image, "clone", OPACITY_OPAQUE_U8);
     p.image->addNode(clone, p.image->root());
 
-    KisTransformMaskSP mask = new KisTransformMask();
+    KisTransformMaskSP mask = new KisTransformMask(p.image, "mask");
     p.image->addNode(mask, clone);
 
     QTransform transform(1, 0, 0,
@@ -474,7 +475,7 @@ void KisTransformMaskTest::testMaskOnCloneLayerWithOffset()
 
 void KisTransformMaskTest::testMultipleMasks()
 {
-    TestUtil::ExternalImageChecker chk("multiple_masks", "transform_mask_updates");
+    TestUtil::ReferenceImageChecker chk("multiple_masks", "transform_mask_updates");
 
     QRect refRect(0,0,512,512);
     QRect fillRect(400,400,100,100);
@@ -488,15 +489,12 @@ void KisTransformMaskTest::testMultipleMasks()
     //KisCloneLayerSP clone = new KisCloneLayer(p.layer, p.image, "clone", OPACITY_OPAQUE_U8);
     //p.image->addNode(clone, p.image->root());
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     p.image->addNode(mask1, p.layer);
 
-    KisTransformMaskSP mask2 = new KisTransformMask();
+    KisTransformMaskSP mask2 = new KisTransformMask(p.image, "mask2");
     p.image->addNode(mask2, p.layer);
 
-    mask1->setName("mask1");
-    mask2->setName("mask2");
-
     p.layer->setDirty(refRect);
     p.image->waitForDone();
     chk.checkImage(p.image, "00_initial_layer_update");
@@ -760,7 +758,7 @@ void KisTransformMaskTest::testMultipleMasks()
 
 void KisTransformMaskTest::testMaskWithOffset()
 {
-    TestUtil::ExternalImageChecker chk("mask_with_offset", "transform_mask_updates");
+    TestUtil::ReferenceImageChecker chk("mask_with_offset", "transform_mask_updates");
 
     QRect refRect(0,0,512,512);
     QRect fillRect(400,400,100,100);
@@ -771,11 +769,9 @@ void KisTransformMaskTest::testMaskWithOffset()
     KisPaintLayerSP player = new KisPaintLayer(p.image, "bg", OPACITY_OPAQUE_U8, p.image->colorSpace());
     p.image->addNode(player, p.image->root(), KisNodeSP());
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     p.image->addNode(mask1, p.layer);
 
-    mask1->setName("mask1");
-
     p.layer->setDirty(refRect);
     p.image->waitForDone();
     chk.checkImage(p.image, "00_initial_layer_update");
@@ -826,8 +822,7 @@ void KisTransformMaskTest::testWeirdFullUpdates()
     player1->paintDevice()->fill(fillRect, KoColor(Qt::red, p.layer->colorSpace()));
     p.image->addNode(player1, p.image->root());
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
-    mask1->setName("mask1");
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     QTransform transform1 =
         QTransform::fromTranslate(256, 0);
     mask1->setTransformParams(KisTransformMaskParamsInterfaceSP(
@@ -840,8 +835,7 @@ void KisTransformMaskTest::testWeirdFullUpdates()
     player2->paintDevice()->fill(fillRect, KoColor(Qt::red, p.layer->colorSpace()));
     p.image->addNode(player2, p.image->root());
 
-    KisTransformMaskSP mask2 = new KisTransformMask();
-    mask2->setName("mask2");
+    KisTransformMaskSP mask2 = new KisTransformMask(p.image, "mask2");
     QTransform transform2 =
         QTransform::fromTranslate(0, 256);
     mask2->setTransformParams(KisTransformMaskParamsInterfaceSP(
@@ -854,8 +848,7 @@ void KisTransformMaskTest::testWeirdFullUpdates()
     player3->paintDevice()->fill(fillRect, KoColor(Qt::red, p.layer->colorSpace()));
     p.image->addNode(player3, p.image->root());
 
-    KisTransformMaskSP mask3 = new KisTransformMask();
-    mask3->setName("mask3");
+    KisTransformMaskSP mask3 = new KisTransformMask(p.image, "mask3");
     QTransform transform3 =
         QTransform::fromTranslate(256, 256);
     mask3->setTransformParams(KisTransformMaskParamsInterfaceSP(
@@ -965,8 +958,7 @@ void KisTransformMaskTest::testTransformHiddenPartsOfTheGroup()
 
     //KIS_DUMP_DEVICE_2(p.image->projection(), imageRect, "image_proj_initial", "dd");
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
-    mask1->setName("mask1");
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     QTransform transform1 =
         QTransform::fromTranslate(0, 1.5 * 256);
     mask1->setTransformParams(KisTransformMaskParamsInterfaceSP(
@@ -986,4 +978,4 @@ void KisTransformMaskTest::testTransformHiddenPartsOfTheGroup()
     //KIS_DUMP_DEVICE_2(p.image->projection(), imageRect, "image_proj_mask", "dd");
 }
 
-QTEST_MAIN(KisTransformMaskTest)
+KISTEST_MAIN(KisTransformMaskTest)
