@@ -26,21 +26,6 @@ void KisVecTest::testCreation()
     KisVector2D v2d = KisVector2D::Zero();
     QVERIFY(v2d.x() == 0.0);
     QVERIFY(v2d.y() == 0.0);
-
-    KisVector3D v3d = KisVector3D::Zero();
-    QVERIFY(v3d.x() == 0.0);
-    QVERIFY(v3d.y() == 0.0);
-    QVERIFY(v3d.z() == 0.0);
-}
-
-
-void KisVecTest::testVec2D()
-{
-}
-
-
-void KisVecTest::testVec3D()
-{
 }
 
 
