@@ -36,8 +36,9 @@
 #include "filter/kis_filter_registry.h"
 #include "kis_filter_mask.h"
 #include "kis_transparency_mask.h"
+#include <sdk/tests/kistest.h>
 
-#define DEBUG_VISITORS
+//#define DEBUG_VISITORS
 
 QString nodeTypeString(KisMergeWalker::NodePosition position);
 QString nodeTypePostfix(KisMergeWalker::NodePosition position);
@@ -61,7 +62,9 @@ public:
 
 protected:
 
-    void registerChangeRect(KisProjectionLeafSP node, NodePosition position) {
+    void registerChangeRect(KisProjectionLeafSP node, NodePosition position) override {
+        Q_UNUSED(position);
+
         QString postfix;
 
         if(!node->isLayer()) {
@@ -69,7 +72,7 @@ protected:
         }
 
 #ifdef DEBUG_VISITORS
-        dbgKrita<< "FW:"<< node->node()->name() <<'\t'<< nodeTypeString(position) << postfix;
+        qDebug()<< "FW:"<< node->node()->name() <<'\t'<< nodeTypeString(position) << postfix;
 #endif
 
         if(postfix.isEmpty()) {
@@ -77,7 +80,7 @@ protected:
         }
     }
 
-    void registerNeedRect(KisProjectionLeafSP node, NodePosition position) {
+    void registerNeedRect(KisProjectionLeafSP node, NodePosition position) override {
         QString postfix;
 
         if(!node->isLayer()) {
@@ -85,7 +88,7 @@ protected:
         }
 
 #ifdef DEBUG_VISITORS
-        dbgKrita<< "BW:"<< node->node()->name() <<'\t'<< nodeTypeString(position) << postfix;
+        qDebug()<< "BW:"<< node->node()->name() <<'\t'<< nodeTypeString(position) << postfix;
 #endif
         if(postfix.isEmpty()) {
             m_order.append(node->node()->name() + nodeTypePostfix(position));
@@ -111,11 +114,11 @@ struct UpdateTestJob {
 
 void reportStartWith(QString nodeName, QRect rect = QRect())
 {
-    dbgKrita;
+    qDebug();
     if(!rect.isEmpty())
-        dbgKrita << "Start with:" << nodeName << rect;
+        qDebug() << "Start with:" << nodeName << rect;
     else
-        dbgKrita << "Start with:" << nodeName;
+        qDebug() << "Start with:" << nodeName;
 }
 
 QString nodeTypeString(KisMergeWalker::NodePosition position)
@@ -195,14 +198,14 @@ void KisWalkersTest::verifyResult(KisBaseRectsWalker &walker, QStringList refere
     QStringList::const_iterator iter = reference.constBegin();
 
     if(reference.size() != list.size()) {
-        dbgKrita << "*** Seems like the walker returned stack of wrong size"
+        qDebug() << "*** Seems like the walker returned stack of wrong size"
                  << "( ref:" << reference.size() << "act:" << list.size() << ")";
-        dbgKrita << "*** We are going to crash soon... just wait...";
+        qDebug() << "*** We are going to crash soon... just wait...";
     }
 
     Q_FOREACH (const KisMergeWalker::JobItem &item, list) {
 #ifdef DEBUG_VISITORS
-        dbgKrita << item.m_leaf->node()->name() << '\t'
+        qDebug() << item.m_leaf->node()->name() << '\t'
                  << item.m_applyRect << '\t'
                  << nodeTypeString(item.m_position);
 #endif
@@ -213,7 +216,7 @@ void KisWalkersTest::verifyResult(KisBaseRectsWalker &walker, QStringList refere
     }
 
 #ifdef DEBUG_VISITORS
-    dbgKrita << "Result AR:\t" << walker.accessRect();
+    qDebug() << "Result AR:\t" << walker.accessRect();
 #endif
 
     QCOMPARE(walker.accessRect(), accessRect);
@@ -325,11 +328,11 @@ void KisWalkersTest::testVisitingWithTopmostMask()
     KisLayerSP adjustmentLayer = new KisAdjustmentLayer(image, "adj", 0, 0);
 
 
-    KisFilterMaskSP filterMask1 = new KisFilterMask();
+    KisFilterMaskSP filterMask1 = new KisFilterMask(image, "mask1");
     filterMask1->initSelection(groupLayer);
     KisFilterSP filter = KisFilterRegistry::instance()->value("blur");
-    Q_ASSERT(filter);
-    KisFilterConfiguration *configuration1 = filter->defaultConfiguration(0);
+    KIS_ASSERT(filter);
+    KisFilterConfigurationSP configuration1 = filter->defaultConfiguration();
     filterMask1->setFilter(configuration1);
 
     image->addNode(paintLayer1, image->rootLayer());
@@ -497,6 +500,71 @@ void KisWalkersTest::testMergeVisiting()
 
 }
 
+#include "kis_psd_layer_style.h"
+
+/*
+  +----------+
+  |root      |
+  | group ls |
+  |  paint 3 |
+  |  paint 2 |
+  | paint 1  |
+  +----------+
+ */
+
+void KisWalkersTest::testComplexGroupVisiting()
+{
+    const KoColorSpace * colorSpace = KoColorSpaceRegistry::instance()->rgb8();
+    KisImageSP image = new KisImage(0, 512, 512, colorSpace, "walker test");
+
+
+    KisPSDLayerStyleSP style(new KisPSDLayerStyle());
+
+    {
+        style->context()->keep_original = true;
+
+        style->dropShadow()->setEffectEnabled(true);
+        style->dropShadow()->setDistance(3);
+        style->dropShadow()->setSpread(1);
+        style->dropShadow()->setSize(7);
+        style->dropShadow()->setNoise(0);
+        style->dropShadow()->setKnocksOut(false);
+        style->dropShadow()->setOpacity(50);
+        style->dropShadow()->setAngle(0);
+    }
+
+    KisLayerSP paintLayer1 = new KisPaintLayer(image, "paint1", OPACITY_OPAQUE_U8);
+    KisLayerSP paintLayer2 = new KisPaintLayer(image, "paint2", OPACITY_OPAQUE_U8);
+    KisLayerSP paintLayer3 = new KisPaintLayer(image, "paint3", OPACITY_OPAQUE_U8);
+
+    KisLayerSP groupLayer = new KisGroupLayer(image, "groupls", OPACITY_OPAQUE_U8);
+
+    image->addNode(paintLayer1, image->rootLayer());
+    image->addNode(groupLayer, image->rootLayer());
+    image->addNode(paintLayer2, groupLayer);
+    image->addNode(paintLayer3, groupLayer);
+
+    groupLayer->setLayerStyle(style);
+
+    QRect testRect(10,10,10,10);
+    // Empty rect to show we don't need any cropping
+    QRect cropRect;
+
+    KisMergeWalker walker(cropRect);
+
+    {
+        QString order("root,groupls,paint1,"
+                      "paint3,paint2");
+        QStringList orderList = order.split(',');
+        QRect accessRect(-8,-8,46,46);
+
+        reportStartWith("paint3");
+        walker.collectRects(paintLayer3, testRect);
+        verifyResult(walker, orderList, accessRect, true, true);
+    }
+}
+
+
     /*
       +------------+
       |root        |
@@ -632,7 +700,7 @@ class TestingRefreshSubtreeWalker : public KisRefreshSubtreeWalker
 {
 public:
     TestingRefreshSubtreeWalker(QRect cropRect) : KisRefreshSubtreeWalker(cropRect) {}
-    UpdateType type() const { return FULL_REFRESH; }
+    UpdateType type() const override { return FULL_REFRESH; }
 };
 
 
@@ -685,11 +753,11 @@ void KisWalkersTest::testRefreshSubtreeVisiting()
         QString order("root,paint5,cplx2,group,paint1,"
                       "paint4,paint3,cplx1,paint2");
         QStringList orderList = order.split(',');
-        QRect accessRect(-10,-10,50,50);
+        QRect accessRect(-4,-4,38,38);
 
         reportStartWith("root");
         walker.collectRects(image->rootLayer(), testRect);
-        verifyResult(walker, orderList, accessRect, true, true);
+        verifyResult(walker, orderList, accessRect, false, true);
     }
 }
 
@@ -746,7 +814,7 @@ void KisWalkersTest::testFullRefreshVisiting()
 
         reportStartWith("root");
         walker.collectRects(groupLayer, testRect);
-        verifyResult(walker, orderList, accessRect, true, true);
+        verifyResult(walker, orderList, accessRect, false, true);
     }
 }
 
@@ -837,14 +905,14 @@ void KisWalkersTest::testMasksVisiting()
     image->addNode(paintLayer1, image->rootLayer());
     image->addNode(paintLayer2, image->rootLayer());
 
-    KisFilterMaskSP filterMask1 = new KisFilterMask();
-    KisFilterMaskSP filterMask2 = new KisFilterMask();
-    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask();
+    KisFilterMaskSP filterMask1 = new KisFilterMask(image, "fmask1");
+    KisFilterMaskSP filterMask2 = new KisFilterMask(image, "fmask2");
+    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask(image, "tmask");
 
     KisFilterSP filter = KisFilterRegistry::instance()->value("blur");
     Q_ASSERT(filter);
-    KisFilterConfiguration *configuration1 = filter->defaultConfiguration(0);
-    KisFilterConfiguration *configuration2 = filter->defaultConfiguration(0);
+    KisFilterConfigurationSP configuration1 = filter->defaultConfiguration();
+    KisFilterConfigurationSP configuration2 = filter->defaultConfiguration();
 
     filterMask1->setFilter(configuration1);
     filterMask2->setFilter(configuration2);
@@ -910,14 +978,14 @@ void KisWalkersTest::testMasksVisitingNoFilthy()
     image->addNode(paintLayer1, image->rootLayer());
     image->addNode(paintLayer2, image->rootLayer());
 
-    KisFilterMaskSP filterMask1 = new KisFilterMask();
-    KisFilterMaskSP filterMask2 = new KisFilterMask();
-    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask();
+    KisFilterMaskSP filterMask1 = new KisFilterMask(image, "fmask1");
+    KisFilterMaskSP filterMask2 = new KisFilterMask(image, "fmask2");
+    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask(image, "tmask");
 
     KisFilterSP filter = KisFilterRegistry::instance()->value("blur");
     Q_ASSERT(filter);
-    KisFilterConfiguration *configuration1 = filter->defaultConfiguration(0);
-    KisFilterConfiguration *configuration2 = filter->defaultConfiguration(0);
+    KisFilterConfigurationSP configuration1 = filter->defaultConfiguration();
+    KisFilterConfigurationSP configuration2 = filter->defaultConfiguration();
 
     filterMask1->setFilter(configuration1);
     filterMask2->setFilter(configuration2);
@@ -985,16 +1053,16 @@ void KisWalkersTest::testMasksOverlapping()
     image->addNode(paintLayer1, image->rootLayer());
     image->addNode(paintLayer2, image->rootLayer());
 
-    KisFilterMaskSP filterMask1 = new KisFilterMask();
-    KisFilterMaskSP filterMask2 = new KisFilterMask();
-    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask();
+    KisFilterMaskSP filterMask1 = new KisFilterMask(image, "fmask1");
+    KisFilterMaskSP filterMask2 = new KisFilterMask(image, "fmask2");
+    KisTransparencyMaskSP transparencyMask = new KisTransparencyMask(image, "tmask");
 
     KisFilterSP blurFilter = KisFilterRegistry::instance()->value("blur");
     KisFilterSP invertFilter = KisFilterRegistry::instance()->value("invert");
     Q_ASSERT(blurFilter);
     Q_ASSERT(invertFilter);
-    KisFilterConfiguration *blurConfiguration = blurFilter->defaultConfiguration(0);
-    KisFilterConfiguration *invertConfiguration = invertFilter->defaultConfiguration(0);
+    KisFilterConfigurationSP blurConfiguration = blurFilter->defaultConfiguration();
+    KisFilterConfigurationSP invertConfiguration = invertFilter->defaultConfiguration();
 
     filterMask1->setFilter(invertConfiguration);
     filterMask2->setFilter(blurConfiguration);
@@ -1113,7 +1181,7 @@ void KisWalkersTest::testMasksOverlapping()
     Q_FOREACH (UpdateTestJob job, updateList) {
         KisMergeWalker walker(cropRect);
         reportStartWith(job.startNode->name(), job.updateRect);
-        dbgKrita << "Area:" << job.updateAreaName;
+        qDebug() << "Area:" << job.updateAreaName;
         walker.collectRects(job.startNode, job.updateRect);
         verifyResult(walker, job);
     }
@@ -1138,27 +1206,27 @@ void KisWalkersTest::testRectsChecksum()
     KisLayerSP paintLayer1 = new KisPaintLayer(image, "paint1", OPACITY_OPAQUE_U8);
     KisAdjustmentLayerSP adjustmentLayer = new KisAdjustmentLayer(image, "adj", 0, 0);
 
-    image->lock();
+    image->barrierLock();
     image->addNode(paintLayer1, image->rootLayer());
     image->addNode(adjustmentLayer, image->rootLayer());
     image->unlock();
 
     KisFilterSP filter = KisFilterRegistry::instance()->value("blur");
     Q_ASSERT(filter);
-    KisFilterConfiguration *configuration;
+    KisFilterConfigurationSP configuration;
 
     KisMergeWalker walker(imageRect);
     walker.collectRects(adjustmentLayer, dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    configuration = filter->defaultConfiguration(0);
+    configuration = filter->defaultConfiguration();
     adjustmentLayer->setFilter(configuration);
     QCOMPARE(walker.checksumValid(), false);
 
     walker.recalculate(dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    configuration = filter->defaultConfiguration(0);
+    configuration = filter->defaultConfiguration();
     configuration->setProperty("halfWidth", 20);
     configuration->setProperty("halfHeight", 20);
     adjustmentLayer->setFilter(configuration);
@@ -1167,7 +1235,7 @@ void KisWalkersTest::testRectsChecksum()
     walker.recalculate(dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    configuration = filter->defaultConfiguration(0);
+    configuration = filter->defaultConfiguration();
     configuration->setProperty("halfWidth", 21);
     configuration->setProperty("halfHeight", 21);
     adjustmentLayer->setFilter(configuration);
@@ -1189,7 +1257,7 @@ void KisWalkersTest::testGraphStructureChecksum()
     KisLayerSP paintLayer1 = new KisPaintLayer(image, "paint1", OPACITY_OPAQUE_U8);
     KisLayerSP paintLayer2 = new KisPaintLayer(image, "paint2", OPACITY_OPAQUE_U8);
 
-    image->lock();
+    image->barrierLock();
     image->addNode(paintLayer1, image->rootLayer());
     image->unlock();
 
@@ -1197,7 +1265,7 @@ void KisWalkersTest::testGraphStructureChecksum()
     walker.collectRects(paintLayer1, dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    image->lock();
+    image->barrierLock();
     image->addNode(paintLayer2, image->rootLayer());
     image->unlock();
     QCOMPARE(walker.checksumValid(), false);
@@ -1205,7 +1273,7 @@ void KisWalkersTest::testGraphStructureChecksum()
     walker.recalculate(dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    image->lock();
+    image->barrierLock();
     image->moveNode(paintLayer1, image->rootLayer(), paintLayer2);
     image->unlock();
     QCOMPARE(walker.checksumValid(), false);
@@ -1213,7 +1281,7 @@ void KisWalkersTest::testGraphStructureChecksum()
     walker.recalculate(dirtyRect);
     QCOMPARE(walker.checksumValid(), true);
 
-    image->lock();
+    image->barrierLock();
     image->removeNode(paintLayer1);
     image->unlock();
     QCOMPARE(walker.checksumValid(), false);
@@ -1222,5 +1290,5 @@ void KisWalkersTest::testGraphStructureChecksum()
     QCOMPARE(walker.checksumValid(), true);
 }
 
-QTEST_MAIN(KisWalkersTest)
+KISTEST_MAIN(KisWalkersTest)
 
