@@ -103,6 +103,14 @@ public:
     void setDefaultTileData(KisTileData *defaultTileData);
     KisTileData* defaultTileData();
 
+    /**
+     * Returns a pointer to the default tile data object with ref counter
+     * increased by one. Make sure you call deref() after you finished using
+     * this object.
+     */
+    KisTileData* refAndFetchDefaultTileData();
+
+
     qint32 numTiles()
     {
         return m_numTiles.load();
@@ -130,7 +138,7 @@ private:
     inline quint32 calculateHash(qint32 col, qint32 row)
     {
 #ifdef SANITY_CHECK
-        KIS_ASSERT_RECOVER_NOOP(row < 0x7FFF && col < 0x7FFF)
+        KIS_ASSERT_RECOVER_NOOP(row < 0x7FFF && col < 0x7FFF);
 #endif // SANITY_CHECK
 
         if (col == 0 && row == 0) {
@@ -161,7 +169,7 @@ private:
 
         m_map.getGC().unlockRawPointerAccess();
 
-        m_map.getGC().update(m_map.migrationInProcess());
+        m_map.getGC().update();
     }
 
     inline bool erase(quint32 idx)
@@ -181,7 +189,7 @@ private:
 
         m_map.getGC().unlockRawPointerAccess();
 
-        m_map.getGC().update(m_map.migrationInProcess());
+        m_map.getGC().update();
         return wasDeleted;
     }
 
@@ -301,7 +309,7 @@ typename KisTileHashTableTraits2<T>::TileTypeSP KisTileHashTableTraits2<T>::getE
     TileTypeSP tile = m_map.get(idx);
     m_map.getGC().unlockRawPointerAccess();
 
-    m_map.getGC().update(m_map.migrationInProcess());
+    m_map.getGC().update();
     return tile;
 }
 
@@ -318,7 +326,7 @@ typename KisTileHashTableTraits2<T>::TileTypeSP KisTileHashTableTraits2<T>::getT
     TileTypeSP tile = m_map.get(idx);
 
     while (!tile) {
-        // we shouldn't try to aquire **any** lock with
+        // we shouldn't try to acquire **any** lock with
         // raw-pointer lock held
         m_map.getGC().unlockRawPointerAccess();
 
@@ -369,7 +377,7 @@ typename KisTileHashTableTraits2<T>::TileTypeSP KisTileHashTableTraits2<T>::getT
     }
     m_map.getGC().unlockRawPointerAccess();
 
-    m_map.getGC().update(m_map.migrationInProcess());
+    m_map.getGC().update();
     return tile;
 }
 
@@ -389,7 +397,7 @@ typename KisTileHashTableTraits2<T>::TileTypeSP KisTileHashTableTraits2<T>::getR
         tile = new TileType(col, row, m_defaultTileData, 0);
     }
 
-    m_map.getGC().update(m_map.migrationInProcess());
+    m_map.getGC().update();
     return tile;
 }
 
@@ -439,7 +447,7 @@ void KisTileHashTableTraits2<T>::clear()
     }
 
     // garbage collection must **not** be run with locks held
-    m_map.getGC().update(false);
+    m_map.getGC().update();
 }
 
 template <class T>
@@ -465,6 +473,15 @@ inline KisTileData* KisTileHashTableTraits2<T>::defaultTileData()
     return m_defaultTileData;
 }
 
+template <class T>
+inline KisTileData* KisTileHashTableTraits2<T>::refAndFetchDefaultTileData()
+{
+    QReadLocker locker(&m_defaultPixelDataLock);
+    m_defaultTileData->ref();
+    return m_defaultTileData;
+}
+
+
 template <class T>
 void KisTileHashTableTraits2<T>::debugPrintInfo()
 {
