@@ -174,7 +174,7 @@ bool KisTileHashTableTraits<T>::unlinkTile(qint32 col, qint32 row, qint32 idx)
              * explicitly
              */
             tile->setNext(TileTypeSP());
-            tile->notifyDead();
+            tile->notifyDetachedFromDataManager();
             tile.clear();
 
             m_numTiles--;
@@ -318,7 +318,7 @@ void KisTileHashTableTraits<T>::clear()
              */
 
             tmp->setNext(TileTypeSP());
-            tmp->notifyDead();
+            tmp->notifyDetachedFromDataManager();
             tmp = 0;
 
             m_numTiles--;
@@ -344,6 +344,15 @@ KisTileData* KisTileHashTableTraits<T>::defaultTileData() const
     return defaultTileDataImp();
 }
 
+template <class T>
+inline KisTileData* KisTileHashTableTraits<T>::refAndFetchDefaultTileData() const
+{
+    QWriteLocker locker(&m_lock);
+    KisTileData *result = defaultTileDataImp();
+    result->ref();
+    return result;
+}
+
 
 /*************** Debugging stuff ***************/
 
@@ -352,12 +361,12 @@ void KisTileHashTableTraits<T>::debugPrintInfo()
 {
     if (!m_numTiles) return;
 
-    qDebug() << "==========================\n"
+    qInfo() << "==========================\n"
              << "TileHashTable:"
              << "\n   def. data:\t\t" << m_defaultTileData
              << "\n   numTiles:\t\t" << m_numTiles;
     debugListLengthDistibution();
-    qDebug() << "==========================\n";
+    qInfo() << "==========================\n";
 }
 
 template<class T>
@@ -406,12 +415,12 @@ void KisTileHashTableTraits<T>::debugListLengthDistibution()
         array[tmp-min]++;
     }
 
-    qDebug() << QString("   minChain: %1\n").arg(min);
-    qDebug() << QString("   maxChain: %1\n").arg(max);
+    qInfo() << QString("   minChain: %1\n").arg(min);
+    qInfo() << QString("   maxChain: %1\n").arg(max);
 
-    qDebug() << "   Chain size distribution:";
+    qInfo() << "   Chain size distribution:";
     for (qint32 i = 0; i < arraySize; i++)
-        qDebug() << QString("      %1: %2").arg(i + min).arg(array[i]);
+        qInfo() << QString("      %1: %2").arg(i + min).arg(array[i]);
 
     delete[] array;
 }
