@@ -22,7 +22,6 @@
 #include "KoStore.h"
 #include "KoStore_p.h"
 
-#include "KoLegacyZipStore.h"
 #include "KoQuaZipStore.h"
 #include "KoDirectoryStore.h"
 
@@ -70,17 +69,7 @@ KoStore* KoStore::createStore(const QString& fileName, Mode mode, const QByteArr
     }
     switch (backend) {
     case Zip:
-        if (mode == KoStore::Read || (appIdentification == "application/x-krita" && KSharedConfig::openConfig()->group("").readEntry<bool>("UseZip64", false))) {
-            return new KoQuaZipStore(fileName, mode, appIdentification, writeMimetype);
-        }
-        else {
-            KoStore *store = new KoLegacyZipStore(fileName, mode, appIdentification, writeMimetype);
-            if (store->bad()) {
-                return new KoQuaZipStore(fileName, mode, appIdentification, writeMimetype);
-            }
-            return store;
-        }
-
+        return new KoQuaZipStore(fileName, mode, appIdentification, writeMimetype);
     case Directory:
         return new KoDirectoryStore(fileName /* should be a dir name.... */, mode, writeMimetype);
     default:
@@ -106,16 +95,7 @@ KoStore* KoStore::createStore(QIODevice *device, Mode mode, const QByteArray & a
         errorStore << "Can't create a Directory store for a memory buffer!" << endl;
         return 0;
     case Zip:
-        if (mode == KoStore::Read || (appIdentification == "application/x-krita" && KSharedConfig::openConfig()->group("").readEntry<bool>("UseZip64", false))) {
-            return new KoQuaZipStore(device, mode, appIdentification, writeMimetype);
-        }
-        else {
-            KoStore *store = new KoLegacyZipStore(device, mode, appIdentification, writeMimetype);
-            if (store->bad()) {
-                return new KoQuaZipStore(device, mode, appIdentification, writeMimetype);
-            }
-            return store;
-        }
+        return new KoQuaZipStore(device, mode, appIdentification, writeMimetype);
     default:
         warnStore << "Unsupported backend requested for KoStore : " << backend;
         return 0;
