@@ -86,11 +86,10 @@ public:
     QUuid id() const;
 
     /**
-     * @brief showView shows the given view. Override this if you want to show
-     * the view in a different way than by making it the central widget, for instance
-     * as an QMdiSubWindow
+     * @brief showView shows the given view, in @p subWindow if not
+     * null, in a new tab otherwise.
      */
-    virtual void showView(KisView *view);
+    virtual void showView(KisView *view, QMdiSubWindow *subWindow = 0);
 
     /**
      * @returns the currently active view
@@ -183,10 +182,17 @@ public:
 
     KisViewManager *viewManager() const;
 
-    KisView *addViewAndNotifyLoadingCompleted(KisDocument *document);
+    KisView *addViewAndNotifyLoadingCompleted(KisDocument *document,
+                                              QMdiSubWindow *subWindow = 0);
 
     QStringList showOpenFileDialog(bool isImporting);
 
+    /**
+     * The top-level window used for a detached canvas.
+     */
+    QWidget *canvasWindow() const;
+    bool canvasDetached() const;
+
     /**
      * Shows if the main window is saving anything right now. If the
      * user presses Ctrl+W too fast, then the document can be close
@@ -228,6 +234,10 @@ Q_SIGNALS:
     /// emitted when the window is migrated among different screens
     void screenChanged();
 
+    /// emitted when the current view has changed
+    void activeViewChanged();
+
+
 public Q_SLOTS:
 
 
@@ -288,7 +298,7 @@ public Q_SLOTS:
      */
     void newOptionWidgets(KoCanvasController *controller, const QList<QPointer<QWidget> > & optionWidgetList);
 
-    KisView *newView(QObject *document);
+    KisView *newView(QObject *document, QMdiSubWindow *subWindow = 0);
 
     void notifyChildViewDestroyed(KisView *view);
 
@@ -304,7 +314,23 @@ public Q_SLOTS:
      */
     void reloadRecentFileList();
 
+    /**
+     * Detach canvas onto a separate window, or restore it back to to main window.
+     */
+    void setCanvasDetached(bool detached);
 
+    /**
+     * @brief Called when a file is picked using Android's Storage Access Framework
+     * @param url
+     */
+    void slotFileSelected(QUrl url);
+
+    void slotEmptyFilePath();
+
+    /**
+     * Toggle full screen on/off.
+     */
+    void viewFullscreen(bool fullScreen);
 
 private Q_SLOTS:
     /**
@@ -337,6 +363,10 @@ private Q_SLOTS:
 
     void importAnimation();
 
+    void renderAnimation();
+
+    void renderAnimationAgain();
+
     /**
      * Show a dialog with author and document information.
      */
@@ -368,16 +398,16 @@ private Q_SLOTS:
      */
     void slotNewToolbarConfig();
 
+    /**
+     * Reset User Configurations.
+     */
+    void slotResetConfigurations();
+
     /**
      *  Shows or hides a toolbar
      */
     void slotToolbarToggled(bool toggle);
 
-    /**
-     * Toggle full screen on/off.
-     */
-    void viewFullscreen(bool fullScreen);
-
     /**
      * Reload file
      */
@@ -430,6 +460,8 @@ private Q_SLOTS:
 
     void slotXmlGuiMakingChanges(bool finished);
 
+    void orientationChanged();
+
 protected:
 
     void closeEvent(QCloseEvent * e) override;
@@ -450,7 +482,7 @@ private:
      * This is a private implementation. For public usage please use
      * newView() and addViewAndNotifyLoadingCompleted().
      */
-    void addView(KisView *view);
+    void addView(KisView *view, QMdiSubWindow *subWindow = 0);
 
     friend class KisPart;
 
@@ -463,13 +495,8 @@ private:
      */
     QDockWidget* createDockWidget(KoDockFactoryBase* factory);
 
-    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags flags = 0);
+    bool openDocumentInternal(const QUrl &url, KisMainWindow::OpenFlags f = KisMainWindow::OpenFlags());
 
-
-    /**
-     * Updates the window caption based on the document info and path.
-     */
-    void updateCaption(const QString & caption, bool modified);
     void updateReloadFileAction(KisDocument *doc);
 
     void saveWindowSettings();
@@ -504,7 +531,6 @@ private:
 
     QString m_errorMessage;
     bool m_dieOnError;
-
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KisMainWindow::OpenFlags)
