@@ -59,8 +59,12 @@
 class KoSectionListItem : public QTreeWidgetItem
 {
 public:
-    KoSectionListItem(QTreeWidget* treeWidget, const QString& name, int sortWeight, int widgetIndex = -1)
-        : QTreeWidgetItem(treeWidget, QStringList() << name), m_sortWeight(sortWeight), m_widgetIndex(widgetIndex) {
+    KoSectionListItem(QTreeWidget* treeWidget, const QString& name, QString untranslatedName, int sortWeight, int widgetIndex = -1)
+        : QTreeWidgetItem(treeWidget, QStringList() << name)
+        , m_sortWeight(sortWeight)
+        , m_widgetIndex(widgetIndex)
+        , m_untranslatedName(untranslatedName)
+    {
         Qt::ItemFlags newFlags = Qt::NoItemFlags;
 
         if(m_widgetIndex >= 0)
@@ -86,9 +90,14 @@ public:
         return m_widgetIndex;
     }
 
+    QString untranslatedName() const {
+        return m_untranslatedName;
+    }
+
 private:
     int m_sortWeight;
     int m_widgetIndex;
+    QString m_untranslatedName;
 };
 
 class KisOpenPanePrivate : public Ui_KisOpenPaneBase
@@ -158,7 +167,7 @@ KisOpenPane::~KisOpenPane()
         if (item) {
             if (!qobject_cast<KisDetailsPane*>(d->m_widgetStack->widget(item->widgetIndex()))) {
                 KConfigGroup cfgGrp( KSharedConfig::openConfig(), "TemplateChooserDialog");
-                cfgGrp.writeEntry("LastReturnType", item->text(0));
+                cfgGrp.writeEntry("LastReturnType", item->untranslatedName());
             }
         }
     }
@@ -195,7 +204,7 @@ void KisOpenPane::initTemplates(const QString& templatesResourcePath)
             }
 
             if (!d->m_templatesSeparator) {
-                d->m_templatesSeparator = new KoSectionListItem(d->m_sectionList, "", 999);
+                d->m_templatesSeparator = new KoSectionListItem(d->m_sectionList, "", "", 999);
             }
 
             KisTemplatesPane* pane = new KisTemplatesPane(this, group->name(),
@@ -209,7 +218,7 @@ void KisOpenPane::initTemplates(const QString& templatesResourcePath)
                     this, SIGNAL(splitterResized(KisDetailsPane*,QList<int>)));
             connect(this, SIGNAL(splitterResized(KisDetailsPane*,QList<int>)),
                     pane, SLOT(resizeSplitter(KisDetailsPane*,QList<int>)));
-            QTreeWidgetItem* item = addPane(group->name(), group->templates().first()->loadPicture(),
+            QTreeWidgetItem* item = addPane(group->name(), "Template", group->templates().first()->loadPicture(),
                                             pane, group->sortingWeight() + templateOffset);
 
 
@@ -234,7 +243,8 @@ void KisOpenPane::initTemplates(const QString& templatesResourcePath)
 
     if (selectItem && (cfgGrp.readEntry("LastReturnType") == "Template")) {
         d->m_sectionList->setCurrentItem(selectItem, 0, QItemSelectionModel::ClearAndSelect);
-    } else if (d->m_sectionList->selectedItems().isEmpty() && firstItem) {
+    }
+    else if (d->m_sectionList->selectedItems().isEmpty() && firstItem) {
         d->m_sectionList->setCurrentItem(firstItem, 0, QItemSelectionModel::ClearAndSelect);
     }
 }
@@ -257,36 +267,32 @@ void KisOpenPane::dropEvent(QDropEvent *event)
     }
 }
 
-void KisOpenPane::addCustomDocumentWidget(QWidget *widget, const QString& title, const QString& icon)
+void KisOpenPane::addCustomDocumentWidget(QWidget *widget, const QString& title, const QString &untranslatedName, const QString& icon)
 {
     Q_ASSERT(widget);
 
-    QString realtitle = title;
-
-    if (realtitle.isEmpty())
-        realtitle = i18n("Custom Document");
-
-    QTreeWidgetItem* item = addPane(realtitle, icon, widget, d->m_freeCustomWidgetIndex);
+    QTreeWidgetItem* item = addPane(title, untranslatedName, icon, widget, d->m_freeCustomWidgetIndex);
     ++d->m_freeCustomWidgetIndex;
     KConfigGroup cfgGrp( KSharedConfig::openConfig(), "TemplateChooserDialog");
 
     QString lastActiveItem = cfgGrp.readEntry("LastReturnType");
     bool showCustomItemByDefault = cfgGrp.readEntry("ShowCustomDocumentWidgetByDefault", false);
-    if (lastActiveItem == realtitle || (lastActiveItem.isEmpty() && showCustomItemByDefault)) {
+    if (lastActiveItem == untranslatedName || (lastActiveItem.isEmpty() && showCustomItemByDefault)) {
         d->m_sectionList->setCurrentItem(item, 0, QItemSelectionModel::ClearAndSelect);
         KoSectionListItem* selectedItem = static_cast<KoSectionListItem*>(item);
         d->m_widgetStack->widget(selectedItem->widgetIndex())->setFocus();
     }
 }
 
-QTreeWidgetItem* KisOpenPane::addPane(const QString &title, const QString &iconName, QWidget *widget, int sortWeight)
+
+QTreeWidgetItem* KisOpenPane::addPane(const QString &title, const QString &untranslatedName, const QString &iconName, QWidget *widget, int sortWeight)
 {
     if (!widget) {
         return 0;
     }
 
     int id = d->m_widgetStack->addWidget(widget);
-    KoSectionListItem* listItem = new KoSectionListItem(d->m_sectionList, title, sortWeight, id);
+    KoSectionListItem* listItem = new KoSectionListItem(d->m_sectionList, title, untranslatedName, sortWeight, id);
 
     // resizes icons so they are a bit smaller
     QIcon icon = KisIconUtils::loadIcon(iconName);
@@ -298,7 +304,7 @@ QTreeWidgetItem* KisOpenPane::addPane(const QString &title, const QString &iconN
     return listItem;
 }
 
-QTreeWidgetItem* KisOpenPane::addPane(const QString& title, const QPixmap& icon, QWidget* widget, int sortWeight)
+QTreeWidgetItem* KisOpenPane::addPane(const QString &title, const QString &untranslatedName, const QPixmap& icon, QWidget* widget, int sortWeight)
 {
     if (!widget) {
         return 0;
@@ -308,7 +314,7 @@ QTreeWidgetItem* KisOpenPane::addPane(const QString& title, const QPixmap& icon,
 
     int iconSize = 32;
 
-    KoSectionListItem* listItem = new KoSectionListItem(d->m_sectionList, title, sortWeight, id);
+    KoSectionListItem* listItem = new KoSectionListItem(d->m_sectionList, title, untranslatedName, sortWeight, id);
 
     if (!icon.isNull()) {
         QImage image = icon.toImage();
