@@ -151,23 +151,18 @@ void KisPaletteEditor::importPalette()
         return;
     }
 
-    QMessageBox messageBox;
-    messageBox.setText(i18n("Do you want to store this palette in your current image?"));
-    messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
-    bool global = (messageBox.exec() == QMessageBox::Yes);
-
     KoColorSet *colorSet = new KoColorSet(filename);
     colorSet->load();
     QString name = filenameFromPath(colorSet->filename());
 
     if (duplicateExistsFilename(name, false)) {
-        colorSet->setFilename(newPaletteFileName(global));
+        colorSet->setFilename(newPaletteFileName(true));
     } else {
         colorSet->setFilename(name);
     }
 
-    colorSet->setIsGlobal(global);
-    m_d->rServer->addResource(colorSet, global);
+    colorSet->setIsGlobal(true);
+    m_d->rServer->addResource(colorSet, true);
     m_d->rServer->removeFromBlacklist(colorSet);
 
     uploadPaletteList();
@@ -417,9 +412,9 @@ void KisPaletteEditor::modifyEntry(const QModelIndex &index)
         KisSwatch entry = m_d->model->getEntry(index);
 
         editableItems->addRow(i18n("ID"), lnIDName);
-        editableItems->addRow(i18nc("Name for a swatch group", "Swatch group name"), lnGroupName);
+        editableItems->addRow(i18nc("Name of the color swatch", "Color swatch name"), lnGroupName);
         editableItems->addRow(i18n("Color"), bnColor);
-        editableItems->addRow(i18n("Spot"), chkSpot);
+        editableItems->addRow(i18n("Spot color"), chkSpot);
 
         lnGroupName->setText(entry.name());
         lnIDName->setText(entry.id());
