@@ -246,6 +246,8 @@ private Q_SLOTS:
     void slotSaveRulersTrackMouseState(bool value);
     void slotToggleFgBg();
     void slotResetFgBg();
+    void slotResetRotation();
+    void slotToggleFullscreen();
 private:
     void createActions();
     void setupManagers();
