@@ -35,6 +35,7 @@
 #include <KoDocumentResourceManager.h>
 #include <KoShapeStroke.h>
 #include <KoDocumentInfo.h>
+#include <KoCanvasBase.h>
 
 #include "KisViewManager.h"
 #include "kis_canvas_resource_provider.h"
@@ -62,7 +63,7 @@
 #include "kis_image_animation_interface.h"
 #include "kis_time_range.h"
 #include "kis_keyframe_channel.h"
-
+#include "kis_node_manager.h"
 
 #include <processing/fill_processing_visitor.h>
 #include <kis_selection_tool_helper.h>
@@ -230,6 +231,7 @@ void KisFillActionFactory::run(const QString &fillSource, KisViewManager *view)
                                   false, // fast mode
                                   usePattern,
                                   true, // fill only selection,
+                                  false,
                                   0, // feathering radius
                                   0, // sizemod
                                   80, // threshold,
@@ -267,9 +269,12 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
     KisImageSP image = view->image();
     if (!image) return;
 
-    bool haveShapesSelected = view->selectionManager()->haveShapesSelected();
+    const bool haveShapesSelected = view->selectionManager()->haveShapesSelected();
 
-    if (haveShapesSelected) {
+    KisNodeSP node = view->activeNode();
+    KisSelectionSP selection = view->selection();
+
+    if (!makeSharpClip && haveShapesSelected) {
         // XXX: "Add saving of XML data for Cut/Copy of shapes"
 
         KisImageBarrierLocker locker(image);
@@ -278,13 +283,7 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
         } else {
             view->canvasBase()->toolProxy()->copy();
         }
-    } else {
-        KisNodeSP node = view->activeNode();
-        if (!node) return;
-
-        KisSelectionSP selection = view->selection();
-        if (selection.isNull()) return;
-
+    } else if (node && selection) {
         {
             KisImageBarrierLocker locker(image);
             KisPaintDeviceSP dev = node->paintDevice();
@@ -373,6 +372,12 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
         KisOperationConfiguration config(id());
         config.setProperty("will-cut", willCut);
         endAction(ap, config.toXML());
+    } else if (!makeSharpClip) {
+        if (willCut) {
+            view->nodeManager()->cutLayersToClipboard();
+        } else {
+            view->nodeManager()->copyLayersToClipboard();
+        }
     }
 }
 
@@ -517,10 +522,11 @@ void KisSelectionToShapeActionFactory::run(KisViewManager *view)
     KoShapeStrokeSP border(new KoShapeStroke(1.0, fgColor.toQColor()));
     shape->setStroke(border);
 
-    view->document()->shapeController()->addShape(shape);
+    KUndo2Command *cmd = view->canvasBase()->shapeController()->addShapeDirect(shape, 0);
+    KisProcessingApplicator::runSingleCommandStroke(view->image(), cmd);
 }
 
-void KisStrokeSelectionActionFactory::run(KisViewManager *view, StrokeSelectionOptions params)
+void KisStrokeSelectionActionFactory::run(KisViewManager *view, const StrokeSelectionOptions& params)
 {
     KisImageWSP image = view->image();
     if (!image) {
@@ -576,13 +582,13 @@ void KisStrokeSelectionActionFactory::run(KisViewManager *view, StrokeSelectionO
         KoShapeStrokeSP border(new KoShapeStroke(size, color));
         shape->setStroke(border);
 
-        view->document()->shapeController()->addShape(shape);
+        KUndo2Command *cmd = view->canvasBase()->shapeController()->addShapeDirect(shape, 0);
+        KisProcessingApplicator::runSingleCommandStroke(view->image(), cmd);
     }
     image->setModified();
-
 }
 
-void KisStrokeBrushSelectionActionFactory::run(KisViewManager *view, StrokeSelectionOptions params)
+void KisStrokeBrushSelectionActionFactory::run(KisViewManager *view, const StrokeSelectionOptions& params)
 {
     KisImageWSP image = view->image();
     if (!image) {
