@@ -194,7 +194,7 @@ KoShapeManager* fetchShapeManagerFromNode(KisNodeSP node)
 }
 }
 
-KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisView *view, KoShapeControllerBase *sc)
+KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisMainWindow *mainWindow, KisView *view, KoShapeControllerBase *sc)
     : KoCanvasBase(sc, resourceManager)
     , m_d(new KisCanvas2Private(this, coordConverter, view, resourceManager))
 {
@@ -204,8 +204,8 @@ KisCanvas2::KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResource
      * light.
      */
     m_d->bootstrapLodBlocked = true;
-    connect(view->mainWindow(), SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
-    connect(view->mainWindow(), SIGNAL(screenChanged()), SLOT(slotConfigChanged()));
+    connect(mainWindow, SIGNAL(guiLoadingFinished()), SLOT(bootstrapFinished()));
+    connect(mainWindow, SIGNAL(screenChanged()), SLOT(slotConfigChanged()));
 
     KisImageConfig config(false);
 
@@ -303,21 +303,25 @@ void KisCanvas2::setCanvasWidget(KisAbstractCanvasWidget *widget)
         m_d->popupPalette->setParent(widget->widget());
     }
 
-    if (m_d->canvasWidget != 0) {
+    if (m_d->canvasWidget) {
+        /**
+         * We are switching the canvas type. We should reinitialize our
+         * connections to decorations and input manager
+         */
+
         widget->setDecorations(m_d->canvasWidget->decorations());
 
-        // Redundant check for the constructor case, see below
-        if(viewManager() != 0)
+        if(viewManager()) {
             viewManager()->inputManager()->removeTrackedCanvas(this);
+            m_d->canvasWidget = widget;
+            viewManager()->inputManager()->addTrackedCanvas(this);
+        } else {
+            m_d->canvasWidget = widget;
+        }
+    } else {
+        m_d->canvasWidget = widget;
     }
 
-    m_d->canvasWidget = widget;
-
-    // Either tmp was null or we are being called by KisCanvas2 constructor that is called by KisView
-    // constructor, so the view manager still doesn't exists.
-    if(m_d->canvasWidget != 0 && viewManager() != 0)
-        viewManager()->inputManager()->addTrackedCanvas(this);
-
     if (!m_d->canvasWidget->decoration(INFINITY_DECORATION_ID)) {
         KisInfinityManager *manager = new KisInfinityManager(m_d->view, this);
         manager->setVisible(true);
@@ -630,8 +634,8 @@ void KisCanvas2::resetCanvas(bool useOpenGL)
 void KisCanvas2::startUpdateInPatches(const QRect &imageRect)
 {
     /**
-     * We don't do patched loading for openGL canvas, becasue it loads
-     * the tiles, which are bascially "patches". Therefore, big chunks
+     * We don't do patched loading for openGL canvas, because it loads
+     * the tiles, which are basically "patches". Therefore, big chunks
      * of memory are never allocated.
      */
     if (m_d->currentCanvasIsOpenGL) {
@@ -784,12 +788,12 @@ void KisCanvas2::updateCanvasProjection()
         if (!m_d->isBatchUpdateActive) {
             // TODO: Implement info->dirtyViewportRect() for KisOpenGLCanvas2 to avoid updating whole canvas
             if (m_d->currentCanvasIsOpenGL) {
-                m_d->savedUpdateRect = QRect();
+                m_d->savedUpdateRect |= vRect;
 
                 // we already had a compression in frameRenderStartCompressor, so force the update directly
                 slotDoCanvasUpdate();
             } else if (/* !m_d->currentCanvasIsOpenGL && */ !vRect.isEmpty()) {
-                m_d->savedUpdateRect = m_d->coordinatesConverter->viewportToWidget(vRect).toAlignedRect();
+                m_d->savedUpdateRect |= m_d->coordinatesConverter->viewportToWidget(vRect).toAlignedRect();
 
                 // we already had a compression in frameRenderStartCompressor, so force the update directly
                 slotDoCanvasUpdate();
@@ -892,12 +896,34 @@ void KisCanvas2::slotDoCanvasUpdate()
         return;
     }
 
-    if (m_d->savedUpdateRect.isEmpty()) {
-        m_d->canvasWidget->widget()->update();
-        emit updateCanvasRequested(m_d->canvasWidget->widget()->rect());
-    } else {
+    if (!m_d->savedUpdateRect.isEmpty()) {
         emit updateCanvasRequested(m_d->savedUpdateRect);
-        m_d->canvasWidget->widget()->update(m_d->savedUpdateRect);
+
+        if (wrapAroundViewingMode()) {
+            const QRectF rc = m_d->savedUpdateRect;
+            const QRectF widgetRect = m_d->canvasWidget->widget()->rect();
+            const QRectF imageRect = m_d->coordinatesConverter->imageRectInWidgetPixels();
+
+            const qreal relX = KisAlgebra2D::wrapValue(rc.x() - imageRect.x(), imageRect.width());
+            const qreal relY = KisAlgebra2D::wrapValue(rc.y() - imageRect.y(), imageRect.height());
+
+            const qreal baseX = std::fmod(imageRect.right(), imageRect.width()) - imageRect.width();
+            const qreal baseY = std::fmod(imageRect.bottom(), imageRect.height()) - imageRect.height();
+
+            for (qreal y = baseY; y < widgetRect.bottom(); y += imageRect.height()) {
+                for (qreal x = baseX; x < widgetRect.right(); x += imageRect.width()) {
+                    const QRectF proposedUpdateRect(x + relX, y + relY,
+                                                    rc.width(), rc.height());
+
+                    const QRect updateRect = (proposedUpdateRect & widgetRect).toAlignedRect();
+                    if (!updateRect.isEmpty()) {
+                        m_d->canvasWidget->widget()->update(updateRect);
+                    }
+                }
+            }
+        } else {
+            m_d->canvasWidget->widget()->update(m_d->savedUpdateRect);
+        }
     }
 
     m_d->savedUpdateRect = QRect();
@@ -905,10 +931,7 @@ void KisCanvas2::slotDoCanvasUpdate()
 
 void KisCanvas2::updateCanvasWidgetImpl(const QRect &rc)
 {
-    if (!m_d->canvasUpdateCompressor.isActive() ||
-        !m_d->savedUpdateRect.isEmpty()) {
-        m_d->savedUpdateRect |= rc;
-    }
+    m_d->savedUpdateRect |= !rc.isEmpty() ? rc : m_d->canvasWidget->widget()->rect();
     m_d->canvasUpdateCompressor.start();
 }
 
@@ -919,17 +942,12 @@ void KisCanvas2::updateCanvas()
 
 void KisCanvas2::updateCanvas(const QRectF& documentRect)
 {
-    if (m_d->currentCanvasIsOpenGL && m_d->canvasWidget->decorations().size() > 0) {
-        updateCanvasWidgetImpl();
-    }
-    else {
-        // updateCanvas is called from tools, never from the projection
-        // updates, so no need to prescale!
-        QRect widgetRect = m_d->coordinatesConverter->documentToWidget(documentRect).toAlignedRect();
-        widgetRect.adjust(-2, -2, 2, 2);
-        if (!widgetRect.isEmpty()) {
-            updateCanvasWidgetImpl(widgetRect);
-        }
+    // updateCanvas is called from tools, never from the projection
+    // updates, so no need to prescale!
+    QRect widgetRect = m_d->coordinatesConverter->documentToWidget(documentRect).toAlignedRect();
+    widgetRect.adjust(-2, -2, 2, 2);
+    if (!widgetRect.isEmpty()) {
+        updateCanvasWidgetImpl(widgetRect);
     }
 }
 
@@ -1084,13 +1102,7 @@ void KisCanvas2::slotConfigChanged()
     //       the coordinates should be able to work around this.
     // FIXME: We should change to associate the display profiles with the screen
     //        model and serial number instead. See https://bugs.kde.org/show_bug.cgi?id=407498
-    QScreen *canvasScreen = this->canvasWidget()->window()->windowHandle()->screen();
-    QPoint canvasScreenCenter = canvasScreen->geometry().center();
-    int canvasScreenNumber = QApplication::desktop()->screenNumber(canvasScreenCenter);
-    if (canvasScreenNumber == -1) {
-        // Fall back to the old way of getting the screenNumber
-        canvasScreenNumber = QApplication::desktop()->screenNumber(this->canvasWidget());
-    }
+    int canvasScreenNumber = QApplication::desktop()->screenNumber(this->canvasWidget());
     if (canvasScreenNumber != -1) {
         setDisplayProfile(cfg.displayProfile(canvasScreenNumber));
     } else {
@@ -1215,13 +1227,7 @@ void KisCanvas2::setWrapAroundViewingMode(bool value)
 
 bool KisCanvas2::wrapAroundViewingMode() const
 {
-    KisCanvasDecorationSP infinityDecoration =
-        m_d->canvasWidget->decoration(INFINITY_DECORATION_ID);
-
-    if (infinityDecoration) {
-        return !(infinityDecoration->visible());
-    }
-    return false;
+    return m_d->canvasWidget->wrapAroundViewingMode();
 }
 
 void KisCanvas2::bootstrapFinished()
