@@ -57,7 +57,7 @@
 #define isOdd(x) ((x) & 0x01)
 
 /**
- * Aligns @value to the lowest integer not smaller than @value and
+ * Aligns @p value to the lowest integer not smaller than @p value and
  * that is a divident of alignment
  */
 inline void alignByPow2Hi(qint32 &value, qint32 alignment)
@@ -68,7 +68,7 @@ inline void alignByPow2Hi(qint32 &value, qint32 alignment)
 }
 
 /**
- * Aligns @value to the lowest integer not smaller than @value and
+ * Aligns @p value to the lowest integer not smaller than @p value and
  * that is, increased by one, a divident of alignment
  */
 inline void alignByPow2ButOneHi(qint32 &value, qint32 alignment)
@@ -78,8 +78,8 @@ inline void alignByPow2ButOneHi(qint32 &value, qint32 alignment)
 }
 
 /**
- * Aligns @value to the highest integer not exceeding @value and
- * that is a divident of @alignment
+ * Aligns @p value to the highest integer not exceeding @p value and
+ * that is a divident of @p alignment
  */
 inline void alignByPow2Lo(qint32 &value, qint32 alignment)
 {
@@ -182,7 +182,7 @@ void KisImagePyramid::setImage(KisImageWSP newImage)
         // Get the full image size
         QRect rc = m_originalImage->projection()->exactBounds();
 
-        KisImageConfig config;
+        KisImageConfig config(true);
 
         int patchWidth = config.updatePatchWidth();
         int patchHeight = config.updatePatchHeight();
@@ -283,43 +283,13 @@ void KisImagePyramid::retrieveImageData(const QRect &rect)
         if (!m_channelFlags.isEmpty() && !m_allChannelsSelected) {
             QScopedArrayPointer<quint8> dst(new quint8[projectionCs->pixelSize() * numPixels]);
 
-            int channelSize = channelInfo[m_selectedChannelIndex]->size();
-            int pixelSize = projectionCs->pixelSize();
-
-            KisConfig cfg;
+            KisConfig cfg(true);
 
             if (m_onlyOneChannelSelected && !cfg.showSingleChannelAsColor()) {
-                int selectedChannelPos = channelInfo[m_selectedChannelIndex]->pos();
-                for (uint pixelIndex = 0; pixelIndex < numPixels; ++pixelIndex) {
-                    for (uint channelIndex = 0; channelIndex < projectionCs->channelCount(); ++channelIndex) {
-
-                        if (channelInfo[channelIndex]->channelType() == KoChannelInfo::COLOR) {
-                            memcpy(dst.data() + (pixelIndex * pixelSize) + (channelIndex * channelSize),
-                                   originalBytes.data() + (pixelIndex * pixelSize) + selectedChannelPos,
-                                   channelSize);
-                        }
-                        else if (channelInfo[channelIndex]->channelType() == KoChannelInfo::ALPHA) {
-                            memcpy(dst.data() + (pixelIndex * pixelSize) + (channelIndex * channelSize),
-                                   originalBytes.data()  + (pixelIndex * pixelSize) + (channelIndex * channelSize),
-                                   channelSize);
-                        }
-                    }
-                }
+                projectionCs->convertChannelToVisualRepresentation(originalBytes.data(), dst.data(), numPixels, m_selectedChannelIndex);
             }
             else {
-                for (uint pixelIndex = 0; pixelIndex < numPixels; ++pixelIndex) {
-                    for (uint channelIndex = 0; channelIndex < projectionCs->channelCount(); ++channelIndex) {
-                        if (m_channelFlags.testBit(channelIndex)) {
-                            memcpy(dst.data() + (pixelIndex * pixelSize) + (channelIndex * channelSize),
-                                   originalBytes.data()  + (pixelIndex * pixelSize) + (channelIndex * channelSize),
-                                   channelSize);
-                        }
-                        else {
-                            memset(dst.data() + (pixelIndex * pixelSize) + (channelIndex * channelSize), 0, channelSize);
-                        }
-                    }
-                }
-
+                projectionCs->convertChannelToVisualRepresentation(originalBytes.data(), dst.data(), numPixels, m_channelFlags);
             }
             originalBytes.swap(dst);
         }
@@ -531,7 +501,7 @@ QImage KisImagePyramid::convertToQImageFast(KisPaintDeviceSP paintDevice,
 
 void KisImagePyramid::configChanged()
 {
-    KisConfig cfg;
+    KisConfig cfg(true);
     m_useOcio = cfg.useOcio();
 }
 
