@@ -104,7 +104,7 @@ struct KisAsyncAnimationRenderDialogBase::Private
     QList<int> framesInProgress;
     int dirtyFramesCount = 0;
     Result result = RenderComplete;
-    QRegion regionOfInterest;
+    KisRegion regionOfInterest;
 
     KisSignalCompressor progressDialogCompressor;
     using ProgressData = QPair<int, QString>;
@@ -187,8 +187,6 @@ KisAsyncAnimationRenderDialogBase::regenerateRange(KisViewManager *viewManager)
 
     const int oldWorkingThreadsLimit = m_d->image->workingThreadsLimit();
 
-    ENTER_FUNCTION() << ppVar(numWorkers) << ppVar(numThreadsPerWorker);
-
     for (int i = 0; i < numWorkers; i++) {
         // reuse the image for one of the workers
         KisImageSP image = i == numWorkers - 1 ? m_d->image : m_d->image->clone(true);
@@ -202,8 +200,6 @@ KisAsyncAnimationRenderDialogBase::regenerateRange(KisViewManager *viewManager)
         m_d->asyncRenderers.push_back(RendererPair(renderer, image));
     }
 
-    ENTER_FUNCTION() << "Copying done in" << m_d->processingTime.elapsed();
-
     tryInitiateFrameRegeneration();
     updateProgressLabel();
 
@@ -211,8 +207,6 @@ KisAsyncAnimationRenderDialogBase::regenerateRange(KisViewManager *viewManager)
         m_d->waitLoop.exec();
     }
 
-    ENTER_FUNCTION() << "Full regeneration done in" << m_d->processingTime.elapsed();
-
     for (auto &pair : m_d->asyncRenderers) {
         KIS_SAFE_ASSERT_RECOVER_NOOP(!pair.renderer->isActive());
         if (viewManager) {
@@ -239,12 +233,12 @@ KisAsyncAnimationRenderDialogBase::regenerateRange(KisViewManager *viewManager)
     return m_d->result;
 }
 
-void KisAsyncAnimationRenderDialogBase::setRegionOfInterest(const QRegion &roi)
+void KisAsyncAnimationRenderDialogBase::setRegionOfInterest(const KisRegion &roi)
 {
     m_d->regionOfInterest = roi;
 }
 
-QRegion KisAsyncAnimationRenderDialogBase::regionOfInterest() const
+KisRegion KisAsyncAnimationRenderDialogBase::regionOfInterest() const
 {
     return m_d->regionOfInterest;
 }
