@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) Boudewijn Rempt <boud@valdyas.org>, (C) 2008
+ *  Copyright (c) 2020 L. E. Segovia <amy@amyspark.me>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,6 +24,7 @@
 #include <QLabel>
 #include <QLayout>
 #include <QGridLayout>
+#include <QPushButton>
 
 #include <klocalizedstring.h>
 
@@ -35,42 +37,51 @@
 #include <KisViewManager.h>
 #include <KisDocument.h>
 
-KisDlgGeneratorLayer::KisDlgGeneratorLayer(const QString & defaultName, KisViewManager *view, QWidget *parent, KisGeneratorLayerSP glayer = 0, const KisFilterConfigurationSP previousConfig = 0)
-        : KoDialog(parent)
+#define UPDATE_DELAY 100 /*ms */
+
+KisDlgGeneratorLayer::KisDlgGeneratorLayer(const QString & defaultName, KisViewManager *view, QWidget *parent, KisGeneratorLayerSP glayer = 0, const KisFilterConfigurationSP previousConfig = 0, const KisStrokeId stroke = KisStrokeId())
+        : QDialog(parent)
+        , layer(glayer)
+        , m_view(view)
+        , isEditing(layer && previousConfig)
         , m_customName(false)
         , m_freezeName(false)
+        , m_stroke(stroke)
+        , m_compressor(UPDATE_DELAY, KisSignalCompressor::FIRST_INACTIVE)
 {
-
-    setButtons(Ok | Cancel);
-    setDefaultButton(Ok);
-    isEditing = glayer && previousConfig;
-
     if(isEditing){
         setModal(false);
-        layer = glayer;
         configBefore = previousConfig;
     }
 
-    QWidget *page = new QWidget(this);
-
-    m_view = view;
-    dlgWidget.setupUi(page);
+    dlgWidget.setupUi(this);
     dlgWidget.wdgGenerator->initialize(m_view);
+    dlgWidget.btnBox->button(QDialogButtonBox::Ok)->setDefault(true);
 
-    setMainWidget(page);
     dlgWidget.txtLayerName->setText( isEditing ? layer->name() : defaultName );
     connect(dlgWidget.txtLayerName, SIGNAL(textChanged(QString)),
             this, SLOT(slotNameChanged(QString)));
     connect(dlgWidget.wdgGenerator, SIGNAL(previewConfiguration()), this, SLOT(previewGenerator()));
+    connect(&m_compressor, SIGNAL(timeout()), this, SLOT(slotDelayedPreviewGenerator()));
+    connect(dlgWidget.btnBox, SIGNAL(accepted()), this, SLOT(accept()));
+    connect(dlgWidget.btnBox, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(this, SIGNAL(accepted()), this, SLOT(saveLayer()));
+    connect(this, SIGNAL(rejected()), this, SLOT(restoreLayer()));
+
+    if (layer && !isEditing) {
+        slotDelayedPreviewGenerator();
+    }
+    restoreGeometry(KisConfig(true).readEntry("generatordialog/geometry", QByteArray()));
 }
 
-KisDlgGeneratorLayer::~KisDlgGeneratorLayer()
+void KisDlgGeneratorLayer::saveLayer()
 {
-    /*Editing a layer should be using the show function with automatic deletion on close.
-     *Because of this, the action should be taken care of when the window is closed and
-     *the user has accepted the changes.*/
-    if(isEditing && result() == QDialog::Accepted) {
-
+    /*
+     * Editing a layer should be using the show function with automatic deletion on close.
+     * Because of this, the action should be taken care of when the window is closed and
+     * the user has accepted the changes.
+     */
+    if (isEditing) {
         layer->setName(layerName());
 
         KisFilterConfigurationSP configAfter(configuration());
@@ -90,25 +101,51 @@ KisDlgGeneratorLayer::~KisDlgGeneratorLayer()
             m_view->undoAdapter()->addCommand(cmd);
             m_view->document()->setModified(true);
         }
+    } else {
+        KIS_ASSERT_RECOVER_RETURN(layer);
+        layer->setFilter(configuration());
     }
-    else if(isEditing && result() == QDialog::Rejected){
+}
+
+void KisDlgGeneratorLayer::restoreLayer()
+{
+    if (isEditing)
+    {
         layer->setFilter(configBefore);
     }
 }
 
+KisDlgGeneratorLayer::~KisDlgGeneratorLayer()
+{
+    KisConfig(false).writeEntry("generatordialog/geometry", saveGeometry());
+}
+
 void KisDlgGeneratorLayer::slotNameChanged(const QString & text)
 {
     if (m_freezeName)
         return;
 
     m_customName = !text.isEmpty();
-    enableButtonOk(m_customName);
+    dlgWidget.btnBox->button(QDialogButtonBox::Ok)->setEnabled(m_customName);
+}
+
+void KisDlgGeneratorLayer::slotDelayedPreviewGenerator()
+{
+    if (!m_stroke.isNull()) {
+        layer->setFilterWithoutUpdate(configuration());
+        layer->previewWithStroke(m_stroke);
+    }
 }
 
 void KisDlgGeneratorLayer::previewGenerator()
 {
-    if (isEditing && layer)
+    if (!m_stroke.isNull()) {
+        m_compressor.start();
+    }
+    else {
+        KIS_ASSERT_RECOVER_RETURN(layer);
         layer->setFilter(configuration());
+    }
 }
 
 void KisDlgGeneratorLayer::setConfiguration(const KisFilterConfigurationSP  config)
