@@ -29,14 +29,14 @@
 #include <QUuid>
 #include <QInputDialog>
 
-
 #include <KoColorPopupButton.h>
+#include <KoColorSpaceRegistry.h>
 #include <KoResourceServerProvider.h>
 
 #include "kis_config.h"
 #include "kis_cmb_contour.h"
 #include "kis_cmb_gradient.h"
-#include "kis_resource_server_provider.h"
+#include "KisResourceServerProvider.h"
 #include "kis_psd_layer_style_resource.h"
 #include "kis_psd_layer_style.h"
 
@@ -47,7 +47,6 @@
 #include <KoFileDialog.h>
 
 
-
 KoAbstractGradient* fetchGradientLazy(KoAbstractGradient *gradient,
                                       KisCanvasResourceProvider *resourceProvider)
 {
@@ -75,6 +74,7 @@ KisDlgLayerStyle::KisDlgLayerStyle(KisPSDLayerStyleSP layerStyle, KisCanvasResou
     QWidget *page = new QWidget(this);
     wdgLayerStyles.setupUi(page);
     setMainWidget(page);
+    wdgLayerStyles.chkPreview->setVisible(false);
 
     connect(wdgLayerStyles.lstStyleSelector, SIGNAL(itemChanged(QListWidgetItem*)), SLOT(notifyGuiConfigChanged()));
 
@@ -101,6 +101,8 @@ KisDlgLayerStyle::KisDlgLayerStyle(KisPSDLayerStyleSP layerStyle, KisCanvasResou
     wdgLayerStyles.stylesStack->addWidget(m_innerGlow);
     connect(m_innerGlow, SIGNAL(configChanged()), SLOT(notifyGuiConfigChanged()));
 
+    // Contour and Texture are sub-styles of Bevel and Emboss
+    // They are only applied to canvas when Bevel and Emboss is active.
     m_contour = new Contour(this);
     m_texture = new Texture(this);
     m_bevelAndEmboss = new BevelAndEmboss(m_contour, m_texture, this);
@@ -109,6 +111,8 @@ KisDlgLayerStyle::KisDlgLayerStyle(KisPSDLayerStyleSP layerStyle, KisCanvasResou
     wdgLayerStyles.stylesStack->addWidget(m_contour);
     wdgLayerStyles.stylesStack->addWidget(m_texture);
 
+    // slotBevelAndEmbossChanged(QListWidgetItem*) enables/disables Contour and Texture on "Bevel and Emboss" toggle.
+    connect(wdgLayerStyles.lstStyleSelector, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(slotBevelAndEmbossChanged(QListWidgetItem*)));
     connect(m_bevelAndEmboss, SIGNAL(configChanged()), SLOT(notifyGuiConfigChanged()));
 
     m_satin = new Satin(this);
@@ -131,7 +135,7 @@ KisDlgLayerStyle::KisDlgLayerStyle(KisPSDLayerStyleSP layerStyle, KisCanvasResou
     wdgLayerStyles.stylesStack->addWidget(m_stroke);
     connect(m_stroke, SIGNAL(configChanged()), SLOT(notifyGuiConfigChanged()));
 
-    KisConfig cfg;
+    KisConfig cfg(true);
     wdgLayerStyles.stylesStack->setCurrentIndex(cfg.readEntry("KisDlgLayerStyle::current", 1));
     wdgLayerStyles.lstStyleSelector->setCurrentRow(cfg.readEntry("KisDlgLayerStyle::current", 1));
 
@@ -139,6 +143,13 @@ KisDlgLayerStyle::KisDlgLayerStyle(KisPSDLayerStyleSP layerStyle, KisCanvasResou
             SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
              this, SLOT(changePage(QListWidgetItem*,QListWidgetItem*)));
 
+    // improve the checkbox visibility by altering the style sheet list a bit
+    // the dark themes make them hard to see
+    QPalette newPalette = palette();
+    newPalette.setColor(QPalette::Active, QPalette::Background, palette().text().color() );
+    wdgLayerStyles.lstStyleSelector->setPalette(newPalette);
+
+
     notifyPredefinedStyleSelected(layerStyle);
 
     connect(m_dropShadow, SIGNAL(globalAngleChanged(int)), SLOT(syncGlobalAngle(int)));
@@ -189,6 +200,32 @@ void KisDlgLayerStyle::notifyPredefinedStyleSelected(KisPSDLayerStyleSP style)
     m_configChangedCompressor->start();
 }
 
+void KisDlgLayerStyle::slotBevelAndEmbossChanged(QListWidgetItem*) {
+    QListWidgetItem *item;
+
+    if (wdgLayerStyles.lstStyleSelector->item(6)->checkState() == Qt::Checked) {
+        // Enable "Contour" (list item 7)
+        item = wdgLayerStyles.lstStyleSelector->item(7);
+        Qt::ItemFlags currentFlags7 = item->flags();
+        item->setFlags(currentFlags7 | Qt::ItemIsEnabled);
+
+        // Enable "Texture" (list item 8)
+        item = wdgLayerStyles.lstStyleSelector->item(8);
+        Qt::ItemFlags currentFlags8 = item->flags();
+        item->setFlags(currentFlags8 | Qt::ItemIsEnabled);
+    }
+    else {
+        // Disable "Contour"
+        item = wdgLayerStyles.lstStyleSelector->item(7);
+        Qt::ItemFlags currentFlags7 = item->flags();
+        item->setFlags(currentFlags7 & (~Qt::ItemIsEnabled));
+
+        // Disable "Texture"
+        item = wdgLayerStyles.lstStyleSelector->item(8);
+        Qt::ItemFlags currentFlags8 = item->flags();
+        item->setFlags(currentFlags8 & (~Qt::ItemIsEnabled));
+    }
+}
 
 void KisDlgLayerStyle::slotNotifyOnAccept()
 {
@@ -282,7 +319,7 @@ void KisDlgLayerStyle::slotSaveStyle()
         new KisPSDLayerStyleCollectionResource(filename));
 
     KisPSDLayerStyleSP newStyle = style()->clone();
-    newStyle->setName(QFileInfo(filename).baseName());
+    newStyle->setName(QFileInfo(filename).completeBaseName());
 
     KisPSDLayerStyleCollectionResource::StylesVector vector = collection->layerStyles();
     vector << newStyle;
@@ -300,10 +337,16 @@ void KisDlgLayerStyle::changePage(QListWidgetItem *current, QListWidgetItem *pre
 
 void KisDlgLayerStyle::setStyle(KisPSDLayerStyleSP style)
 {
-    *m_layerStyle = *style;
+    // we may self-assign style is some cases
+    if (style != m_layerStyle) {
+        *m_layerStyle = *style;
+    }
     m_sanityLayerStyleDirty = false;
 
-    m_stylesSelector->notifyExternalStyleChanged(m_layerStyle->name(), m_layerStyle->uuid());
+    {
+        KisSignalsBlocker b(m_stylesSelector);
+        m_stylesSelector->notifyExternalStyleChanged(m_layerStyle->name(), m_layerStyle->uuid());
+    }
 
     QListWidgetItem *item;
     item = wdgLayerStyles.lstStyleSelector->item(2);
@@ -518,7 +561,7 @@ void StylesSelector::loadCollection(const QString &fileName)
     collection->load();
 
     KoResourceServer<KisPSDLayerStyleCollectionResource> *server = KisResourceServerProvider::instance()->layerStyleCollectionServer();
-    collection->setFilename(server->saveLocation() + QDir::separator() + collection->name());
+    collection->setFilename(server->saveLocation() + '/' + collection->name());
     server->addResource(collection);
 
     refillCollections();
@@ -590,6 +633,7 @@ BevelAndEmboss::BevelAndEmboss(Contour *contour, Texture *texture, QWidget *pare
 
     ui.intSize->setRange(0, 250);
     ui.intSize->setSuffix(i18n(" px"));
+    ui.intSize->setExponentRatio(2.0);
 
     ui.intSoften->setRange(0, 18);
     ui.intSoften->setSuffix(i18n(" px"));
@@ -608,25 +652,22 @@ BevelAndEmboss::BevelAndEmboss(Contour *contour, Texture *texture, QWidget *pare
     ui.intOpacity2->setRange(0, 100);
     ui.intOpacity2->setSuffix(i18n(" %"));
 
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SLOT(slotDialAngleChanged(int)));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SLOT(slotIntAngleChanged(int)));
-    connect(ui.chkUseGlobalLight, SIGNAL(toggled(bool)), SLOT(slotGlobalLightToggled()));
+    ui.angleSelector->enableGlobalLight(true);
+    connect(ui.angleSelector, SIGNAL(globalAngleChanged(int)), SIGNAL(globalAngleChanged(int)));
+    connect(ui.angleSelector, SIGNAL(configChanged()), SIGNAL(configChanged()));
 
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.chkUseGlobalLight, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.intAltitude, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
     connect(ui.cmbContour, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.chkAntiAliased, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.cmbHighlightMode, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
-    connect(ui.bnHighlightColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
+    connect(ui.bnHighlightColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
     connect(ui.intOpacity, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
     connect(ui.cmbShadowMode, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
-    connect(ui.bnShadowColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
-    connect(ui.intOpacity2, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));;
+    connect(ui.bnShadowColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
+    connect(ui.intOpacity2, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 
     // Contour
-    m_contour->ui.intRange->setRange(0, 100);
+    m_contour->ui.intRange->setRange(1, 100);
     m_contour->ui.intRange->setSuffix(i18n(" %"));
 
     connect(m_contour->ui.cmbContour, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
@@ -656,18 +697,21 @@ void BevelAndEmboss::setBevelAndEmboss(const psd_layer_effects_bevel_emboss *bev
     ui.intSize->setValue(bevelAndEmboss->size());
     ui.intSoften->setValue(bevelAndEmboss->soften());
 
-    ui.dialAngle->setValue(bevelAndEmboss->angle());
-    ui.intAngle->setValue(bevelAndEmboss->angle());
-    ui.chkUseGlobalLight->setChecked(bevelAndEmboss->useGlobalLight());
+    ui.angleSelector->setValue(bevelAndEmboss->angle());
+    ui.angleSelector->setUseGlobalLight(bevelAndEmboss->useGlobalLight());
+
     ui.intAltitude->setValue(bevelAndEmboss->altitude());
     // FIXME: curve editing
     // ui.cmbContour;
     ui.chkAntiAliased->setChecked(bevelAndEmboss->glossAntiAliased());
     ui.cmbHighlightMode->selectCompositeOp(KoID(bevelAndEmboss->highlightBlendMode()));
-    ui.bnHighlightColor->setColor(bevelAndEmboss->highlightColor());
+    KoColor highlightshadow(KoColorSpaceRegistry::instance()->rgb8());
+    highlightshadow.fromQColor(bevelAndEmboss->highlightColor());
+    ui.bnHighlightColor->setColor(highlightshadow);
     ui.intOpacity->setValue(bevelAndEmboss->highlightOpacity());
     ui.cmbShadowMode->selectCompositeOp(KoID(bevelAndEmboss->shadowBlendMode()));
-    ui.bnShadowColor->setColor(bevelAndEmboss->shadowColor());
+    highlightshadow.fromQColor(bevelAndEmboss->shadowColor());
+    ui.bnShadowColor->setColor(highlightshadow);
     ui.intOpacity2->setValue(bevelAndEmboss->shadowOpacity());
 
     // FIXME: curve editing
@@ -691,15 +735,15 @@ void BevelAndEmboss::fetchBevelAndEmboss(psd_layer_effects_bevel_emboss *bevelAn
     bevelAndEmboss->setSize(ui.intSize->value());
     bevelAndEmboss->setSoften(ui.intSoften->value());
 
-    bevelAndEmboss->setAngle(ui.dialAngle->value());
-    bevelAndEmboss->setUseGlobalLight(ui.chkUseGlobalLight->isChecked());
+    bevelAndEmboss->setAngle(ui.angleSelector->value());
+    bevelAndEmboss->setUseGlobalLight(ui.angleSelector->useGlobalLight());
     bevelAndEmboss->setAltitude(ui.intAltitude->value());
     bevelAndEmboss->setGlossAntiAliased(ui.chkAntiAliased->isChecked());
     bevelAndEmboss->setHighlightBlendMode(ui.cmbHighlightMode->selectedCompositeOp().id());
-    bevelAndEmboss->setHighlightColor(ui.bnHighlightColor->color());
+    bevelAndEmboss->setHighlightColor(ui.bnHighlightColor->color().toQColor());
     bevelAndEmboss->setHighlightOpacity(ui.intOpacity->value());
     bevelAndEmboss->setShadowBlendMode(ui.cmbShadowMode->selectedCompositeOp().id());
-    bevelAndEmboss->setShadowColor(ui.bnShadowColor->color());
+    bevelAndEmboss->setShadowColor(ui.bnShadowColor->color().toQColor());
     bevelAndEmboss->setShadowOpacity(ui.intOpacity2->value());
 
     // FIXME: curve editing
@@ -713,32 +757,6 @@ void BevelAndEmboss::fetchBevelAndEmboss(psd_layer_effects_bevel_emboss *bevelAn
     bevelAndEmboss->setTextureAlignWithLayer(m_texture->ui.chkLinkWithLayer->isChecked());
 }
 
-void BevelAndEmboss::slotDialAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.intAngle);
-    ui.intAngle->setValue(value);
-
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(value);
-    }
-}
-
-void BevelAndEmboss::slotIntAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.dialAngle);
-    ui.dialAngle->setValue(value);
-
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(value);
-    }
-}
-
-void BevelAndEmboss::slotGlobalLightToggled()
-{
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(ui.intAngle->value());
-    }
-}
 
 /********************************************************************/
 /***** Texture          *********************************************/
@@ -792,21 +810,23 @@ ColorOverlay::ColorOverlay(QWidget *parent)
 
     connect(ui.cmbCompositeOp, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.intOpacity, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.bnColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
+    connect(ui.bnColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
 }
 
 void ColorOverlay::setColorOverlay(const psd_layer_effects_color_overlay *colorOverlay)
 {
     ui.cmbCompositeOp->selectCompositeOp(KoID(colorOverlay->blendMode()));
     ui.intOpacity->setValue(colorOverlay->opacity());
-    ui.bnColor->setColor(colorOverlay->color());
+    KoColor color(KoColorSpaceRegistry::instance()->rgb8());
+    color.fromQColor(colorOverlay->color());
+    ui.bnColor->setColor(color);
 }
 
 void ColorOverlay::fetchColorOverlay(psd_layer_effects_color_overlay *colorOverlay) const
 {
     colorOverlay->setBlendMode(ui.cmbCompositeOp->selectedCompositeOp().id());
     colorOverlay->setOpacity(ui.intOpacity->value());
-    colorOverlay->setColor(ui.bnColor->color());
+    colorOverlay->setColor(ui.bnColor->color().toQColor());
 }
 
 
@@ -832,22 +852,19 @@ DropShadow::DropShadow(Mode mode, QWidget *parent)
 
     ui.intSize->setRange(0, 250);
     ui.intSize->setSuffix(i18n(" px"));
+    ui.intSize->setExponentRatio(2.0);
 
     ui.intNoise->setRange(0, 100);
     ui.intNoise->setSuffix(i18n(" %"));
 
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SLOT(slotDialAngleChanged(int)));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SLOT(slotIntAngleChanged(int)));
-    connect(ui.chkUseGlobalLight, SIGNAL(toggled(bool)), SLOT(slotGlobalLightToggled()));
+    ui.angleSelector->enableGlobalLight(true);
+    connect(ui.angleSelector, SIGNAL(globalAngleChanged(int)), SIGNAL(globalAngleChanged(int)));
+    connect(ui.angleSelector, SIGNAL(configChanged()), SIGNAL(configChanged()));
 
     // connect everything to configChanged() signal
     connect(ui.cmbCompositeOp, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.intOpacity, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.bnColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
-
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.chkUseGlobalLight, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
+    connect(ui.bnColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
 
     connect(ui.intDistance, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
     connect(ui.intSpread, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
@@ -866,42 +883,16 @@ DropShadow::DropShadow(Mode mode, QWidget *parent)
     }
 }
 
-void DropShadow::slotDialAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.intAngle);
-    ui.intAngle->setValue(value);
-
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(value);
-    }
-}
-
-void DropShadow::slotIntAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.dialAngle);
-    ui.dialAngle->setValue(value);
-
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(value);
-    }
-}
-
-void DropShadow::slotGlobalLightToggled()
-{
-    if (ui.chkUseGlobalLight->isChecked()) {
-        emit globalAngleChanged(ui.intAngle->value());
-    }
-}
-
 void DropShadow::setShadow(const psd_layer_effects_shadow_common *shadow)
 {
     ui.cmbCompositeOp->selectCompositeOp(KoID(shadow->blendMode()));
     ui.intOpacity->setValue(shadow->opacity());
-    ui.bnColor->setColor(shadow->color());
+    KoColor color(KoColorSpaceRegistry::instance()->rgb8());
+    color.fromQColor(shadow->color());
+    ui.bnColor->setColor(color);
 
-    ui.dialAngle->setValue(shadow->angle());
-    ui.intAngle->setValue(shadow->angle());
-    ui.chkUseGlobalLight->setChecked(shadow->useGlobalLight());
+    ui.angleSelector->setValue(shadow->angle());
+    ui.angleSelector->setUseGlobalLight(shadow->useGlobalLight());
 
     ui.intDistance->setValue(shadow->distance());
     ui.intSpread->setValue(shadow->spread());
@@ -925,10 +916,10 @@ void DropShadow::fetchShadow(psd_layer_effects_shadow_common *shadow) const
 {
     shadow->setBlendMode(ui.cmbCompositeOp->selectedCompositeOp().id());
     shadow->setOpacity(ui.intOpacity->value());
-    shadow->setColor(ui.bnColor->color());
+    shadow->setColor(ui.bnColor->color().toQColor());
 
-    shadow->setAngle(ui.dialAngle->value());
-    shadow->setUseGlobalLight(ui.chkUseGlobalLight->isChecked());
+    shadow->setAngle(ui.angleSelector->value());
+    shadow->setUseGlobalLight(ui.angleSelector->useGlobalLight());
 
     shadow->setDistance(ui.intDistance->value());
     shadow->setSpread(ui.intSpread->value());
@@ -1000,8 +991,7 @@ GradientOverlay::GradientOverlay(KisCanvasResourceProvider *resourceProvider, QW
     ui.intScale->setRange(0, 100);
     ui.intScale->setSuffix(i18n(" %"));
 
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SLOT(slotDialAngleChanged(int)));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SLOT(slotIntAngleChanged(int)));
+    connect(ui.angleSelector, SIGNAL(configChanged()), SIGNAL(configChanged()));
 
     connect(ui.cmbCompositeOp, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.intOpacity, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
@@ -1009,8 +999,6 @@ GradientOverlay::GradientOverlay(KisCanvasResourceProvider *resourceProvider, QW
     connect(ui.chkReverse, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.cmbStyle, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.chkAlignWithLayer, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
     connect(ui.intScale, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 }
 
@@ -1026,11 +1014,10 @@ void GradientOverlay::setGradientOverlay(const psd_layer_effects_gradient_overla
         ui.cmbGradient->setGradient(gradient);
     }
 
-    ui.chkReverse->setChecked(config->antiAliased());
+    ui.chkReverse->setChecked(config->reverse());
     ui.cmbStyle->setCurrentIndex((int)config->style());
     ui.chkAlignWithLayer->setCheckable(config->alignWithLayer());
-    ui.dialAngle->setValue(config->angle());
-    ui.intAngle->setValue(config->angle());
+    ui.angleSelector->setValue(config->angle());
     ui.intScale->setValue(config->scale());
 }
 
@@ -1042,21 +1029,10 @@ void GradientOverlay::fetchGradientOverlay(psd_layer_effects_gradient_overlay *c
     config->setReverse(ui.chkReverse->isChecked());
     config->setStyle((psd_gradient_style)ui.cmbStyle->currentIndex());
     config->setAlignWithLayer(ui.chkAlignWithLayer->isChecked());
-    config->setAngle(ui.dialAngle->value());
+    config->setAngle(ui.angleSelector->value());
     config->setScale(ui.intScale->value());
 }
 
-void GradientOverlay::slotDialAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.intAngle);
-    ui.intAngle->setValue(value);
-}
-
-void GradientOverlay::slotIntAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.dialAngle);
-    ui.dialAngle->setValue(value);
-}
 
 /********************************************************************/
 /***** Innner Glow      *********************************************/
@@ -1080,8 +1056,9 @@ InnerGlow::InnerGlow(Mode mode, KisCanvasResourceProvider *resourceProvider, QWi
 
     ui.intSize->setRange(0, 250);
     ui.intSize->setSuffix(i18n(" px"));
+    ui.intSize->setExponentRatio(2.0);
 
-    ui.intRange->setRange(0, 100);
+    ui.intRange->setRange(1, 100);
     ui.intRange->setSuffix(i18n(" %"));
 
     ui.intJitter->setRange(0, 100);
@@ -1092,7 +1069,7 @@ InnerGlow::InnerGlow(Mode mode, KisCanvasResourceProvider *resourceProvider, QWi
     connect(ui.intNoise, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 
     connect(ui.radioColor, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
-    connect(ui.bnColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
+    connect(ui.bnColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
     connect(ui.radioGradient, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.cmbGradient, SIGNAL(gradientChanged(KoAbstractGradient*)), SIGNAL(configChanged()));
 
@@ -1121,7 +1098,9 @@ void InnerGlow::setConfig(const psd_layer_effects_glow_common *config)
     ui.intNoise->setValue(config->noise());
 
     ui.radioColor->setChecked(config->fillType() == psd_fill_solid_color);
-    ui.bnColor->setColor(config->color());
+    KoColor color(KoColorSpaceRegistry::instance()->rgb8());
+    color.fromQColor(config->color());
+    ui.bnColor->setColor(color);
     ui.radioGradient->setChecked(config->fillType() == psd_fill_gradient);
 
     KoAbstractGradient *gradient = fetchGradientLazy(
@@ -1164,7 +1143,7 @@ void InnerGlow::fetchConfig(psd_layer_effects_glow_common *config) const
         config->setFillType(psd_fill_gradient);
     }
 
-    config->setColor(ui.bnColor->color());
+    config->setColor(ui.bnColor->color().toQColor());
     config->setGradient(GradientPointerConverter::resourceToStyle(ui.cmbGradient->gradient()));
     config->setTechnique((psd_technique_type)ui.cmbTechnique->currentIndex());
     config->setSpread(ui.intChoke->value());
@@ -1246,16 +1225,13 @@ Satin::Satin(QWidget *parent)
 
     ui.intSize->setRange(0, 250);
     ui.intSize->setSuffix(i18n(" px"));
-
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SLOT(slotDialAngleChanged(int)));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SLOT(slotIntAngleChanged(int)));
+    ui.intSize->setExponentRatio(2.0);
 
     connect(ui.cmbCompositeOp, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
-    connect(ui.bnColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
+    connect(ui.bnColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
     connect(ui.intOpacity, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
+    connect(ui.angleSelector, SIGNAL(configChanged()), SIGNAL(configChanged()));
     connect(ui.intDistance, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
     connect(ui.intSize, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 
@@ -1265,26 +1241,15 @@ Satin::Satin(QWidget *parent)
 
 }
 
-void Satin::slotDialAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.intAngle);
-    ui.intAngle->setValue(value);
-}
-
-void Satin::slotIntAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.dialAngle);
-    ui.dialAngle->setValue(value);
-}
-
 void Satin::setSatin(const psd_layer_effects_satin *satin)
 {
     ui.cmbCompositeOp->selectCompositeOp(KoID(satin->blendMode()));
-    ui.bnColor->setColor(satin->color());
+    KoColor color(KoColorSpaceRegistry::instance()->rgb8());
+    color.fromQColor(satin->color());
+    ui.bnColor->setColor(color);
     ui.intOpacity->setValue(satin->opacity());
 
-    ui.dialAngle->setValue(satin->angle());
-    ui.intAngle->setValue(satin->angle());
+    ui.angleSelector->setValue(satin->angle());
 
     ui.intDistance->setValue(satin->distance());
     ui.intSize->setValue(satin->size());
@@ -1301,9 +1266,9 @@ void Satin::fetchSatin(psd_layer_effects_satin *satin) const
 {
     satin->setBlendMode(ui.cmbCompositeOp->selectedCompositeOp().id());
     satin->setOpacity(ui.intOpacity->value());
-    satin->setColor(ui.bnColor->color());
+    satin->setColor(ui.bnColor->color().toQColor());
 
-    satin->setAngle(ui.dialAngle->value());
+    satin->setAngle(ui.angleSelector->value());
 
     satin->setDistance(ui.intDistance->value());
     satin->setSize(ui.intSize->value());
@@ -1326,6 +1291,7 @@ Stroke::Stroke(KisCanvasResourceProvider *resourceProvider, QWidget *parent)
 
     ui.intSize->setRange(0, 250);
     ui.intSize->setSuffix(i18n(" px"));
+    ui.intSize->setExponentRatio(2.0);
 
     ui.intOpacity->setRange(0, 100);
     ui.intOpacity->setSuffix(i18n(" %"));
@@ -1345,16 +1311,16 @@ Stroke::Stroke(KisCanvasResourceProvider *resourceProvider, QWidget *parent)
 
     connect(ui.cmbFillType, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
 
-    connect(ui.bnColor, SIGNAL(changed(QColor)), SIGNAL(configChanged()));
+    connect(ui.bnColor, SIGNAL(changed(KoColor)), SIGNAL(configChanged()));
 
     connect(ui.cmbGradient, SIGNAL(gradientChanged(KoAbstractGradient*)), SIGNAL(configChanged()));
     connect(ui.chkReverse, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.cmbStyle, SIGNAL(currentIndexChanged(int)), SIGNAL(configChanged()));
     connect(ui.chkAlignWithLayer, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
-    connect(ui.dialAngle, SIGNAL(valueChanged(int)), SLOT(slotDialAngleChanged(int)));
-    connect(ui.intAngle, SIGNAL(valueChanged(int)), SLOT(slotIntAngleChanged(int)));
     connect(ui.intScale, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
 
+    connect(ui.angleSelector, SIGNAL(configChanged()), SIGNAL(configChanged()));
+
     connect(ui.patternChooser, SIGNAL(resourceSelected(KoResource*)), SIGNAL(configChanged()));
     connect(ui.chkLinkWithLayer, SIGNAL(toggled(bool)), SIGNAL(configChanged()));
     connect(ui.intScale_2, SIGNAL(valueChanged(int)), SIGNAL(configChanged()));
@@ -1363,19 +1329,6 @@ Stroke::Stroke(KisCanvasResourceProvider *resourceProvider, QWidget *parent)
     ui.fillStack->setCurrentIndex(ui.cmbFillType->currentIndex());
 }
 
-void Stroke::slotDialAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.intAngle);
-    ui.intAngle->setValue(value);
-}
-
-void Stroke::slotIntAngleChanged(int value)
-{
-    KisSignalsBlocker b(ui.dialAngle);
-    ui.dialAngle->setValue(value);
-}
-
-
 void Stroke::setStroke(const psd_layer_effects_stroke *stroke)
 {
 
@@ -1385,8 +1338,9 @@ void Stroke::setStroke(const psd_layer_effects_stroke *stroke)
     ui.intOpacity->setValue(stroke->opacity());
 
     ui.cmbFillType->setCurrentIndex((int)stroke->fillType());
-
-    ui.bnColor->setColor(stroke->color());
+    KoColor color(KoColorSpaceRegistry::instance()->rgb8());
+    color.fromQColor(stroke->color());
+    ui.bnColor->setColor(color);
 
     KoAbstractGradient *gradient =
         fetchGradientLazy(GradientPointerConverter::styleToResource(stroke->gradient()), m_resourceProvider);
@@ -1398,8 +1352,7 @@ void Stroke::setStroke(const psd_layer_effects_stroke *stroke)
     ui.chkReverse->setChecked(stroke->antiAliased());
     ui.cmbStyle->setCurrentIndex((int)stroke->style());
     ui.chkAlignWithLayer->setCheckable(stroke->alignWithLayer());
-    ui.dialAngle->setValue(stroke->angle());
-    ui.intAngle->setValue(stroke->angle());
+    ui.angleSelector->setValue(stroke->angle());
     ui.intScale->setValue(stroke->scale());
 
     ui.patternChooser->setCurrentPattern(stroke->pattern());
@@ -1417,13 +1370,13 @@ void Stroke::fetchStroke(psd_layer_effects_stroke *stroke) const
 
     stroke->setFillType((psd_fill_type)ui.cmbFillType->currentIndex());
 
-    stroke->setColor(ui.bnColor->color());
+    stroke->setColor(ui.bnColor->color().toQColor());
 
     stroke->setGradient(GradientPointerConverter::resourceToStyle(ui.cmbGradient->gradient()));
     stroke->setReverse(ui.chkReverse->isChecked());
     stroke->setStyle((psd_gradient_style)ui.cmbStyle->currentIndex());
     stroke->setAlignWithLayer(ui.chkAlignWithLayer->isChecked());
-    stroke->setAngle(ui.dialAngle->value());
+    stroke->setAngle(ui.angleSelector->value());
     stroke->setScale(ui.intScale->value());
 
     stroke->setPattern(static_cast<KoPattern*>(ui.patternChooser->currentResource()));
