@@ -25,6 +25,7 @@
 #include <kis_types.h>
 
 class KisDocument;
+class KoCanvasBase;
 
 class KRITAUI_EXPORT KisReferenceImagesLayer : public KisShapeLayer
 {
@@ -74,5 +75,7 @@ private:
     friend class ReferenceImagesCanvas;
 };
 
+typedef KisSharedPtr<KisReferenceImagesLayer> KisReferenceImagesLayerSP;
+
 
 #endif //KRITA_KISREFERENCEIMAGESLAYER_H
