@@ -38,6 +38,7 @@ class KoViewConverter;
 class KoShapeControllerBase;
 class KoDocumentResourceManager;
 class KisShapeLayerCanvasBase;
+class KoSelectedShapesProxy;
 
 const QString KIS_SHAPE_LAYER_ID = "KisShapeLayer";
 /**
@@ -76,7 +77,9 @@ public:
 protected:
     KisShapeLayer(KoShapeControllerBase* shapeController, KisImageWSP image, const QString &name, quint8 opacity, KisShapeLayerCanvasBase *canvas);
 private:
-    void initShapeLayer(KoShapeControllerBase* controller, KisPaintDeviceSP copyFromProjection = 0, KisShapeLayerCanvasBase *canvas = 0);
+    void initShapeLayerImpl(KoShapeControllerBase* controller, KisPaintDeviceSP newProjectionDevice, KisShapeLayerCanvasBase *overrideCanvas);
+    void initNewShapeLayer(KoShapeControllerBase* controller, const KoColorSpace *projectionColorSpace, KisDefaultBoundsBaseSP bounds, KisShapeLayerCanvasBase *overrideCanvas = 0);
+    void initClonedShapeLayer(KoShapeControllerBase* controller, KisPaintDeviceSP copyFromProjection, KisShapeLayerCanvasBase *overrideCanvas = 0);
 public:
     KisNodeSP clone() const override {
         return new KisShapeLayer(*this);
@@ -163,6 +166,13 @@ public:
 
     void forceUpdateHiddenAreaOnOriginal() override;
 
+    /**
+     * @brief selectedShapesProxy
+     * @return returns the selectedShapesProxy of the KoCanvasBase of this layer,
+     * used for certain undo commands.
+     */
+    KoSelectedShapesProxy* selectedShapesProxy();
+
 protected:
     using KoShape::isVisible;
 
@@ -173,6 +183,8 @@ protected:
 
     KoShapeControllerBase *shapeController() const;
 
+    friend class TransformShapeLayerDeferred;
+
 Q_SIGNALS:
     /**
      * These signals are forwarded from the local shape manager
@@ -195,6 +207,7 @@ Q_SIGNALS:
 
 private Q_SLOTS:
     void slotMoveShapes(const QPointF &diff);
+    void slotTransformShapes(const QTransform &transform);
 
 private:
     QList<KoShape*> shapesToBeTransformed();
