@@ -61,6 +61,7 @@
 
 #include "kis_extended_modifiers_mapper.h"
 #include "kis_input_manager_p.h"
+#include "kis_algebra_2d.h"
 
 template <typename T>
 uint qHash(QPointer<T> value) {
@@ -79,10 +80,21 @@ KisInputManager::KisInputManager(QObject *parent)
 
     QApplication::instance()->
             installEventFilter(new Private::ProximityNotifier(d, this));
+
+    // on macos global Monitor listen to keypresses when krita is not in focus
+    // and local monitor listen presses when krita is in focus.
+#ifdef Q_OS_MACOS
+    KisExtendedModifiersMapper::setGlobalMonitor(true);
+    KisExtendedModifiersMapper::setLocalMonitor(true, &d->matcher);
+#endif
 }
 
 KisInputManager::~KisInputManager()
 {
+#ifdef Q_OS_MACOS
+    KisExtendedModifiersMapper::setGlobalMonitor(false);
+    KisExtendedModifiersMapper::setLocalMonitor(false);
+#endif
     delete d;
 }
 
@@ -145,11 +157,6 @@ void KisInputManager::setupAsEventFilter(QObject *receiver)
     }
 }
 
-void KisInputManager::stopIgnoringEvents()
-{
-    d->allowMouseEvents();
-}
-
 #if defined (__clang__)
 #pragma GCC diagnostic ignored "-Wswitch"
 #endif
@@ -201,7 +208,7 @@ bool KisInputManager::eventFilter(QObject* object, QEvent* event)
     return eventFilterImpl(event);
 }
 
-// Qt's events do not have copy-ctors yes, so we should emulate them
+// Qt's events do not have copy-ctors yet, so we should emulate them
 // See https://bugreports.qt.io/browse/QTBUG-72488
 
 template <class Event> void copyEventHack(Event *src, QScopedPointer<QEvent> &dst);
@@ -254,6 +261,32 @@ bool KisInputManager::compressMoveEventCommon(Event *event)
                   std::is_same<Event, QTouchEvent>::value,
                   "event should be a mouse or a tablet event");
 
+#ifdef Q_OS_WIN32
+    /**
+     * On Windows, when the user presses some global window manager shortcuts,
+     * e.g. Alt+Space (to show window title menu), events for these key presses
+     * and releases are not delivered (see bug 424319). This code is a workaround
+     * for this problem. It checks consistency of standard modifiers and resets
+     * shortcut's matcher state in case of a trouble.
+     */
+    if (event->type() == QEvent::MouseButtonPress ||
+        event->type() == QEvent::MouseButtonRelease ||
+        event->type() == QEvent::MouseMove ||
+        event->type() == QEvent::TabletMove ||
+        event->type() == QEvent::TabletPress ||
+        event->type() == QEvent::TabletRelease) {
+
+        QInputEvent *inputEvent = static_cast<QInputEvent*>(event);
+        if (!d->matcher.sanityCheckModifiersCorrectness(inputEvent->modifiers())) {
+            qWarning() << "WARNING: modifiers state became inconsistent! Trying to fix that...";
+            qWarning() << "    " << ppVar(inputEvent->modifiers());
+            qWarning() << "    " << ppVar(d->matcher.debugPressedKeys());
+
+            d->fixShortcutMatcherModifiersState();
+        }
+    }
+#endif
+
     bool retval = false;
 
     /**
@@ -355,14 +388,12 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             retval = d->matcher.autoRepeatedKeyPressed(key);
         }
 
-        /**
-         * Workaround for temporary switching of tools by
-         * KoCanvasControllerWidget. We don't need this switch because
-         * we handle it ourselves.
-         */
-        retval |= !d->forwardAllEventsToTool &&
-                (keyEvent->key() == Qt::Key_Space ||
-                 keyEvent->key() == Qt::Key_Escape);
+        // In case we matched ashortcut we should accept the event to
+        // notify Qt that it shouldn't try to trigger its partially matched
+        // shortcuts.
+        if (retval) {
+            keyEvent->setAccepted(true);
+        }
 
         break;
     }
@@ -433,17 +464,26 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             }
         }
 
-        if (qAbs(d->accumulatedScrollDelta) >= QWheelEvent::DefaultDeltasPerStep) {
+        bool wasScrolled = false;
+
+        while (qAbs(d->accumulatedScrollDelta) >= QWheelEvent::DefaultDeltasPerStep) {
             //Make sure the input actions know we are active.
             KisAbstractInputAction::setInputManager(this);
             retval = d->matcher.wheelEvent(action, wheelEvent);
+            d->accumulatedScrollDelta -=
+                KisAlgebra2D::signPZ(d->accumulatedScrollDelta) *
+                QWheelEvent::DefaultDeltasPerStep;
+            wasScrolled = true;
+        }
+
+        if (wasScrolled) {
             d->accumulatedScrollDelta = 0;
         }
-        else {
-            retval = true;
-        }
+
+        retval = !wasScrolled;
         break;
     }
+#ifndef Q_OS_ANDROID
     case QEvent::Enter:
         d->debugEvent<QEvent, false>(event);
         //Make sure the input actions know we are active.
@@ -454,7 +494,6 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             d->allowMouseEvents();
             d->touchHasBlockedPressEvents = false;
         }
-
         d->matcher.enterEvent();
         break;
     case QEvent::Leave:
@@ -470,6 +509,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 
         d->matcher.leaveEvent();
         break;
+#endif
     case QEvent::FocusIn:
         d->debugEvent<QEvent, false>(event);
         KisAbstractInputAction::setInputManager(this);
@@ -565,7 +605,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
         break;
     }
     case QEvent::TabletRelease: {
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MAC) || defined(Q_OS_ANDROID)
         d->allowMouseEvents();
 #endif
         d->debugEvent<QTabletEvent, false>(event);
@@ -586,6 +626,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 
     case QEvent::TouchBegin:
     {
+        d->debugEvent<QTouchEvent, false>(event);
         if (startTouch(retval)) {
             QTouchEvent *touchEvent = static_cast<QTouchEvent *> (event);
             KisAbstractInputAction::setInputManager(this);
@@ -595,6 +636,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             {
                 d->previousPos = touchEvent->touchPoints().at(0).pos();
                 d->buttonPressed = false;
+                d->resetCompressor();
             }
             else {
                 retval = d->matcher.touchBeginEvent(touchEvent);
@@ -611,6 +653,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
     case QEvent::TouchUpdate:
     {
         QTouchEvent *touchEvent = static_cast<QTouchEvent*>(event);
+        d->debugEvent<QTouchEvent, false>(event);
 
 #ifdef Q_OS_MAC
         int count = 0;
@@ -646,7 +689,6 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
                 d->touchStrokeStarted = true;
                 retval = compressMoveEventCommon(touchEvent);
                 d->blockMouseEvents();
-                d->resetCompressor();
             }
             else if (!d->touchStrokeStarted){
                 KisAbstractInputAction::setInputManager(this);
@@ -667,6 +709,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
 
     case QEvent::TouchEnd:
     {
+        d->debugEvent<QTouchEvent, false>(event);
         endTouch();
         QTouchEvent *touchEvent = static_cast<QTouchEvent*>(event);
         retval = d->matcher.touchEndEvent(touchEvent);
@@ -685,6 +728,18 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
         event->accept();
         break;
     }
+    case QEvent::TouchCancel:
+    {
+        d->debugEvent<QTouchEvent, false>(event);
+        endTouch();
+        d->matcher.touchCancelEvent(d->previousPos);
+        // reset state
+        d->previousPos = {0, 0};
+        d->touchStrokeStarted = false;
+        retval = true;
+        event->accept();
+        break;
+    }
 
     case QEvent::NativeGesture:
     {
