@@ -23,6 +23,8 @@
 #include <QScopedPointer>
 #include <QtGlobal>
 
+#include <boost/preprocessor/repeat_from_to.hpp>
+
 #include "kis_input_manager.h"
 #include "kis_config.h"
 #include "kis_abstract_input_action.h"
@@ -227,6 +229,13 @@ void KisInputManager::Private::CanvasSwitcher::addCanvas(KisCanvas2 *canvas)
 
     if (!canvasResolver.contains(canvasWidget)) {
         canvasResolver.insert(canvasWidget, canvas);
+    } else {
+        // just a sanity cheeck to find out if we are
+        // trying to add two canvases concurrently.
+        KIS_SAFE_ASSERT_RECOVER_NOOP(d->canvas == canvas);
+    }
+
+    if (canvas != d->canvas) {
         d->q->setupAsEventFilter(canvasWidget);
         canvasWidget->installEventFilter(this);
 
@@ -235,8 +244,6 @@ void KisInputManager::Private::CanvasSwitcher::addCanvas(KisCanvas2 *canvas)
 
         d->canvas = canvas;
         d->toolProxy = qobject_cast<KisToolProxy*>(canvas->toolProxy());
-    } else {
-        KIS_ASSERT_RECOVER_RETURN(d->canvas == canvas);
     }
 }
 
@@ -251,6 +258,11 @@ void KisInputManager::Private::CanvasSwitcher::removeCanvas(KisCanvas2 *canvas)
     }
 
     widget->removeEventFilter(this);
+
+    if (d->canvas == canvas) {
+        d->canvas = 0;
+        d->toolProxy = 0;
+    }
 }
 
 bool isInputWidget(QWidget *w)
@@ -418,6 +430,11 @@ bool KisInputManager::Private::ProximityNotifier::eventFilter(QObject* object, Q
     return QObject::eventFilter(object, event);
 }
 
+#define EXTRA_BUTTON(z, n, _) \
+    if(buttons & Qt::ExtraButton##n) { \
+        buttonSet << Qt::ExtraButton##n; \
+    }
+
 void KisInputManager::Private::addStrokeShortcut(KisAbstractInputAction* action, int index,
                                                  const QList<Qt::Key> &modifiers,
                                                  Qt::MouseButtons buttons)
@@ -425,25 +442,25 @@ void KisInputManager::Private::addStrokeShortcut(KisAbstractInputAction* action,
     KisStrokeShortcut *strokeShortcut =
         new KisStrokeShortcut(action, index);
 
-    QList<Qt::MouseButton> buttonList;
+    QSet<Qt::MouseButton> buttonSet;
     if(buttons & Qt::LeftButton) {
-        buttonList << Qt::LeftButton;
+        buttonSet << Qt::LeftButton;
     }
     if(buttons & Qt::RightButton) {
-        buttonList << Qt::RightButton;
+        buttonSet << Qt::RightButton;
     }
     if(buttons & Qt::MidButton) {
-        buttonList << Qt::MidButton;
-    }
-    if(buttons & Qt::XButton1) {
-        buttonList << Qt::XButton1;
-    }
-    if(buttons & Qt::XButton2) {
-        buttonList << Qt::XButton2;
+        buttonSet << Qt::MidButton;
     }
 
-    if (buttonList.size() > 0) {
-        strokeShortcut->setButtons(QSet<Qt::Key>::fromList(modifiers), QSet<Qt::MouseButton>::fromList(buttonList));
+BOOST_PP_REPEAT_FROM_TO(1, 25, EXTRA_BUTTON, _)
+
+    if (!buttonSet.empty()) {
+#if QT_VERSION >= QT_VERSION_CHECK(5,14,0)
+        strokeShortcut->setButtons(QSet<Qt::Key>(modifiers.begin(), modifiers.end()), buttonSet);
+#else
+        strokeShortcut->setButtons(QSet<Qt::Key>::fromList(modifiers), buttonSet);
+#endif
         matcher.addShortcut(strokeShortcut);
     }
     else {
@@ -612,6 +629,27 @@ void KisInputManager::Private::blockMouseEvents()
 
 void KisInputManager::Private::allowMouseEvents()
 {
+    /**
+     * On Windows tablet events may arrive asynchronously to the
+     * mouse events (in WinTab mode). The problem is that Qt
+     * generates Enter/Leave and FocusIn/Out events via mouse
+     * events only. It means that TabletPress may come much before
+     * Enter and FocusIn event and start the stroke. In such a case
+     * we shouldn't unblock mouse events.
+     *
+     * See https://bugs.kde.org/show_bug.cgi?id=417040
+     *
+     * PS:
+     * Ideally, we should fix Qt to generate Enter/Leave and
+     * FocusIn/Out events based on tablet events as well, but
+     * it is a lot of work.
+     */
+#ifdef Q_OS_WIN32
+    if (eventEater.hungry && matcher.hasRunningShortcut()) {
+        return;
+    }
+#endif
+
     eventEater.deactivate();
 }
 
@@ -640,16 +678,7 @@ bool KisInputManager::Private::handleCompressedTabletEvent(QEvent *event)
      */
     QWidget *recievingWidget = dynamic_cast<QWidget*>(eventsReceiver);
     if (recievingWidget && !recievingWidget->hasFocus()) {
-        QVector<Qt::Key> guessedKeys;
-
-        KisExtendedModifiersMapper mapper;
-        Qt::KeyboardModifiers modifiers = mapper.queryStandardModifiers();
-        Q_FOREACH (Qt::Key key, mapper.queryExtendedModifiers()) {
-            QKeyEvent kevent(QEvent::ShortcutOverride, key, modifiers);
-            guessedKeys << KisExtendedModifiersMapper::workaroundShiftAltMetaHell(&kevent);
-        }
-
-        matcher.recoveryModifiersWithoutFocus(guessedKeys);
+        fixShortcutMatcherModifiersState();
     }
 
     if (!matcher.pointerMoved(event) && toolProxy) {
@@ -661,6 +690,19 @@ bool KisInputManager::Private::handleCompressedTabletEvent(QEvent *event)
     return retval;
 }
 
+void KisInputManager::Private::fixShortcutMatcherModifiersState()
+{
+    QVector<Qt::Key> guessedKeys;
+    KisExtendedModifiersMapper mapper;
+    Qt::KeyboardModifiers modifiers = mapper.queryStandardModifiers();
+    Q_FOREACH (Qt::Key key, mapper.queryExtendedModifiers()) {
+        QKeyEvent kevent(QEvent::ShortcutOverride, key, modifiers);
+        guessedKeys << KisExtendedModifiersMapper::workaroundShiftAltMetaHell(&kevent);
+    }
+
+    matcher.recoveryModifiersWithoutFocus(guessedKeys);
+}
+
 qint64 KisInputManager::Private::TabletLatencyTracker::currentTimestamp() const
 {
     // on OS X, we need to compute the timestamp that compares correctly against the native event timestamp,
