@@ -212,6 +212,14 @@ const QList<KisAnimationFrameCache *> KisAnimationFrameCache::caches()
     return Private::caches.values();
 }
 
+const KisAnimationFrameCacheSP KisAnimationFrameCache::cacheForImage(KisImageWSP image)
+{
+    auto it = std::find_if(Private::caches.begin(), Private::caches.end(),
+                           [image] (KisAnimationFrameCache *cache) { return cache->image() == image; });
+
+    return it != Private::caches.end() ? *it : nullptr;
+}
+
 KisAnimationFrameCache::KisAnimationFrameCache(KisOpenGLImageTexturesSP textures)
     : m_d(new Private(textures))
 {
@@ -307,7 +315,7 @@ KisOpenGLUpdateInfoSP KisAnimationFrameCache::Private::fetchFrameDataImpl(KisIma
     }
 }
 
-KisOpenGLUpdateInfoSP KisAnimationFrameCache::fetchFrameData(int time, KisImageSP image, const QRegion &requestedRegion) const
+KisOpenGLUpdateInfoSP KisAnimationFrameCache::fetchFrameData(int time, KisImageSP image, const KisRegion &requestedRegion) const
 {
     if (time != image->animationInterface()->currentTime()) {
         qWarning() << "WARNING: KisAnimationFrameCache::frameReady image's time doesn't coincide with the requested time!";
