@@ -47,7 +47,6 @@ KisCanvasResourceProvider::KisCanvasResourceProvider(KisViewManager * view)
     : m_view(view)
 {
     m_fGChanged = true;
-    m_enablefGChange = true;    // default to true, so that colour history is working without popup palette
 }
 
 KisCanvasResourceProvider::~KisCanvasResourceProvider()
@@ -297,21 +296,13 @@ void KisCanvasResourceProvider::slotOnScreenResolutionChanged()
 void KisCanvasResourceProvider::slotCanvasResourceChanged(int key, const QVariant & res)
 {
     if(key == KoCanvasResourceProvider::ForegroundColor || key == KoCanvasResourceProvider::BackgroundColor) {
-        KoAbstractGradient* resource = KoResourceServerProvider::instance()->gradientServer()->resources()[0];
-        KoStopGradient* stopGradient = dynamic_cast<KoStopGradient*>(resource);
-        if(stopGradient) {
-            QList<KoGradientStop> stops;
-            stops << KoGradientStop(0.0, fgColor()) << KoGradientStop(1.0,  KoColor(QColor(0, 0, 0, 0), fgColor().colorSpace()));
-            stopGradient->setStops(stops);
-            KoResourceServerProvider::instance()->gradientServer()->updateResource(resource);
-        }
-        resource = KoResourceServerProvider::instance()->gradientServer()->resources()[1];
-        stopGradient = dynamic_cast<KoStopGradient*>(resource);
-        if(stopGradient) {
-            QList<KoGradientStop> stops;
-            stops << KoGradientStop(0.0, fgColor()) << KoGradientStop(1.0, bgColor());
-            stopGradient->setStops(stops);
-            KoResourceServerProvider::instance()->gradientServer()->updateResource(resource);
+        QList<KoAbstractGradient*> resources = KoResourceServerProvider::instance()->gradientServer()->resources();
+        for (int i = 0; i < resources.count(); i++) {
+            KoAbstractGradient* gradient = resources[i];
+            if(gradient->hasVariableColors()){
+                gradient->setVariableColors(fgColor(), bgColor());
+                KoResourceServerProvider::instance()->gradientServer()->updateResource(gradient);
+            }
         }
     }
     switch (key) {
@@ -365,7 +356,7 @@ void KisCanvasResourceProvider::setEraserMode(bool value)
 
 void KisCanvasResourceProvider::slotPainting()
 {
-    if (m_fGChanged && m_enablefGChange) {
+    if (m_fGChanged) {
         emit sigFGColorUsed(fgColor());
         m_fGChanged = false;
     }
@@ -400,11 +391,6 @@ void KisCanvasResourceProvider::slotGamutMaskDeactivate()
     emit sigGamutMaskDeactivated();
 }
 
-void KisCanvasResourceProvider::slotResetEnableFGChange(bool b)
-{
-    m_enablefGChange = b;
-}
-
 QList<QPointer<KisAbstractPerspectiveGrid> > KisCanvasResourceProvider::perspectiveGrids() const
 {
     return m_perspectiveGrids;
@@ -531,6 +517,16 @@ qreal KisCanvasResourceProvider::size() const
     return m_resourceManager->resource(Size).toReal();
 }
 
+void KisCanvasResourceProvider::setPatternSize(qreal size)
+{
+    m_resourceManager->setResource(PatternSize, size);
+}
+
+qreal KisCanvasResourceProvider::patternSize() const
+{
+    return m_resourceManager->resource(PatternSize).toReal();
+}
+
 void KisCanvasResourceProvider::setGlobalAlphaLock(bool lock)
 {
     m_resourceManager->setResource(GlobalAlphaLock, lock);
