@@ -21,7 +21,7 @@
 
 
 #include <QHash>
-#include <QSignalMapper>
+#include <KisSignalMapper.h>
 
 #include <QMessageBox>
 #include <kactionmenu.h>
@@ -52,25 +52,23 @@
 
 struct KisFilterManager::Private {
     Private()
-        : reapplyAction(0)
-        , actionCollection(0)
-        , actionManager(0)
-        , view(0)
     {
     }
-    KisAction* reapplyAction;
+
+    KisAction* reapplyAction {0};
+    KisAction* reapplyActionReprompt {0};
     QHash<QString, KActionMenu*> filterActionMenus;
     QHash<KisFilter*, QAction *> filters2Action;
-    KActionCollection *actionCollection;
-    KisActionManager *actionManager;
-    KisViewManager *view;
+    KActionCollection *actionCollection {0};
+    KisActionManager *actionManager {0};
+    KisViewManager *view {0};
 
     KisFilterConfigurationSP lastConfiguration;
     KisFilterConfigurationSP currentlyAppliedConfiguration;
     KisStrokeId currentStrokeId;
     QRect initialApplyRect;
 
-    QSignalMapper actionsMapper;
+    KisSignalMapper actionsMapper;
 
     QPointer<KisDlgFilter> filterDialog;
 };
@@ -101,9 +99,15 @@ void KisFilterManager::setup(KActionCollection * ac, KisActionManager *actionMan
     d->reapplyAction = d->actionManager->createAction("filter_apply_again");
     d->reapplyAction->setActivationFlags(KisAction::ACTIVE_DEVICE);
     d->reapplyAction->setEnabled(false);
-    connect(d->reapplyAction, SIGNAL(triggered()), SLOT(reapplyLastFilter()));
 
-    connect(&d->actionsMapper, SIGNAL(mapped(const QString&)), SLOT(showFilterDialog(const QString&)));
+    d->reapplyActionReprompt = d->actionManager->createAction("filter_apply_reprompt");
+    d->reapplyActionReprompt->setActivationFlags(KisAction::ACTIVE_DEVICE);
+    d->reapplyActionReprompt->setEnabled(false);
+
+    connect(d->reapplyAction, SIGNAL(triggered()), SLOT(reapplyLastFilter()));
+    connect(d->reapplyActionReprompt, SIGNAL(triggered()), SLOT(reapplyLastFilterReprompt()));
+
+    connect(&d->actionsMapper, SIGNAL(mapped(QString)), SLOT(showFilterDialog(QString)));
 
     // Setup list of filters
     QStringList keys = KisFilterRegistry::instance()->keys();
@@ -112,7 +116,7 @@ void KisFilterManager::setup(KActionCollection * ac, KisActionManager *actionMan
         insertFilter(filterName);
     }
 
-    connect(KisFilterRegistry::instance(), SIGNAL(filterAdded(QString)), SLOT(insertFilter(const QString &)));
+    connect(KisFilterRegistry::instance(), SIGNAL(filterAdded(QString)), SLOT(insertFilter(QString)));
 }
 
 void KisFilterManager::insertFilter(const QString & filterName)
@@ -176,7 +180,15 @@ void KisFilterManager::reapplyLastFilter()
     finish();
 }
 
-void KisFilterManager::showFilterDialog(const QString &filterId)
+void KisFilterManager::reapplyLastFilterReprompt()
+{
+    if (!d->lastConfiguration) return;
+
+    showFilterDialog(d->lastConfiguration->name(), d->lastConfiguration);
+    finish();
+}
+
+void KisFilterManager::showFilterDialog(const QString &filterId, KisFilterConfigurationSP overrideDefaultConfig)
 {
     if (!d->view->activeNode()->isEditable()) {
         d->view->showFloatingMessage(i18n("Cannot apply filter to locked layer."),
@@ -186,7 +198,7 @@ void KisFilterManager::showFilterDialog(const QString &filterId)
 
     if (d->filterDialog && d->filterDialog->isVisible()) {
         KisFilterSP filter = KisFilterRegistry::instance()->value(filterId);
-        d->filterDialog->setFilter(filter);
+        d->filterDialog->setFilter(filter, overrideDefaultConfig);
         return;
     }
 
@@ -245,10 +257,13 @@ void KisFilterManager::showFilterDialog(const QString &filterId)
             d->filterDialog = new KisDlgFilter(d->view , d->view->activeNode(), this, d->view->mainWindow());
             d->filterDialog->setAttribute(Qt::WA_DeleteOnClose);
         }
-        d->filterDialog->setFilter(filter);
+
+        d->filterDialog->setFilter(filter, overrideDefaultConfig);
         d->filterDialog->setVisible(true);
     } else {
-        apply(KisFilterConfigurationSP(filter->defaultConfiguration()));
+        KisFilterConfigurationSP defaultConfiguration =
+            overrideDefaultConfig ? overrideDefaultConfig : filter->defaultConfiguration();
+        apply(defaultConfiguration);
         finish();
     }
 }
@@ -276,8 +291,8 @@ void KisFilterManager::apply(KisFilterConfigurationSP filterConfig)
         applyRect |= image->bounds();
     }
 
-    KoCanvasResourceManager *resourceManager =
-        d->view->resourceProvider()->resourceManager();
+    KoCanvasResourceProvider *resourceManager =
+        d->view->canvasResourceProvider()->resourceManager();
 
     KisResourcesSnapshotSP resources =
         new KisResourcesSnapshot(image,
