@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2006 Boudewijn Rempt <boud@valdyas.org>
+ *  Copyright (c) 2020 L. E. Segovia <amy@amyspark.me>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -102,20 +103,14 @@
 #include "kis_layer_utils.h"
 #include "lazybrush/kis_colorize_mask.h"
 #include "kis_processing_applicator.h"
+#include "kis_projection_leaf.h"
 
 #include "KisSaveGroupVisitor.h"
 
 KisLayerManager::KisLayerManager(KisViewManager * view)
     : m_view(view)
-    , m_imageView(0)
-    , m_imageFlatten(0)
-    , m_imageMergeLayer(0)
-    , m_groupLayersSave(0)
-    , m_imageResizeToLayer(0)
-    , m_flattenLayer(0)
-    , m_rasterizeLayer(0)
     , m_commandsAdapter(new KisNodeCommandsAdapter(m_view))
-    , m_layerStyle(0)
+
 {
 }
 
@@ -148,7 +143,6 @@ KisPaintDeviceSP KisLayerManager::activeDevice()
 void KisLayerManager::activateLayer(KisLayerSP layer)
 {
     if (m_imageView) {
-        emit sigLayerActivated(layer);
         layersUpdated();
         if (layer) {
             m_view->canvasResourceProvider()->slotNodeActivated(layer.data());
@@ -242,9 +236,11 @@ void KisLayerManager::layerProperties()
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
     const bool multipleLayersSelected = selectedNodes.size() > 1;
 
+    if (!m_view->nodeManager()->canModifyLayers(selectedNodes)) return;
+
     KisAdjustmentLayerSP adjustmentLayer = KisAdjustmentLayerSP(dynamic_cast<KisAdjustmentLayer*>(layer.data()));
-    KisGeneratorLayerSP groupLayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()));
-    KisFileLayerSP filterLayer = KisFileLayerSP(dynamic_cast<KisFileLayer*>(layer.data()));
+    KisGeneratorLayerSP generatorLayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()));
+    KisFileLayerSP fileLayer = KisFileLayerSP(dynamic_cast<KisFileLayer*>(layer.data()));
 
     if (adjustmentLayer && !multipleLayersSelected) {
 
@@ -292,28 +288,27 @@ void KisLayerManager::layerProperties()
             }
         }
     }
-    else if (groupLayer && !multipleLayersSelected) {
-        KisFilterConfigurationSP configBefore(groupLayer->filter());
+    else if (generatorLayer && !multipleLayersSelected) {
+        KisFilterConfigurationSP configBefore(generatorLayer->filter());
         Q_ASSERT(configBefore);
         QString xmlBefore = configBefore->toXML();
 
-        KisDlgGeneratorLayer *dlg = new KisDlgGeneratorLayer(groupLayer->name(), m_view, m_view->mainWindow(), groupLayer, configBefore);
-        dlg->setCaption(i18n("Fill Layer Properties"));
+        KisDlgGeneratorLayer *dlg = new KisDlgGeneratorLayer(generatorLayer->name(), m_view, m_view->mainWindow(), generatorLayer, configBefore, KisStrokeId());
+        dlg->setWindowTitle(i18n("Fill Layer Properties"));
         dlg->setAttribute(Qt::WA_DeleteOnClose);
 
         dlg->setConfiguration(configBefore.data());
-        dlg->resize(dlg->minimumSizeHint());
 
         Qt::WindowFlags flags = dlg->windowFlags();
         dlg->setWindowFlags(flags | Qt::Tool | Qt::Dialog);
         dlg->show();
 
     }
-    else if (filterLayer && !multipleLayersSelected){
+    else if (fileLayer && !multipleLayersSelected){
         QString basePath = QFileInfo(m_view->document()->url().toLocalFile()).absolutePath();
-        QString fileNameOld = filterLayer->fileName();
-        KisFileLayer::ScalingMethod scalingMethodOld = filterLayer->scalingMethod();
-        KisDlgFileLayer dlg(basePath, filterLayer->name(), m_view->mainWindow());
+        QString fileNameOld = fileLayer->fileName();
+        KisFileLayer::ScalingMethod scalingMethodOld = fileLayer->scalingMethod();
+        KisDlgFileLayer dlg(basePath, fileLayer->name(), m_view->mainWindow());
         dlg.setCaption(i18n("File Layer Properties"));
         dlg.setFileName(fileNameOld);
         dlg.setScalingMethod(scalingMethodOld);
@@ -326,11 +321,11 @@ void KisLayerManager::layerProperties()
                 QMessageBox::critical(m_view->mainWindow(), i18nc("@title:window", "Krita"), i18n("No file name specified"));
                 return;
             }
-            filterLayer->setName(dlg.layerName());
+            fileLayer->setName(dlg.layerName());
 
             if (fileNameOld!= fileNameNew || scalingMethodOld != scalingMethodNew) {
                 KisChangeFileLayerCmd *cmd
-                        = new KisChangeFileLayerCmd(filterLayer,
+                        = new KisChangeFileLayerCmd(fileLayer,
                                                     basePath,
                                                     fileNameOld,
                                                     scalingMethodOld,
@@ -373,6 +368,8 @@ void KisLayerManager::changeCloneSource()
         return;
     }
 
+    if (!m_view->nodeManager()->canModifyLayers(implicitCastList<KisNodeSP>(cloneLayers))) return;
+
     KisDlgChangeCloneSource *dialog = new KisDlgChangeCloneSource(cloneLayers, m_view);
     dialog->setCaption(i18n("Change Clone Layer"));
     dialog->resize(dialog->minimumSizeHint());
@@ -387,6 +384,8 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     KisImageWSP image = m_view->image();
     if (!image) return;
 
+    // this precondition must be checked at higher level
+    KIS_SAFE_ASSERT_RECOVER_RETURN(source->isEditable(false));
 
     KisLayer *srcLayer = qobject_cast<KisLayer*>(source.data());
     if (srcLayer && (srcLayer->inherits("KisGroupLayer") || srcLayer->layerStyle() || srcLayer->childCount() > 0)) {
@@ -394,12 +393,15 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
         return;
     }
 
-
     KisPaintDeviceSP srcDevice =
             source->paintDevice() ? source->projection() : source->original();
 
     bool putBehind = false;
-    QString newCompositeOp = source->compositeOpId();
+
+    QString newCompositeOp =
+        source->projectionLeaf()->isLayer() ?
+            source->compositeOpId() : COMPOSITE_OVER;
+
     KisColorizeMask *colorizeMask = dynamic_cast<KisColorizeMask*>(source.data());
     if (colorizeMask) {
         srcDevice = colorizeMask->coloringProjection();
@@ -417,10 +419,12 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
             *srcDevice->compositionSourceColorSpace()) {
 
         clone = new KisPaintDevice(srcDevice->compositionSourceColorSpace());
+        clone->setDefaultPixel(
+            srcDevice->defaultPixel().convertedTo(
+                srcDevice->compositionSourceColorSpace()));
 
         QRect rc(srcDevice->extent());
         KisPainter::copyAreaOptimized(rc.topLeft(), srcDevice, clone, rc);
-
     } else {
         clone = new KisPaintDevice(*srcDevice);
     }
@@ -429,6 +433,17 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
                                          source->name(),
                                          source->opacity(),
                                          clone);
+
+    if (srcDevice->framesInterface()) {
+        KisKeyframeChannel *cloneKeyChannel = layer->getKeyframeChannel(KisKeyframeChannel::Content.id(), true);
+        layer->enableAnimation();
+        KisKeyframeChannel *sourceKeyChannel = srcDevice->keyframeChannel();
+
+        foreach (const int &index, sourceKeyChannel->allKeyframeIds()) {
+            cloneKeyChannel->copyExternalKeyframe(sourceKeyChannel, index, index);
+        }
+    }
+
     layer->setCompositeOpId(newCompositeOp);
 
     KisNodeSP parent = source->parent();
@@ -447,7 +462,6 @@ void KisLayerManager::convertNodeToPaintLayer(KisNodeSP source)
     m_commandsAdapter->removeNode(source);
     m_commandsAdapter->addNode(layer, parent, above);
     m_commandsAdapter->endMacro();
-
 }
 
 void KisLayerManager::convertGroupToAnimated()
@@ -455,6 +469,8 @@ void KisLayerManager::convertGroupToAnimated()
     KisGroupLayerSP group = dynamic_cast<KisGroupLayer*>(activeLayer().data());
     if (group.isNull()) return;
 
+    if (!m_view->nodeManager()->canModifyLayer(group)) return;
+
     KisPaintLayerSP animatedLayer = new KisPaintLayer(m_view->image(), group->name(), OPACITY_OPAQUE_U8);
     animatedLayer->enableAnimation();
     KisRasterKeyframeChannel *contentChannel = dynamic_cast<KisRasterKeyframeChannel*>(
@@ -481,6 +497,9 @@ void KisLayerManager::convertLayerToFileLayer(KisNodeSP source)
     KisImageSP image = m_view->image();
     if (!image) return;
 
+    // this precondition must be checked at higher level
+    KIS_SAFE_ASSERT_RECOVER_RETURN(source->isEditable(false));
+
     QStringList listMimeFilter = KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export);
 
     KoDialog dlg;
@@ -573,7 +592,7 @@ void KisLayerManager::adjustLayerPosition(KisNodeSP node, KisNodeSP activeNode,
     }
 
     while (parent &&
-           (!parent->allowAsChild(node) || parent->userLocked())) {
+           (!parent->allowAsChild(node) || !parent->isEditable(false))) {
 
         above = parent;
         parent = parent->parent();
@@ -603,8 +622,9 @@ void KisLayerManager::addLayerCommon(KisNodeSP activeNode, KisNodeSP layer, bool
 
 KisLayerSP KisLayerManager::addPaintLayer(KisNodeSP activeNode)
 {
-    KisImageWSP image = m_view->image();
-    KisLayerSP layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, image->colorSpace());
+    KisImageWSP image = m_view->image();    
+    KisLayerSP layer = new KisPaintLayer(image.data(),  image->nextLayerName( i18n("Paint Layer") ), OPACITY_OPAQUE_U8, image->colorSpace());
+
     addLayerCommon(activeNode, layer, false, 0);
 
     return layer;
@@ -613,7 +633,7 @@ KisLayerSP KisLayerManager::addPaintLayer(KisNodeSP activeNode)
 KisNodeSP KisLayerManager::addGroupLayer(KisNodeSP activeNode)
 {
     KisImageWSP image = m_view->image();
-    KisGroupLayerSP group = new KisGroupLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
+    KisGroupLayerSP group = new KisGroupLayer(image.data(), image->nextLayerName( i18n("Group") ), OPACITY_OPAQUE_U8);
     addLayerCommon(activeNode, group, false, 0);
     return group;
 }
@@ -629,7 +649,7 @@ KisNodeSP KisLayerManager::addCloneLayer(KisNodeList nodes)
 
     KisNodeSP node, lastClonedNode;
     Q_FOREACH (node, filteredNodes) {
-        lastClonedNode = new KisCloneLayer(qobject_cast<KisLayer*>(node.data()), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
+        lastClonedNode = new KisCloneLayer(qobject_cast<KisLayer*>(node.data()), image.data(), image->nextLayerName( i18n("Clone Layer") ), OPACITY_OPAQUE_U8);
         addLayerCommon(newAbove, lastClonedNode, true, 0 );
     }
 
@@ -642,7 +662,7 @@ KisNodeSP KisLayerManager::addShapeLayer(KisNodeSP activeNode)
     if (!m_view->document()) return 0;
 
     KisImageWSP image = m_view->image();
-    KisShapeLayerSP layer = new KisShapeLayer(m_view->document()->shapeController(), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
+    KisShapeLayerSP layer = new KisShapeLayer(m_view->document()->shapeController(), image.data(), image->nextLayerName(i18n("Vector Layer")), OPACITY_OPAQUE_U8);
 
     addLayerCommon(activeNode, layer, false, 0);
 
@@ -664,7 +684,7 @@ KisNodeSP KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode)
 
     KisPaintDeviceSP previewDevice = new KisPaintDevice(*adjl->original());
 
-    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), previewDevice, image->nextLayerName(), i18n("New Filter Layer"), m_view, qApp->activeWindow());
+    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), previewDevice, image->nextLayerName(i18n("Filter Layer")), i18n("New Filter Layer"), m_view, qApp->activeWindow());
     dlg.resize(dlg.minimumSizeHint());
 
     // ensure that the device may be free'd by the dialog
@@ -694,30 +714,39 @@ KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP activeNode, c
     return layer;
 }
 
+KisGeneratorLayerSP KisLayerManager::addGeneratorLayer(KisNodeSP activeNode, const QString &name, KisFilterConfigurationSP filter, KisSelectionSP selection, KisProcessingApplicator *applicator)
+{
+    KisImageWSP image = m_view->image();
+    auto layer = new KisGeneratorLayer(image, name, filter, selection);
+    addLayerCommon(activeNode, layer, true, applicator);
+
+    return layer;
+}
+
 KisNodeSP KisLayerManager::addGeneratorLayer(KisNodeSP activeNode)
 {
     KisImageWSP image = m_view->image();
+    KisSelectionSP selection = m_view->selection();
     QColor currentForeground = m_view->canvasResourceProvider()->fgColor().toQColor();
 
+    KisProcessingApplicator applicator(image, 0, KisProcessingApplicator::NONE, KisImageSignalVector() << ModifiedSignal, kundo2_i18n("Add Layer"));
 
-    KisDlgGeneratorLayer dlg(image->nextLayerName(), m_view, m_view->mainWindow(), 0, 0);
+    KisGeneratorLayerSP node = addGeneratorLayer(activeNode, QString(), nullptr, selection, &applicator);
+
+    KisDlgGeneratorLayer dlg(image->nextLayerName(i18n("Fill Layer")), m_view, m_view->mainWindow(), node, nullptr, applicator.getStroke());
     KisFilterConfigurationSP defaultConfig = dlg.configuration();
     defaultConfig->setProperty("color", currentForeground);
     dlg.setConfiguration(defaultConfig);
 
-    dlg.resize(dlg.minimumSizeHint());
     if (dlg.exec() == QDialog::Accepted) {
-        KisSelectionSP selection = m_view->selection();
-        KisFilterConfigurationSP  generator = dlg.configuration();
-        QString name = dlg.layerName();
-
-        KisNodeSP node = new KisGeneratorLayer(image, name, generator, selection);
-
-        addLayerCommon(activeNode, node, true, 0);
-
+        node->setName(dlg.layerName());
+        applicator.end();
         return node;
     }
-    return 0;
+    else {
+        applicator.cancel();
+        return nullptr;
+    }
 }
 
 void KisLayerManager::flattenImage()
@@ -797,6 +826,24 @@ void KisLayerManager::mergeLayer()
     if (!m_view->blockUntilOperationsFinished(image)) return;
 
     QList<KisNodeSP> selectedNodes = m_view->nodeManager()->selectedNodes();
+
+    // check if all the layers are a part of a locked group
+    bool hasEditableLayer = false;
+    Q_FOREACH (KisNodeSP node, selectedNodes) {
+        if (node->isEditable(false)) {
+            hasEditableLayer = true;
+            break;
+        }
+    }
+
+    if (!hasEditableLayer) {
+        m_view->showFloatingMessage(
+                    i18ncp("floating message in layer manager",
+                          "Layer is locked ", "Layers are locked", selectedNodes.size()),
+                    QIcon(), 2000, KisFloatingMessage::Low);
+        return;
+    }
+
     if (selectedNodes.size() > 1) {
         image->mergeMultipleLayers(selectedNodes, m_view->activeNode());
     }
@@ -839,6 +886,7 @@ void KisLayerManager::flattenLayer()
     if (!layer) return;
 
     if (!m_view->blockUntilOperationsFinished(image)) return;
+    if (!m_view->nodeManager()->canModifyLayer(layer)) return;
 
     convertNodeToPaintLayer(layer);
     m_view->updateGUI();
@@ -853,6 +901,7 @@ void KisLayerManager::rasterizeLayer()
     if (!layer) return;
 
     if (!m_view->blockUntilOperationsFinished(image)) return;
+    if (!m_view->nodeManager()->canModifyLayer(layer)) return;
 
     KisPaintLayerSP paintLayer = new KisPaintLayer(image, layer->name(), layer->opacity());
     KisPainter gc(paintLayer->paintDevice());
@@ -942,7 +991,7 @@ KisNodeSP KisLayerManager::addFileLayer(KisNodeSP activeNode)
     }
     KisImageWSP image = m_view->image();
 
-    KisDlgFileLayer dlg(basePath, image->nextLayerName(), m_view->mainWindow());
+    KisDlgFileLayer dlg(basePath, image->nextLayerName(i18n("File Layer")), m_view->mainWindow());
     dlg.resize(dlg.minimumSizeHint());
 
     if (dlg.exec() == QDialog::Accepted) {
@@ -976,6 +1025,7 @@ void KisLayerManager::layerStyle()
     if (!layer) return;
 
     if (!m_view->blockUntilOperationsFinished(image)) return;
+    if (!m_view->nodeManager()->canModifyLayer(layer)) return;
 
     KisPSDLayerStyleSP oldStyle;
     if (layer->layerStyle()) {
