@@ -100,6 +100,10 @@ KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage)
         rc |= node->exactBounds();
     }
 
+    if (rc.isEmpty()) {
+        rc = srcImage->bounds();
+    }
+
     KisImageSP image = new KisImage(0, rc.width(), rc.height(), nodes.first()->colorSpace(), nodes.first()->name());
     image->setAllowMasksOnRootNode(true);
 
@@ -144,7 +148,7 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
     if (mimetype == "application/x-qt-image") {
         KisConfig cfg(true);
 
-        KisDocument *doc = createDocument(m_nodes, m_image);
+        QScopedPointer<KisDocument> doc(createDocument(m_nodes, m_image));
 
         return doc->image()->projection()->convertToQImage(cfg.displayProfile(QApplication::desktop()->screenNumber(qApp->activeWindow())),
                                                            KoColorConversionTransformation::internalRenderingIntent(),
@@ -162,7 +166,7 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
         QByteArray ba = serializeToByteArray(m_nodes, m_image);
 
         QString temporaryPath =
-                QDir::tempPath() + QDir::separator() +
+                QDir::tempPath() + '/' +
                 QString("krita_tmp_dnd_layer_%1_%2.kra")
                 .arg(QApplication::applicationPid())
                 .arg(qrand());
@@ -310,20 +314,23 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
 
     if (nodes.isEmpty() && data->hasFormat("application/x-krita-node-url")) {
         QByteArray ba = data->data("application/x-krita-node-url");
+        QUrl url = QUrl::fromEncoded(ba);
+        Q_ASSERT(url.isLocalFile());
+
         KisDocument *tempDoc = KisPart::instance()->createDocument();
-        Q_ASSERT(QUrl::fromEncoded(ba).isLocalFile());
-        bool result = tempDoc->openUrl(QUrl::fromEncoded(ba));
+        bool result = tempDoc->openUrl(url);
 
         if (result) {
-            KisImageWSP tempImage = tempDoc->image();
+            KisImageSP tempImage = tempDoc->image();
             Q_FOREACH (KisNodeSP node, tempImage->root()->childNodes(QStringList(), KoProperties())) {
                 tempImage->removeNode(node);
                 initializeExternalNode(&node, image, shapeController);
                 nodes << node;
             }
         }
+
         delete tempDoc;
-        QFile::remove(QUrl::fromEncoded(ba).toLocalFile());
+        QFile::remove(url.toLocalFile());
     }
 
     if (nodes.isEmpty() && data->hasImage()) {
@@ -383,7 +390,7 @@ bool nodeAllowsAsChild(KisNodeSP parent, KisNodeList nodes)
 {
     bool result = true;
     Q_FOREACH (KisNodeSP node, nodes) {
-        if (!parent->allowAsChild(node)) {
+        if (!parent->allowAsChild(node) || !parent->isEditable(false)) {
             result = false;
             break;
         }
