@@ -20,9 +20,8 @@
 
 #include <QStandardPaths>
 #include <QMessageBox>
-#include <QSignalMapper>
+#include <KisSignalMapper.h>
 #include <QApplication>
-#include <QMessageBox>
 
 #include <kactioncollection.h>
 
@@ -123,8 +122,8 @@ struct KisNodeManager::Private {
 
     bool activateNodeImpl(KisNodeSP node);
 
-    QSignalMapper nodeCreationSignalMapper;
-    QSignalMapper nodeConversionSignalMapper;
+    KisSignalMapper nodeCreationSignalMapper;
+    KisSignalMapper nodeConversionSignalMapper;
 
     bool lastRequestedIsolatedModeStatus;
 
@@ -197,8 +196,6 @@ bool KisNodeManager::Private::activateNodeImpl(KisNodeSP node)
 KisNodeManager::KisNodeManager(KisViewManager *view)
     : m_d(new Private(this, view))
 {
-
-    connect(&m_d->layerManager, SIGNAL(sigLayerActivated(KisLayerSP)), SIGNAL(sigLayerActivated(KisLayerSP)));
 }
 
 KisNodeManager::~KisNodeManager()
@@ -274,9 +271,15 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     action = actionManager->createAction("activateNextLayer");
     connect(action, SIGNAL(triggered()), this, SLOT(activateNextNode()));
 
+    action = actionManager->createAction("activateNextSiblingLayer");
+    connect(action, SIGNAL(triggered()), this, SLOT(activateNextSiblingNode()));
+
     action = actionManager->createAction("activatePreviousLayer");
     connect(action, SIGNAL(triggered()), this, SLOT(activatePreviousNode()));
 
+    action = actionManager->createAction("activatePreviousSiblingLayer");
+    connect(action, SIGNAL(triggered()), this, SLOT(activatePreviousSiblingNode()));
+
     action = actionManager->createAction("switchToPreviouslyActiveNode");
     connect(action, SIGNAL(triggered()), this, SLOT(switchToPreviouslyActiveNode()));
 
@@ -373,7 +376,7 @@ void KisNodeManager::setup(KActionCollection * actionCollection, KisActionManage
     connect(&m_d->nodeConversionSignalMapper, SIGNAL(mapped(QString)),
             this, SLOT(convertNode(QString)));
 
-    action = actionManager->createAction("isolate_layer");
+    action = actionManager->createAction("isolate_active_layer");
     connect(action, SIGNAL(triggered(bool)), this, SLOT(toggleIsolateMode(bool)));
 
     action = actionManager->createAction("toggle_layer_visibility");
@@ -435,6 +438,59 @@ const KoColorSpace* KisNodeManager::activeColorSpace()
     }
 }
 
+bool KisNodeManager::canModifyLayers(KisNodeList nodes, bool showWarning)
+{
+    KisNodeSP lockedNode;
+    Q_FOREACH (KisNodeSP node, nodes) {
+        if (!node->isEditable(false)) {
+            lockedNode = node;
+            break;
+        }
+    }
+
+    if (lockedNode && showWarning) {
+        QString errorMessage;
+
+        if (nodes.size() <= 1) {
+            errorMessage = i18n("Layer is locked");
+        } else {
+            errorMessage = i18n("Layer \"%1\" is locked", lockedNode->name());
+        }
+
+        m_d->view->showFloatingMessage(errorMessage, QIcon());
+    }
+
+    return !lockedNode;
+}
+
+bool KisNodeManager::canModifyLayer(KisNodeSP node, bool showWarning)
+{
+    return canModifyLayers({node}, showWarning);
+}
+
+bool KisNodeManager::canMoveLayers(KisNodeList nodes, bool showWarning)
+{
+    KisNodeSP lockedNode;
+    Q_FOREACH (KisNodeSP node, nodes) {
+        if (node->parent() && !node->parent()->isEditable(false)) {
+            lockedNode = node->parent();
+            break;
+        }
+    }
+
+    if (lockedNode && showWarning) {
+        QString errorMessage = i18n("Layer \"%1\" is locked", lockedNode->name());
+        m_d->view->showFloatingMessage(errorMessage, QIcon());
+    }
+
+    return !lockedNode;
+}
+
+bool KisNodeManager::canMoveLayer(KisNodeSP node, bool showWarning)
+{
+    return canMoveLayers({node}, showWarning);
+}
+
 void KisNodeManager::moveNodeAt(KisNodeSP node, KisNodeSP parent, int index)
 {
     if (parent->allowAsChild(node)) {
@@ -495,7 +551,7 @@ void KisNodeManager::toggleIsolateMode(bool checked)
                 activeNode->inherits("KisColorizeMask")) return;
 
         if (!image->startIsolatedMode(activeNode)) {
-            KisAction *action = m_d->view->actionManager()->actionByName("isolate_layer");
+            KisAction *action = m_d->view->actionManager()->actionByName("isolate_active_layer");
             action->setChecked(false);
         }
     } else {
@@ -519,7 +575,7 @@ void KisNodeManager::slotUpdateIsolateModeActionImageStatusChange()
 
 void KisNodeManager::slotUpdateIsolateModeAction()
 {
-    KisAction *action = m_d->view->actionManager()->actionByName("isolate_layer");
+    KisAction *action = m_d->view->actionManager()->actionByName("isolate_active_layer");
     Q_ASSERT(action);
 
     KisNodeSP activeNode = this->activeNode();
@@ -555,6 +611,10 @@ KisNodeSP  KisNodeManager::createNode(const QString & nodeType, bool quiet, KisP
 
     KIS_ASSERT_RECOVER_RETURN_VALUE(activeNode, 0);
 
+    /// the check for editability happens inside the functions
+    /// themselves, because layers can be created anyway (in a
+    /// different position), but masks cannot.
+
     // XXX: make factories for this kind of stuff,
     //      with a registry
 
@@ -618,6 +678,8 @@ void KisNodeManager::convertNode(const QString &nodeType)
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) return;
 
+    if (!canModifyLayer(activeNode)) return;
+
     if (nodeType == "KisPaintLayer") {
         m_d->layerManager.convertNodeToPaintLayer(activeNode);
     } else if (nodeType == "KisSelectionMask" ||
@@ -883,7 +945,7 @@ bool KisNodeManager::trySetNodeProperties(KisNodeSP node, KisImageSP image, KisB
         }
     }
 
-    KisNodePropertyListCommand::setNodePropertiesNoUndo(node, image, properties);
+    KisNodePropertyListCommand::setNodePropertiesAutoUndo(node, image, properties);
 
     return true;
 }
@@ -927,6 +989,8 @@ KisNodeJugglerCompressed* KisNodeManager::Private::lazyGetJuggler(const KUndo2Ma
 
 void KisNodeManager::raiseNode()
 {
+    if (!canMoveLayers(selectedNodes())) return;
+
     KUndo2MagicString actionName = kundo2_i18n("Raise Nodes");
     KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
     juggler->raiseNode(selectedNodes());
@@ -934,6 +998,8 @@ void KisNodeManager::raiseNode()
 
 void KisNodeManager::lowerNode()
 {
+    if (!canMoveLayers(selectedNodes())) return;
+
     KUndo2MagicString actionName = kundo2_i18n("Lower Nodes");
     KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
     juggler->lowerNode(selectedNodes());
@@ -952,6 +1018,8 @@ void KisNodeManager::removeSingleNode(KisNodeSP node)
 
 void KisNodeManager::removeSelectedNodes(KisNodeList nodes)
 {
+    if (!canModifyLayers(nodes)) return;
+
     KUndo2MagicString actionName = kundo2_i18n("Remove Nodes");
     KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
     juggler->removeNode(nodes);
@@ -1002,56 +1070,74 @@ void KisNodeManager::mirrorAllNodesY()
                Qt::Vertical, m_d->view->selection());
 }
 
-void KisNodeManager::activateNextNode()
+void KisNodeManager::activateNextNode(bool siblingsOnly)
 {
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) return;
 
-    KisNodeSP node = activeNode->nextSibling();
+    KisNodeSP nextNode = activeNode->nextSibling();
 
-    while (node && node->childCount() > 0) {
-        node = node->firstChild();
+    if (!siblingsOnly) {
+        // Recurse groups...
+        while (nextNode && nextNode->childCount() > 0) {
+            nextNode = nextNode->firstChild();
+        }
+
+        // Out of nodes? Back out of group...
+        if (!nextNode && activeNode->parent()) {
+            nextNode = activeNode->parent();
+        }
     }
 
-    if (!node && activeNode->parent() && activeNode->parent()->parent()) {
-        node = activeNode->parent();
+    // Skip nodes hidden from tree view..
+    while (nextNode && isNodeHidden(nextNode, m_d->nodeDisplayModeAdapter->showGlobalSelectionMask())) {
+        nextNode = nextNode->nextSibling();
     }
 
-    while(node && isNodeHidden(node, m_d->nodeDisplayModeAdapter->showGlobalSelectionMask())) {
-        node = node->nextSibling();
-    }
-
-    if (node) {
-        slotNonUiActivatedNode(node);
+    // Select node, unless root..
+    if (nextNode && nextNode->parent()) {
+        slotNonUiActivatedNode(nextNode);
     }
 }
 
-void KisNodeManager::activatePreviousNode()
+void KisNodeManager::activateNextSiblingNode()
+{
+    activateNextNode(true);
+}
+
+void KisNodeManager::activatePreviousNode(bool siblingsOnly)
 {
     KisNodeSP activeNode = this->activeNode();
     if (!activeNode) return;
 
-    KisNodeSP node;
+    KisNodeSP nextNode = activeNode->prevSibling();
 
-    if (activeNode->childCount() > 0) {
-        node = activeNode->lastChild();
-    }
-    else {
-        node = activeNode->prevSibling();
+    if (!siblingsOnly) {
+        // Enter groups..
+        if (activeNode->childCount() > 0) {
+            nextNode = activeNode->lastChild();
+        }
+
+        // Out of nodes? Back out of group...
+        if (!nextNode && activeNode->parent()) {
+            nextNode = activeNode->parent()->prevSibling();
+        }
     }
 
-    while (!node && activeNode->parent()) {
-        node = activeNode->parent()->prevSibling();
-        activeNode = activeNode->parent();
+    // Skip nodes hidden from tree view..
+    while (nextNode && isNodeHidden(nextNode, m_d->nodeDisplayModeAdapter->showGlobalSelectionMask())) {
+        nextNode = nextNode->prevSibling();
     }
 
-    while(node && isNodeHidden(node, m_d->nodeDisplayModeAdapter->showGlobalSelectionMask())) {
-        node = node->prevSibling();
+    // Select node, unless root..
+    if (nextNode && nextNode->parent()) {
+        slotNonUiActivatedNode(nextNode);
     }
+}
 
-    if (node) {
-        slotNonUiActivatedNode(node);
-    }
+void KisNodeManager::activatePreviousSiblingNode()
+{
+    activatePreviousNode(true);
 }
 
 void KisNodeManager::switchToPreviouslyActiveNode()
@@ -1066,6 +1152,8 @@ void KisNodeManager::mirrorNode(KisNodeSP node,
                                 Qt::Orientation orientation,
                                 KisSelectionSP selection)
 {
+    if (!canModifyLayer(node)) return;
+
     KisImageSignalVector emitSignals;
     emitSignals << ModifiedSignal;
 
@@ -1146,10 +1234,16 @@ void KisNodeManager::saveNodeAsImage()
         return;
     }
 
-    KisImageWSP image = m_d->view->image();
+    KisPaintDeviceSP saveDevice = node->projection();
+
+    if (!saveDevice) {
+        return;
+    }
+
+    KisImageSP image = m_d->view->image();
     QRect saveRect = image->bounds() | node->exactBounds();
 
-    m_d->saveDeviceAsImage(node->projection(),
+    m_d->saveDeviceAsImage(saveDevice,
                            node->name(),
                            saveRect,
                            image->xRes(), image->yRes(),
@@ -1193,6 +1287,7 @@ void KisNodeManager::saveVectorLayerAsImage()
 void KisNodeManager::slotSplitAlphaIntoMask()
 {
     KisNodeSP node = activeNode();
+    if (!canModifyLayer(node)) return;
 
     // guaranteed by KisActionManager
     KIS_ASSERT_RECOVER_RETURN(node->hasEditablePaintDevice());
@@ -1234,7 +1329,7 @@ void KisNodeManager::Private::mergeTransparencyMaskAsAlpha(bool writeToLayers)
     // guaranteed by KisActionManager
     KIS_ASSERT_RECOVER_RETURN(node->inherits("KisTransparencyMask"));
 
-    if (writeToLayers && !parentNode->hasEditablePaintDevice()) {
+    if (writeToLayers && (!parentNode->hasEditablePaintDevice() || !node->isEditable(false))) {
         QMessageBox::information(view->mainWindow(),
                                  i18nc("@title:window", "Layer %1 is not editable", parentNode->name()),
                                  i18n("Cannot write alpha channel of "
@@ -1383,9 +1478,11 @@ void KisNodeManager::cutLayersToClipboard()
 
     KisClipboard::instance()->setLayers(nodes, m_d->view->image(), false);
 
-    KUndo2MagicString actionName = kundo2_i18n("Cut Nodes");
-    KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
-    juggler->removeNode(nodes);
+    if (canModifyLayers(nodes)) {
+        KUndo2MagicString actionName = kundo2_i18n("Cut Nodes");
+        KisNodeJugglerCompressed *juggler = m_d->lazyGetJuggler(actionName);
+        juggler->removeNode(nodes);
+    }
 }
 
 void KisNodeManager::copyLayersToClipboard()
@@ -1421,16 +1518,18 @@ void KisNodeManager::pasteLayersFromClipboard()
                                   nodeInsertionAdapter());
 }
 
-void KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
+bool KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
                                           const QString &overrideGroupName,
                                           KisNodeSP *newGroup,
                                           KisNodeSP *newLastChild)
 {
     KisNodeSP active = activeNode();
-    if (!active) return;
+    if (!active) return false;
+
+    if (!canMoveLayer(active)) return false;
 
     KisImageSP image = m_d->view->image();
-    QString groupName = !overrideGroupName.isEmpty() ? overrideGroupName : image->nextLayerName();
+    QString groupName = !overrideGroupName.isEmpty() ? overrideGroupName : image->nextLayerName(i18n("Group"));
     KisGroupLayerSP group = new KisGroupLayer(image.data(), groupName, OPACITY_OPAQUE_U8);
 
     KisNodeList nodes1;
@@ -1440,7 +1539,7 @@ void KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
     nodes2 = KisLayerUtils::sortMergableNodes(image->root(), selectedNodes());
     KisLayerUtils::filterMergableNodes(nodes2);
 
-    if (nodes2.size() == 0) return;
+    if (nodes2.size() == 0) return false;
 
     if (KisLayerUtils::checkIsChildOf(active, nodes2)) {
         active = nodes2.first();
@@ -1454,6 +1553,8 @@ void KisNodeManager::createQuickGroupImpl(KisNodeJugglerCompressed *juggler,
 
     *newGroup = group;
     *newLastChild = nodes2.last();
+
+    return true;
 }
 
 void KisNodeManager::createQuickGroup()
@@ -1476,12 +1577,12 @@ void KisNodeManager::createQuickClippingGroup()
     KisNodeSP above;
 
     KisImageSP image = m_d->view->image();
-    createQuickGroupImpl(juggler, image->nextLayerName(i18nc("default name for a clipping group layer", "Clipping Group")), &parent, &above);
+    if (createQuickGroupImpl(juggler, image->nextLayerName(i18nc("default name for a clipping group layer", "Clipping Group")), &parent, &above)) {
+        KisPaintLayerSP maskLayer = new KisPaintLayer(image.data(), i18nc("default name for quick clip group mask layer", "Mask Layer"), OPACITY_OPAQUE_U8, image->colorSpace());
+        maskLayer->disableAlphaChannel(true);
 
-    KisPaintLayerSP maskLayer = new KisPaintLayer(image.data(), i18nc("default name for quick clip group mask layer", "Mask Layer"), OPACITY_OPAQUE_U8, image->colorSpace());
-    maskLayer->disableAlphaChannel(true);
-
-    juggler->addNode(KisNodeList() << maskLayer, parent, above);
+        juggler->addNode(KisNodeList() << maskLayer, parent, above);
+    }
 }
 
 void KisNodeManager::quickUngroup()
@@ -1489,6 +1590,8 @@ void KisNodeManager::quickUngroup()
     KisNodeSP active = activeNode();
     if (!active) return;
 
+    if (!canModifyLayer(active)) return;
+
     KisNodeSP parent = active->parent();
     KisNodeSP aboveThis = active;
 
