@@ -372,6 +372,7 @@ void KisNodeModel::slotBeginRemoveDummy(KisNodeDummy *dummy)
 
     if (itemIndex.isValid()) {
         connectDummy(dummy, false);
+        emit sigBeforeBeginRemoveRows(parentIndex, itemIndex.row(), itemIndex.row());
         beginRemoveRows(parentIndex, itemIndex.row(), itemIndex.row());
         m_d->needFinishRemoveRows = true;
     }
@@ -502,19 +503,23 @@ QVariant KisNodeModel::data(const QModelIndex &index, int role) const
     case KisNodeModel::ColorLabelIndexRole: {
         return node->colorLabelIndex();
     }
+    case KisNodeModel::DropReasonRole: {
+        QString result;
+        KisProjectionLeaf::NodeDropReason reason = node->projectionLeaf()->dropReason();
+
+        if (reason == KisProjectionLeaf::DropPassThroughMask) {
+            result = i18nc("@info:tooltip", "Disabled: masks on pass-through groups are not supported!");
+        } else if (reason == KisProjectionLeaf::DropPassThroughClone) {
+            result = i18nc("@info:tooltip", "Disabled: cloning pass-through groups is not supported!");
+        }
+
+        return result;
+    }
     default:
         if (role >= int(KisNodeModel::BeginThumbnailRole) && belongsToIsolatedGroup(node)) {
 
             const int maxSize = role - int(KisNodeModel::BeginThumbnailRole);
-
-            QSize size = node->extent().size();
-            size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
-            if (size.width() == 0 || size.height() == 0) {
-                // No thumbnail can be shown if there isn't width or height...
-                return QVariant();
-            }
-
-            return node->createThumbnail(size.width(), size.height());
+            return node->createThumbnail(maxSize, maxSize, Qt::KeepAspectRatio);
         } else {
             return QVariant();
         }
@@ -654,12 +659,16 @@ QStringList KisNodeModel::mimeTypes() const
 
 QMimeData * KisNodeModel::mimeData(const QModelIndexList &indexes) const
 {
+    bool hasLockedLayer = false;
     KisNodeList nodes;
     Q_FOREACH (const QModelIndex &idx, indexes) {
-        nodes << nodeFromIndex(idx);
+        KisNodeSP node = nodeFromIndex(idx);
+
+        nodes << node;
+        hasLockedLayer |= !node->isEditable(false);
     }
 
-    return KisMimeData::mimeForLayers(nodes, m_d->image);
+    return KisMimeData::mimeForLayers(nodes, m_d->image, hasLockedLayer);
 }
 
 bool KisNodeModel::dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent)
@@ -724,7 +733,7 @@ void KisNodeModel::updateDropEnabled(const QList<KisNodeSP> &nodes, QModelIndex
 
         bool dropEnabled = true;
         Q_FOREACH (const KisNodeSP &node, nodes) {
-            if (!target->allowAsChild(node)) {
+            if (!target->allowAsChild(node) || !target->isEditable(false)) {
                 dropEnabled = false;
                 break;
             }
