@@ -130,6 +130,7 @@ struct KisPaintingAssistant::Private {
     explicit Private(const Private &rhs);
     KisPaintingAssistantHandleSP reuseOrCreateHandle(QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap, KisPaintingAssistantHandleSP origHandle, KisPaintingAssistant *q, bool registerAssistant = true);
     QList<KisPaintingAssistantHandleSP> handles, sideHandles;
+
     KisPaintingAssistantHandleSP topLeft, bottomLeft, topRight, bottomRight, topMiddle, bottomMiddle, rightMiddle, leftMiddle;
 
     // share everything except handles between the clones
@@ -138,6 +139,7 @@ struct KisPaintingAssistant::Private {
         QString name;
         bool isSnappingActive;
         bool outlineVisible;
+
         KisCanvas2* m_canvas = 0;
 
         QPixmapCache::Key cached;
@@ -488,6 +490,20 @@ QByteArray KisPaintingAssistant::saveXml(QMap<KisPaintingAssistantHandleSP, int>
         xml.writeEndElement();
     }
     xml.writeEndElement();
+    if (!d->sideHandles.isEmpty()) { // for vanishing points only
+	xml.writeStartElement("sidehandles");
+	QMap<KisPaintingAssistantHandleSP, int> sideHandleMap;
+	Q_FOREACH (KisPaintingAssistantHandleSP handle, d->sideHandles) {
+	    int id = sideHandleMap.size();
+	    sideHandleMap.insert(handle, id);
+	    xml.writeStartElement("sidehandle");
+	    xml.writeAttribute("id", QString::number(id));
+	    xml.writeAttribute("x", QString::number(double(handle->x()), 'f', 3));
+	    xml.writeAttribute("y", QString::number(double(handle->y()), 'f', 3));
+	    xml.writeEndElement();
+      }
+    }
+
     xml.writeEndElement();
     xml.writeEndDocument();
     return data;
@@ -505,6 +521,7 @@ void KisPaintingAssistant::loadXml(KoStore* store, QMap<int, KisPaintingAssistan
     store->open(path);
     QByteArray data = store->read(store->size());
     QXmlStreamReader xml(data);
+    QMap<int, KisPaintingAssistantHandleSP> sideHandleMap;
     while (!xml.atEnd()) {
         switch (xml.readNext()) {
         case QXmlStreamReader::StartElement:
@@ -549,6 +566,20 @@ void KisPaintingAssistant::loadXml(KoStore* store, QMap<int, KisPaintingAssistan
                     }
                 }
                 addHandle(handleMap.value(id), HandleType::NORMAL);
+            } else if (xml.name() == "sidehandle") {
+                QString strId = xml.attributes().value("id").toString(),
+                        strX = xml.attributes().value("x").toString(),
+                        strY = xml.attributes().value("y").toString();
+                if (!strId.isEmpty() && !strX.isEmpty() && !strY.isEmpty()) {
+                    id = strId.toInt();
+                    x = strX.toDouble();
+                    y = strY.toDouble();
+                    if (!sideHandleMap.contains(id)) {
+                        sideHandleMap.insert(id, new KisPaintingAssistantHandle(x, y));
+                    }
+                }
+                addHandle(sideHandleMap.value(id), HandleType::SIDE);
+
             }
             break;
         default:
