@@ -78,6 +78,13 @@ public:
     void setType(char type);
     char handleType() const;
 
+    /**
+     * Returns the pointer to the "chief" assistant,
+     * which is supposed to handle transformations of the
+     * handle, when all the assistants are transformed
+     */
+    KisPaintingAssistant* chiefAssistant() const;
+
 private:
     void registerAssistant(KisPaintingAssistant*);
     void unregisterAssistant(KisPaintingAssistant*);
@@ -110,7 +117,9 @@ public:
      * @param strokeBegin the coordinates of the beginning of the stroke
      */
     virtual QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin) = 0;
-    virtual void endStroke() { }
+    virtual void endStroke() {}
+    virtual void setAdjustedBrushPosition(const QPointF position) { Q_UNUSED(position) }
+    virtual void setFollowBrushPosition(bool follow) { Q_UNUSED(follow) }
     virtual QPointF getEditorPosition() const = 0; // Returns editor widget position in document-space coordinates.
     virtual int numHandles() const = 0;
 
@@ -179,6 +188,11 @@ public:
     /// it will return false if we are in the middle of creating the assistant.
     virtual bool isAssistantComplete() const;
 
+    /// Transform the assistant using the given \p transform. Please note that \p transform
+    /// should be in 'document' coordinate system.
+    /// Used with image-wide transformations.
+    virtual void transform(const QTransform &transform);
+
 public:
     /**
      * This will render the final output. The drawCache does rendering most of the time so be sure to check that
@@ -199,6 +213,11 @@ protected:
     QList<KisPaintingAssistantHandleSP> m_handles;
 
     QPointF pixelToView(const QPoint pixelCoords) const;
+public:
+    /// clones the list of assistants
+    /// the originally shared handles will still be shared
+    /// the cloned assistants do not share any handle with the original assistants
+    static QList<KisPaintingAssistantSP> cloneAssistantList(const QList<KisPaintingAssistantSP> &list);
 
 private:
     struct Private;
