@@ -43,6 +43,7 @@ struct AssistantEditorData {
     const QPointF moveIconPosition = QPointF(15, 15);
     const QPointF snapIconPosition = QPointF(54, 20);
     const QPointF deleteIconPosition = QPointF(83, 18);
+    const QSize boundingSize = QSize(110, 40);
 };
 
 /**
@@ -64,11 +65,15 @@ public:
     void addAssistant(KisPaintingAssistantSP assistant);
     void removeAssistant(KisPaintingAssistantSP assistant);
     void removeAll();
+    void setAssistants(const QList<KisPaintingAssistantSP> &assistants);
     QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin);
+    void setAdjustedBrushPosition(const QPointF position);
     void endStroke();
     QList<KisPaintingAssistantHandleSP> handles();
     QList<KisPaintingAssistantSP> assistants() const;
 
+    bool hasPaintableAssistants() const;
+
 
     /// getter and setter functions for what assistant is currently selected
     /// this is used to control some tool options that are specific to a assistant
@@ -129,6 +134,8 @@ public Q_SLOTS:
     QPointF snapToGuide(KoPointerEvent *e, const QPointF &offset, bool useModifiers);
     QPointF snapToGuide(const QPointF& pt, const QPointF &offset);
 
+    void slotUpdateDecorationVisibility();
+
 protected:
     void drawDecoration(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter *converter,KisCanvas2* canvas) override;
     void drawHandles(KisPaintingAssistantSP assistant, QPainter& gc, const KisCoordinatesConverter *converter);
