@@ -33,7 +33,12 @@
 
 #include <KoColorProfile.h>
 #include <KoColorSpace.h>
-#include "kis_icon_utils.h"
+#include <KoToolManager.h>
+#include <KoViewConverter.h>
+
+#include <KisUsageLogger.h>
+
+#include <kis_icon_utils.h>
 
 #include <kis_types.h>
 #include <kis_image.h>
@@ -42,60 +47,78 @@
 #include <kis_selection_manager.h>
 #include "kis_memory_statistics_server.h"
 
-#include <KisView.h>
+#include "KisView.h"
+#include "KisDocument.h"
 #include "KisViewManager.h"
 #include "canvas/kis_canvas2.h"
 #include "kis_progress_widget.h"
 #include "kis_zoom_manager.h"
 
-#include <KoToolManager.h>
-#include <KoViewConverter.h>
-#include <KisMainWindow.h>
+#include "KisMainWindow.h"
+#include "kis_config.h"
+
+#include "widgets/KisMemoryReportButton.h"
 
 enum {
     IMAGE_SIZE_ID,
     POINTER_POSITION_ID
 };
 
-KisStatusBar::KisStatusBar(KisViewManager *view)
-        : m_view(view),
-          m_imageView(0),
-          m_statusBar(0)
+KisStatusBar::KisStatusBar(KisViewManager *viewManager)
+    : m_viewManager(viewManager)
+    , m_imageView(0)
+    , m_statusBar(0)
 {
 }
 
 void KisStatusBar::setup()
 {
     m_selectionStatus = new QToolButton();
+    m_selectionStatus->setObjectName("selection status");
     m_selectionStatus->setIconSize(QSize(16,16));
     m_selectionStatus->setAutoRaise(true);
-    m_selectionStatus->setEnabled(false);   
+    m_selectionStatus->setEnabled(false);
     updateSelectionIcon();
 
-    m_statusBar = m_view->mainWindow()->statusBar();
+    m_statusBar = m_viewManager->mainWindow()->statusBar();
 
-    connect(m_selectionStatus, SIGNAL(clicked()), m_view->selectionManager(), SLOT(slotToggleSelectionDecoration()));
-    connect(m_view->selectionManager(), SIGNAL(displaySelectionChanged()), SLOT(updateSelectionToolTip()));
-    connect(m_view->mainWindow(), SIGNAL(themeChanged()), this, SLOT(updateSelectionIcon()));
+    connect(m_selectionStatus, SIGNAL(clicked()), m_viewManager->selectionManager(), SLOT(slotToggleSelectionDecoration()));
+    connect(m_viewManager->selectionManager(), SIGNAL(displaySelectionChanged()), SLOT(updateSelectionToolTip()));
+    connect(m_viewManager->mainWindow(), SIGNAL(themeChanged()), this, SLOT(updateSelectionIcon()));
 
     addStatusBarItem(m_selectionStatus);
     m_selectionStatus->setVisible(false);
 
+#ifdef Q_OS_ANDROID
+    m_fullscreenToggle = new QToolButton;
+    m_fullscreenToggle->setObjectName("Toggle Fullscreen");
+    m_fullscreenToggle->setCheckable(false);
+    m_fullscreenToggle->setToolTip(i18n("Toggle Fullscreen"));
+    m_fullscreenToggle->setAutoRaise(true);
+    m_fullscreenToggle->setIcon(KisIconUtils::loadIcon("zoom-horizontal"));
+    addStatusBarItem(m_fullscreenToggle);
+    m_fullscreenToggle->setVisible(true);
+    connect(m_fullscreenToggle, SIGNAL(clicked()), m_viewManager, SLOT(slotToggleFullscreen()));
+#endif
+
     m_statusBarStatusLabel = new KSqueezedTextLabel();
+    m_statusBarStatusLabel->setObjectName("statsBarStatusLabel");
     m_statusBarStatusLabel->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
     m_statusBarStatusLabel->setContentsMargins(5, 5, 5, 5);
-    connect(KoToolManager::instance(), SIGNAL(changedStatusText(const QString &)),
-            m_statusBarStatusLabel, SLOT(setText(const QString &)));
+    connect(KoToolManager::instance(), SIGNAL(changedStatusText(QString)),
+            m_statusBarStatusLabel, SLOT(setText(QString)));
     addStatusBarItem(m_statusBarStatusLabel, 2);
     m_statusBarStatusLabel->setVisible(false);
 
     m_statusBarProfileLabel = new KSqueezedTextLabel();
+    m_statusBarProfileLabel->setObjectName("statsBarProfileLabel");
     m_statusBarProfileLabel->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
     m_statusBarProfileLabel->setContentsMargins(5, 5, 5, 5);
     addStatusBarItem(m_statusBarProfileLabel, 3);
     m_statusBarProfileLabel->setVisible(false);
 
     m_progress = new KisProgressWidget();
+    m_progress->setObjectName("ProgressBar");
     addStatusBarItem(m_progress);
     m_progress->setVisible(false);
     connect(m_progress, SIGNAL(sigCancellationRequested()), this, SIGNAL(sigCancellationRequested()));
@@ -103,7 +126,8 @@ void KisStatusBar::setup()
     m_progressUpdater.reset(new KisProgressUpdater(m_progress, m_progress->progressProxy()));
     m_progressUpdater->setAutoNestNames(true);
 
-    m_memoryReportBox = new QPushButton();
+    m_memoryReportBox = new KisMemoryReportButton();
+    m_memoryReportBox->setObjectName("memoryReportBox");
     m_memoryReportBox->setFlat(true);
     m_memoryReportBox->setContentsMargins(5, 5, 5, 5);
     m_memoryReportBox->setMinimumWidth(120);
@@ -113,6 +137,7 @@ void KisStatusBar::setup()
     connect(m_memoryReportBox, SIGNAL(clicked()), SLOT(showMemoryInfoToolTip()));
 
     m_pointerPositionLabel = new QLabel(QString());
+    m_pointerPositionLabel->setObjectName("pointerPositionLabel");
     m_pointerPositionLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     m_pointerPositionLabel->setMinimumWidth(100);
     m_pointerPositionLabel->setContentsMargins(5,5, 5, 5);
@@ -122,6 +147,18 @@ void KisStatusBar::setup()
     connect(KisMemoryStatisticsServer::instance(),
             SIGNAL(sigUpdateMemoryStatistics()),
             SLOT(imageSizeChanged()));
+
+    m_resetAngleButton = new QToolButton;
+    m_resetAngleButton->setObjectName("Reset Rotation");
+    m_resetAngleButton->setCheckable(false);
+    m_resetAngleButton->setToolTip(i18n("Reset Rotation"));
+    m_resetAngleButton->setAutoRaise(true);
+    m_resetAngleButton->setIcon(KisIconUtils::loadIcon("rotate-canvas-left"));
+    addStatusBarItem(m_resetAngleButton);
+
+    connect(m_resetAngleButton, SIGNAL(clicked()), m_viewManager, SLOT(slotResetRotation()));
+    m_resetAngleButton->setVisible(false);
+
 }
 
 KisStatusBar::~KisStatusBar()
@@ -142,16 +179,19 @@ void KisStatusBar::setView(QPointer<KisView> imageView)
 
     if (imageView) {
         m_imageView = imageView;
-
+        m_resetAngleButton->setVisible(true);
         connect(m_imageView, SIGNAL(sigColorSpaceChanged(const KoColorSpace*)),
                 this, SLOT(updateStatusBarProfileLabel()));
         connect(m_imageView, SIGNAL(sigProfileChanged(const KoColorProfile*)),
                 this, SLOT(updateStatusBarProfileLabel()));
-        connect(m_imageView, SIGNAL(sigSizeChanged(const QPointF&, const QPointF&)),
+        connect(m_imageView, SIGNAL(sigSizeChanged(QPointF,QPointF)),
                 this, SLOT(imageSizeChanged()));
         updateStatusBarProfileLabel();
         addStatusBarItem(m_imageView->zoomManager()->zoomActionWidget());
     }
+    else {
+        m_resetAngleButton->setVisible(false);
+    }
 
     imageSizeChanged();
 }
@@ -165,8 +205,6 @@ void KisStatusBar::addStatusBarItem(QWidget *widget, int stretch, bool permanent
     else {
         m_statusBar->addWidget(widget, stretch);
     }
-
-    sbItem.show();
     widget->setVisible(true);
     m_statusBarItems.append(sbItem);
 }
@@ -207,9 +245,9 @@ void KisStatusBar::documentMousePositionChanged(const QPointF &pos)
 
     QPoint pixelPos = m_imageView->image()->documentToImagePixelFloored(pos);
 
-    pixelPos.setX(qBound(0, pixelPos.x(), m_view->image()->width() - 1));
-    pixelPos.setY(qBound(0, pixelPos.y(), m_view->image()->height() - 1));
-    m_pointerPositionLabel->setText(QString("%1, %2").arg(pixelPos.x()).arg(pixelPos.y()));
+    pixelPos.setX(qBound(0, pixelPos.x(), m_viewManager->image()->width() - 1));
+    pixelPos.setY(qBound(0, pixelPos.y(), m_viewManager->image()->height() - 1));
+    m_pointerPositionLabel->setText(i18nc("@info mouse position (x, y)", "%1, %2", pixelPos.x(), pixelPos.y()));
 }
 
 void KisStatusBar::imageSizeChanged()
@@ -221,7 +259,7 @@ void KisStatusBar::imageSizeChanged()
     if (image) {
         qint32 w = image->width();
         qint32 h = image->height();
-        sizeText = QString("%1 x %2 (%3)").arg(w).arg(h).arg(m_shortMemoryTag);
+        sizeText = i18nc("@info:status width x height (file size)", "%1 &x %2 (%3)", w, h, m_shortMemoryTag);
     } else {
         sizeText = m_shortMemoryTag;
     }
@@ -234,9 +272,9 @@ void KisStatusBar::imageSizeChanged()
 void KisStatusBar::updateSelectionIcon()
 {
     QIcon icon;
-    if (!m_view->selectionManager()->displaySelection()) {
+    if (!m_viewManager->selectionManager()->displaySelection()) {
         icon = KisIconUtils::loadIcon("selection-mode_invisible");
-    } else if (m_view->selectionManager()->showSelectionAsMask()) {
+    } else if (m_viewManager->selectionManager()->showSelectionAsMask()) {
         icon = KisIconUtils::loadIcon("selection-mode_mask");
     } else /* if (!m_view->selectionManager()->showSelectionAsMask()) */ {
         icon = KisIconUtils::loadIcon("selection-mode_ants");
@@ -247,8 +285,8 @@ void KisStatusBar::updateSelectionIcon()
 void KisStatusBar::updateMemoryStatus()
 {
     KisMemoryStatisticsServer::Statistics stats =
-        KisMemoryStatisticsServer::instance()
-        ->fetchMemoryStatistics(m_imageView ? m_imageView->image() : 0);
+            KisMemoryStatisticsServer::instance()
+            ->fetchMemoryStatistics(m_imageView ? m_imageView->image() : 0);
     const KFormat format;
 
     const QString imageStatsMsg =
@@ -289,20 +327,31 @@ void KisStatusBar::updateMemoryStatus()
     const qint64 warnLevel = stats.tilesHardLimit - stats.tilesHardLimit / 8;
 
     if (stats.imageSize > warnLevel ||
-        stats.realMemorySize > warnLevel) {
+            stats.realMemorySize > warnLevel) {
 
-        icon = KisIconUtils::loadIcon("dialog-warning");
+        if (!m_memoryWarningLogged) {
+            m_memoryWarningLogged = true;
+            KisUsageLogger::log(QString("WARNING: %1 is running out of memory:%2\n").arg(m_imageView->document()->url().toLocalFile()).arg(longStats));
+        }
+
+        icon = KisIconUtils::loadIcon("warning");
         QString suffix =
-            i18nc("tooltip on statusbar memory reporting button",
-                  "\n\nWARNING:\tOut of memory! Swapping has been started.\n"
-                  "\t\tPlease configure more RAM for Krita in Settings dialog");
+                i18nc("tooltip on statusbar memory reporting button",
+                      "\n\nWARNING:\tOut of memory! Swapping has been started.\n"
+                      "\t\tPlease configure more RAM for Krita in Settings dialog");
         longStats += suffix;
+
+
     }
 
     m_shortMemoryTag = shortStats;
     m_longMemoryTag = longStats;
     m_memoryStatusIcon = icon;
 
+    m_memoryReportBox->setMaximumMemory(stats.totalMemoryLimit);
+    m_memoryReportBox->setCurrentMemory(stats.totalMemorySize);
+    m_memoryReportBox->setImageWeight(stats.imageSize);
+
     emit memoryStatusUpdated();
 }
 
@@ -315,22 +364,22 @@ void KisStatusBar::updateSelectionToolTip()
 {
     updateSelectionIcon();
 
-    KisSelectionSP selection = m_view->selection();
+    KisSelectionSP selection = m_viewManager->selection();
     if (selection) {
         m_selectionStatus->setEnabled(true);
 
         QRect r = selection->selectedExactRect();
 
         QString displayMode =
-            !m_view->selectionManager()->displaySelection() ?
-            i18n("Hidden") :
-            (m_view->selectionManager()->showSelectionAsMask() ?
-             i18n("Mask") : i18n("Ants"));
+                !m_viewManager->selectionManager()->displaySelection() ?
+                    i18n("Hidden") :
+                    (m_viewManager->selectionManager()->showSelectionAsMask() ?
+                         i18n("Mask") : i18n("Ants"));
 
         m_selectionStatus->setToolTip(
-            i18n("Selection: x = %1 y = %2 width = %3 height = %4\n"
-                 "Display Mode: %5",
-                 r.x(), r.y(), r.width(), r.height(), displayMode));
+                    i18n("Selection: x = %1 y = %2 width = %3 height = %4\n"
+                         "Display Mode: %5",
+                         r.x(), r.y(), r.width(), r.height(), displayMode));
     } else {
         m_selectionStatus->setEnabled(false);
         m_selectionStatus->setToolTip(i18n("No Selection"));
@@ -350,11 +399,10 @@ void KisStatusBar::setProfile(KisImageWSP image)
     }
 
     if (!image) return;
-
     if (image->profile() == 0) {
         m_statusBarProfileLabel->setText(i18n("No profile"));
     } else {
-        m_statusBarProfileLabel->setText(image->colorSpace()->name() + "  " + image->profile()->name());
+        m_statusBarProfileLabel->setText(i18nc("<color space> <image profile>", "%1  %2", image->colorSpace()->name(), image->profile()->name()));
     }
 
 }
