@@ -84,6 +84,7 @@ KisZoomManager::KisZoomManager(QPointer<KisView> view, KoZoomHandler * zoomHandl
         , m_physicalDpiX(72.0)
         , m_physicalDpiY(72.0)
         , m_devicePixelRatio(1.0)
+        , m_guiUpdateCompressor(80, KisSignalCompressor::FIRST_ACTIVE)
 {
 }
 
@@ -189,6 +190,8 @@ void KisZoomManager::setup(KActionCollection * actionCollection)
             this, SLOT(changeAspectMode(bool)));
 
     applyRulersUnit(m_view->document()->unit());
+
+    connect(&m_guiUpdateCompressor, SIGNAL(timeout()), SLOT(slotUpdateGuiAfterZoomChange()));
 }
 
 void KisZoomManager::updateImageBoundsSnapping()
@@ -221,6 +224,14 @@ void KisZoomManager::updateImageBoundsSnapping()
     }
 }
 
+void KisZoomManager::updateCurrentZoomResource()
+{
+    const qreal effectiveZoom =
+        m_view->canvasBase()->coordinatesConverter()->effectiveZoom();
+
+    m_view->canvasBase()->resourceManager()->setResource(KisCanvasResourceProvider::EffectiveZoom, effectiveZoom);
+}
+
 void KisZoomManager::updateMouseTrackingConnections()
 {
     bool value = m_horizontalRuler->isVisible() &&
@@ -256,6 +267,18 @@ qreal KisZoomManager::zoom() const
     return zoomX;
 }
 
+qreal KisZoomManager::resolutionX() const
+{
+    KisImageSP image = m_view->image();
+    return m_aspectMode ? image->xRes() / m_devicePixelRatio : POINT_TO_INCH(m_physicalDpiX);
+}
+
+qreal KisZoomManager::resolutionY() const
+{
+    KisImageSP image = m_view->image();
+    return m_aspectMode ? image->yRes() / m_devicePixelRatio : POINT_TO_INCH(m_physicalDpiY);
+}
+
 void KisZoomManager::mousePositionChanged(const QPoint &viewPos)
 {
     QPoint pt = viewPos - m_rulersOffset;
@@ -298,6 +321,28 @@ void KisZoomManager::setRulersPixelMultiple2(bool enabled)
     }
 }
 
+void KisZoomManager::slotUpdateGuiAfterZoomChange()
+{
+    const qreal effectiveZoom =
+        m_view->canvasBase()->coordinatesConverter()->effectiveZoom();
+
+    const qreal humanZoom = effectiveZoom * 100.0;
+
+    // XXX: KOMVC -- this is very irritating in MDI mode
+
+    if (m_view->viewManager()) {
+        m_view->viewManager()->
+                showFloatingMessage(
+                    i18nc("floating message about zoom", "Zoom: %1 %",
+                          KritaUtils::prettyFormatReal(humanZoom)),
+                    QIcon(), 500, KisFloatingMessage::Low, Qt::AlignCenter);
+    }
+
+    updateCurrentZoomResource();
+
+    m_view->canvasBase()->resourceManager()->setResource(KisCanvasResourceProvider::EffectiveZoom, effectiveZoom);
+}
+
 void KisZoomManager::setMinMaxZoom()
 {
     KisImageWSP image = m_view->image();
@@ -312,7 +357,7 @@ void KisZoomManager::setMinMaxZoom()
 
 }
 
-void KisZoomManager::updateGUI()
+void KisZoomManager::updateGuiAfterDocumentSize()
 {
     QRectF widgetRect = m_view->canvasBase()->coordinatesConverter()->imageRectInWidgetPixels();
     QSize documentSize = m_view->canvasBase()->viewConverter()->viewToDocument(widgetRect).toAlignedRect().size();
@@ -333,34 +378,18 @@ void KisZoomManager::slotZoomChanged(KoZoomMode::Mode mode, qreal zoom)
     Q_UNUSED(mode);
     Q_UNUSED(zoom);
     m_view->canvasBase()->notifyZoomChanged();
-
-    qreal humanZoom = zoom * 100.0;
-
-// XXX: KOMVC -- this is very irritating in MDI mode
-
-    if (m_view->viewManager()) {
-        m_view->viewManager()->
-                showFloatingMessage(
-                    i18nc("floating message about zoom", "Zoom: %1 %",
-                          KritaUtils::prettyFormatReal(humanZoom)),
-                    QIcon(), 500, KisFloatingMessage::Low, Qt::AlignCenter);
-    }
-
-    const qreal effectiveZoom =
-        m_view->canvasBase()->coordinatesConverter()->effectiveZoom();
-
-    m_view->canvasBase()->resourceManager()->setResource(KisCanvasResourceProvider::EffectiveZoom, effectiveZoom);
+    m_guiUpdateCompressor.start();
 }
 
 void KisZoomManager::slotScrollAreaSizeChanged()
 {
     pageOffsetChanged();
-    updateGUI();
+    updateGuiAfterDocumentSize();
 }
 
 void KisZoomManager::changeAspectMode(bool aspectMode)
 {
-    KisImageWSP image = m_view->image();
+    KisImageSP image = m_view->image();
 
     // changeAspectMode is called with the same aspectMode when the window is
     // moved across screens. Preserve the old zoomMode if this is the case.
@@ -368,14 +397,8 @@ void KisZoomManager::changeAspectMode(bool aspectMode)
             aspectMode == m_aspectMode ? m_zoomHandler->zoomMode() : KoZoomMode::ZOOM_CONSTANT;
     const qreal newZoom = m_zoomHandler->zoom();
 
-    const qreal resolutionX =
-        aspectMode ? image->xRes() / m_devicePixelRatio : POINT_TO_INCH(m_physicalDpiX);
-
-    const qreal resolutionY =
-        aspectMode ? image->yRes() / m_devicePixelRatio : POINT_TO_INCH(m_physicalDpiY);
-
     m_aspectMode = aspectMode;
-    m_zoomController->setZoom(newMode, newZoom, resolutionX, resolutionY);
+    m_zoomController->setZoom(newMode, newZoom, resolutionX(), resolutionY());
     m_view->canvasBase()->notifyZoomChanged();
 }
 
