@@ -18,7 +18,13 @@
 
 #include "opengl/kis_opengl_image_textures.h"
 
+#ifdef HAS_ONLY_OPENGL_ES
+#include <qopengl.h>
+#endif
+
+#ifndef HAS_ONLY_OPENGL_ES
 #include <QOpenGLFunctions>
+#endif
 #include <QOpenGLContext>
 
 #include <QMessageBox>
@@ -62,7 +68,6 @@ KisOpenGLImageTextures::KisOpenGLImageTextures()
     : m_image(0)
     , m_monitorProfile(0)
     , m_internalColorManagementActive(true)
-    , m_checkerTexture(0)
     , m_glFuncs(0)
     , m_useOcio(false)
     , m_initialized(false)
@@ -85,7 +90,6 @@ KisOpenGLImageTextures::KisOpenGLImageTextures(KisImageWSP image,
     , m_renderingIntent(renderingIntent)
     , m_conversionFlags(conversionFlags)
     , m_internalColorManagementActive(true)
-    , m_checkerTexture(0)
     , m_glFuncs(0)
     , m_useOcio(false)
     , m_initialized(false)
@@ -108,7 +112,8 @@ void KisOpenGLImageTextures::initGL(QOpenGLFunctions *f)
     static KisTextureTileInfoPoolRegistry s_poolRegistry;
     m_updateInfoBuilder.setTextureInfoPool(s_poolRegistry.getPool(m_texturesInfo.width, m_texturesInfo.height));
 
-    m_glFuncs->glGenTextures(1, &m_checkerTexture);
+    m_checkerTexture = GLuint();
+    m_glFuncs->glGenTextures(1, &(*m_checkerTexture));
     recreateImageTextureTiles();
 
     KisOpenGLUpdateInfoSP info = updateCache(m_image->bounds(), m_image);
@@ -127,7 +132,9 @@ KisOpenGLImageTextures::~KisOpenGLImageTextures()
     }
 
     destroyImageTextureTiles();
-    m_glFuncs->glDeleteTextures(1, &m_checkerTexture);
+    if (m_checkerTexture) {
+        m_glFuncs->glDeleteTextures(1, &(*m_checkerTexture));
+    }
 }
 
 KisImageSP KisOpenGLImageTextures::image() const
@@ -140,8 +147,8 @@ bool KisOpenGLImageTextures::imageCanShareTextures()
     KisConfig cfg(true);
     if (cfg.useOcio()) return false;
     if (KisPart::instance()->mainwindowCount() == 1) return true;
-    if (qApp->desktop()->screenCount() == 1) return true;
-    for (int i = 1; i < qApp->desktop()->screenCount(); i++) {
+    if (QGuiApplication::screens().count() == 1) return true;
+    for (int i = 1; i < QGuiApplication::screens().count(); i++) {
         if (cfg.displayProfile(i) != cfg.displayProfile(i - 1)) {
             return false;
         }
@@ -332,10 +339,11 @@ void KisOpenGLImageTextures::generateCheckerTexture(const QImage &checkImage)
 GLuint KisOpenGLImageTextures::checkerTexture()
 {
     if (m_glFuncs) {
-        if (m_checkerTexture == 0) {
-            m_glFuncs->glGenTextures(1, &m_checkerTexture);
+        if (!m_checkerTexture) {
+            m_checkerTexture = GLuint();
+            m_glFuncs->glGenTextures(1, &(*m_checkerTexture));
         }
-        return m_checkerTexture;
+        return *m_checkerTexture;
     }
     else {
         dbgUI << "Tried to access checker texture before OpenGL was initialized";
@@ -353,6 +361,22 @@ void KisOpenGLImageTextures::updateConfig(bool useBuffer, int NumMipmapLevels)
     }
 }
 
+void KisOpenGLImageTextures::testingForceInitialized()
+{
+    m_initialized = true;
+    m_updateInfoBuilder.setTextureInfoPool(toQShared(new KisTextureTileInfoPool(256, 256)));
+
+    ConversionOptions options;
+    options.m_destinationColorSpace = KoColorSpaceRegistry::instance()->rgb8();
+    options.m_conversionFlags = KoColorConversionTransformation::internalConversionFlags();
+    options.m_renderingIntent = KoColorConversionTransformation::internalRenderingIntent();
+    options.m_needsConversion = false;
+    m_updateInfoBuilder.setConversionOptions(options);
+
+    m_updateInfoBuilder.setTextureBorder(4);
+    m_updateInfoBuilder.setEffectiveTextureSize(QSize(248, 248));
+}
+
 void KisOpenGLImageTextures::slotImageSizeChanged(qint32 /*w*/, qint32 /*h*/)
 {
     recreateImageTextureTiles();
@@ -477,12 +501,18 @@ void initializeRGBA16FTextures(QOpenGLContext *ctx, KisGLTexturesInfo &texturesI
         texturesInfo.internalFormat = GL_RGBA16F;
         dbgUI << "Using half (GLES or GL3)";
     } else if (ctx->hasExtension("GL_ARB_texture_float")) {
+#ifndef HAS_ONLY_OPENGL_ES
         texturesInfo.internalFormat = GL_RGBA16F_ARB;
         dbgUI << "Using ARB half";
     }
     else if (ctx->hasExtension("GL_ATI_texture_float")) {
         texturesInfo.internalFormat = GL_RGBA_FLOAT16_ATI;
         dbgUI << "Using ATI half";
+#else
+        KIS_ASSERT_X(false, "initializeRGBA16FTextures",
+                "Unexpected KisOpenGL::hasOpenGLES and \
+                 KisOpenGL::hasOpenGL3 returned false");
+#endif
     }
 
     bool haveBuiltInOpenExr = false;
@@ -495,6 +525,7 @@ void initializeRGBA16FTextures(QOpenGLContext *ctx, KisGLTexturesInfo &texturesI
         destinationColorDepthId = Float16BitsColorDepthID;
         dbgUI << "Pixel type half (GLES or GL3)";
     } else if (haveBuiltInOpenExr && ctx->hasExtension("GL_ARB_half_float_pixel")) {
+#ifndef HAS_ONLY_OPENGL_ES
         texturesInfo.type = GL_HALF_FLOAT_ARB;
         destinationColorDepthId = Float16BitsColorDepthID;
         dbgUI << "Pixel type half";
@@ -502,6 +533,11 @@ void initializeRGBA16FTextures(QOpenGLContext *ctx, KisGLTexturesInfo &texturesI
         texturesInfo.type = GL_FLOAT;
         destinationColorDepthId = Float32BitsColorDepthID;
         dbgUI << "Pixel type float";
+#else
+        KIS_ASSERT_X(false, "KisOpenGLCanvas2::paintToolOutline",
+                "Unexpected KisOpenGL::hasOpenGLES and \
+                 KisOpenGL::hasOpenGL3 returned false");
+#endif
     }
     texturesInfo.format = GL_RGBA;
 }
@@ -513,10 +549,20 @@ void KisOpenGLImageTextures::updateTextureFormat()
     if (!(m_image && ctx)) return;
 
     if (!KisOpenGL::hasOpenGLES()) {
+#ifndef HAS_ONLY_OPENGL_ES
         m_texturesInfo.internalFormat = GL_RGBA8;
         m_texturesInfo.type = GL_UNSIGNED_BYTE;
         m_texturesInfo.format = GL_BGRA;
+#else
+        KIS_ASSERT_X(false, "KisOpenGLImageTextures::updateTextureFormat",
+                "Unexpected KisOpenGL::hasOpenGLES returned false");
+#endif
     } else {
+#ifdef HAS_ONLY_OPENGL_ES
+        m_texturesInfo.internalFormat = GL_RGBA8;
+        m_texturesInfo.type = GL_UNSIGNED_BYTE;
+        m_texturesInfo.format = GL_RGBA;
+#else
         m_texturesInfo.internalFormat = GL_BGRA8_EXT;
         m_texturesInfo.type = GL_UNSIGNED_BYTE;
         m_texturesInfo.format = GL_BGRA_EXT;
@@ -527,6 +573,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
             m_texturesInfo.type = GL_UNSIGNED_BYTE;
             m_texturesInfo.format = GL_RGBA;
         }
+#endif
     }
 
     const bool useHDRMode = KisOpenGLModeProber::instance()->useHDRMode();
@@ -547,11 +594,17 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 m_texturesInfo.internalFormat = GL_RGBA32F;
                 dbgUI << "Using float (GLES or GL3)";
             } else if (ctx->hasExtension("GL_ARB_texture_float")) {
+#ifndef HAS_ONLY_OPENGL_ES
                 m_texturesInfo.internalFormat = GL_RGBA32F_ARB;
                 dbgUI << "Using ARB float";
             } else if (ctx->hasExtension("GL_ATI_texture_float")) {
                 m_texturesInfo.internalFormat = GL_RGBA_FLOAT32_ATI;
                 dbgUI << "Using ATI float";
+#else
+        KIS_ASSERT_X(false, "KisOpenGLCanvas2::updateTextureFormat",
+                "Unexpected KisOpenGL::hasOpenGLES and \
+                 KisOpenGL::hasOpenGL3 returned false");
+#endif
             }
 
             m_texturesInfo.type = GL_FLOAT;
@@ -559,6 +612,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
             destinationColorDepthId = Float32BitsColorDepthID;
         }
         else if (colorDepthId == Integer16BitsColorDepthID) {
+#ifndef HAS_ONLY_OPENGL_ES
             if (!KisOpenGL::hasOpenGLES()) {
                 m_texturesInfo.internalFormat = GL_RGBA16;
                 m_texturesInfo.type = GL_UNSIGNED_SHORT;
@@ -566,17 +620,23 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 destinationColorDepthId = Integer16BitsColorDepthID;
                 dbgUI << "Using 16 bits rgba";
             }
+#endif
             // TODO: for ANGLE, see if we can convert to 16f to support 10-bit display
         }
     }
     else {
         // We will convert the colorspace to 16 bits rgba, instead of 8 bits
         if (colorDepthId == Integer16BitsColorDepthID && !KisOpenGL::hasOpenGLES()) {
+#ifndef HAS_ONLY_OPENGL_ES
             m_texturesInfo.internalFormat = GL_RGBA16;
             m_texturesInfo.type = GL_UNSIGNED_SHORT;
             m_texturesInfo.format = GL_BGRA;
             destinationColorDepthId = Integer16BitsColorDepthID;
             dbgUI << "Using conversion to 16 bits rgba";
+#else
+            KIS_ASSERT_X(false, "KisOpenGLCanvas2::updateTextureFormat",
+                    "Unexpected KisOpenGL::hasOpenGLES returned false");
+#endif
         } else if (colorDepthId == Float16BitsColorDepthID && KisOpenGL::hasOpenGLES()) {
             // TODO: try removing opengl es limit
             initializeRGBA16FTextures(ctx, m_texturesInfo, destinationColorDepthId);
@@ -592,10 +652,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
 
         if (cm != KisConfig::INTERNAL) {
             emit sigShowFloatingMessage(
-                i18n("You enabled OpenColorIO based color management, but your image is not an RGB image.\n"
-                     "OpenColorIO-based color management only works with RGB images.\n"
-                     "Please check the settings in the LUT docker.\n"
-                     "OpenColorIO will now be deactivated."), 5000, true);
+                i18n("OpenColorIO is disabled: image color space is not supported"), 5000, true);
         }
 
         warnUI << "WARNING: Internal color management was forcibly enabled";
