@@ -16,7 +16,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#import <AppKit/Appkit.h>
+#import <AppKit/AppKit.h>
 
 extern "C" {
     bool isMouseCoalescingEnabled();
