@@ -21,6 +21,8 @@
 
 #include <QtTest>
 
+class KisAnimationFrameCache;
+
 class KisAnimationFrameCacheTest : public QObject
 {
     Q_OBJECT
@@ -28,6 +30,11 @@ class KisAnimationFrameCacheTest : public QObject
 private Q_SLOTS:
     void testCache();
 
+    void slotFrameGerenationFinished(int time);
+
+private:
+    KisAnimationFrameCache *m_globalAnimationCache = 0;
+
 };
 #endif
 
