@@ -55,11 +55,9 @@ void KisFileLayerTest::testFileLayerPlusTransformMaskOffImage()
     waitForMaskUpdates(p.image->root());
     p.image->waitForDone();
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     p.image->addNode(mask1, flayer);
 
-    mask1->setName("mask1");
-
     flayer->setDirty(refRect);
     p.image->waitForDone();
     chk.checkImage(p.image, "00_initial_layer_update");
@@ -123,11 +121,9 @@ void KisFileLayerTest::testFileLayerPlusTransformMaskSmallFileBigOffset()
     // initialized properly
     QCOMPARE(flayer->original()->defaultBounds()->bounds(), p.image->bounds());
 
-    KisTransformMaskSP mask1 = new KisTransformMask();
+    KisTransformMaskSP mask1 = new KisTransformMask(p.image, "mask1");
     p.image->addNode(mask1, flayer);
 
-    mask1->setName("mask1");
-
     flayer->setDirty(refRect);
     p.image->waitForDone();
     chk.checkImage(p.image, "00_initial_layer_update");
