@@ -27,17 +27,12 @@
 
 #include <sdk/tests/testutil.h>
 
-KisShapeControllerTest::~KisShapeControllerTest()
-{
-}
-
 KisDummiesFacadeBase* KisShapeControllerTest::dummiesFacadeFactory()
 {
-
     m_doc = KisPart::instance()->createDocument();
 
     m_nameServer = new KisNameServer();
-    return new KisShapeController(m_doc, m_nameServer);
+    return new KisShapeController(m_nameServer, m_doc->undoStack());
 }
 
 void KisShapeControllerTest::destroyDummiesFacade(KisDummiesFacadeBase *dummiesFacade)
