@@ -32,8 +32,9 @@ KisFigurePaintingToolHelper::KisFigurePaintingToolHelper(const KUndo2MagicString
                                                          KisImageWSP image,
                                                          KisNodeSP currentNode,
                                                          KoCanvasResourceProvider *resourceManager,
-                                                         KisPainter::StrokeStyle strokeStyle,
-                                                         KisPainter::FillStyle fillStyle)
+                                                         KisToolShapeUtils::StrokeStyle strokeStyle,
+                                                         KisToolShapeUtils::FillStyle fillStyle,
+                                                         QTransform fillTransform)
 {
     m_strokesFacade = image.data();
 
@@ -42,8 +43,7 @@ KisFigurePaintingToolHelper::KisFigurePaintingToolHelper(const KUndo2MagicString
                                  currentNode,
                                  resourceManager);
 
-    m_resources->setStrokeStyle(strokeStyle);
-    m_resources->setFillStyle(fillStyle);
+    setupPaintStyles(m_resources, strokeStyle, fillStyle, fillTransform);
 
     KisFreehandStrokeInfo *strokeInfo = new KisFreehandStrokeInfo();
 
@@ -53,6 +53,56 @@ KisFigurePaintingToolHelper::KisFigurePaintingToolHelper(const KUndo2MagicString
     m_strokeId = m_strokesFacade->startStroke(stroke);
 }
 
+void KisFigurePaintingToolHelper::setupPaintStyles(KisResourcesSnapshotSP resources,
+                                                   KisToolShapeUtils::StrokeStyle strokeStyle,
+                                                   KisToolShapeUtils::FillStyle fillStyle,
+                                                   QTransform fillTransform)
+{
+    using namespace KisToolShapeUtils;
+
+    const KoColor fgColor = resources->currentFgColor();
+    const KoColor bgColor = resources->currentBgColor();
+
+    switch (strokeStyle) {
+    case StrokeStyleNone:
+        resources->setStrokeStyle(KisPainter::StrokeStyleNone);
+        break;
+    case StrokeStyleForeground:
+        resources->setStrokeStyle(KisPainter::StrokeStyleBrush);
+        break;
+    case StrokeStyleBackground:
+        resources->setStrokeStyle(KisPainter::StrokeStyleBrush);
+
+        resources->setFGColorOverride(bgColor);
+        resources->setBGColorOverride(fgColor);
+
+        if (fillStyle == FillStyleForegroundColor) {
+            fillStyle = FillStyleBackgroundColor;
+        } else if (fillStyle == FillStyleBackgroundColor) {
+            fillStyle = FillStyleForegroundColor;
+        }
+
+        break;
+    };
+
+    switch (fillStyle) {
+    case FillStyleForegroundColor:
+        resources->setFillStyle(KisPainter::FillStyleForegroundColor);
+        break;
+    case FillStyleBackgroundColor:
+        resources->setFillStyle(KisPainter::FillStyleBackgroundColor);
+        break;
+    case FillStylePattern:
+        resources->setFillStyle(KisPainter::FillStylePattern);
+        break;
+    case FillStyleNone:
+        resources->setFillStyle(KisPainter::FillStyleNone);
+        break;
+    }
+
+    resources->setFillTransform(fillTransform);
+}
+
 KisFigurePaintingToolHelper::~KisFigurePaintingToolHelper()
 {
     m_strokesFacade->addJob(m_strokeId,
