@@ -37,19 +37,20 @@ public:
     void connectMaskSignals(KisCanvasResourceProvider* resourceProvider);
 
 Q_SIGNALS:
-    void sigGamutMaskToggle(bool state);
     void sigGamutMaskChanged(KoGamutMask*);
+    void sigGamutMaskDeactivated();
 
 public Q_SLOTS:
     void slotGamutMaskSet(KoGamutMask* mask);
     void slotGamutMaskUnset();
+    void slotGamutMaskDeactivate();
 
 private Q_SLOTS:
     void slotGamutMaskToggle(bool state);
     void slotGamutMaskRotate(int angle);
 
 private:
-    Ui_wdgGamutMaskToolbar* m_ui;
+    QScopedPointer<Ui_wdgGamutMaskToolbar> m_ui;
     KoGamutMask* m_selectedMask;
 
     QIcon m_iconMaskOff;
@@ -57,6 +58,8 @@ private:
 
     QString m_textNoMask;
     QString m_textMaskDisabled;
+
+    bool m_selfUpdate;
 };
 
 #endif // KISGAMUTMASKTOOLBAR_H
