@@ -25,17 +25,17 @@
 #include "kis_floating_message.h"
 
 #include <QApplication>
-#include <QDesktopWidget>
+#include <QScreen>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QTimer>
 #include <QRegExp>
+#include <QDesktopWidget>
 
 #include <kis_icon.h>
 #include <kis_debug.h>
 #include "kis_global.h"
 
-
 /* Code copied from kshadowengine.cpp
  *
  * Copyright (C) 2003 Laur Ivan <laurivan@eircom.net>
@@ -128,6 +128,7 @@ KisFloatingMessage::KisFloatingMessage(const QString &message, QWidget *parent,
     , m_timeout(timeout)
     , m_priority(priority)
     , m_alignment(alignment)
+    , widgetQueuedForDeletion(false)
 {
     m_icon = KisIconUtils::loadIcon("krita").pixmap(256, 256).toImage();
 
@@ -139,6 +140,7 @@ KisFloatingMessage::KisFloatingMessage(const QString &message, QWidget *parent,
 
     m_timer.setSingleShot( true );
     connect(&m_timer, SIGNAL(timeout()), SLOT(startFade()));
+    connect(this, SIGNAL(destroyed()), SLOT(widgetDeleted()));
 }
 
 void KisFloatingMessage::tryOverrideMessage(const QString message,
@@ -160,8 +162,12 @@ void KisFloatingMessage::tryOverrideMessage(const QString message,
 
 void KisFloatingMessage::showMessage()
 {
-
+    if (widgetQueuedForDeletion) return;
+#if QT_VERSION >= QT_VERSION_CHECK(5,13,0)
+    setGeometry(determineMetrics(fontMetrics().horizontalAdvance('x')));
+#else
     setGeometry(determineMetrics(fontMetrics().width('x')));
+#endif
     setWindowOpacity(OSD_WINDOW_OPACITY);
 
     QWidget::setVisible(true);
@@ -189,7 +195,21 @@ QRect KisFloatingMessage::determineMetrics( const int M )
     // determine a sensible maximum size, don't cover the whole desktop or cross the screen
     const QSize margin( (M + MARGIN) * 2, (M + MARGIN) * 2); //margins
     const QSize image = m_icon.isNull() ? QSize(0, 0) : minImageSize;
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+    QRect geom = parentWidget()->geometry();
+    QPoint p(geom.width() / 2 + geom.left(), geom.height() / 2 + geom.top());
+    QScreen *s = qApp->screenAt(p);
+    QSize max;
+    if (s) {
+        max = QSize(s->availableGeometry().size() - margin);
+    }
+    else {
+        max = QSize(1024, 768);
+    }
+#else
     const QSize max = QApplication::desktop()->availableGeometry(parentWidget()).size() - margin;
+#endif
+
 
     // If we don't do that, the boundingRect() might not be suitable for drawText() (Qt issue N67674)
     m_message.replace(QRegExp( " +\n"), "\n");
@@ -220,7 +240,18 @@ QRect KisFloatingMessage::determineMetrics( const int M )
 
 
     const QSize newSize = rect.size();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+    QRect screen;
+    if (s) {
+        screen = s->availableGeometry();
+    }
+    else {
+        screen = QRect(0, 0, 1024, 768);
+    }
+#else
     QRect screen = QApplication::desktop()->screenGeometry(parentWidget());
+#endif
+
 
     QPoint newPos(MARGIN, MARGIN);
 
@@ -320,6 +351,7 @@ void KisFloatingMessage::removeMessage()
 {
     m_timer.stop();
     m_fadeTimeLine.stop();
+    widgetQueuedForDeletion = true;
 
     hide();
     deleteLater();
@@ -329,3 +361,8 @@ void KisFloatingMessage::updateOpacity(int /*value*/)
 {
     setWindowOpacity(OSD_WINDOW_OPACITY - 0.1);
 }
+
+void KisFloatingMessage::widgetDeleted()
+{
+    widgetQueuedForDeletion = false;
+}
