@@ -19,6 +19,7 @@
 #define KIS_GRADIENT_CHOOSER_H_
 
 #include <KoDialog.h>
+#include <KoColor.h>
 
 #include <QFrame>
 #include <QToolButton>
@@ -40,11 +41,11 @@ class KisCustomGradientDialog : public KoDialog
 
 public:
 
-    KisCustomGradientDialog(KoAbstractGradient* gradient, QWidget *parent, const char *name);
+    KisCustomGradientDialog(KoAbstractGradient* gradient, QWidget *parent, const char *name, const KoColor &fgColor, const KoColor &bgColor);
 
 private:
 
-    QWidget * m_page;
+    QWidget *m_page {0};
 
 };
 
@@ -70,6 +71,8 @@ Q_SIGNALS:
 
 public Q_SLOTS:
     void slotUpdateIcons();
+    void setForegroundColor(KoColor);
+    void setBackgroundColor(KoColor);
 
 private Q_SLOTS:
     virtual void update(KoResource * resource);
@@ -83,6 +86,8 @@ private:
     QLabel *m_lbName;
     KoResourceItemChooser * m_itemChooser;
 
+    KoColor m_foregroundColor, m_backgroundColor;
+
     QToolButton* m_addGradient;
     QPushButton* m_editGradient;
 };
