@@ -40,10 +40,10 @@ public:
     /**
      * Constructs a KoResourcePopupAction (gradient or pattern) with the specified parent.
      *
-     * @param gradientResourceAdapter pointer to the gradient or pattern
+     * @param resourceAdapter pointer to the gradient or pattern
      * @param parent The parent for this action.
      */
-    explicit KoResourcePopupAction(QSharedPointer<KoAbstractResourceServerAdapter>gradientResourceAdapter, QObject *parent = 0);
+    explicit KoResourcePopupAction(QSharedPointer<KoAbstractResourceServerAdapter>resourceAdapter, QObject *parent = 0);
 
     /**
      * Destructor
@@ -56,6 +56,8 @@ public:
     void setCurrentResource(KoResource *resource);
     KoResource *currentResource() const;
 
+    void keepAspectRatio(bool keepRatio);
+
 Q_SIGNALS:
     /// Emitted when a resource was selected
     void resourceSelected(QSharedPointer<KoShapeBackground>  background);
