@@ -47,31 +47,25 @@ public:
 KoTagFilterWidget::KoTagFilterWidget(QWidget* parent): QWidget(parent)
 ,d( new Private())
 {
-    d->tagSearchBarTooltip_saving_disabled = i18nc (
+    QString searchTooltipMaintext = i18nc(
             "@info:tooltip",
-            "Entering search terms here will add to, or remove resources from the current tag view."
+            "<p>Enter search terms here to add resources to, or remove them from, the current tag view.</p>"
             "<p>To filter based on the partial, case insensitive name of a resource:<br/>"
-            "<tt>partialname</tt> or <tt>!partialname</tt>.</p>"
-            "<p>In-/exclusion of other tag sets:<br/>"
-            "<tt>[Tagname]</tt> or <tt>![Tagname]</tt>.</p>"
-            "<p>Case sensitive and full name matching in-/exclusion:<br/>"
-            "<tt>\"ExactMatch\"</tt> or <tt>!\"ExactMatch\"</tt>.</p>"
-            "Filter results cannot be saved for the <b>All Presets</b> view.<br/>"
-            "In this view, pressing <b>Enter</b> or clearing the filter box will restore all items.<br/>"
-            "Create and/or switch to a different tag if you want to save filtered resources into named sets."
-            );
+            "<tt>partialname</tt> or <tt>!partialname</tt></p>"
+            "<p>To include or exclude other tag sets:<br/>"
+            "<tt>[Tagname]</tt> or <tt>![Tagname]</tt></p>"
+            "<p>For case sensitive and full name matching in-/exclusion:<br/>"
+            "<tt>\"ExactMatch\"</tt> or <tt>!\"ExactMatch\"</tt></p>");
 
-    d->tagSearchBarTooltip_saving_enabled = i18nc (
+    d->tagSearchBarTooltip_saving_disabled = searchTooltipMaintext + i18nc(
             "@info:tooltip",
-            "Entering search terms here will add to, or remove resources from the current tag view."
-            "<p>To filter based on the partial, case insensitive name of a resource:<br/>"
-            "<tt>partialname</tt> or <tt>!partialname</tt>.</p>"
-            "<p>In-/exclusion of other tag sets:<br/>"
-            "<tt>[Tagname]</tt> or <tt>![Tagname]</tt>.</p>"
-            "<p>Case sensitive and full name matching in-/exclusion:<br/>"
-            "<tt>\"ExactMatch\"</tt> or <tt>!\"ExactMatch\"</tt>.</p>"
-            "Pressing <b>Enter</b> or clicking the <b>Save</b> button will save the changes."
-            );
+            "<p>Filter results cannot be saved for the <b>All Presets</b> view. "
+            "In this view, pressing <b>Enter</b> or clearing the filter box will restore all items. "
+            "Create and/or switch to a different tag if you want to save filtered resources into named sets.</p>");
+
+    d->tagSearchBarTooltip_saving_enabled = searchTooltipMaintext + i18nc(
+            "@info:tooltip",
+            "<p>Pressing <b>Enter</b> or clicking the <b>Save</b> button will save the changes.</p>");
 
     QGridLayout* filterBarLayout = new QGridLayout;
 
@@ -127,11 +121,6 @@ void KoTagFilterWidget::clear()
     d->tagSearchSaveButton->setEnabled(false);
 }
 
-QLineEdit *KoTagFilterWidget::searchField()
-{
-    return d->tagSearchLineEdit;
-}
-
 void KoTagFilterWidget::onTextChanged(const QString& lineEditText)
 {
     d->tagSearchSaveButton->setEnabled(!lineEditText.isEmpty());
