@@ -60,6 +60,7 @@ public:
     QString caption;
     QString defaultDirectory;
     QString proposedFileName;
+    QUrl defaultUri;
     QStringList filterList;
     QString defaultFilter;
     QScopedPointer<QFileDialog> fileDialog;
@@ -102,6 +103,11 @@ void KoFileDialog::setDefaultDir(const QString &defaultDir, bool force)
     }
 }
 
+void KoFileDialog::setDirectoryUrl(const QUrl &defaultUri)
+{
+    d->defaultUri = defaultUri;
+}
+
 void KoFileDialog::setImageFilters()
 {
     QStringList imageFilters;
@@ -147,9 +153,15 @@ QString KoFileDialog::selectedMimeType() const
 void KoFileDialog::createFileDialog()
 {
     d->fileDialog.reset(new QFileDialog(d->parent, d->caption, d->defaultDirectory + "/" + d->proposedFileName));
+    if (!d->defaultUri.isEmpty()) {
+        d->fileDialog->setDirectoryUrl(d->defaultUri);
+    }
     KConfigGroup group = KSharedConfig::openConfig()->group("File Dialogs");
 
     bool dontUseNative = true;
+#ifdef Q_OS_ANDROID
+    dontUseNative = false;
+#endif
 #ifdef Q_OS_UNIX
     if (qgetenv("XDG_CURRENT_DESKTOP") == "KDE") {
         dontUseNative = false;
@@ -177,6 +189,11 @@ void KoFileDialog::createFileDialog()
     if (d->type == SaveFile) {
         d->fileDialog->setAcceptMode(QFileDialog::AcceptSave);
         d->fileDialog->setFileMode(QFileDialog::AnyFile);
+
+#ifdef Q_OS_ANDROID
+        // HACK: discovered by looking into the code
+        d->fileDialog->setWindowTitle(d->proposedFileName.isEmpty() ? "Untitled.kra" : d->proposedFileName);
+#endif
     }
     else { // open / import
 
@@ -198,6 +215,7 @@ void KoFileDialog::createFileDialog()
         }
     }
 
+#ifndef Q_OS_ANDROID
     d->fileDialog->setNameFilters(d->filterList);
 
     if (!d->proposedFileName.isEmpty()) {
@@ -214,6 +232,7 @@ void KoFileDialog::createFileDialog()
     else if (!d->defaultFilter.isEmpty()) {
         d->fileDialog->selectNameFilter(d->defaultFilter);
     }
+#endif
 
     if (d->type == ImportDirectory ||
             d->type == ImportFile || d->type == ImportFiles ||
@@ -260,7 +279,10 @@ QString KoFileDialog::filename()
             if (!(extension.contains(".") || url.endsWith("."))) {
                 extension = "." + extension;
             }
+// We can only write to the Uri that was returned, we don't have permission to change the Uri.
+#ifndef Q_OS_ANDROID
             url = url + extension;
+#endif
         }
 
         d->mimeType = KisMimeDatabase::mimeTypeForFile(url, d->type == KoFileDialog::SaveFile ? false : true);
