@@ -61,22 +61,24 @@ public:
                  KoFileDialog::DialogType type,
                  const QString &dialogName);
 
-    ~KoFileDialog();
+    ~KoFileDialog() override;
 
     void setCaption(const QString &caption);
 
     /**
-     * @brief setDefaultDir set the default directory to defaultDir
+     * @brief setDefaultDir set the default directory to defaultDir.
      *
      * @param defaultDir a path to a file or directory
      */
-    void setDefaultDir(const QString &defaultDir, bool override = false);
+    void setDefaultDir(const QString &defaultDir, bool force = false);
 
     /**
-     * @brief setOverrideDir override both the default dir and the saved dir found by dialogName
-     * @param overrideDir a path to a file or directory
+     * @brief setDirectoryUrl set the default URI to defaultUri.
+     * @param defaultUri a Uri to a file from some ContentProvider
+     *
+     * Used only on Android.
      */
-    void setOverrideDir(const QString &overrideDir);
+    void setDirectoryUrl(const QUrl &defaultUri);
 
     /**
      * @brief setImageFilters sets the name filters for the file dialog to all
@@ -84,10 +86,13 @@ public:
      */
     void setImageFilters();
 
-    void setMimeTypeFilters(const QStringList &filterList,
-                            QString defaultFilter = QString());
-
-    void setHideNameFilterDetailsOption();
+    /**
+     * @brief setMimeTypeFilters Update the list of file filters from mime types.
+     * @param mimeTypeList A list of mime types that forms the basis of this dialog's file filters
+     * @param defaultMimeType Sets the default filter based on this mime type
+     */
+    void setMimeTypeFilters(const QStringList &mimeTypeList,
+                            QString defaultMimeType = QString());
 
     /// Get the file names the user selected in the file dialog
     QStringList filenames();
@@ -104,10 +109,6 @@ public:
 
     QString selectedMimeType() const;
 
-private Q_SLOTS:
-
-    void filterSelected(const QString &filter);
-
 private:
     /**
      * @brief splitNameFilter take a single line of a QDialog name filter and split it
