@@ -283,7 +283,11 @@ void KRecentFilesAction::loadEntries(const KConfigGroup &_config)
     // read file list
     for (int i = 1; i <= d->m_maxItems; i++) {
         key = QString("File%1").arg(i);
+#ifdef Q_OS_ANDROID
+        value = cg.readEntry(key, QString());
+#else
         value = cg.readPathEntry(key, QString());
+#endif
         if (value.isEmpty()) {
             continue;
         }
@@ -341,11 +345,20 @@ void KRecentFilesAction::saveEntries(const KConfigGroup &_cg)
     for (int i = 1; i <= selectableActionGroup()->actions().count(); i++) {
         key = QString("File%1").arg(i);
         // i - 1 because we started from 1
+#ifdef Q_OS_ANDROID
+        value = d->m_urls[ selectableActionGroup()->actions()[ i - 1 ] ].toDisplayString();
+        cg.writeEntry(key, value);
+#else
         value = d->m_urls[ selectableActionGroup()->actions()[ i - 1 ] ].toDisplayString(QUrl::PreferLocalFile);
         cg.writePathEntry(key, value);
+#endif
         key = QString("Name%1").arg(i);
         value = d->m_shortNames[ selectableActionGroup()->actions()[ i - 1 ] ];
+#ifdef Q_OS_ANDROID
+        cg.writeEntry(key, value);
+#else
         cg.writePathEntry(key, value);
+#endif
     }
 
 }
