@@ -26,6 +26,7 @@
 #include <klocalizedstring.h>
 #include <KisShortcutsDialog.h>
 #include <KConfigGroup>
+#include <qdom.h>
 
 #include "kis_debug.h"
 #include "KoResourcePaths.h"
@@ -80,24 +81,72 @@ namespace {
         bool m_explicitlyReset = false;
     };
 
+    // Convenience macros to extract a child node.
+    QDomElement getChild(QDomElement xml, QString node) {
+        return xml.firstChildElement(node);
+    }
+
     // Convenience macros to extract text of a child node.
     QString getChildContent(QDomElement xml, QString node) {
         return xml.firstChildElement(node).text();
     }
 
+    QString getChildContentForOS(QDomElement xml, QString tagName, QString os = QString()) {
+        bool found = false;
+
+        QDomElement node = xml.firstChildElement(tagName);
+        QDomElement nodeElse;
+
+        while(!found && !node.isNull()) {
+            if (node.attribute("operatingSystem") == os) {
+                found = true;
+                break;
+            }
+            else if (node.hasAttribute("operatingSystemElse")) {
+                nodeElse = node;
+            }
+            else if (nodeElse.isNull()) {
+                nodeElse = node;
+            }
+
+            node = node.nextSiblingElement(tagName);
+        }
+
+        if (!found && !nodeElse.isNull()) {
+            return nodeElse.text();
+        }
+        return node.text();
+    }
+
     // Use Krita debug logging categories instead of KDE's default qDebug() for
     // harmless empty strings and translations
-    QString quietlyTranslate(const QString &s) {
-        if (s.isEmpty()) {
-            return s;
+    QString quietlyTranslate(const QDomElement &s) {
+        if (s.isNull() || s.text().isEmpty()) {
+            return QString();
         }
-        QString translatedString = i18nc("action", s.toUtf8());
-        if (translatedString == s) {
-            translatedString = i18n(s.toUtf8());
+        QString translatedString;
+        const QString attrContext = QStringLiteral("context");
+        const QString attrDomain = QStringLiteral("translationDomain");
+        QString context = QStringLiteral("action");
+
+        if (!s.attribute(attrContext).isEmpty()) {
+            context = s.attribute(attrContext);
+        }
+
+        QByteArray domain = s.attribute(attrDomain).toUtf8();
+        if (domain.isEmpty()) {
+            domain = s.ownerDocument().documentElement().attribute(attrDomain).toUtf8();
+            if (domain.isEmpty()) {
+                domain = KLocalizedString::applicationDomain();
+            }
+        }
+        translatedString = i18ndc(domain.constData(), context.toUtf8().constData(), s.text().toUtf8().constData());
+        if (translatedString == s.text()) {
+            translatedString = i18n(s.text().toUtf8().constData());
         }
         if (translatedString.isEmpty()) {
-            dbgAction << "No translation found for" << s;
-            return s;
+            dbgAction << "No translation found for" << s.text();
+            return s.text();
         }
 
         return translatedString;
@@ -155,6 +204,10 @@ KisActionRegistry::KisActionRegistry()
     loadCustomShortcuts();
 }
 
+KisActionRegistry::~KisActionRegistry()
+{
+}
+
 KisActionRegistry::ActionCategory KisActionRegistry::fetchActionCategory(const QString &name) const
 {
     if (!d->actionInfoList.contains(name)) return ActionCategory();
@@ -264,7 +317,7 @@ bool KisActionRegistry::propertizeAction(const QString &name, QAction * a)
     QDomElement actionXml = info.xmlData;
     if (!actionXml.text().isEmpty()) {
         // i18n requires converting format from QString.
-        auto getChildContent_i18n = [=](QString node){return quietlyTranslate(getChildContent(actionXml, node));};
+        auto getChildContent_i18n = [=](QString node){return quietlyTranslate(getChild(actionXml, node));};
 
         // Note: the fields in the .action documents marked for translation are determined by extractrc.
         QString icon      = getChildContent(actionXml, "icon");
@@ -315,6 +368,7 @@ void KisActionRegistry::Private::loadActionFiles()
 
     // Extract actions all XML .action files.
     Q_FOREACH (const QString &actionDefinition, actionDefinitions)  {
+
         QDomDocument doc;
         QFile f(actionDefinition);
         f.open(QFile::ReadOnly);
@@ -324,7 +378,7 @@ void KisActionRegistry::Private::loadActionFiles()
         QString collectionName = base.attribute("name");
         QString version        = base.attribute("version");
         if (version != "2") {
-            errAction << ".action XML file" << actionDefinition << "has incorrect version; skipping.";
+            qDebug() << ".action XML file" << actionDefinition << "has incorrect version; skipping.";
             continue;
         }
 
@@ -335,11 +389,15 @@ void KisActionRegistry::Private::loadActionFiles()
 
             // <text> field
             QDomElement categoryTextNode = actions.firstChild().toElement();
-            QString categoryName         = quietlyTranslate(categoryTextNode.text());
+            QString categoryName         = quietlyTranslate(categoryTextNode);
 
             // <action></action> tags
             QDomElement actionXml  = categoryTextNode.nextSiblingElement();
 
+            if (actionXml.isNull()) {
+                qWarning() << actionDefinition << "does not contain any valid actios! (Or the text element was left empty...)";
+            }
+
             // Loop over individual actions
             while (!actionXml.isNull()) {
                 if (actionXml.tagName() == "Action") {
@@ -348,7 +406,7 @@ void KisActionRegistry::Private::loadActionFiles()
 
                     // Bad things
                     if (name.isEmpty()) {
-                        errAction << "Unnamed action in definitions file " << actionDefinition;
+                        qWarning() << "Unnamed action in definitions file " << actionDefinition;
                     }
 
                     else if (actionInfoList.contains(name)) {
@@ -360,7 +418,11 @@ void KisActionRegistry::Private::loadActionFiles()
                         info.xmlData         = actionXml;
 
                         // Use empty list to signify no shortcut
-                        QString shortcutText = getChildContent(actionXml, "shortcut");
+#ifdef Q_OS_MACOS
+                        QString shortcutText = getChildContentForOS(actionXml, "shortcut", "macos");
+#else
+                        QString shortcutText = getChildContentForOS(actionXml, "shortcut");
+#endif
                         if (!shortcutText.isEmpty()) {
                             info.setDefaultShortcuts(QKeySequence::listFromString(shortcutText));
                         }
