@@ -121,7 +121,7 @@ QIcon loadIcon(const QString &name)
 
 
 bool useDarkIcons() {
-    QColor background = qApp->palette().background().color();
+    QColor background = qApp->palette().window().color();
     return  background.value() > 100;
 }
 
