@@ -23,7 +23,7 @@
 
 #include "khelpmenu.h"
 #include "config-xmlgui.h"
-#include <QtCore/QTimer>
+#include <QTimer>
 #include <QAction>
 #include <QApplication>
 #include <QDialogButtonBox>
@@ -45,7 +45,6 @@
 #include "kswitchlanguagedialog_p.h"
 
 #include <kaboutdata.h>
-#include <kauthorized.h>
 #include <klocalizedstring.h>
 #include <kstandardaction.h>
 
