@@ -35,9 +35,9 @@
 #include "ktoolbar.h"
 
 #include <QApplication>
-#include <QtCore/QList>
-#include <QtCore/QObject>
-#include <QtCore/QTimer>
+#include <QList>
+#include <QObject>
+#include <QTimer>
 #include <QCloseEvent>
 #include <QDesktopWidget>
 #include <QDockWidget>
@@ -86,8 +86,8 @@ class DockResizeListener : public QObject
 {
 public:
     DockResizeListener(KMainWindow *win);
-    virtual ~DockResizeListener();
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    ~DockResizeListener() override;
+    bool eventFilter(QObject *watched, QEvent *event) override;
 
 private:
     KMainWindow *m_win;
@@ -182,11 +182,6 @@ KMainWindow::KMainWindow(KMainWindowPrivate &dd, QWidget *parent, Qt::WindowFlag
 
 void KMainWindowPrivate::init(KMainWindow *_q)
 {
-    // We set allow quit to true when the first mainwindow is created, so that when the refcounting
-    // reaches 0 the application can quit. We don't want this to happen before the first mainwindow
-    // is created, otherwise running a job in main would exit the app too early.
-    QCoreApplication::setQuitLockEnabled(true);
-
     q = _q;
 
     q->setAnimated(q->style()->styleHint(QStyle::SH_Widget_Animate, 0, q));
@@ -233,16 +228,16 @@ static bool endsWithHashNumber(const QString &s)
     }
     return false;
 }
-
+#ifdef HAVE_DBUS
 static inline bool isValidDBusObjectPathCharacter(const QChar &c)
 {
-    register ushort u = c.unicode();
+    ushort u = c.unicode();
     return (u >= QLatin1Char('a') && u <= QLatin1Char('z'))
            || (u >= QLatin1Char('A') && u <= QLatin1Char('Z'))
            || (u >= QLatin1Char('0') && u <= QLatin1Char('9'))
            || (u == QLatin1Char('_')) || (u == QLatin1Char('/'));
 }
-
+#endif
 void KMainWindowPrivate::polish(KMainWindow *q)
 {
     // Set a unique object name. Required by session management, window management, and for the dbus interface.
@@ -411,25 +406,6 @@ bool KMainWindow::restore(int , bool )
     return false;
 }
 
-void KMainWindow::setCaption(const QString &caption)
-{
-    setPlainCaption(caption);
-}
-
-void KMainWindow::setCaption(const QString &caption, bool modified)
-{
-    QString title = caption;
-    if (!title.contains(QStringLiteral("[*]")) && !title.isEmpty()) { // append the placeholder so that the modified mechanism works
-        title.append(QStringLiteral(" [*]"));
-    }
-    setPlainCaption(title);
-    setWindowModified(modified);
-}
-
-void KMainWindow::setPlainCaption(const QString &caption)
-{
-    setWindowTitle(caption);
-}
 
 void KMainWindow::appHelpActivated(void)
 {
@@ -458,6 +434,10 @@ void KMainWindow::closeEvent(QCloseEvent *e)
     }
 
     if (queryClose()) {
+        // widgets will start destroying themselves at this point and we don't
+        // want to save state anymore after this as it might be incorrect
+        d->autoSaveSettings = false;
+        d->letDirtySettings = false;
         e->accept();
     } else {
         e->ignore();    //if the window should not be closed, don't close it
@@ -509,7 +489,7 @@ void KMainWindow::saveMainWindowSettings(KConfigGroup &cg)
     }
 
     // One day will need to save the version number, but for now, assume 0
-    // Utilise the QMainWindow::saveState() functionality.
+    // Utilize the QMainWindow::saveState() functionality.
     const QByteArray state = saveState();
     cg.writeEntry("State", state.toBase64());
 
