@@ -32,7 +32,7 @@
 #include <kritawidgetutils_export.h>
 
 #include <QMainWindow>
-#include <QtCore/QMetaClassInfo>
+#include <QMetaClassInfo>
 
 class QMenu;
 class KConfig;
@@ -47,9 +47,6 @@ class KToolBar;
     inline const classname ## Private *k_func() const { return reinterpret_cast<classname ## Private *>(k_ptr); } \
     friend class classname ## Private;
 
-// This is mostly from KDE3. TODO KDE5: remove the constructor parameter.
-#define KDE_DEFAULT_WINDOWFLAGS 0
-
 /**
  * @short %KDE top level main window
  *
@@ -143,7 +140,7 @@ public:
      * for the composer windows "composer#".
      *
      */
-    explicit KMainWindow(QWidget *parent = 0, Qt::WindowFlags f = KDE_DEFAULT_WINDOWFLAGS);
+    explicit KMainWindow(QWidget *parent = 0, Qt::WindowFlags f = Qt::WindowFlags());
 
     /**
      * \brief Destructor.
@@ -151,7 +148,7 @@ public:
      * Will also destroy the toolbars, and menubar if
      * needed.
      */
-    virtual ~KMainWindow();
+    ~KMainWindow() override;
 
     /**
      * If the session did contain so high a number, @p true is returned,
@@ -252,7 +249,7 @@ public:
      * of restoring and saving automatically. Make sure you call this
      * _after all_ your *bars have been created.
      *
-     * To make sure that KMainWindow propertly obtains the default
+     * To make sure that KMainWindow properly obtains the default
      * size of the window you should do the following:
      * - Remove hard coded resize() calls in the constructor or main, they
      *   should be removed in favor of letting the automatic resizing
@@ -314,7 +311,7 @@ public:
      *
      * @param config Config group to read the settings from.
      * KF5 porting note: the unused bool argument was removed, make sure to remove it from your
-     * reimplementations too! And add a Q_DECL_OVERRIDE for good measure.
+     * reimplementations too! And add a override for good measure.
      */
     virtual void applyMainWindowSettings(const KConfigGroup &config);
 
@@ -333,32 +330,6 @@ public:
     QString dbusName() const;
 
 public Q_SLOTS:
-    /**
-     * Makes a KDE compliant caption (window title).
-     *
-     * @param caption Your caption. @em Do @em not include the application name
-     * in this string. It will be added automatically according to the KDE
-     * standard.
-     */
-    virtual void setCaption(const QString &caption);
-    /**
-     * Makes a KDE compliant caption.
-     *
-     * @param caption Your caption. @em Do @em not include the application name
-     * in this string. It will be added automatically according to the KDE
-     * standard.
-     * @param modified Specify whether the document is modified. This displays
-     * an additional sign in the title bar, usually "**".
-     */
-    virtual void setCaption(const QString &caption, bool modified);
-
-    /**
-     * Make a plain caption without any modifications.
-     *
-     * @param caption Your caption. This is the string that will be
-     * displayed in the window title.
-     */
-    virtual void setPlainCaption(const QString &caption);
 
     /**
      * Open the help page for the application.
@@ -396,7 +367,7 @@ protected:
      * if needed, once all constructor code for the main window has run.
      * Also reimplemented to catch when a QDockWidget is added or removed.
      */
-    bool event(QEvent *event) Q_DECL_OVERRIDE;
+    bool event(QEvent *event) override;
 
     /**
      * Reimplemented to autosave settings and call queryClose().
@@ -405,7 +376,7 @@ protected:
      * If you do it anyway, ensure to call the base implementation to keep
      * the feature of auto-saving window settings working.
      */
-    void closeEvent(QCloseEvent *) Q_DECL_OVERRIDE;
+    void closeEvent(QCloseEvent *) override;
 
     /**
        Called before the window is closed, either by the user or indirectly by
