@@ -44,7 +44,6 @@
 #endif
 #include <QDebug>
 
-#include <kauthorized.h>
 #include <kconfig.h>
 #include <ksharedconfig.h>
 #ifdef HAVE_ICONTHEMES
@@ -262,11 +261,7 @@ void KToolBar::Private::init(bool readConfig, bool _isMainToolBar)
                 q->mainWindow(), SLOT(setSettingsDirty()));
     }
 
-    if (!KAuthorized::authorize(QStringLiteral("movable_toolbars"))) {
-        q->setMovable(false);
-    } else {
-        q->setMovable(!KToolBar::toolBarsLocked());
-    }
+    q->setMovable(!KToolBar::toolBarsLocked());
 
     connect(q, SIGNAL(movableChanged(bool)),
             q, SLOT(slotMovableChanged(bool)));
@@ -525,21 +520,6 @@ void KToolBar::Private::loadKDESettings()
         toolButtonStyleSettings[Level_KDEDefault] = toolButtonStyleSetting();
     } else {
         const QString fallBack = toolButtonStyleToString(Qt::ToolButtonTextBesideIcon);
-        /**
-          TODO: if we get complaints about text beside icons on small screens,
-                try the following code out on such systems - aseigo.
-        // if we are on a small screen with a non-landscape ratio, then
-        // we revert to text under icons since width is probably not our
-        // friend in such cases
-        QDesktopWidget *desktop = QApplication::desktop();
-        QRect screenGeom = desktop->screenGeometry(desktop->primaryScreen());
-        qreal ratio = screenGeom.width() / qreal(screenGeom.height());
-
-        if (screenGeom.width() < 1024 && ratio <= 1.4) {
-            fallBack = "TextUnderIcon";
-        }
-        **/
-
         KConfigGroup group(KSharedConfig::openConfig(), "Toolbar style");
         const QString value = group.readEntry("ToolButtonStyleOtherToolbars", fallBack);
         toolButtonStyleSettings[Level_KDEDefault] = KToolBar::Private::toolButtonStyleFromString(value);
@@ -1037,9 +1017,7 @@ int KToolBar::iconSizeDefault() const
 
 void KToolBar::slotMovableChanged(bool movable)
 {
-    if (movable && !KAuthorized::authorize(QStringLiteral("movable_toolbars"))) {
-        setMovable(false);
-    }
+    setMovable(movable);
 }
 
 void KToolBar::dragEnterEvent(QDragEnterEvent *event)
@@ -1204,7 +1182,7 @@ void KToolBar::mouseMoveEvent(QMouseEvent *event)
 
     drag->setMimeData(mimeData);
 
-    Qt::DropAction dropAction = drag->start(Qt::MoveAction);
+    Qt::DropAction dropAction = drag->exec(Qt::MoveAction);
 
     if (dropAction == Qt::MoveAction)
         // Only remove from this toolbar if it was moved to another toolbar
