@@ -26,7 +26,6 @@
 #include "kxmlguiwindow.h"
 #include "kmenumenuhandler_p.h"
 
-#include <kauthorized.h>
 #include <klocalizedstring.h>
 
 #include <QDomElement>
@@ -49,10 +48,10 @@ using namespace KDEPrivate;
 class KXMLGUIBuilderPrivate
 {
 public:
-    KXMLGUIBuilderPrivate() : m_client(0L)  {}
+    KXMLGUIBuilderPrivate() { }
     ~KXMLGUIBuilderPrivate() { }
 
-    QWidget *m_widget;
+    QWidget *m_widget {0};
 
     QString tagMainWindow;
     QString tagMenuBar;
@@ -74,9 +73,9 @@ public:
 
     QString attrIcon;
 
-    KXMLGUIClient *m_client;
+    KXMLGUIClient *m_client {0};
 
-    KMenuMenuHandler *m_menumenuhandler;
+    KMenuMenuHandler *m_menumenuhandler {0};
 };
 
 KXMLGUIBuilder::KXMLGUIBuilder(QWidget *widget)
@@ -174,10 +173,6 @@ QWidget *KXMLGUIBuilder::createContainer(QWidget *parent, int index, const QDomE
 
         QString name = element.attribute(d->attrName);
 
-        if (!KAuthorized::authorizeAction(name)) {
-            return 0;
-        }
-
         QMenu *popup = new QMenu(p);
         popup->setObjectName(name);
 
