@@ -48,6 +48,8 @@ for arg in "${@}"; do
     elif [[ "${arg}" = --debug ]]; then
         OSXBUILD_TYPE="Debug"
         OSXBUILD_TESTING="ON"
+    elif [[ "${arg}" = --install_tarball ]]; then
+        OSXBUILD_TARBALLINSTALL="TRUE"
     else
         parsed_args="${parsed_args} ${arg}"
     fi
@@ -287,7 +289,6 @@ build_3rdparty () {
         ext_eigen3 \
         ext_exiv2 \
         ext_fftw3 \
-        ext_ilmbase \
         ext_jpeg \
         ext_lcms2 \
         ext_ocio \
@@ -341,6 +342,7 @@ build_3rdparty () {
         ext_kwindowsystem \
         ext_quazip
 
+    cmake_3rdparty ext_seexpr
 
     ## All builds done, creating a new install onlydeps install dir
     dir_clean "${KIS_INSTALL_DIR}.onlydeps"
@@ -389,12 +391,10 @@ rebuild_3rdparty () {
         ext_expat \
         ext_exiv2 \
         ext_fftw3 \
-        ext_ilmbase \
         ext_jpeg \
         ext_patch \
         ext_lcms2 \
         ext_ocio \
-        ext_ilmbase \
         ext_openexr
         #ext_openjpeg
 
@@ -432,6 +432,9 @@ rebuild_3rdparty () {
         ext_kimageformats \
         ext_kwindowsystem \
         ext_quazip
+
+    build_install_ext \
+        ext_seexpr
 }
 
 #not tested
@@ -511,9 +514,6 @@ build_krita_tarball () {
     build_krita
 
     print_msg "Build done!"
-    print_msg "to install run
-osxbuild.sh install ${KIS_BUILD_DIR}"
-
 }
 
 install_krita () {
@@ -543,7 +543,7 @@ install_krita () {
 
     # compile integrations
     if test ${OSTYPE} == "darwin*"; then
-        cd ${KIS_BUILD_DIR}/krita/integration/kritaquicklook
+        cd ${KIS_BUILD_DIR}/krita/integration
         make install
     fi
 }
@@ -648,9 +648,23 @@ script_run() {
         # This is not on by default as build success requires all
         # deps installed in the given dir beforehand.
         # KIS_INSTALL_DIR=${3}
-        OSXBUILD_DIR=$(get_directory_fromargs "${@}")
 
-        build_krita_tarball "${OSXBUILD_DIR}"
+        if [[ -f "${2}" && "${2:(-7)}" == ".tar.gz" ]]; then
+            TARBALL_FILE="${2}"
+            build_krita_tarball "${TARBALL_FILE}"
+
+            if [[ -n "${OSXBUILD_TARBALLINSTALL}" ]]; then
+                install_krita "${KIS_BUILD_DIR}"
+            else
+                print_msg "to install run
+osxbuild.sh install ${KIS_BUILD_DIR}"
+            fi
+            exit
+        else
+            log "File not a tarball tar.gz file"
+        fi
+
+        
 
     elif [[ ${1} = "clean" ]]; then
         # remove all build and install directories to start
