@@ -23,6 +23,7 @@
 #include <klocalizedstring.h>
 #include "kis_debug.h"
 #include <QPainter>
+#include <QPainterPath>
 #include <QLinearGradient>
 #include <QTransform>
 #include <kis_canvas2.h>
@@ -32,7 +33,9 @@
 #include <math.h>
 
 ConcentricEllipseAssistant::ConcentricEllipseAssistant()
-        : KisPaintingAssistant("concentric ellipse", i18n("Concentric Ellipse assistant"))
+    : KisPaintingAssistant("concentric ellipse", i18n("Concentric Ellipse assistant"))
+    , m_followBrushPosition(false)
+    , m_adjustedPositionValid(false)
 {
 }
 
@@ -45,21 +48,42 @@ ConcentricEllipseAssistant::ConcentricEllipseAssistant(const ConcentricEllipseAs
     : KisPaintingAssistant(rhs, handleMap)
     , m_ellipse(rhs.m_ellipse)
     , m_extraEllipse(rhs.m_extraEllipse)
+    , m_followBrushPosition(rhs.m_followBrushPosition)
+    , m_adjustedPositionValid(rhs.m_adjustedPositionValid)
+    , m_adjustedBrushPosition(rhs.m_adjustedBrushPosition)
 {
 }
 
+void ConcentricEllipseAssistant::setAdjustedBrushPosition(const QPointF position)
+{
+    m_adjustedPositionValid = true;
+    m_adjustedBrushPosition = position;
+}
+
+void ConcentricEllipseAssistant::endStroke()
+{
+    // Brush stroke ended, guides should follow the brush position again.
+    m_followBrushPosition = false;
+    m_adjustedPositionValid = false;
+}
+
+void ConcentricEllipseAssistant::setFollowBrushPosition(bool follow)
+{
+    m_followBrushPosition = follow;
+}
+
 QPointF ConcentricEllipseAssistant::project(const QPointF& pt, const QPointF& strokeBegin) const
 {
     Q_ASSERT(isAssistantComplete());
     m_ellipse.set(*handles()[0], *handles()[1], *handles()[2]);
 
-    qreal
-            dx = pt.x() - strokeBegin.x(),
-            dy = pt.y() - strokeBegin.y();
-        if (dx * dx + dy * dy < 4.0) {
-            // allow some movement before snapping
-            return strokeBegin;
-        }
+    qreal dx = pt.x() - strokeBegin.x();
+    qreal dy = pt.y() - strokeBegin.y();
+    if (dx * dx + dy * dy < 4.0) {
+        // allow some movement before snapping
+        return strokeBegin;
+    }
+    //
     //calculate ratio
     QPointF initial = m_ellipse.project(strokeBegin);
     QPointF center = m_ellipse.boundingRect().center();
@@ -101,11 +125,18 @@ void ConcentricEllipseAssistant::drawAssistant(QPainter& gc, const QRectF& updat
         dbgFile<<"canvas does not exist in the ellipse assistant, you may have passed arguments incorrectly:"<<canvas;
     }
 
-    QTransform initialTransform = converter->documentToWidgetTransform();
+
 
     if (isSnappingActive() && previewVisible == true){
 
         if (isAssistantComplete()){
+
+            QTransform initialTransform = converter->documentToWidgetTransform();
+
+            if (m_followBrushPosition && m_adjustedPositionValid) {
+                mousePos = initialTransform.map(m_adjustedBrushPosition);
+            }
+
             if (m_ellipse.set(*handles()[0], *handles()[1], *handles()[2])) {
                 QPointF initial = m_ellipse.project(initialTransform.inverted().map(mousePos));
                 QPointF center = m_ellipse.boundingRect().center();
@@ -183,6 +214,25 @@ bool ConcentricEllipseAssistant::isAssistantComplete() const
     return handles().size() >= 3;
 }
 
+void ConcentricEllipseAssistant::transform(const QTransform &transform)
+{
+    m_ellipse.set(*handles()[0], *handles()[1], *handles()[2]);
+
+    QPointF newAxes;
+    QTransform newTransform;
+
+    std::tie(newAxes, newTransform) = KisAlgebra2D::transformEllipse(QPointF(m_ellipse.semiMajor(), m_ellipse.semiMinor()), m_ellipse.getInverse() * transform);
+
+    const QPointF p1 = newTransform.map(QPointF(newAxes.x(), 0));
+    const QPointF p2 = newTransform.map(QPointF(-newAxes.x(), 0));
+    const QPointF p3 = newTransform.map(QPointF(0, newAxes.y()));
+
+    *handles()[0] = p1;
+    *handles()[1] = p2;
+    *handles()[2] = p3;
+
+    uncache();
+}
 
 ConcentricEllipseAssistantFactory::ConcentricEllipseAssistantFactory()
 {
