@@ -37,8 +37,10 @@ public:
     ParallelRulerAssistant();
     KisPaintingAssistantSP clone(QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap) const override;
     QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin) override;
-    //virtual void endStroke();
-    QPointF buttonPosition() const override;
+    void setAdjustedBrushPosition(const QPointF position) override;
+    void setFollowBrushPosition(bool follow) override;
+    void endStroke() override;
+    QPointF getEditorPosition() const override;
     int numHandles() const override { return 2; }
     bool isAssistantComplete() const override;
 
@@ -48,6 +50,12 @@ protected:
 private:
     QPointF project(const QPointF& pt, const QPointF& strokeBegin);
     explicit ParallelRulerAssistant(const ParallelRulerAssistant &rhs, QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap);
+
+    // Needed to make sure that when we are in the middle of a brush stroke, the
+    // guides follow the brush position, not the cursor position.
+    bool m_followBrushPosition;
+    bool m_adjustedPositionValid;
+    QPointF m_adjustedBrushPosition;
 };
 
 class ParallelRulerAssistantFactory : public KisPaintingAssistantFactory
