@@ -21,12 +21,14 @@
 #include <kis_assistant_tool.h>
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QStandardPaths>
 #include <QFile>
 #include <QLineF>
 
+
 #include <kis_debug.h>
 #include <klocalizedstring.h>
 #include <KColorButton>
@@ -101,7 +103,7 @@ void KisAssistantTool::deactivate()
 void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
 {
     setMode(KisTool::PAINT_MODE);
-    m_origAssistantList = cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants());
+    m_origAssistantList = KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants());
 
     bool newAssistantAllowed = true;
 
@@ -269,11 +271,12 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
         } else if (m_handleDrag && assistant->handles().size()>2 && (assistant->id() == "ellipse" ||
                                     assistant->id() == "concentric ellipse" ||
                                     assistant->id() == "fisheye-point")){
-            m_snapIsRadial = false;
             if (m_handleDrag == assistant->handles()[0]) {
                 m_dragStart = *assistant->handles()[1];
+                m_snapIsRadial = false;
             } else if (m_handleDrag == assistant->handles()[1]) {
                 m_dragStart = *assistant->handles()[0];
+                m_snapIsRadial = false;
             } else if (m_handleDrag == assistant->handles()[2]) {
                 m_dragStart = assistant->getEditorPosition();
                 m_radius = QLineF(m_dragStart, *assistant->handles()[0]);
@@ -374,15 +377,10 @@ void KisAssistantTool::continuePrimaryAction(KoPointerEvent *event)
 
     if (m_handleDrag) {
         *m_handleDrag = event->point;
-        //ported from the gradient tool... we need to think about this more in the future.
-        if (event->modifiers() == Qt::ShiftModifier && m_snapIsRadial) {
-            QLineF dragRadius = QLineF(m_dragStart, event->point);
-            dragRadius.setLength(m_radius.length());
-            *m_handleDrag = dragRadius.p2();
-        } else if (event->modifiers() == Qt::ShiftModifier ) {
-            QPointF move = snapToClosestAxis(event->point - m_dragStart);
-            *m_handleDrag = m_dragStart + move;
-        } else {
+
+        KisPaintingAssistantSP selectedAssistant = m_canvas->paintingAssistantsDecoration()->selectedAssistant();
+
+        if (!snap(event)) {
             *m_handleDrag = canvasDecoration->snapToGuide(event, QPointF(), false);
         }
         m_handleDrag->uncache();
@@ -406,7 +404,7 @@ void KisAssistantTool::continuePrimaryAction(KoPointerEvent *event)
         m_canvas->updateCanvas();
     } else if (m_assistantDrag) {
         QPointF newAdjustment = canvasDecoration->snapToGuide(event, QPointF(), false) - m_cursorStart;
-        if (event->modifiers() == Qt::ShiftModifier ) {
+        if (event->modifiers() & Qt::ShiftModifier ) {
             newAdjustment = snapToClosestAxis(newAdjustment);
         }
         Q_FOREACH (KisPaintingAssistantHandleSP handle, m_assistantDrag->handles()) {
@@ -525,7 +523,7 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
             m_assistantDrag.clear();
         }
         dbgUI << "creating undo command...";
-        KUndo2Command *command = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
+        KUndo2Command *command = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
         m_canvas->viewManager()->undoAdapter()->addCommand(command);
         dbgUI << "done";
     } else if(m_internalMode == MODE_DRAGGING_TRANSLATING_TWONODES) {
@@ -539,18 +537,6 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
     m_canvas->updateCanvas(); // TODO update only the relevant part of the canvas
 }
 
-QList<KisPaintingAssistantSP> KisAssistantTool::cloneAssistantList(const QList<KisPaintingAssistantSP> &list) const
-{
-    QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> handleMap;
-    QList<KisPaintingAssistantSP> clonedList;
-    dbgUI << "cloning assistants...";
-    for (auto i = list.begin(); i != list.end(); ++i) {
-        clonedList << (*i)->clone(handleMap);
-    }
-    dbgUI << "done";
-    return clonedList;
-}
-
 void KisAssistantTool::addAssistant()
 {
     m_canvas->paintingAssistantsDecoration()->addAssistant(m_newAssistant);
@@ -561,7 +547,7 @@ void KisAssistantTool::addAssistant()
     }
 
     QList<KisPaintingAssistantSP> assistants = m_canvas->paintingAssistantsDecoration()->assistants();
-    KUndo2Command *addAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(assistants), EditAssistantsCommand::ADD, assistants.indexOf(m_newAssistant));
+    KUndo2Command *addAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(assistants), EditAssistantsCommand::ADD, assistants.indexOf(m_newAssistant));
     m_canvas->viewManager()->undoAdapter()->addCommand(addAssistantCmd);
 
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
@@ -581,7 +567,7 @@ void KisAssistantTool::removeAssistant(KisPaintingAssistantSP assistant)
     }
     m_canvas->paintingAssistantsDecoration()->removeAssistant(assistant);
 
-    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()), EditAssistantsCommand::REMOVE, assistants.indexOf(assistant));
+    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()), EditAssistantsCommand::REMOVE, assistants.indexOf(assistant));
     m_canvas->viewManager()->undoAdapter()->addCommand(removeAssistantCmd);
 
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
@@ -613,16 +599,21 @@ void KisAssistantTool::updateToolOptionsUI()
          // load custom color settings from assistant (this happens when changing assistant
          m_options.useCustomAssistantColor->setChecked(m_selectedAssistant->useCustomColor());
          m_options.customAssistantColorButton->setColor(m_selectedAssistant->assistantCustomColor());
-         float opacity = (float)m_selectedAssistant->assistantCustomColor().alpha()/255.0 * 100.0 ;
-         m_options.customColorOpacitySlider->setValue(opacity);
+
+
+         double opacity = (double)m_selectedAssistant->assistantCustomColor().alpha()/(double)255.00 * (double)100.00 ;
+         opacity = ceil(opacity); // helps keep the 0-100% slider from shifting
+
+         m_options.customColorOpacitySlider->blockSignals(true);
+         m_options.customColorOpacitySlider->setValue((double)opacity);
+         m_options.customColorOpacitySlider->blockSignals(false);
+
+
      } else {
          m_options.vanishingPointAngleSpinbox->setVisible(false); //
      }
 
      // show/hide elements if an assistant is selected or not
-      m_options.assistantsGlobalOpacitySlider->setVisible(hasActiveAssistant);
-      m_options.assistantsColor->setVisible(hasActiveAssistant);
-      m_options.globalColorLabel->setVisible(hasActiveAssistant);
       m_options.useCustomAssistantColor->setVisible(hasActiveAssistant);
 
       // hide custom color options if use custom color is not selected
@@ -660,7 +651,11 @@ void KisAssistantTool::slotChangeVanishingPointAngle(double value)
 void KisAssistantTool::mouseMoveEvent(KoPointerEvent *event)
 {
     if (m_newAssistant && m_internalMode == MODE_CREATION) {
-        *m_newAssistant->handles().back() = event->point;
+
+        KisPaintingAssistantHandleSP new_handle = m_newAssistant->handles().back();
+        if (!snap(event)) {
+            *new_handle = event->point;
+        }
 
     } else if (m_newAssistant && m_internalMode == MODE_DRAGGING_TRANSLATING_TWONODES) {
         QPointF translate = event->point - m_dragEnd;
@@ -672,6 +667,22 @@ void KisAssistantTool::mouseMoveEvent(KoPointerEvent *event)
      m_canvas->updateCanvas();
 }
 
+
+void KisAssistantTool::keyPressEvent(QKeyEvent *event)
+{
+    // When the user is in the middle of creating a new
+    // assistant the escape key can be used to cancel this process.
+    if (event->key()==Qt::Key_Escape && (m_newAssistant)) {
+        // Clear shared pointer to the assistant being created so
+        // it gets cleaned-up
+        m_newAssistant.clear();
+        m_canvas->updateCanvas();
+        event->accept();
+    } else {
+        event->ignore();
+    }
+}
+
 void KisAssistantTool::paint(QPainter& _gc, const KoViewConverter &_converter)
 {
     QRectF canvasSize = QRectF(QPointF(0, 0), QSizeF(m_canvas->image()->size()));
@@ -721,8 +732,14 @@ void KisAssistantTool::paint(QPainter& _gc, const KoViewConverter &_converter)
 
 void KisAssistantTool::removeAllAssistants()
 {
+    m_origAssistantList = m_canvas->paintingAssistantsDecoration()->assistants();
+
     m_canvas->viewManager()->canvasResourceProvider()->clearPerspectiveGrids();
     m_canvas->paintingAssistantsDecoration()->removeAll();
+
+    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, KisPaintingAssistant::cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
+    m_canvas->viewManager()->undoAdapter()->addCommand(removeAssistantCmd);
+
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
     m_canvas->updateCanvas();
 
@@ -746,6 +763,7 @@ void KisAssistantTool::loadAssistants()
     QByteArray data = file.readAll();
     QXmlStreamReader xml(data);
     QMap<int, KisPaintingAssistantHandleSP> handleMap;
+    QMap<int, KisPaintingAssistantHandleSP> sideHandleMap;
     KisPaintingAssistantSP assistant;
     bool errors = false;
     while (!xml.atEnd()) {
@@ -779,6 +797,30 @@ void KisAssistantTool::loadAssistants()
                         errors = true;
                     }
                 }
+	    // for vanishing point assistant
+            } else if (xml.name() == "sidehandle"){
+
+	      // read in sidehandles
+              if (!xml.attributes().value("id").isEmpty()) {
+		  QString strId = xml.attributes().value("id").toString(),
+			  strX = xml.attributes().value("x").toString(),
+			  strY = xml.attributes().value("y").toString();
+		  if (!strId.isEmpty() && !strX.isEmpty() && !strY.isEmpty()) {
+		      int id = strId.toInt();
+		      double x = strX.toDouble();
+		      double y = strY.toDouble();
+		      if (!sideHandleMap.contains(id)) {
+			  sideHandleMap.insert(id, new KisPaintingAssistantHandle(x,y));
+		      }}
+              }
+	      // addHandle to assistant
+              if (!xml.attributes().value("ref").isEmpty() && assistant) {
+		  KisPaintingAssistantHandleSP handle = sideHandleMap.value(xml.attributes().value("ref").toString().toInt());
+		  if (handle) {
+		      assistant->addHandle(handle, HandleType::SIDE);
+		  }
+	      }
+
             } else if (xml.name() == "assistant") {
                 const KisPaintingAssistantFactory* factory = KisPaintingAssistantFactoryRegistry::instance()->get(xml.attributes().value("type").toString());
 
@@ -822,8 +864,8 @@ void KisAssistantTool::loadAssistants()
             if (xml.name() == "assistant") {
                 if (assistant) {
                     if (assistant->handles().size() == assistant->numHandles()) {
-                        if (assistant->id() == "vanishing point"){
-                        //ideally we'd save and load side-handles as well, but this is all I've got//
+		        if (assistant->id() == "vanishing point" && sideHandleMap.empty()){
+                        // Create side handles if the saved vp assistant doesn't have any.
                             QPointF pos = *assistant->handles()[0];
                             assistant->addHandle(new KisPaintingAssistantHandle(pos+QPointF(-70,0)), HandleType::SIDE);
                             assistant->addHandle(new KisPaintingAssistantHandle(pos+QPointF(-140,0)), HandleType::SIDE);
@@ -890,6 +932,19 @@ void KisAssistantTool::saveAssistants()
         xml.writeEndElement();
     }
     xml.writeEndElement();
+    xml.writeStartElement("sidehandles");
+    QMap<KisPaintingAssistantHandleSP, int> sideHandleMap;
+    Q_FOREACH (KisPaintingAssistantSP assistant, m_canvas->paintingAssistantsDecoration()->assistants()) {
+	Q_FOREACH (KisPaintingAssistantHandleSP handle, assistant->sideHandles()) {
+	    int id = sideHandleMap.size();
+	    sideHandleMap.insert(handle, id);
+	    xml.writeStartElement("sidehandle");
+	    xml.writeAttribute("id", QString::number(id));
+	    xml.writeAttribute("x", QString::number(double(handle->x()), 'f', 3));
+	    xml.writeAttribute("y", QString::number(double(handle->y()), 'f', 3));
+	    xml.writeEndElement();
+	}
+    }
     xml.writeStartElement("assistants");
 
 
@@ -912,6 +967,15 @@ void KisAssistantTool::saveAssistants()
             xml.writeEndElement();
         }
         xml.writeEndElement();
+        if (!sideHandleMap.empty()) {
+	    xml.writeStartElement("sidehandles");
+	    Q_FOREACH (const KisPaintingAssistantHandleSP handle, assistant->sideHandles()) {
+		xml.writeStartElement("sidehandle");
+		xml.writeAttribute("ref", QString::number(sideHandleMap.value(handle)));
+		xml.writeEndElement();
+	    }
+	    xml.writeEndElement();
+        }
         xml.writeEndElement();
     }
     xml.writeEndElement();
@@ -963,11 +1027,8 @@ QWidget *KisAssistantTool::createOptionWidget()
         connect(m_options.assistantsColor, SIGNAL(changed(QColor)), SLOT(slotGlobalAssistantsColorChanged(QColor)));
         connect(m_options.assistantsGlobalOpacitySlider, SIGNAL(valueChanged(int)), SLOT(slotGlobalAssistantOpacityChanged()));
 
-
         connect(m_options.vanishingPointAngleSpinbox, SIGNAL(valueChanged(double)), this, SLOT(slotChangeVanishingPointAngle(double)));
 
-        //ENTER_FUNCTION() << ppVar(m_canvas) << ppVar(m_canvas && m_canvas->paintingAssistantsDecoration());
-
         // initialize UI elements with existing data if possible
         if (m_canvas && m_canvas->paintingAssistantsDecoration()) {
             const QColor color = m_canvas->paintingAssistantsDecoration()->globalAssistantsColor();
@@ -975,8 +1036,6 @@ QWidget *KisAssistantTool::createOptionWidget()
             QColor opaqueColor = color;
             opaqueColor.setAlpha(255);
 
-            //ENTER_FUNCTION() << ppVar(opaqueColor);
-
             m_options.assistantsColor->setColor(opaqueColor);
             m_options.customAssistantColorButton->setColor(opaqueColor);
             m_options.assistantsGlobalOpacitySlider->setValue(color.alphaF() * 100.0);
@@ -1002,7 +1061,7 @@ QWidget *KisAssistantTool::createOptionWidget()
         m_options.vanishingPointAngleSpinbox->setPrefix(i18n("Density: "));
         m_options.vanishingPointAngleSpinbox->setSuffix(QChar(Qt::Key_degree));
         m_options.vanishingPointAngleSpinbox->setRange(1.0, 180.0);
-        m_options.vanishingPointAngleSpinbox->setSingleStep(0.5);
+        m_options.vanishingPointAngleSpinbox->setSingleStep(1.0);
 
 
         m_options.vanishingPointAngleSpinbox->setVisible(false);
@@ -1073,3 +1132,101 @@ void KisAssistantTool::slotCustomOpacityChanged()
     m_canvas->paintingAssistantsDecoration()->uncache();
     m_canvas->canvasWidget()->update();
 }
+
+void KisAssistantTool::beginAlternateAction(KoPointerEvent *event, AlternateAction action)
+{
+    Q_UNUSED(action);
+    setMode(KisTool::PAINT_MODE);
+
+    if (m_newAssistant && m_internalMode == MODE_CREATION) {
+        KisPaintingAssistantsDecorationSP canvasDecoration = m_canvas->paintingAssistantsDecoration();
+
+        *m_newAssistant->handles().back() = event->point;
+
+        if (!snap(event)) {
+            *m_newAssistant->handles().back() = canvasDecoration->snapToGuide(event, QPointF(), false);
+        }
+
+        if (m_newAssistant->handles().size() == m_newAssistant->numHandles()) {
+            addAssistant();
+        } else {
+            m_newAssistant->addHandle(new KisPaintingAssistantHandle(canvasDecoration->snapToGuide(event, QPointF(), false)), HandleType::NORMAL);
+        }
+        m_canvas->updateCanvas();
+        return;
+    }
+}
+
+void KisAssistantTool::continueAlternateAction(KoPointerEvent *event, AlternateAction action)
+{
+    Q_UNUSED(action);
+    event->ignore();
+}
+
+void KisAssistantTool::endAlternateAction(KoPointerEvent *event, AlternateAction action)
+{
+    Q_UNUSED(action);
+    setMode(KisTool::HOVER_MODE);
+    event->ignore();
+    m_canvas->updateCanvas();
+}
+
+bool KisAssistantTool::snap(KoPointerEvent *event)
+{
+    if ((event->modifiers() & Qt::ShiftModifier) == false) {
+        return false;
+    } else {
+
+        if (m_handleDrag) {
+            if (m_snapIsRadial == true) {
+                QLineF dragRadius = QLineF(m_dragStart, event->point);
+                dragRadius.setLength(m_radius.length());
+                *m_handleDrag = dragRadius.p2();
+            } else {
+                QPointF snap_point = snapToClosestAxis(event->point - m_dragStart);
+                *m_handleDrag = m_dragStart + snap_point;
+            }
+
+        } else {
+            if (m_newAssistant && m_internalMode == MODE_CREATION) {
+                QList<KisPaintingAssistantHandleSP> handles = m_newAssistant->handles();
+                KisPaintingAssistantHandleSP handle_snap = handles.back();
+                // for any assistant, snap 2nd handle to x or y axis relative to first handle
+                if (handles.size() == 2) {
+                    QPointF snap_point = snapToClosestAxis(event->point - *handles[0]);
+                    *handle_snap =  *handles[0] + snap_point;
+                } else {
+                    bool was_snapped = false;
+                    if (m_newAssistant->id() == "spline") {
+                        KisPaintingAssistantHandleSP start;
+                        handles.size() == 3 ? start = handles[0] : start = handles[1];
+                        QPointF snap_point = snapToClosestAxis(event->point - *start);
+                        *handle_snap =  *start + snap_point;
+                        was_snapped = true;
+                    }
+
+                    if (m_newAssistant->id() == "ellipse" ||
+                        m_newAssistant->id() == "concentric ellipse" ||
+                        m_newAssistant->id() == "fisheye-point") {
+                        QPointF center = QLineF(*handles[0], *handles[1]).center();
+                        QLineF radius = QLineF(center,*handles[0]);
+                        QLineF dragRadius = QLineF(center, event->point);
+                        dragRadius.setLength(radius.length());
+                        *handle_snap = dragRadius.p2();
+                        was_snapped = true;
+                    }
+
+                    if (m_newAssistant->id() == "perspective") {
+                        KisPaintingAssistantHandleSP start;
+                        handles.size() == 3 ? start = handles[1] : start = handles[2];
+                        QPointF snap_point = snapToClosestAxis(event->point - *start);
+                        *handle_snap =  *start + snap_point;
+                        was_snapped = true;
+                    }
+                    return was_snapped;
+                }
+            }
+        }
+        return true;
+    }
+}
