@@ -219,11 +219,6 @@ void KisColorSelectorBase::mousePressEvent(QMouseEvent* event)
 {
     event->accept();
 
-
-//this boolean here is to check if the color selector is updating the resource, so it won't update itself when the resource is updated//
-   if (m_colorUpdateSelf==false)
-   {m_colorUpdateSelf=true;}
-
     if(!m_isPopup && m_popupOnMouseClick &&
        event->button() == Qt::MidButton) {
 
@@ -246,6 +241,7 @@ void KisColorSelectorBase::mousePressEvent(QMouseEvent* event)
         if(y+m_popup->height()>availRect.y()+availRect.height())
             y = availRect.y()+availRect.height()-m_popup->height();
 
+        m_colorUpdateSelf=false;
         m_popup->move(x, y);
         m_popup->setHidingTime(200);
         showPopup(DontMove);
@@ -256,6 +252,7 @@ void KisColorSelectorBase::mousePressEvent(QMouseEvent* event)
         }
         hide();
     } else {
+        m_colorUpdateSelf=true;
         showColorPreview();
         event->ignore();
     }
@@ -395,7 +392,7 @@ void KisColorSelectorBase::lazyCreatePopup()
 #ifdef Q_OS_MACOS
                                 Qt::Popup |
 #else
-                                Qt::Tool |
+                                Qt::Window |
 #endif
                                 Qt::NoDropShadowWindowHint |
                                 Qt::BypassWindowManagerHint);
