@@ -25,8 +25,6 @@
 #include <kconfiggroup.h>
 #include <ksharedconfig.h>
 
-#include <KisGamutMaskViewConverter.h>
-
 #include "kis_display_color_converter.h"
 #include "kis_acs_pixel_cache_renderer.h"
 
@@ -39,7 +37,6 @@ KisColorSelectorWheel::KisColorSelectorWheel(KisColorSelector *parent) :
     m_renderAreaOffsetY(0.0),
     m_toRenderArea(QTransform())
 {
-    m_viewConverter = new KisGamutMaskViewConverter();
 }
 
 KoColor KisColorSelectorWheel::selectColor(int x, int y)
@@ -186,7 +183,8 @@ void KisColorSelectorWheel::paint(QPainter* painter)
 
     if(isDirty()) {
         KisPaintDeviceSP realPixelCache;
-        Acs::PixelCacheRenderer::render(this, m_parent->converter(), QRect(0, 0, width(), height()), realPixelCache, m_pixelCache, m_pixelCacheOffset);
+        Acs::PixelCacheRenderer::render(this, m_parent->converter(), QRect(0, 0, width(), height()), realPixelCache,
+                                        m_pixelCache, m_pixelCacheOffset, painter->device()->devicePixelRatioF());
 
         //antialiasing for wheel
         QPainter tmpPainter(&m_pixelCache);
@@ -201,11 +199,6 @@ void KisColorSelectorWheel::paint(QPainter* painter)
         m_toRenderArea.reset();
         m_toRenderArea.translate(-m_renderAreaOffsetX,-m_renderAreaOffsetY);
 
-        m_viewConverter->setViewSize(m_renderAreaSize);
-        if (m_currentGamutMask) {
-           m_viewConverter->setMaskSize(m_currentGamutMask->maskSize());
-        }
-
         QPoint ellipseCenter(width() / 2 - size / 2, height() / 2 - size / 2);
         ellipseCenter -= m_pixelCacheOffset;
 
@@ -216,7 +209,8 @@ void KisColorSelectorWheel::paint(QPainter* painter)
 
     // draw gamut mask
     if (m_gamutMaskOn && m_currentGamutMask) {
-        QImage maskBuffer  = QImage(m_renderAreaSize.width(), m_renderAreaSize.height(), QImage::Format_ARGB32_Premultiplied);
+        QImage maskBuffer = QImage(m_renderAreaSize*painter->device()->devicePixelRatioF(), QImage::Format_ARGB32_Premultiplied);
+        maskBuffer.setDevicePixelRatio(painter->device()->devicePixelRatioF());
         maskBuffer.fill(0);
         QPainter maskPainter(&maskBuffer);
 
@@ -224,8 +218,8 @@ void KisColorSelectorWheel::paint(QPainter* painter)
         maskPainter.setRenderHint(QPainter::Antialiasing, true);
 
         maskPainter.resetTransform();
-        maskPainter.translate(rect.width()/2, rect.height()/2);
-        maskPainter.scale(rect.width()/2, rect.height()/2);
+        maskPainter.translate(rect.width()*0.5, rect.height()*0.5);
+        maskPainter.scale(rect.width()*0.5, rect.height()*0.5);
 
         maskPainter.setPen(QPen(QBrush(Qt::white), 0.002));
         maskPainter.setBrush(QColor(128,128,128,255)); // middle gray
@@ -233,12 +227,13 @@ void KisColorSelectorWheel::paint(QPainter* painter)
         maskPainter.drawEllipse(QPointF(0,0), 1.0, 1.0);
 
         maskPainter.resetTransform();
+        maskPainter.setTransform(m_currentGamutMask->maskToViewTransform(m_renderAreaSize.width()));
 
         maskPainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-        m_currentGamutMask->paint(maskPainter, *m_viewConverter, m_maskPreviewActive);
+        m_currentGamutMask->paint(maskPainter, m_maskPreviewActive);
 
         maskPainter.setCompositionMode(QPainter::CompositionMode_SourceOver);
-        m_currentGamutMask->paintStroke(maskPainter, *m_viewConverter, m_maskPreviewActive);
+        m_currentGamutMask->paintStroke(maskPainter, m_maskPreviewActive);
 
         painter->drawImage(m_renderAreaOffsetX, m_renderAreaOffsetY, maskBuffer);
     }
@@ -259,7 +254,7 @@ void KisColorSelectorWheel::paint(QPainter* painter)
     }
 }
 
-KoColor KisColorSelectorWheel::colorAt(int x, int y, bool forceValid)
+KoColor KisColorSelectorWheel::colorAt(float x, float y, bool forceValid)
 {
     KoColor color(Qt::transparent, m_parent->colorSpace());
 
@@ -319,7 +314,13 @@ KoColor KisColorSelectorWheel::colorAt(int x, int y, bool forceValid)
 
 bool KisColorSelectorWheel::allowsColorSelectionAtPoint(const QPoint &pt) const
 {
-    return !m_gamutMaskOn || !m_currentGamutMask ||
-        m_currentGamutMask->coordIsClear(m_toRenderArea.map(QPointF(pt)),
-                                         *m_viewConverter, m_maskPreviewActive);
+    if (!m_gamutMaskOn || !m_currentGamutMask) {
+        return true;
+    }
+
+    QPointF colorCoord = m_toRenderArea.map(QPointF(pt));
+    QPointF translatedPoint = m_currentGamutMask->viewToMaskTransform(m_renderAreaSize.width()).map(colorCoord);
+    bool isClear = m_currentGamutMask->coordIsClear(translatedPoint, m_maskPreviewActive);
+
+    return isClear;
 }
