@@ -31,38 +31,36 @@
 #include "kis_scalar_keyframe_channel.h"
 #include "kis_zoom_button.h"
 #include "kis_custom_modifiers_catcher.h"
+#include "krita_utils.h"
+
 
 const int VERTICAL_PADDING = 30;
 
 struct KisAnimationCurvesView::Private
 {
     Private()
-        : model(0)
-        , isDraggingKeyframe(false)
-        , isAdjustingHandle(false)
-        , panning(false)
     {}
 
-    KisAnimationCurvesModel *model;
-    TimelineRulerHeader *horizontalHeader;
-    KisAnimationCurvesValueRuler *verticalHeader;
-    KisAnimationCurvesKeyframeDelegate *itemDelegate;
-    KisZoomButton *horizontalZoomButton;
-    KisZoomButton *verticalZoomButton;
-    KisCustomModifiersCatcher *modifiersCatcher;
+    KisAnimationCurvesModel *model {0};
+    TimelineRulerHeader *horizontalHeader {0};
+    KisAnimationCurvesValueRuler *verticalHeader {0};
+    KisAnimationCurvesKeyframeDelegate *itemDelegate {0};
+    KisZoomButton *horizontalZoomButton {0};
+    KisZoomButton *verticalZoomButton {0};
+    KisCustomModifiersCatcher *modifiersCatcher {0};
 
-    bool isDraggingKeyframe;
-    bool isAdjustingHandle;
-    int adjustedHandle; // 0 = left, 1 = right
+    bool isDraggingKeyframe {false};
+    bool isAdjustingHandle {false};
+    int adjustedHandle {0}; // 0 = left, 1 = right
     QPoint dragStart;
     QPoint dragOffset;
 
-    int horizontalZoomStillPointIndex;
-    int horizontalZoomStillPointOriginalOffset;
-    qreal verticalZoomStillPoint;
-    qreal verticalZoomStillPointOriginalOffset;
+    int horizontalZoomStillPointIndex {0};
+    int horizontalZoomStillPointOriginalOffset {0};
+    qreal verticalZoomStillPoint {0.0};
+    qreal verticalZoomStillPointOriginalOffset {0.0};
 
-    bool panning;
+    bool panning {false};
     QPoint panStartOffset;
 };
 
@@ -71,6 +69,7 @@ KisAnimationCurvesView::KisAnimationCurvesView(QWidget *parent)
     , m_d(new Private())
 {
     m_d->horizontalHeader = new TimelineRulerHeader(this);
+
     m_d->verticalHeader = new KisAnimationCurvesValueRuler(this);
     m_d->itemDelegate = new KisAnimationCurvesKeyframeDelegate(m_d->horizontalHeader, m_d->verticalHeader, this);
 
@@ -87,6 +86,7 @@ KisAnimationCurvesView::KisAnimationCurvesView(QWidget *parent)
         connect(scroller, SIGNAL(stateChanged(QScroller::State)),
                 this, SLOT(slotScrollerStateChanged(QScroller::State)));
     }
+
 }
 
 KisAnimationCurvesView::~KisAnimationCurvesView()
@@ -110,6 +110,7 @@ void KisAnimationCurvesView::setModel(QAbstractItemModel *model)
 
     connect(model, &QAbstractItemModel::headerDataChanged,
             this, &KisAnimationCurvesView::slotHeaderDataChanged);
+
 }
 
 void KisAnimationCurvesView::setZoomButtons(KisZoomButton *horizontal, KisZoomButton *vertical)
@@ -168,10 +169,45 @@ void KisAnimationCurvesView::paintEvent(QPaintEvent *e)
     int lastFrame = m_d->horizontalHeader->logicalIndexAt(r.right());
     if (lastFrame == -1) lastFrame = model()->columnCount();
 
+
+    paintFrames(painter);
+
     paintCurves(painter, firstFrame, lastFrame);
     paintKeyframes(painter, firstFrame, lastFrame);
 }
 
+void KisAnimationCurvesView::paintFrames(QPainter &painter)
+{
+    const QColor textColor = qApp->palette().text().color();
+    const QColor backgroundColor = qApp->palette().background().color();
+
+
+    // paint vertical lines so it is easier to tell where each frame starts/stops
+    QColor blendedColor = KritaUtils::blendColors(textColor, backgroundColor, 0.2);
+    painter.setPen(QPen(blendedColor, 1));
+    int channels = model()->rowCount();
+    for (int channel = 0; channel < channels; channel++) {
+
+        // draw border around entire frame, so override the height and Y position
+        for (int time = 0; time <= model()->columnCount(); time++) {
+
+            QModelIndex index = model()->index(channel, time);
+
+            int offset = 0;
+            if (m_d->horizontalHeader && m_d->horizontalHeader->offset()) {
+                offset = m_d->horizontalHeader->offset();
+            }
+
+            int horizontalStepSize = m_d->horizontalHeader->defaultSectionSize();
+            int xPosition = horizontalStepSize * time - offset;
+            QRect frameRect = QRect(xPosition, -10, horizontalStepSize, 9999);
+
+            painter.drawLine(frameRect.topRight(), frameRect.bottomRight());
+        }
+
+    }
+}
+
 void KisAnimationCurvesView::paintCurves(QPainter &painter, int firstFrame, int lastFrame)
 {
     int channels = model()->rowCount();
@@ -475,6 +511,7 @@ void KisAnimationCurvesView::mouseMoveEvent(QMouseEvent *e)
             m_d->verticalZoomButton->continueZoom(QPoint(0, e->pos().y()));
         }
     } else if (e->buttons() & Qt::LeftButton) {
+
         m_d->dragOffset = e->pos() - m_d->dragStart;
 
         if (m_d->isAdjustingHandle) {
@@ -497,6 +534,7 @@ void KisAnimationCurvesView::mouseMoveEvent(QMouseEvent *e)
 
 void KisAnimationCurvesView::mouseReleaseEvent(QMouseEvent *e)
 {
+
     if (e->button() == Qt::LeftButton) {
         m_d->panning = false;
 
@@ -553,6 +591,7 @@ void KisAnimationCurvesView::scrollContentsBy(int dx, int dy)
 
     scrollDirtyRegion(dx, dy);
     viewport()->scroll(dx, dy);
+    viewport()->update();
 }
 
 void KisAnimationCurvesView::updateGeometries()
@@ -588,8 +627,14 @@ void KisAnimationCurvesView::slotDataChanged(const QModelIndex &topLeft, const Q
 {
     Q_UNUSED(topLeft);
     Q_UNUSED(bottomRight);
+
     updateVerticalRange();
     viewport()->update();
+
+    // this forces the horizontal ruler to refresh. Repaint() doesn't do it for some reason
+    // If you remove this, scrubbing the timeline will probably stop updating the indicator
+    m_d->horizontalHeader->resize(m_d->horizontalHeader->width()-1, m_d->horizontalHeader->height());
+    m_d->horizontalHeader->resize(m_d->horizontalHeader->width()+1, m_d->horizontalHeader->height());
 }
 
 void KisAnimationCurvesView::slotHeaderDataChanged(Qt::Orientation orientation, int first, int last)
@@ -724,6 +769,7 @@ void KisAnimationCurvesView::removeKeyframes()
     m_d->model->removeFrames(selectedIndexes());
 }
 
+
 void KisAnimationCurvesView::zoomToFit()
 {
     if (!model()) return;
