@@ -33,6 +33,8 @@
 #include "KisImageBarrierLockerWithFeedback.h"
 #include "commands_new/kis_switch_current_time_command.h"
 #include "kis_command_utils.h"
+#include "KisPart.h"
+#include "kis_animation_cache_populator.h"
 
 struct KisTimeBasedItemModel::Private
 {
@@ -106,8 +108,6 @@ void KisTimeBasedItemModel::setImage(KisImageWSP image)
     if (image) {
         KisImageAnimationInterface *ai = image->animationInterface();
 
-        slotCurrentTimeChanged(ai->currentUITime());
-
         connect(ai, SIGNAL(sigFramerateChanged()), SLOT(slotFramerateChanged()));
         connect(ai, SIGNAL(sigUiTimeChanged(int)), SLOT(slotCurrentTimeChanged(int)));
     }
@@ -422,6 +422,15 @@ void KisTimeBasedItemModel::slotInternalScrubPreviewRequested(int time)
 void KisTimeBasedItemModel::setScrubState(bool active)
 {
     if (!m_d->scrubInProgress && active) {
+
+        if (m_d->framesCache) {
+            const int currentFrame = m_d->image->animationInterface()->currentUITime();
+            const bool hasCurrentFrameInCache = m_d->framesCache->frameStatus(currentFrame) == KisAnimationFrameCache::Cached;
+            if(!hasCurrentFrameInCache) {
+                KisPart::instance()->prioritizeFrameForCache(m_d->image, currentFrame);
+            }
+        }
+
         m_d->scrubStartFrame = m_d->activeFrameIndex;
         m_d->scrubInProgress = true;
     }
@@ -455,11 +464,6 @@ void KisTimeBasedItemModel::scrubTo(int time, bool preview)
     }
 }
 
-void KisTimeBasedItemModel::slotFramerateChanged()
-{
-    emit headerDataChanged(Qt::Horizontal, 0, columnCount() - 1);
-}
-
 void KisTimeBasedItemModel::slotCurrentTimeChanged(int time)
 {
     if (time != m_d->activeFrameIndex) {
@@ -467,6 +471,11 @@ void KisTimeBasedItemModel::slotCurrentTimeChanged(int time)
     }
 }
 
+void KisTimeBasedItemModel::slotFramerateChanged()
+{
+    emit headerDataChanged(Qt::Horizontal, 0, columnCount() - 1);
+}
+
 void KisTimeBasedItemModel::slotCacheChanged()
 {
     const int numFrames = columnCount();
