@@ -34,6 +34,7 @@
 #include "kis_undo_adapter.h"
 #include "kis_node_dummies_graph.h"
 #include "kis_dummies_facade_base.h"
+#include "KisNodeDisplayModeAdapter.h"
 #include "kis_signal_compressor.h"
 #include "kis_signal_compressor_with_param.h"
 #include "kis_keyframe_channel.h"
@@ -251,7 +252,9 @@ QMap<QString, KisKeyframeChannel*> TimelineFramesModel::channelsAt(QModelIndex i
     return srcDummy->node()->keyframeChannels();
 }
 
-void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade, KisImageSP image)
+void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade,
+                                           KisImageSP image,
+                                           KisNodeDisplayModeAdapter *displayModeAdapter)
 {
     KisDummiesFacadeBase *oldDummiesFacade = m_d->dummiesFacade;
 
@@ -268,7 +271,7 @@ void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade,
     m_d->converter.reset();
 
     if (m_d->dummiesFacade) {
-        m_d->converter.reset(new TimelineNodeListKeeper(this, m_d->dummiesFacade));
+        m_d->converter.reset(new TimelineNodeListKeeper(this, m_d->dummiesFacade, displayModeAdapter));
         connect(m_d->dummiesFacade, SIGNAL(sigDummyChanged(KisNodeDummy*)),
                 SLOT(slotDummyChanged(KisNodeDummy*)));
         connect(m_d->image->animationInterface(),
@@ -288,6 +291,7 @@ void TimelineFramesModel::setDummiesFacade(KisDummiesFacadeBase *dummiesFacade,
     if (m_d->dummiesFacade) {
         emit sigInfiniteTimelineUpdateNeeded();
         emit sigAudioChannelChanged();
+        slotCurrentTimeChanged(m_d->image->animationInterface()->currentUITime());
     }
 }
 
@@ -395,13 +399,7 @@ QVariant TimelineFramesModel::data(const QModelIndex &index, int role) const
         }
         const int maxSize = 200;
 
-        QSize size = dummy->node()->extent().size();
-        size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
-        if (size.width() == 0 || size.height() == 0) {
-            // No thumbnail can be shown if there isn't width or height...
-            return QVariant();
-        }
-        QImage image(dummy->node()->createThumbnailForFrame(size.width(), size.height(), index.column()));
+        QImage image(dummy->node()->createThumbnailForFrame(maxSize, maxSize, index.column(), Qt::KeepAspectRatio));
         return image;
     }
     }
@@ -623,6 +621,8 @@ bool TimelineFramesModel::canDropFrameData(const QMimeData */*data*/, const QMod
 {
     if (!index.isValid()) return false;
 
+    if ( !m_d->layerEditable(index.row()) ) return false;
+
     /**
      * Now we support D&D around any layer, so just return 'true' all
      * the time.
@@ -880,7 +880,7 @@ bool TimelineFramesModel::insertFrames(int dstColumn, const QList<int> &dstRows,
     return true;
 }
 
-bool TimelineFramesModel::insertHoldFrames(QModelIndexList selectedIndexes, int count)
+bool TimelineFramesModel::insertHoldFrames(const QModelIndexList &selectedIndexes, int count)
 {
     if (selectedIndexes.isEmpty() || count == 0) return true;
 
