@@ -28,6 +28,7 @@
 #include <QAction>
 #include <QStandardPaths>
 #include <QMenu>
+#include <QAction>
 
 #include <klocalizedstring.h>
 #include <kactioncollection.h>
@@ -49,6 +50,14 @@
 #include <kis_action_manager.h>
 #include <kis_action_registry.h>
 
+#include <dialogs/KisAsyncAnimationFramesSaveDialog.h>
+#include <animation/KisAnimationRenderingOptions.h>
+#include <animation/KisAnimationRender.h>
+#include <kis_image_animation_interface.h>
+#include <kis_time_range.h>
+#include <KisMimeDatabase.h>
+
+
 #include "compositionmodel.h"
 
 
@@ -63,11 +72,14 @@ CompositionDockerDock::CompositionDockerDock( )
     compositionView->installEventFilter(this);
     deleteButton->setIcon(KisIconUtils::loadIcon("edit-delete"));
     saveButton->setIcon(KisIconUtils::loadIcon("list-add"));
-    exportButton->setIcon(KisIconUtils::loadIcon("document-export"));
+    moveUpButton->setIcon(KisIconUtils::loadIcon("arrow-up"));
+    moveDownButton->setIcon(KisIconUtils::loadIcon("arrow-down"));
 
     deleteButton->setToolTip(i18n("Delete Composition"));
     saveButton->setToolTip(i18n("New Composition"));
-    exportButton->setToolTip(i18n("Export Composition"));
+    exportCompositions->setToolTip(i18n("Export Composition"));
+    moveUpButton->setToolTip(i18n("Move Composition Up"));
+    moveDownButton->setToolTip(i18n("Move Composition Down"));
 
     setWidget(widget);
 
@@ -80,7 +92,27 @@ CompositionDockerDock::CompositionDockerDock( )
 
     connect( deleteButton, SIGNAL(clicked(bool)), this, SLOT(deleteClicked()));
     connect( saveButton, SIGNAL(clicked(bool)), this, SLOT(saveClicked()));
-    connect( exportButton, SIGNAL(clicked(bool)), this, SLOT(exportClicked()));
+    connect( moveUpButton, SIGNAL(clicked(bool)), this, SLOT(moveCompositionUp()));
+    connect( moveDownButton, SIGNAL(clicked(bool)), this, SLOT(moveCompositionDown()));
+
+    QAction* imageAction = new QAction(KisIconUtils::loadIcon("document-export"), i18n("Export Images"), this);
+    connect(imageAction, SIGNAL(triggered(bool)), this, SLOT(exportImageClicked()));
+
+    QAction* animationAction = new QAction(KisIconUtils::loadIcon("addblankframe"), i18n("Export Animations"), this);
+    connect(animationAction, SIGNAL(triggered(bool)), this, SLOT(exportAnimationClicked()));
+
+    exportCompositions->setDefaultAction(imageAction);
+
+    QMenu* exportMenu = new QMenu(this);
+    exportMenu->addAction(imageAction);
+    exportMenu->addAction(animationAction);
+
+    exportCompositions->setMenu(exportMenu);
+
+    connect(exportMenu, &QMenu::triggered, [this](QAction* triggered){
+        exportCompositions->setDefaultAction(triggered);
+    });
+
     saveNameEdit->setPlaceholderText(i18n("Insert Name"));
 
     QScroller *scroller = KisKineticScroller::createPreconfiguredScroller(compositionView);
@@ -116,6 +148,7 @@ void CompositionDockerDock::setCanvas(KoCanvasBase * canvas)
             KisAction *renameAction = m_canvas->viewManager()->actionManager()->createAction("rename_composition");
             connect(renameAction, SIGNAL(triggered()), this, SLOT(renameComposition()));
             m_actions.append(renameAction);
+
         } else {
             Q_FOREACH (KisAction *action, m_actions) {
                 m_canvas->viewManager()->actionManager()->addAction(action->objectName(), action);
@@ -179,6 +212,28 @@ void CompositionDockerDock::saveClicked()
     image->setModified();
 }
 
+void CompositionDockerDock::moveCompositionUp()
+{
+    QModelIndex index = compositionView->currentIndex();
+    if (m_canvas && m_canvas->viewManager() && m_canvas->viewManager()->image() && index.isValid()) {
+        KisLayerCompositionSP composition = m_model->compositionFromIndex(index);
+        m_canvas->viewManager()->image()->moveCompositionUp(composition);
+        updateModel();
+        compositionView->setCurrentIndex(m_model->index(m_canvas->viewManager()->image()->compositions().indexOf(composition),0));
+    }
+}
+
+void CompositionDockerDock::moveCompositionDown()
+{
+    QModelIndex index = compositionView->currentIndex();
+    if (m_canvas && m_canvas->viewManager() && m_canvas->viewManager()->image() && index.isValid()) {
+        KisLayerCompositionSP composition = m_model->compositionFromIndex(index);
+        m_canvas->viewManager()->image()->moveCompositionDown(composition);
+        updateModel();
+        compositionView->setCurrentIndex(m_model->index(m_canvas->viewManager()->image()->compositions().indexOf(composition),0));
+    }
+}
+
 void CompositionDockerDock::updateModel()
 {
     if (m_model && m_canvas && m_canvas->viewManager() && m_canvas->viewManager()->image()) {
@@ -186,7 +241,7 @@ void CompositionDockerDock::updateModel()
     }
 }
 
-void CompositionDockerDock::exportClicked()
+void CompositionDockerDock::exportImageClicked()
 {
     if (m_canvas && m_canvas->viewManager() && m_canvas->viewManager()->image()) {
         QString path;
@@ -239,11 +294,10 @@ void CompositionDockerDock::exportClicked()
 
             d->setFileBatchMode(true);
 
-
             d->exportDocumentSync(QUrl::fromLocalFile(path + composition->name() + ".png"), "image/png");
             d->deleteLater();
-
         }
+
         currentComposition->apply();
         image->waitForDone();
         image->refreshGraph();
@@ -251,6 +305,75 @@ void CompositionDockerDock::exportClicked()
 
 }
 
+void CompositionDockerDock::exportAnimationClicked()
+{
+    KisConfig cfg(true);
+    KisPropertiesConfigurationSP settings = cfg.exportConfiguration("ANIMATION_EXPORT");
+    KisAnimationRenderingOptions exportOptions;
+    exportOptions.fromProperties(settings);
+
+    if (m_canvas &&
+        m_canvas->viewManager() &&
+        m_canvas->viewManager()->image() &&
+        m_canvas->viewManager()->image()->animationInterface() &&
+        m_canvas->viewManager()->document()) {
+
+        QString path;
+
+        KoFileDialog dialog(0, KoFileDialog::OpenDirectory, "compositiondockerdock");
+        dialog.setCaption(i18n("Select a Directory"));
+        dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation));
+        path = dialog.filename();
+
+
+        if (path.isNull()) return;
+
+        if (!path.endsWith('/')) {
+            path.append('/');
+        }
+
+        KisImageSP image = m_canvas->viewManager()->image();
+        QString filename = m_canvas->viewManager()->document()->localFilePath();
+        if (!filename.isEmpty()) {
+            QFileInfo info(filename);
+            path += info.completeBaseName();
+        }
+
+        KisLayerCompositionSP currentComposition = toQShared(new KisLayerComposition(image, "temp"));
+        currentComposition->store();
+
+        const QString frameMimeType = settings->getPropertyLazy("frame_mimetype", frameMimeType);
+        const QString imageExtension = KisMimeDatabase::suffixesForMimeType(frameMimeType).first();
+        const QString videoExtension = KisMimeDatabase::suffixesForMimeType(exportOptions.videoMimeType).first();
+
+        Q_FOREACH (KisLayerCompositionSP composition, image->compositions()) {
+            if(!composition->isExportEnabled())
+                continue;
+
+            composition->apply();
+            image->waitForDone();
+            image->refreshGraph();
+
+            KisTimeRange range = image->animationInterface()->fullClipRange();
+
+            exportOptions.firstFrame = range.start();
+            exportOptions.lastFrame = range.end();
+            exportOptions.width = image->width();
+            exportOptions.height = image->height();
+            exportOptions.videoFileName = QString("%1/%2/video.%3").arg(path).arg(composition->name()).arg(videoExtension);
+            exportOptions.directory = QString("%1/%2").arg(path).arg(composition->name());
+            exportOptions.basename = QString("frame");
+            exportOptions.wantsOnlyUniqueFrameSequence = true;
+
+            KisAnimationRender::render(m_canvas->viewManager()->document(), m_canvas->viewManager(), exportOptions);
+        }
+
+        currentComposition->apply();
+        image->waitForDone();
+        image->refreshGraph();
+    }
+}
+
 bool CompositionDockerDock::eventFilter(QObject* obj, QEvent* event)
 {
     if (event->type() == QEvent::KeyPress ) {
