@@ -68,7 +68,7 @@
 #include <kis_action.h>
 #include "kis_action_manager.h"
 #include "widgets/kis_cmb_composite.h"
-#include "widgets/kis_slider_spin_box.h"
+#include "kis_slider_spin_box.h"
 #include "KisViewManager.h"
 #include "kis_node_manager.h"
 #include "kis_node_model.h"
@@ -194,7 +194,7 @@ LayerBox::LayerBox()
         m_wdgLayerBox->doubleOpacity->setPrefix(QString("%1:  ").arg(i18n("Opacity")));
     }
     m_wdgLayerBox->doubleOpacity->setRange(0, 100, 0);
-    m_wdgLayerBox->doubleOpacity->setSuffix("%");
+    m_wdgLayerBox->doubleOpacity->setSuffix(i18n("%"));
 
     connect(m_wdgLayerBox->doubleOpacity, SIGNAL(valueChanged(qreal)), SLOT(slotOpacitySliderMoved(qreal)));
     connect(&m_opacityDelayTimer, SIGNAL(timeout()), SLOT(slotOpacityChanged()));
@@ -221,6 +221,15 @@ LayerBox::LayerBox()
     connect(m_nodeModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(updateUI()));
     connect(m_nodeModel, SIGNAL(modelReset()), SLOT(slotModelReset()));
 
+    connect(m_nodeModel, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(slotForgetAboutSavedNodeBeforeEditSelectionMode()));
+    connect(m_nodeModel, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(slotForgetAboutSavedNodeBeforeEditSelectionMode()));
+    connect(m_nodeModel, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)), SLOT(slotForgetAboutSavedNodeBeforeEditSelectionMode()));
+    connect(m_nodeModel, SIGNAL(modelReset()), SLOT(slotForgetAboutSavedNodeBeforeEditSelectionMode()));
+
+    // we should update expanded state of the nodes on adding the nodes
+    connect(m_nodeModel, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(slotNodeCollapsedChanged()));
+    connect(m_nodeModel, SIGNAL(modelReset()), SLOT(slotNodeCollapsedChanged()));
+
     KisAction *showGlobalSelectionMask = new KisAction(i18n("&Show Global Selection Mask"), this);
     showGlobalSelectionMask->setObjectName("show-global-selection-mask");
     showGlobalSelectionMask->setActivationFlags(KisAction::ACTIVE_IMAGE);
@@ -242,8 +251,8 @@ LayerBox::LayerBox()
     m_wdgLayerBox->listLayers->setModel(m_filteringModel);
     // this connection should be done *after* the setModel() call to
     // happen later than the internal selection model
-    connect(m_filteringModel.data(), &KisNodeFilterProxyModel::rowsAboutToBeRemoved,
-            this, &LayerBox::slotAboutToRemoveRows);
+    connect(m_filteringModel.data(), &KisNodeFilterProxyModel::sigBeforeBeginRemoveRows,
+            this, &LayerBox::slotAdjustCurrentBeforeRemoveRows);
 
     connect(m_wdgLayerBox->cmbFilter, SIGNAL(selectedColorsChanged()), SLOT(updateLayerFiltering()));
 
@@ -334,6 +343,10 @@ void LayerBox::setViewManager(KisViewManager* kisview)
 {
     m_nodeManager = kisview->nodeManager();
 
+    if (m_nodeManager) {
+        connect(m_nodeManager, SIGNAL(sigNodeActivated(KisNodeSP)), SLOT(slotForgetAboutSavedNodeBeforeEditSelectionMode()));
+    }
+
     Q_FOREACH (KisAction *action, m_actions) {
         kisview->actionManager()->
                 addAction(action->objectName(),
@@ -494,17 +507,26 @@ void LayerBox::updateUI()
     KisNodeSP activeNode = m_nodeManager->activeNode();
 
     if (activeNode != m_activeNode) {
-        if( !m_activeNode.isNull() )
-            m_activeNode->disconnect(this);
+        m_activeNodeConnections.clear();
         m_activeNode = activeNode;
 
         if (activeNode) {
+            KisPaintDeviceSP parentLayerDevice = activeNode->parent() ? activeNode->parent()->original() : 0;
+            if (parentLayerDevice) {
+                // update blending modes availability
+                m_activeNodeConnections.addConnection(
+                     parentLayerDevice, SIGNAL(colorSpaceChanged(const KoColorSpace*)),
+                     this, SLOT(updateUI()));
+            }
+
             KisKeyframeChannel *opacityChannel = activeNode->getKeyframeChannel(KisKeyframeChannel::Opacity.id(), false);
             if (opacityChannel) {
                 watchOpacityChannel(opacityChannel);
             } else {
                 watchOpacityChannel(0);
-                connect(activeNode.data(), &KisNode::keyframeChannelAdded, this, &LayerBox::slotKeyframeChannelAdded);
+                m_activeNodeConnections.addConnection(
+                    activeNode, SIGNAL(keyframeChannelAdded(KisKeyframeChannel*)),
+                    this, SLOT(slotKeyframeChannelAdded(KisKeyframeChannel*)));
             }
         }
     }
@@ -517,11 +539,9 @@ void LayerBox::updateUI()
     m_wdgLayerBox->doubleOpacity->setEnabled(activeNode && activeNode->isEditable(false));
 
     m_wdgLayerBox->cmbComposite->setEnabled(activeNode && activeNode->isEditable(false));
-    m_wdgLayerBox->cmbComposite->validate(m_image->colorSpace());
 
     if (activeNode) {
-        if (activeNode->inherits("KisColorizeMask") ||
-                activeNode->inherits("KisLayer")) {
+        if (activeNode->inherits("KisColorizeMask") || activeNode->inherits("KisLayer")) {
 
             m_wdgLayerBox->doubleOpacity->setEnabled(true);
 
@@ -531,6 +551,9 @@ void LayerBox::updateUI()
 
             const KoCompositeOp* compositeOp = activeNode->compositeOp();
             if (compositeOp) {
+                /// the composite op works in the color space of the parent layer,
+                /// not the active one.
+                m_wdgLayerBox->cmbComposite->validate(compositeOp->colorSpace());
                 slotSetCompositeOp(compositeOp);
             } else {
                 m_wdgLayerBox->cmbComposite->setEnabled(false);
@@ -636,6 +659,8 @@ void LayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex &in
             menu.addAction(m_removeAction);
             addActionToMenu(&menu, "duplicatelayer");
             addActionToMenu(&menu, "merge_layer");
+            addActionToMenu(&menu, "new_from_visible");
+
 
             if (singleLayer) {
                 addActionToMenu(&menu, "flatten_image");
@@ -666,6 +691,9 @@ void LayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex &in
                 addActionToMenu(addLayerMenu, "add_new_colorize_mask");
                 addActionToMenu(addLayerMenu, "add_new_transform_mask");
                 addActionToMenu(addLayerMenu, "add_new_selection_mask");
+                addLayerMenu->addSeparator();
+                addActionToMenu(addLayerMenu, "add_new_clone_layer");
+
 
                 QMenu *convertToMenu = menu.addMenu(i18n("&Convert"));
                 addActionToMenu(convertToMenu, "convert_to_paint_layer");
@@ -678,6 +706,9 @@ void LayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex &in
                 addActionToMenu(splitAlphaMenu, "split_alpha_into_mask");
                 addActionToMenu(splitAlphaMenu, "split_alpha_write");
                 addActionToMenu(splitAlphaMenu, "split_alpha_save_merged");
+            } else {
+                QMenu *addLayerMenu = menu.addMenu(i18n("&Add"));
+                addActionToMenu(addLayerMenu, "add_new_clone_layer");
             }
 
             menu.addSeparator();
@@ -687,7 +718,7 @@ void LayerBox::slotContextMenuRequested(const QPoint &pos, const QModelIndex &in
             if (singleLayer) {
                 KisNodeSP node = m_filteringModel->nodeFromIndex(index);
                 if (node && !node->inherits("KisTransformMask")) {
-                    addActionToMenu(&menu, "isolate_layer");
+                    addActionToMenu(&menu, "isolate_active_layer");
                 }
 
                 addActionToMenu(&menu, "selectopaque");
@@ -795,7 +826,9 @@ void LayerBox::slotSelectOpaque()
 
 void LayerBox::slotNodeCollapsedChanged()
 {
-    expandNodesRecursively(m_image->rootLayer(), m_filteringModel, m_wdgLayerBox->listLayers);
+    if (m_nodeModel->hasDummiesFacade()) {
+        expandNodesRecursively(m_image->rootLayer(), m_filteringModel, m_wdgLayerBox->listLayers);
+    }
 }
 
 inline bool isSelectionMask(KisNodeSP node)
@@ -843,7 +876,10 @@ void LayerBox::slotEditGlobalSelection(bool showSelections)
     KisSelectionMaskSP globalSelectionMask;
 
     if (!showSelections) {
-        activateNode = findNonHidableNode(activateNode);
+        activateNode =
+            m_savedNodeBeforeEditSelectionMode ?
+                KisNodeSP(m_savedNodeBeforeEditSelectionMode) :
+                findNonHidableNode(activateNode);
     }
 
     m_nodeModel->setShowGlobalSelection(showSelections);
@@ -912,6 +948,9 @@ void LayerBox::slotEditGlobalSelection(bool showSelections)
         applicator.end();
     }
 
+    if (showSelections) {
+        m_savedNodeBeforeEditSelectionMode = lastActiveNode;
+    }
 }
 
 void LayerBox::selectionChanged(const QModelIndexList selection)
@@ -941,17 +980,17 @@ void LayerBox::selectionChanged(const QModelIndexList selection)
     updateUI();
 }
 
-void LayerBox::slotAboutToRemoveRows(const QModelIndex &parent, int start, int end)
+void LayerBox::slotAdjustCurrentBeforeRemoveRows(const QModelIndex &parent, int start, int end)
 {
     /**
      * Qt has changed its behavior when deleting an item. Previously
      * the selection priority was on the next item in the list, and
      * now it has shanged to the previous item. Here we just adjust
-     * the selected item after the node removal. Please take care that
-     * this method overrides what was done by the corresponding method
-     * of QItemSelectionModel, which *has already done* its work. That
-     * is why we use (start - 1) and (end + 1) in the activation
-     * condition.
+     * the selected item after the node removal.
+     *
+     * This method is called right before the Qt's beginRemoveRows()
+     * is called, that is we make sure that Qt will never have to
+     * adjust the position of the removed cursor.
      *
      * See bug: https://bugs.kde.org/show_bug.cgi?id=345601
      */
@@ -960,11 +999,12 @@ void LayerBox::slotAboutToRemoveRows(const QModelIndex &parent, int start, int e
     QAbstractItemModel *model = m_filteringModel;
 
     if (currentIndex.isValid() && parent == currentIndex.parent()
-            && currentIndex.row() >= start - 1 && currentIndex.row() <= end + 1) {
+            && currentIndex.row() >= start && currentIndex.row() <= end) {
         QModelIndex old = currentIndex;
+
         if (model && end < model->rowCount(parent) - 1) // there are rows left below the change
             currentIndex = model->index(end + 1, old.column(), parent);
-        else if (start > 0) // there are rows left above the change
+        else if (model && start > 0) // there are rows left above the change
             currentIndex = model->index(start - 1, old.column(), parent);
         else // there are no rows left in the table
             currentIndex = QModelIndex();
@@ -1074,6 +1114,11 @@ void LayerBox::slotImageTimeChanged(int time)
     updateUI();
 }
 
+void LayerBox::slotForgetAboutSavedNodeBeforeEditSelectionMode()
+{
+    m_savedNodeBeforeEditSelectionMode = 0;
+}
+
 void LayerBox::slotUpdateIcons() {
     m_wdgLayerBox->bnAdd->setIcon(KisIconUtils::loadIcon("addlayer"));
     m_wdgLayerBox->bnRaise->setIcon(KisIconUtils::loadIcon("arrowupblr"));
