@@ -79,6 +79,9 @@ public:
 
     bool stasisIsDirty(const QModelIndex &root, const OptionalProperty &clickedProperty, bool on = false, bool off = false);
     void resetPropertyStateRecursive(const QModelIndex &root, const OptionalProperty &clickedProperty);
+    void restorePropertyInStasisRecursive(const QModelIndex &root, const OptionalProperty &clickedProperty);
+
+    bool checkImmediateStasis(const QModelIndex &root, const OptionalProperty &clickedProperty);
 
     void getParentsIndex(QList<QModelIndex> &items, const QModelIndex &index);
     void getChildrenIndex(QList<QModelIndex> &items, const QModelIndex &index);
@@ -323,17 +326,17 @@ void NodeDelegate::drawThumbnail(QPainter *p, const QStyleOptionViewItem &option
     KisNodeViewColorScheme scm;
 
     const int thumbSize = scm.thumbnailSize();
+    const qreal devicePixelRatio = p->device()->devicePixelRatioF();
+    const int thumbSizeHighRes = thumbSize*devicePixelRatio;
+
     const qreal oldOpacity = p->opacity(); // remember previous opacity
 
-    QImage img = index.data(int(KisNodeModel::BeginThumbnailRole) + thumbSize).value<QImage>();
+    QImage img = index.data(int(KisNodeModel::BeginThumbnailRole) + thumbSizeHighRes).value<QImage>();
+    img.setDevicePixelRatio(devicePixelRatio);
     if (!(option.state & QStyle::State_Enabled)) {
         p->setOpacity(0.35);
     }
 
-    QRect fitRect = thumbnailClickRect(option, index);
-    // Shrink to icon rect
-    fitRect = kisGrowRect(fitRect, -(scm.thumbnailMargin()+scm.border()));
-
     // paint in a checkerboard pattern behind the layer contents to represent transparent
     const int step = scm.thumbnailSize() / 6;
     QImage checkers(2 * step, 2 * step, QImage::Format_ARGB32);
@@ -343,13 +346,25 @@ void NodeDelegate::drawThumbnail(QPainter *p, const QStyleOptionViewItem &option
     gc.fillRect(QRect(step, step, step, step), d->checkersColor1);
     gc.fillRect(QRect(0, step, step, step), d->checkersColor2);
 
-    QBrush brush(checkers);
-    p->fillRect(fitRect, brush);
 
-    p->drawImage(fitRect, img);
+    QRect fitRect = thumbnailClickRect(option, index);
+    // Shrink to icon rect
+    fitRect = kisGrowRect(fitRect, -(scm.thumbnailMargin()+scm.border()));
+
+    QPoint offset;
+    offset.setX((fitRect.width() - img.width()/devicePixelRatio) / 2);
+    offset.setY((fitRect.height() - img.height()/devicePixelRatio) / 2);
+    offset += fitRect.topLeft();
+
+    QBrush brush(checkers);
+    p->setBrushOrigin(offset);
+    QRect imageRectLowRes = QRect(img.rect().topLeft(), img.rect().size()/devicePixelRatio);
+    p->fillRect(imageRectLowRes.translated(offset), brush);
+
+    p->drawImage(offset, img);
     p->setOpacity(oldOpacity); // restore old opacity
 
-    QRect borderRect = kisGrowRect(fitRect, 1);
+    QRect borderRect = kisGrowRect(imageRectLowRes, 1).translated(offset);
     KritaUtils::renderExactRect(p, borderRect, scm.gridColor(option, d->view));
 }
 
@@ -528,11 +543,21 @@ void NodeDelegate::Private::toggleProperty(KisBaseNode::PropertyList &props, Opt
         togglePropertyRecursive(root, clickedProperty, items, record, mode);
 
     } else {
-        shiftClickedIndexes.clear();
-        resetPropertyStateRecursive(root, clickedProperty);
-        clickedProperty->state = !clickedProperty->state.toBool();
-        clickedProperty->isInStasis = false;
-        view->model()->setData(index, QVariant::fromValue(props), KisNodeModel::PropertiesRole);
+        // If we have properties in stasis, we need to cancel stasis to avoid overriding
+        // values in stasis.
+        // IMPORTANT -- we also need to check the first row of nodes to determine
+        // if a stasis is currently active in some cases.
+        const bool hasPropInStasis = (shiftClickedIndexes.count() > 0 || checkImmediateStasis(root, clickedProperty));
+        if (clickedProperty->canHaveStasis && hasPropInStasis) {
+            shiftClickedIndexes.clear();
+            restorePropertyInStasisRecursive(root, clickedProperty);
+        } else {
+            shiftClickedIndexes.clear();
+            resetPropertyStateRecursive(root, clickedProperty);
+            clickedProperty->state = !clickedProperty->state.toBool();
+            clickedProperty->isInStasis = false;
+            view->model()->setData(index, QVariant::fromValue(props), KisNodeModel::PropertiesRole);
+        }
     }
 }
 
@@ -555,7 +580,7 @@ void NodeDelegate::Private::togglePropertyRecursive(const QModelIndex &root, con
         if (record == StasisOperation::Review || record ==  StasisOperation::Record) {
             prop->isInStasis = true;
             if(mode) { //include mode
-                prop->state = (items.contains(idx))? QVariant(true) : QVariant(false);
+                prop->state = (items.contains(idx)) ? QVariant(true) : QVariant(false);
             } else { // exclude
                 prop->state = (!items.contains(idx))? prop->state :
                               (items.at(0) == idx)? QVariant(true) : QVariant(false);
@@ -619,6 +644,45 @@ void NodeDelegate::Private::resetPropertyStateRecursive(const QModelIndex &root,
     }
 }
 
+void NodeDelegate::Private::restorePropertyInStasisRecursive(const QModelIndex &root, const OptionalProperty &clickedProperty)
+{
+    if (!clickedProperty->canHaveStasis) return;
+    int rowCount = view->model()->rowCount(root);
+
+    for (int i = 0; i < rowCount; i++) {
+        QModelIndex idx = view->model()->index(i, 0, root);
+        KisBaseNode::PropertyList props = idx.data(KisNodeModel::PropertiesRole).value<KisBaseNode::PropertyList>();
+        OptionalProperty prop = findProperty(props, clickedProperty);
+
+        if (prop->isInStasis) {
+            prop->isInStasis = false;
+            prop->state = QVariant(prop->stateInStasis);
+        }
+
+        view->model()->setData(idx, QVariant::fromValue(props), KisNodeModel::PropertiesRole);
+
+        restorePropertyInStasisRecursive(idx, clickedProperty);
+    }
+}
+
+bool NodeDelegate::Private::checkImmediateStasis(const QModelIndex &root, const OptionalProperty &clickedProperty)
+{
+    if (!clickedProperty->canHaveStasis) return false;
+
+    const int rowCount = view->model()->rowCount(root);
+    for (int i = 0; i < rowCount; i++){
+        QModelIndex idx = view->model()->index(i, 0, root);
+        KisBaseNode::PropertyList props = idx.data(KisNodeModel::PropertiesRole).value<KisBaseNode::PropertyList>();
+        OptionalProperty prop = findProperty(props, clickedProperty);
+
+        if (prop->isInStasis) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void NodeDelegate::Private::getParentsIndex(QList<QModelIndex> &items, const QModelIndex &index)
 {
     if (!index.isValid()) return;
