@@ -27,40 +27,12 @@
 
 #include <QMutex>
 #include "kis_idle_watcher.h"
-#include "kis_simple_stroke_strategy.h"
 
 #include <kis_canvas2.h>
 
 class KisSignalCompressor;
 class KoCanvasBase;
 
-class OverviewThumbnailStrokeStrategy : public QObject, public KisSimpleStrokeStrategy
-{
-    Q_OBJECT
-public:
-    OverviewThumbnailStrokeStrategy(KisImageWSP image);
-    ~OverviewThumbnailStrokeStrategy() override;
-
-    static QList<KisStrokeJobData*> createJobsData(KisPaintDeviceSP dev, const QRect& imageRect, KisPaintDeviceSP thumbDev, const QSize &thumbnailSize);
-
-private:
-    void initStrokeCallback() override;
-    void doStrokeCallback(KisStrokeJobData *data) override;
-    void finishStrokeCallback() override;
-    void cancelStrokeCallback() override;
-
-Q_SIGNALS:
-    //Emitted when thumbnail is updated and overviewImage is fully generated.
-    void thumbnailUpdated(QImage pixmap);
-
-
-private:
-    struct Private;
-    const QScopedPointer<Private> m_d;
-    QMutex m_thumbnailMergeMutex;
-    KisImageSP m_image;
-};
-
 class OverviewWidget : public QWidget
 {
     Q_OBJECT
@@ -93,19 +65,38 @@ protected:
     void wheelEvent(QWheelEvent* event) override;
 
 private:
-    QSize recalculatePreviewSize();
-    QPointF previewOrigin();
+    void recalculatePreviewDimensions();
+    ///
+    /// \brief isPixelArt checks if the preview is bigger than the image itself
+    ///
+    /// We assume that if the preview is bigger than the image itself,
+    /// (1) the image is very small,
+    ///     (might not be true if the docker is huge and the user is on 4k display...)
+    /// (2) the image is pixel art, so they would like to see pixels.
+    /// In that case we'd like to use a different algorithm (Nearest Neighbour, called "Box")
+    ///    in the OverviewThumbnailStrokeStrategy.
+    /// \return
+    ///
+    bool isPixelArt();
+
+    QPointF calculatePreviewOrigin(QSize previewSize);
     QTransform canvasToPreviewTransform();
     QTransform previewToCanvasTransform();
     QPolygonF previewPolygon();
 
-    qreal m_previewScale;
+    qreal m_previewScale {1.0};
     QPixmap m_oldPixmap;
     QPixmap m_pixmap;
+    QImage m_image;
     QPointer<KisCanvas2> m_canvas;
 
-    bool m_dragging;
-    QPointF m_lastPos;
+
+    QPointF m_previewOrigin; // in the same coordinates space as m_previewSize
+    QSize m_previewSize {QSize(100, 100)};
+
+
+    bool m_dragging {false};
+    QPointF m_lastPos {QPointF(0, 0)};
 
     QColor m_outlineColor;
     KisIdleWatcher m_imageIdleWatcher;
