@@ -36,6 +36,7 @@
 #include <kis_canvas2.h>
 #include <kis_mainwindow_observer.h>
 #include <KisView.h>
+#include <kis_signal_auto_connection.h>
 
 class KisViewManager;
 class KisCanvasResourceProvider;
@@ -46,7 +47,7 @@ class KisPaletteModel;
 class KisPaletteEditor;
 class Ui_WdgPaletteDock;
 
-class PaletteDockerDock : public QDockWidget, public KisMainwindowObserver
+class PaletteDockerDock : public QDockWidget, public KisMainwindowObserver, public KoResourceServerObserver<KoColorSet>
 {
     Q_OBJECT
 public:
@@ -60,6 +61,15 @@ public: // QDockWidget
 public: // KisMainWindowObserver
     void setViewManager(KisViewManager* kisview) override;
 
+public: //KoResourceServerObserver
+    void unsetResourceServer() override;
+    void resourceAdded(KoColorSet *resource) override;
+    void removingResource(KoColorSet *resource) override;
+    void resourceChanged(KoColorSet *resource) override;
+    void syncTaggedResourceView() override;
+    void syncTagAddition(const QString& tag) override;
+    void syncTagRemoval(const QString& tag) override;
+
 private Q_SLOTS:
     void slotContextMenu(const QModelIndex &);
 
@@ -83,6 +93,7 @@ private Q_SLOTS:
     void loadFromWorkspace(KisWorkspaceResource* workspace);
 
     void slotFGColorResourceChanged(const KoColor& color);
+    void slotUpdatePaletteList(const QList<KoColorSet *> &oldPaletteList, const QList<KoColorSet *> &newPaletteList);
 
 private:
     void setEntryByForeground(const QModelIndex &index);
@@ -109,6 +120,8 @@ private /* member variables */:
     QMenu m_viewContextMenu;
 
     bool m_colorSelfUpdate;
+
+    KisSignalAutoConnectionsStore m_connections;
 };
 
 
