@@ -22,14 +22,14 @@
 #include <QVariant>
 
 #include <QUrl>
-#include <kis_view_plugin.h>
+#include <KisActionPlugin.h>
 
-class LayerSplit : public KisViewPlugin
+class LayerSplit : public KisActionPlugin
 {
     Q_OBJECT
 public:
     LayerSplit(QObject *parent, const QVariantList &);
-    virtual ~LayerSplit();
+    ~LayerSplit() override;
 
 private Q_SLOTS:
 
