@@ -88,21 +88,6 @@ namespace PyKrita
         Python::ensureInitialized();
         Python py = Python();
 
-        // NOTE: This code is not needed on Python 3.
-        //       This might also fail if private sip module for PyQt5 is used,
-        //       as required by newer PyQt5. It's fine since any exceptions
-        //       in this script will be ignored.
-        PyRun_SimpleString(
-                "import sip\n"
-                        "sip.setapi('QDate', 2)\n"
-                        "sip.setapi('QTime', 2)\n"
-                        "sip.setapi('QDateTime', 2)\n"
-                        "sip.setapi('QUrl', 2)\n"
-                        "sip.setapi('QTextStream', 2)\n"
-                        "sip.setapi('QString', 2)\n"
-                        "sip.setapi('QVariant', 2)\n"
-        );
-
         // Initialize 'plugins' dict of module 'pykrita'
         PyObject* plugins = PyDict_New();
         py.itemStringSet("plugins", plugins);
@@ -332,7 +317,14 @@ bool Python::setPath(const QStringList& scriptPaths)
 //             << (!qgetenv("APPDIR").isNull() && KoResourcePaths::getApplicationRoot().contains(qgetenv("APPDIR")));
 
 
-    bool runningInBundle = ((!qgetenv("APPDIR").isNull() && KoResourcePaths::getApplicationRoot().contains(qgetenv("APPDIR"))) || KoResourcePaths::getApplicationRoot().toLower().contains("krita.app"));
+#if defined Q_OS_WIN
+    bool runningInBundle = false;
+#elif defined Q_OS_MAC
+    bool runningInBundle = KoResourcePaths::getApplicationRoot().toLower().contains("krita.app");
+#else
+    bool runningInBundle = (!qgetenv("APPDIR").isNull() &&
+                             KoResourcePaths::getApplicationRoot().contains(qgetenv("APPDIR")));
+#endif
     dbgScript << "Python::setPath. Script paths:" << scriptPaths << runningInBundle;
 
 #ifdef Q_OS_WIN
@@ -372,7 +364,7 @@ bool Python::setPath(const QStringList& scriptPaths)
         dbgScript << "Found bundled Python at" << pythonDir.absolutePath();
         // The default paths for Windows embeddable Python is ./python38.zip;./
         // HACK: Assuming bundled Python is version 3.8.*
-        // FIXME: Should we read python38._pth for the paths or use Py_GetPath?
+        // FIXME: Should we read python36._pth for the paths or use Py_GetPath?
         paths.append(pythonDir.absoluteFilePath("python38.zip"));
         paths.append(pythonDir.absolutePath());
     } else {
