@@ -61,6 +61,7 @@
 #include "kis_qmic_simple_convertor.h"
 #include "kis_import_qmic_processing_visitor.h"
 #include <PluginSettings.h>
+#include <kis_image_barrier_locker.h>
 
 #include "kis_qmic_applicator.h"
 
@@ -391,7 +392,7 @@ bool QMic::prepareCroppedImages(QByteArray *message, QRectF &rc, int inputMode)
 {
     if (!viewManager()) return false;
 
-    viewManager()->image()->lock();
+    KisImageBarrierLocker locker(viewManager()->image());
 
     m_inputMode = (InputLayerMode)inputMode;
 
@@ -400,7 +401,6 @@ bool QMic::prepareCroppedImages(QByteArray *message, QRectF &rc, int inputMode)
     KisInputOutputMapper mapper(viewManager()->image(), viewManager()->activeNode());
     KisNodeListSP nodes = mapper.inputNodes(m_inputMode);
     if (nodes->isEmpty()) {
-        viewManager()->image()->unlock();
         return false;
     }
 
@@ -454,8 +454,6 @@ bool QMic::prepareCroppedImages(QByteArray *message, QRectF &rc, int inputMode)
 
     dbgPlugins << QString::fromUtf8(*message);
 
-    viewManager()->image()->unlock();
-
     return true;
 }
 
