@@ -28,8 +28,8 @@ K_PLUGIN_FACTORY_WITH_JSON(DodgeBurnPluginFactory, "kritadodgeburn.json", regist
 DodgeBurnPlugin::DodgeBurnPlugin(QObject *parent, const QVariantList &)
 {
     Q_UNUSED(parent);
-    KisFilterRegistry::instance()->add(new KisFilterDodgeBurn("dodge", "Dodge", i18n("Dodge")));
-    KisFilterRegistry::instance()->add(new KisFilterDodgeBurn("burn", "Burn", i18n("Burn")));
+    KisFilterRegistry::instance()->add(new KisFilterDodgeBurn("dodge", "Dodge", i18n("Dodge...")));
+    KisFilterRegistry::instance()->add(new KisFilterDodgeBurn("burn", "Burn", i18n("Burn...")));
 }
 
 DodgeBurnPlugin::~DodgeBurnPlugin()
