@@ -107,9 +107,9 @@ void KisEdgeDetectionFilter::processImpl(KisPaintDeviceSP device, const QRect &r
                                                config->getBool("transparency", false));
 }
 
-KisFilterConfigurationSP KisEdgeDetectionFilter::factoryConfiguration() const
+KisFilterConfigurationSP KisEdgeDetectionFilter::defaultConfiguration() const
 {
-    KisFilterConfigurationSP config = new KisFilterConfiguration(id().id(), 1);
+    KisFilterConfigurationSP config = factoryConfiguration();
     config->setProperty("horizRadius", 1);
     config->setProperty("vertRadius", 1);
     config->setProperty("type", "prewitt");
