@@ -33,7 +33,6 @@
 #include <kis_paint_device.h>
 #include <kundo2command.h>
 #include <KoColorSpaceRegistry.h>
-#include <KisImportExportManager.h>
 #include <kis_file_name_requester.h>
 #include "ui_wdgfastcolortransfer.h"
 
@@ -41,8 +40,8 @@ KisWdgFastColorTransfer::KisWdgFastColorTransfer(QWidget * parent) : KisConfigWi
 {
     m_widget = new Ui_WdgFastColorTransfer();
     m_widget->setupUi(this);
-    m_widget->fileNameURLRequester->setMimeTypeFilters(KisImportExportManager::mimeFilter("application/x-krita", KisImportExportManager::Import));
-    connect(m_widget->fileNameURLRequester, SIGNAL(textChanged(const QString&)), this, SIGNAL(sigConfigurationItemChanged()));
+    m_widget->fileNameURLRequester->setMimeTypeFilters(KisImportExportManager::supportedMimeTypes(KisImportExportManager::Import));
+    connect(m_widget->fileNameURLRequester, SIGNAL(textChanged(QString)), this, SIGNAL(sigConfigurationItemChanged()));
 }
 
 
@@ -51,7 +50,7 @@ KisWdgFastColorTransfer::~KisWdgFastColorTransfer()
     delete m_widget;
 }
 
-void KisWdgFastColorTransfer::setConfiguration(const KisPropertiesConfiguration* config)
+void KisWdgFastColorTransfer::setConfiguration(const KisPropertiesConfigurationSP config)
 {
     QVariant value;
     if (config->getProperty("filename", value)) {
@@ -60,9 +59,9 @@ void KisWdgFastColorTransfer::setConfiguration(const KisPropertiesConfiguration*
 
 }
 
-KisPropertiesConfiguration* KisWdgFastColorTransfer::configuration() const
+KisPropertiesConfigurationSP KisWdgFastColorTransfer::configuration() const
 {
-    KisFilterConfiguration* config = new KisFilterConfiguration("colortransfer", 1);
+    KisFilterConfigurationSP config = new KisFilterConfiguration("colortransfer", 1);
     QString fileName = this->widget()->fileNameURLRequester->fileName();
 
     if (fileName.isEmpty()) return config;
@@ -74,9 +73,8 @@ KisPropertiesConfiguration* KisWdgFastColorTransfer::configuration() const
     KisDocument *d = KisPart::instance()->createDocument();
 
     KisImportExportManager manager(d);
-    KisImportExportFilter::ConversionStatus status;
-    QString s = manager.importDocument(fileName, QString(), status);
-    dbgPlugins << "import returned" << s << "and status" << status;
+    KisImportExportErrorCode status = manager.importDocument(fileName, QString());
+    dbgPlugins << "import returned status" << status.errorMessage();
     KisImageWSP importedImage = d->image();
 
     if (importedImage) {
@@ -97,15 +95,14 @@ KisPropertiesConfiguration* KisWdgFastColorTransfer::configuration() const
     }
 
     dbgPlugins << "convert ref to lab";
-    KUndo2Command* cmd = ref->convertTo(labCS, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
-    delete cmd;
+    ref->convertTo(labCS, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
 
     // Compute the means and sigmas of ref
     double meanL_ref = 0., meanA_ref = 0., meanB_ref = 0.;
     double sigmaL_ref = 0., sigmaA_ref = 0., sigmaB_ref = 0.;
 
     KisSequentialConstIterator refIt(ref, importedImage->bounds());
-    do {
+    while (refIt.nextPixel()) {
         const quint16* data = reinterpret_cast<const quint16*>(refIt.oldRawData());
 
         quint32 L = data[0];
@@ -120,7 +117,7 @@ KisPropertiesConfiguration* KisWdgFastColorTransfer::configuration() const
         sigmaA_ref += A * A;
         sigmaB_ref += B * B;
 
-    } while (refIt.nextPixel());
+    }
 
     double totalSize = 1. / (importedImage->width() * importedImage->height());
 
