@@ -190,13 +190,13 @@ KisConfigWidget * KisOilPaintFilter::createConfigurationWidget(QWidget* parent,
     param.push_back(KisIntegerWidgetParam(1, 5, 1, i18n("Brush size"), "brushSize"));
     param.push_back(KisIntegerWidgetParam(10, 255, 30, i18nc("smooth out the painting strokes the filter creates", "Smooth"), "smooth"));
     KisMultiIntegerFilterWidget * w = new KisMultiIntegerFilterWidget(id().id(),  parent,  id().id(),  param);
-    w->setConfiguration(factoryConfiguration());
+    w->setConfiguration(defaultConfiguration());
     return w;
 }
 
-KisFilterConfigurationSP KisOilPaintFilter::factoryConfiguration() const
+KisFilterConfigurationSP KisOilPaintFilter::defaultConfiguration() const
 {
-    KisFilterConfigurationSP config = new KisFilterConfiguration("oilpaint", 1);
+    KisFilterConfigurationSP config = factoryConfiguration();
     config->setProperty("brushSize", 1);
     config->setProperty("smooth", 30);
 
