@@ -28,6 +28,9 @@ class KisAnimatedBrushAnnotation : public KisAnnotation
 {
 public:
     KisAnimatedBrushAnnotation(const KisPipeBrushParasite &parasite);
+    KisAnnotation* clone() const Q_DECL_OVERRIDE {
+        return new KisAnimatedBrushAnnotation(*this);
+    }
 };
 
 #endif // KISANIMATEDBRUSHANNOTATION_H
