@@ -217,7 +217,7 @@ bool KisKraLoadVisitor::visit(KisPaintLayer *layer)
             if (!pixelSelection->read(m_store->device())) {
                 pixelSelection->disconnect();
             } else {
-                KisTransparencyMask* mask = new KisTransparencyMask();
+                KisTransparencyMask* mask = new KisTransparencyMask(m_image, i18n("Transparency Mask"));
                 mask->setSelection(selection);
                 m_image->addNode(mask, layer, layer->firstChild());
             }
@@ -230,6 +230,8 @@ bool KisKraLoadVisitor::visit(KisPaintLayer *layer)
 
 bool KisKraLoadVisitor::visit(KisGroupLayer *layer)
 {
+    loadNodeKeyframes(layer);
+
     if (*layer->colorSpace() != *m_image->colorSpace()) {
         layer->resetCache(m_image->colorSpace());
     }
@@ -261,11 +263,18 @@ bool KisKraLoadVisitor::visit(KisAdjustmentLayer* layer)
         // We use the default, empty selection
     }
 
+    if (!result) {
+        m_warningMessages.append(i18nc("Warning during loading a kra file with a filter layer",
+                                       "Selection on layer %s couldn't be loaded. It will be replaced by an empty selection.", layer->name()));
+        // otherwise ignore and just use what is there already
+        // (most probably an empty selection)
+    }
+
     if (!loadMetaData(layer)) {
         return false;
     }
 
-    loadFilterConfiguration(layer->filter(), getLocation(layer, DOT_FILTERCONFIG));
+    loadFilterConfiguration(layer, getLocation(layer, DOT_FILTERCONFIG));
     fixOldFilterConfigurations(layer->filter());
 
     result = visitAll(layer);
@@ -284,9 +293,8 @@ bool KisKraLoadVisitor::visit(KisGeneratorLayer *layer)
 
     // HACK ALERT: we set the same filter again to ensure the layer
     // is correctly updated
-    KisFilterConfigurationSP filter = layer->filter();
-    result = loadFilterConfiguration(filter.data(), getLocation(layer, DOT_FILTERCONFIG));
-    layer->setFilter(filter);
+    result = loadFilterConfiguration(layer, getLocation(layer, DOT_FILTERCONFIG));
+    layer->resetCache();
 
     result = visitAll(layer);
     return result;
@@ -344,7 +352,7 @@ bool KisKraLoadVisitor::visit(KisFilterMask *mask)
 
     bool result = true;
     result = loadSelection(getLocation(mask), mask->selection());
-    result = loadFilterConfiguration(mask->filter(), getLocation(mask, DOT_FILTERCONFIG));
+    result = loadFilterConfiguration(mask, getLocation(mask, DOT_FILTERCONFIG));
     fixOldFilterConfigurations(mask->filter());
     return result;
 }
@@ -432,8 +440,13 @@ bool KisKraLoadVisitor::visit(KisColorizeMask *mask)
         return false;
 
     QVector<KisLazyFillTools::KeyStroke> strokes;
-    if (!KisDomUtils::loadValue(doc.documentElement(), COLORIZE_KEYSTROKES_SECTION, &strokes, mask->colorSpace()))
+    if (!KisDomUtils::loadValue(doc.documentElement(),
+                                COLORIZE_KEYSTROKES_SECTION,
+                                &strokes,
+                                mask->colorSpace(),
+                                QPoint(mask->x(), mask->y()))) {
         return false;
+    }
 
     int i = 0;
     Q_FOREACH (const KisLazyFillTools::KeyStroke &stroke, strokes) {
@@ -444,6 +457,35 @@ bool KisKraLoadVisitor::visit(KisColorizeMask *mask)
     mask->setKeyStrokesDirect(QList<KisLazyFillTools::KeyStroke>::fromVector(strokes));
 
     loadPaintDevice(mask->coloringProjection(), COLORIZE_COLORING_DEVICE);
+
+    const KoColorProfile *profile =
+        loadProfile(getLocation(mask, DOT_ICC), mask->colorSpace()->colorModelId().id(), mask->colorSpace()->colorDepthId().id());
+
+    if (!profile) {
+        KisNodeSP parent = mask->parent();
+        KIS_SAFE_ASSERT_RECOVER(parent) {
+            parent = m_image->root();
+        }
+
+        if (parent->colorSpace()->colorModelId() == mask->colorSpace()->colorModelId() &&
+            parent->colorSpace()->colorDepthId() == mask->colorSpace()->colorDepthId()) {
+
+            profile = parent->colorSpace()->profile();
+        }
+    }
+
+    if (!profile) {
+        if (m_image->colorSpace()->colorModelId() == mask->colorSpace()->colorModelId() &&
+            m_image->colorSpace()->colorDepthId() == mask->colorSpace()->colorDepthId()) {
+
+            profile = m_image->colorSpace()->profile();
+        }
+    }
+
+    if (profile) {
+        mask->setProfile(profile, 0);
+    }
+
     mask->resetCache();
 
     m_store->popDirectory();
@@ -559,11 +601,27 @@ bool KisKraLoadVisitor::loadPaintDeviceFrame(KisPaintDeviceSP device, const QStr
 
 bool KisKraLoadVisitor::loadProfile(KisPaintDeviceSP device, const QString& location)
 {
+    const KoColorProfile *profile = loadProfile(location, device->colorSpace()->colorModelId().id(), device->colorSpace()->colorDepthId().id());
+
+    if (profile) {
+        // TODO: check result!
+        device->setProfile(profile, 0);
+    } else {
+        m_warningMessages << i18n("Could not load profile: %1.", location);
+    }
+
+    return true;
+}
+
+const KoColorProfile *KisKraLoadVisitor::loadProfile(const QString &location, const QString &colorModelId, const QString &colorDepthId)
+{
+    const KoColorProfile *result = 0;
+
     if (m_store->hasFile(location)) {
         m_store->open(location);
         QByteArray data;
         data.resize(m_store->size());
-        dbgFile << "Data to load: " << m_store->size() << " from " << location << " with color space " << device->colorSpace()->id();
+        dbgFile << "Data to load: " << m_store->size() << " from " << location << " with color space " << colorModelId << colorDepthId;
         int read = m_store->read(data.data(), m_store->size());
         dbgFile << "Profile size: " << data.size() << " " << m_store->atEnd() << " " << m_store->device()->bytesAvailable() << " " << read;
         m_store->close();
@@ -572,26 +630,23 @@ bool KisKraLoadVisitor::loadProfile(KisPaintDeviceSP device, const QString& loca
         QByteArray hash = hashGenerator->generateHash(data);
 
         if (m_profileCache.contains(hash)) {
-            if (device->setProfile(m_profileCache[hash], 0)) {
-                return true;
-            }
+            result = m_profileCache[hash];
         }
         else {
             // Create a colorspace with the embedded profile
-            const KoColorProfile *profile = KoColorSpaceRegistry::instance()->createColorProfile(device->colorSpace()->colorModelId().id(), device->colorSpace()->colorDepthId().id(), data);
+            const KoColorProfile *profile = KoColorSpaceRegistry::instance()->createColorProfile(colorModelId, colorDepthId, data);
             m_profileCache[hash] = profile;
-            if (device->setProfile(profile, 0)) {
-                return true;
-            }
-
+            result = profile;
         }
     }
-    m_warningMessages << i18n("Could not load profile: %1.", location);
-    return true;
+
+    return result;
 }
 
-bool KisKraLoadVisitor::loadFilterConfiguration(KisFilterConfigurationSP kfc, const QString& location)
+bool KisKraLoadVisitor::loadFilterConfiguration(KisNodeFilterInterface *nodeInterface, const QString& location)
 {
+    KisFilterConfigurationSP kfc = nodeInterface->filter();
+
     if (m_store->hasFile(location)) {
         QByteArray data;
         m_store->open(location);
@@ -666,17 +721,7 @@ bool KisKraLoadVisitor::loadSelection(const QString& location, KisSelectionSP ds
         pixelSelection->setDefaultPixel(transparent);
     }
 
-    // Pixel selection
     bool result = true;
-    QString pixelSelectionLocation = location + DOT_PIXEL_SELECTION;
-    if (m_store->hasFile(pixelSelectionLocation)) {
-        KisPixelSelectionSP pixelSelection = dstSelection->pixelSelection();
-        result = loadPaintDevice(pixelSelection, pixelSelectionLocation);
-        if (!result) {
-            m_warningMessages << i18n("Could not load raster selection %1.", location);
-        }
-        pixelSelection->invalidateOutlineCache();
-    }
 
     // Shape selection
     QString shapeSelectionLocation = location + DOT_SHAPE_SELECTION;
@@ -687,13 +732,45 @@ bool KisKraLoadVisitor::loadSelection(const QString& location, KisSelectionSP ds
         m_store->enterDirectory(shapeSelectionLocation) ;
 
         KisShapeSelection* shapeSelection = new KisShapeSelection(m_shapeController, m_image, dstSelection);
-        dstSelection->setShapeSelection(shapeSelection);
+        dstSelection->convertToVectorSelectionNoUndo(shapeSelection);
         result = shapeSelection->loadSelection(m_store);
+
+        /**
+         * We need to explicitly call updateProjection() here, because
+         * KisUpdateSelectionJob that is put into the queue will not update
+         * actual layers that own this selection. In normal situation,
+         * the setDirty() call is done explicitly while painting.
+         *
+         * TODO: consider adding a proper setDirty() call to
+         *       KisUpdateSelectionJob. Though it doesn't seem to be needed
+         *       until we allow modifying vector selections on non-selection
+         *       masks.
+         */
+        dstSelection->updateProjection();
         m_store->popDirectory();
         if (!result) {
             m_warningMessages << i18n("Could not load vector selection %1.", location);
         }
+    } else {
+        /**
+         * NOTE: since loading a vector selection discards all the contents
+         * of a raster projection by reincarnating the paint device, there is
+         * no need to load pixel selection in case any vector selection
+         * is present.
+         */
+
+        // Pixel selection
+        QString pixelSelectionLocation = location + DOT_PIXEL_SELECTION;
+        if (m_store->hasFile(pixelSelectionLocation)) {
+            KisPixelSelectionSP pixelSelection = dstSelection->pixelSelection();
+            result = loadPaintDevice(pixelSelection, pixelSelectionLocation);
+            if (!result) {
+                m_warningMessages << i18n("Could not load raster selection %1.", location);
+            }
+            pixelSelection->invalidateOutlineCache();
+        }
     }
+
     return true;
 }
 
