@@ -28,7 +28,7 @@ class psdExport : public KisImportExportFilter {
         psdExport(QObject *parent, const QVariantList &);
         ~psdExport() override;
     public:
-        KisImportExportFilter::ConversionStatus convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration = 0) override;
+        KisImportExportErrorCode convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration = 0) override;
         void initializeCapabilities() override;
 };
 
