@@ -25,7 +25,7 @@
 #include <QFileInfo>
 
 #include "kis_types.h"
-#include <KisImageBuilderResult.h>
+#include <KisImportExportErrorCode.h>
 class KisDocument;
 
 class PSDLoader : public QObject {
@@ -35,11 +35,11 @@ class PSDLoader : public QObject {
 public:
 
     PSDLoader(KisDocument *doc);
-    virtual ~PSDLoader();
+    ~PSDLoader() override;
 
-    KisImageBuilder_Result buildImage(const QString &filename);
+    KisImportExportErrorCode buildImage(QIODevice *io);
 
-    KisImageWSP image();
+    KisImageSP image();
 
 public Q_SLOTS:
 
@@ -47,11 +47,11 @@ public Q_SLOTS:
 
 private:
 
-    KisImageBuilder_Result decode(const QString &filename);
+    KisImportExportErrorCode decode(QIODevice *io);
 
 private:
 
-    KisImageWSP m_image;
+    KisImageSP m_image;
     KisDocument *m_doc;
     bool m_stop;
 };
