@@ -27,8 +27,9 @@
 #include <KoColorSpaceMaths.h>
 #include <KoColorSpaceTraits.h>
 #include <KoCmykColorSpaceTraits.h>
+#include <colorspaces/KoAlphaColorSpace.h>
 
-#include <netinet/in.h> // htonl
+#include <QtEndian>
 
 #include "kis_global.h"
 #include <asl/kis_asl_writer_utils.h>
@@ -49,6 +50,22 @@ template <class Traits>
 typename Traits::channels_type convertByteOrder(typename Traits::channels_type value);
 // default implementation is undefined for every color space should be added manually
 
+template <>
+inline quint8 convertByteOrder<AlphaU8Traits>(quint8 value) {
+    return value;
+}
+
+template <>
+inline quint16 convertByteOrder<AlphaU16Traits>(quint16 value) {
+    return qFromBigEndian((quint16)value);
+}
+
+template <>
+inline float convertByteOrder<AlphaF32Traits>(float value) {
+    return qFromBigEndian((quint32)value);
+}
+
+
 template <>
 inline quint8 convertByteOrder<KoGrayU8Traits>(quint8 value) {
     return value;
@@ -56,12 +73,12 @@ inline quint8 convertByteOrder<KoGrayU8Traits>(quint8 value) {
 
 template <>
 inline quint16 convertByteOrder<KoGrayU16Traits>(quint16 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint16)value);
 }
 
 template <>
 inline quint32 convertByteOrder<KoGrayU32Traits>(quint32 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint32)value);
 }
 
 template <>
@@ -71,12 +88,12 @@ inline quint8 convertByteOrder<KoBgrU8Traits>(quint8 value) {
 
 template <>
 inline quint16 convertByteOrder<KoBgrU16Traits>(quint16 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint16)value);
 }
 
 template <>
 inline quint32 convertByteOrder<KoBgrU32Traits>(quint32 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint32)value);
 }
 
 template <>
@@ -86,12 +103,12 @@ inline quint8 convertByteOrder<KoCmykU8Traits>(quint8 value) {
 
 template <>
 inline quint16 convertByteOrder<KoCmykU16Traits>(quint16 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint16)value);
 }
 
 template <>
 inline float convertByteOrder<KoCmykF32Traits>(float value) {
-    return ntohs(value);
+    return qFromBigEndian((quint32)value);
 }
 
 template <>
@@ -101,12 +118,55 @@ inline quint8 convertByteOrder<KoLabU8Traits>(quint8 value) {
 
 template <>
 inline quint16 convertByteOrder<KoLabU16Traits>(quint16 value) {
-    return ntohs(value);
+    return qFromBigEndian((quint16)value);
 }
 
 template <>
 inline float convertByteOrder<KoLabF32Traits>(float value) {
-    return ntohs(value);
+    return qFromBigEndian((quint32)value);
+}
+
+template <class Traits>
+void readAlphaMaskPixel(const QMap<quint16, QByteArray> &channelBytes,
+                        int col, quint8 *dstPtr);
+
+template <>
+void readAlphaMaskPixel<AlphaU8Traits>(const QMap<quint16, QByteArray> &channelBytes,
+                                       int col, quint8 *dstPtr)
+{
+    *dstPtr = reinterpret_cast<const quint8*>(channelBytes.first().constData())[col];
+}
+
+template <>
+void readAlphaMaskPixel<AlphaU16Traits>(const QMap<quint16, QByteArray> &channelBytes,
+                                       int col, quint8 *dstPtr)
+{
+    *dstPtr = reinterpret_cast<const quint16*>(channelBytes.first().constData())[col] >> 8;
+}
+
+template <>
+void readAlphaMaskPixel<AlphaF32Traits>(const QMap<quint16, QByteArray> &channelBytes,
+                                        int col, quint8 *dstPtr)
+{
+    *dstPtr = reinterpret_cast<const float*>(channelBytes.first().constData())[col] * 255;
+}
+
+template <class Traits>
+inline typename Traits::channels_type readChannelValue(const QMap<quint16, QByteArray> &channelBytes,
+                                       quint16 channelId, int col, typename Traits::channels_type defaultValue)
+{
+    typedef typename Traits::channels_type channels_type;
+
+    if (channelBytes.contains(channelId)) {
+        const QByteArray &bytes = channelBytes[channelId];
+        if (col < bytes.size()) {
+            return convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(bytes.constData())[col]);
+        }
+
+        dbgFile << "col index out of range channelId: "<< channelId << " col:" << col;
+    }
+
+    return defaultValue;
 }
 
 template <class Traits>
@@ -117,17 +177,10 @@ void readGrayPixel(const QMap<quint16, QByteArray> &channelBytes,
     typedef typename Traits::channels_type channels_type;
 
     const channels_type unitValue = KoColorSpaceMathsTraits<channels_type>::unitValue;
-    channels_type opacity = unitValue;
-    if (channelBytes.contains(-1)) {
-        opacity = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[-1].constData())[col]);
-    }
-
     Pixel *pixelPtr = reinterpret_cast<Pixel*>(dstPtr);
 
-    channels_type gray = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[0].constData())[col]);
-
-    pixelPtr->gray = gray;
-    pixelPtr->alpha = opacity;
+    pixelPtr->gray  = readChannelValue<Traits>(channelBytes, 0, col, unitValue);
+    pixelPtr->alpha = readChannelValue<Traits>(channelBytes, -1, col, unitValue);
 }
 
 template <class Traits>
@@ -138,21 +191,13 @@ void readRgbPixel(const QMap<quint16, QByteArray> &channelBytes,
     typedef typename Traits::channels_type channels_type;
 
     const channels_type unitValue = KoColorSpaceMathsTraits<channels_type>::unitValue;
-    channels_type opacity = unitValue;
-    if (channelBytes.contains(-1)) {
-        opacity = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[-1].constData())[col]);
-    }
-
     Pixel *pixelPtr = reinterpret_cast<Pixel*>(dstPtr);
 
-    channels_type blue = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[2].constData())[col]);
-    channels_type green = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[1].constData())[col]);
-    channels_type red = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[0].constData())[col]);
+    pixelPtr->blue  = readChannelValue<Traits>(channelBytes, 2, col, unitValue);
+    pixelPtr->green = readChannelValue<Traits>(channelBytes, 1, col, unitValue);
+    pixelPtr->red   = readChannelValue<Traits>(channelBytes, 0, col, unitValue);
+    pixelPtr->alpha = readChannelValue<Traits>(channelBytes, -1, col, unitValue);
 
-    pixelPtr->blue = blue;
-    pixelPtr->green = green;
-    pixelPtr->red = red;
-    pixelPtr->alpha = opacity;
 }
 
 template <class Traits>
@@ -163,23 +208,13 @@ void readCmykPixel(const QMap<quint16, QByteArray> &channelBytes,
     typedef typename Traits::channels_type channels_type;
 
     const channels_type unitValue = KoColorSpaceMathsTraits<channels_type>::unitValue;
-    channels_type opacity = unitValue;
-    if (channelBytes.contains(-1)) {
-        opacity = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[-1].constData())[col]);
-    }
-
     Pixel *pixelPtr = reinterpret_cast<Pixel*>(dstPtr);
 
-    channels_type cyan = unitValue - convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[0].constData())[col]);
-    channels_type magenta = unitValue - convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[1].constData())[col]);
-    channels_type yellow = unitValue - convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[2].constData())[col]);
-    channels_type black = unitValue - convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[3].constData())[col]);
-
-    pixelPtr->cyan = cyan;
-    pixelPtr->magenta = magenta;
-    pixelPtr->yellow = yellow;
-    pixelPtr->black = black;
-    pixelPtr->alpha = opacity;
+    pixelPtr->cyan    = unitValue - readChannelValue<Traits>(channelBytes, 0, col, unitValue);
+    pixelPtr->magenta = unitValue - readChannelValue<Traits>(channelBytes, 1, col, unitValue);
+    pixelPtr->yellow  = unitValue - readChannelValue<Traits>(channelBytes, 2, col, unitValue);
+    pixelPtr->black   = unitValue - readChannelValue<Traits>(channelBytes, 3, col, unitValue);
+    pixelPtr->alpha   = readChannelValue<Traits>(channelBytes, -1, col, unitValue);
 }
 
 template <class Traits>
@@ -190,21 +225,12 @@ void readLabPixel(const QMap<quint16, QByteArray> &channelBytes,
     typedef typename Traits::channels_type channels_type;
 
     const channels_type unitValue = KoColorSpaceMathsTraits<channels_type>::unitValue;
-    channels_type opacity = unitValue;
-    if (channelBytes.contains(-1)) {
-        opacity = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[-1].constData())[col]);
-    }
-
     Pixel *pixelPtr = reinterpret_cast<Pixel*>(dstPtr);
 
-    channels_type L = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[0].constData())[col]);
-    channels_type a = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[1].constData())[col]);
-    channels_type b = convertByteOrder<Traits>(reinterpret_cast<const channels_type *>(channelBytes[2].constData())[col]);
-
-    pixelPtr->L = L;
-    pixelPtr->a = a;
-    pixelPtr->b = b;
-    pixelPtr->alpha = opacity;
+    pixelPtr->L = readChannelValue<Traits>(channelBytes, 0, col, unitValue);
+    pixelPtr->a = readChannelValue<Traits>(channelBytes, 1, col, unitValue);
+    pixelPtr->b = readChannelValue<Traits>(channelBytes, 2, col, unitValue);
+    pixelPtr->alpha = readChannelValue<Traits>(channelBytes, -1, col, unitValue);
 }
 
 void readRgbPixelCommon(int channelSize,
@@ -259,6 +285,19 @@ void readLabPixelCommon(int channelSize,
     }
 }
 
+void readAlphaMaskPixelCommon(int channelSize,
+                                const QMap<quint16, QByteArray> &channelBytes,
+                                int col, quint8 *dstPtr)
+{
+    if (channelSize == 1) {
+        readAlphaMaskPixel<AlphaU8Traits>(channelBytes, col, dstPtr);
+    } else if (channelSize == 2) {
+        readAlphaMaskPixel<AlphaU16Traits>(channelBytes, col, dstPtr);
+    } else if (channelSize == 4) {
+        readAlphaMaskPixel<AlphaF32Traits>(channelBytes, col, dstPtr);
+    }
+}
+
 /**********************************************************************/
 /* Two functions copied from the abandoned PSDParse library (GPL)     */
 /* See: http://www.telegraphics.com.au/svn/psdparse/trunk/psd_zip.c   */
@@ -271,7 +310,7 @@ typedef quint8 psd_uchar;
 typedef int psd_int;
 typedef quint8 Bytef;
 
-psd_status psd_unzip_without_prediction(psd_uchar *src_buf, psd_int src_len, 
+psd_status psd_unzip_without_prediction(psd_uchar *src_buf, psd_int src_len,
                                         psd_uchar *dst_buf, psd_int dst_len)
 {
 #ifdef HAVE_ZLIB
@@ -293,7 +332,7 @@ psd_status psd_unzip_without_prediction(psd_uchar *src_buf, psd_int src_len,
         state = inflate(&stream, Z_PARTIAL_FLUSH);
         if(state == Z_STREAM_END)
             break;
-        if(state == Z_DATA_ERROR || state != Z_OK)
+        if(state != Z_OK)
             break;
     }  while (stream.avail_out > 0);
 
@@ -307,8 +346,8 @@ psd_status psd_unzip_without_prediction(psd_uchar *src_buf, psd_int src_len,
     return 0;
 }
 
-psd_status psd_unzip_with_prediction(psd_uchar *src_buf, psd_int src_len, 
-                                     psd_uchar *dst_buf, psd_int dst_len, 
+psd_status psd_unzip_with_prediction(psd_uchar *src_buf, psd_int src_len,
+                                     psd_uchar *dst_buf, psd_int dst_len,
                                      psd_int row_size, psd_int color_depth)
 {
     psd_status status;
@@ -353,7 +392,7 @@ psd_status psd_unzip_with_prediction(psd_uchar *src_buf, psd_int src_len,
 /**********************************************************************/
 
 QMap<quint16, QByteArray> fetchChannelsBytes(QIODevice *io, QVector<ChannelInfo*> channelInfoRecords,
-                                            int row, int width, int channelSize)
+                                            int row, int width, int channelSize, bool processMasks)
 {
     const int uncompressedLength = width * channelSize;
 
@@ -361,7 +400,7 @@ QMap<quint16, QByteArray> fetchChannelsBytes(QIODevice *io, QVector<ChannelInfo*
 
     Q_FOREACH (ChannelInfo *channelInfo, channelInfoRecords) {
         // user supplied masks are ignored here
-        if (channelInfo->channelId < -1) continue;
+        if (!processMasks && channelInfo->channelId < -1) continue;
 
         io->seek(channelInfo->channelDataStart + channelInfo->channelOffset);
 
@@ -393,7 +432,8 @@ void readCommon(KisPaintDeviceSP dev,
                 const QRect &layerRect,
                 QVector<ChannelInfo*> infoRecords,
                 int channelSize,
-                PixelFunc pixelFunc)
+                PixelFunc pixelFunc,
+                bool processMasks)
 {
     KisOffsetKeeper keeper(io);
 
@@ -439,10 +479,10 @@ void readCommon(KisPaintDeviceSP dev,
 
         KisSequentialIterator it(dev, layerRect);
         int col = 0;
-        do {
+        while (it.nextPixel()) {
             pixelFunc(channelSize, channelBytes, col, it.rawData());
             col++;
-        } while(it.nextPixel());
+        }
 
     } else {
         KisHLineIteratorSP it = dev->createHLineIteratorNG(layerRect.left(), layerRect.top(), layerRect.width());
@@ -450,7 +490,8 @@ void readCommon(KisPaintDeviceSP dev,
             QMap<quint16, QByteArray> channelBytes;
 
             channelBytes = fetchChannelsBytes(io, infoRecords,
-                                              i, layerRect.width(), channelSize);
+                                              i, layerRect.width(),
+                                              channelSize, processMasks);
 
             for (qint64 col = 0; col < layerRect.width(); col++){
                 pixelFunc(channelSize, channelBytes, col, it->rawData());
@@ -470,16 +511,16 @@ void readChannels(QIODevice *io,
 {
     switch (colorMode) {
     case Grayscale:
-        readCommon(device, io, layerRect, infoRecords, channelSize, &readGrayPixelCommon);
+        readCommon(device, io, layerRect, infoRecords, channelSize, &readGrayPixelCommon, false);
         break;
     case RGB:
-        readCommon(device, io, layerRect, infoRecords, channelSize, &readRgbPixelCommon);
+        readCommon(device, io, layerRect, infoRecords, channelSize, &readRgbPixelCommon, false);
         break;
     case CMYK:
-        readCommon(device, io, layerRect, infoRecords, channelSize, &readCmykPixelCommon);
+        readCommon(device, io, layerRect, infoRecords, channelSize, &readCmykPixelCommon, false);
         break;
     case Lab:
-        readCommon(device, io, layerRect, infoRecords, channelSize, &readLabPixelCommon);
+        readCommon(device, io, layerRect, infoRecords, channelSize, &readLabPixelCommon, false);
         break;
     case Bitmap:
     case Indexed:
@@ -492,6 +533,16 @@ void readChannels(QIODevice *io,
     }
 }
 
+void readAlphaMaskChannels(QIODevice *io,
+                           KisPaintDeviceSP device,
+                           int channelSize,
+                           const QRect &layerRect,
+                           QVector<ChannelInfo*> infoRecords)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN(infoRecords.size() == 1);
+    readCommon(device, io, layerRect, infoRecords, channelSize, &readAlphaMaskPixelCommon, true);
+}
+
 void writeChannelDataRLE(QIODevice *io, const quint8 *plane, const int channelSize, const QRect &rc, const qint64 sizeFieldOffset, const qint64 rleBlockOffset, const bool writeCompressionType)
 {
     typedef KisAslWriterUtils::OffsetStreamPusher<quint32> Pusher;
@@ -560,7 +611,7 @@ inline void preparePixelForWrite(quint8 *dataPlane,
             quint16 *pixelPtr = reinterpret_cast<quint16*>(dataPlane) + i;
 
             val = *pixelPtr;
-            val = ntohs(val);
+            val = qFromBigEndian(val);
             if (channelId >= 0 && (colorMode == CMYK || colorMode == CMYK64)) {
                 val = quint16_MAX - val;
             }
@@ -573,7 +624,7 @@ inline void preparePixelForWrite(quint8 *dataPlane,
             quint32 *pixelPtr = reinterpret_cast<quint32*>(dataPlane) + i;
 
             val = *pixelPtr;
-            val = ntohl(val);
+            val = qFromBigEndian(val);
             if (channelId >= 0 && (colorMode == CMYK || colorMode == CMYK64)) {
                 val = quint16_MAX - val;
             }
@@ -608,10 +659,10 @@ void writePixelDataCommon(QIODevice *io,
             int channelIndex = KoChannelInfo::displayPositionToChannelIndex(ch->displayPosition(), origChannels);
 
             quint8 *holder = 0;
-            qSwap(holder, tmp[channelIndex]);
+            std::swap(holder, tmp[channelIndex]);
 
             if (ch->channelType() == KoChannelInfo::ALPHA) {
-                qSwap(holder, alphaPlanePtr);
+                std::swap(holder, alphaPlanePtr);
             } else {
                 planes.append(holder);
             }
@@ -653,13 +704,17 @@ void writePixelDataCommon(QIODevice *io,
         }
 
     } catch (KisAslWriterUtils::ASLWriteException &e) {
-        qDeleteAll(planes);
+        Q_FOREACH (quint8 *plane, planes) {
+            delete[] plane;
+        }
         planes.clear();
 
         throw KisAslWriterUtils::ASLWriteException(PREPEND_METHOD(e.what()));
     }
 
-    qDeleteAll(planes);
+    Q_FOREACH (quint8 *plane, planes) {
+        delete[] plane;
+    }
     planes.clear();
 }
 
