@@ -24,6 +24,7 @@
 #define KIS_BRUSHOP_SETTINGS_WIDGET_H_
 
 #include <kis_brush_based_paintop_options_widget.h>
+#include <kis_curve_option_widget.h>
 
 
 class KisBrushOpSettingsWidget : public KisBrushBasedPaintopOptionWidget
@@ -38,6 +39,12 @@ public:
     ~KisBrushOpSettingsWidget() override;
 
     KisPropertiesConfigurationSP configuration() const override;
+
+protected:
+    void notifyPageChanged() override;
+
+private:
+    KisCurveOptionWidget* m_lightnessStrengthOptionWidget;
 };
 
 
