@@ -19,13 +19,25 @@
 #ifndef KIS_EXPERIMENT_PAINTOP_SETTINGS_H_
 #define KIS_EXPERIMENT_PAINTOP_SETTINGS_H_
 
-#include <brushengine/kis_paintop_settings.h>
+#include <brushengine/kis_no_size_paintop_settings.h>
+#include <QScopedPointer>
 
-class KisExperimentPaintOpSettings : public KisPaintOpSettings
+class KisExperimentPaintOpSettings : public KisNoSizePaintOpSettings
 {
 public:
-    bool paintIncremental();
-    QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode) const;
+    KisExperimentPaintOpSettings();
+    ~KisExperimentPaintOpSettings() override;
+
+    bool lodSizeThresholdSupported() const override;
+    bool paintIncremental() override;
+    QPainterPath brushOutline(const KisPaintInformation &info, const OutlineMode &mode, qreal alignForZoom) override;
+
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
+
+private:
+
+    struct Private;
+    const QScopedPointer<Private> m_d;
 };
 
 #endif
