@@ -19,6 +19,8 @@
 #ifndef KIS_GRID_PAINTOP_SETTINGS_H_
 #define KIS_GRID_PAINTOP_SETTINGS_H_
 
+#include <QScopedPointer>
+
 #include <brushengine/kis_paintop_settings.h>
 #include <kis_types.h>
 
@@ -31,10 +33,23 @@ class KisGridPaintOpSettings : public KisOutlineGenerationPolicy<KisPaintOpSetti
 {
 public:
     KisGridPaintOpSettings();
+    ~KisGridPaintOpSettings() override;
 
-    QPainterPath brushOutline(const KisPaintInformation &info, OutlineMode mode) const;
-    bool paintIncremental();
+    void setPaintOpSize(qreal value) override;
+    qreal paintOpSize() const override;
+
+    QPainterPath brushOutline(const KisPaintInformation &info, const OutlineMode &mode, qreal alignForZoom) override;
+    bool paintIncremental() override;
+
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
+
+private:
+
+    struct Private;
+    const QScopedPointer<Private> m_d;
 
 };
 
+typedef KisSharedPtr<KisGridPaintOpSettings> KisGridPaintOpSettingsSP;
+
 #endif
