@@ -21,10 +21,12 @@
 #include "kis_pressure_size_option.h"
 #include "kis_pressure_rotation_option.h"
 #include "kis_pressure_mirror_option.h"
+#include "kis_pressure_sharpness_option.h"
 #include <brushengine/kis_paintop_settings.h>
 #include <kis_properties_configuration.h>
 #include "kis_paintop_settings.h"
 #include <QElapsedTimer>
+#include "kis_algebra_2d.h"
 
 #define NOISY_UPDATE_SPEED 50  // Time in ms for outline updates to noisy brushes
 
@@ -42,11 +44,16 @@ struct KisCurrentOutlineFetcher::Private {
         if (optionsAvailable & MIRROR_OPTION) {
             mirrorOption.reset(new KisPressureMirrorOption());
         }
+
+        if (optionsAvailable & SHARPNESS_OPTION) {
+            sharpnessOption.reset(new KisPressureSharpnessOption());
+        }
     }
 
     QScopedPointer<KisPressureSizeOption> sizeOption;
     QScopedPointer<KisPressureRotationOption> rotationOption;
     QScopedPointer<KisPressureMirrorOption> mirrorOption;
+    QScopedPointer<KisPressureSharpnessOption> sharpnessOption;
 
     bool isDirty;
     QElapsedTimer lastUpdateTime;
@@ -75,6 +82,7 @@ QPainterPath KisCurrentOutlineFetcher::fetchOutline(const KisPaintInformation &i
                                                     const KisPaintOpSettingsSP settings,
                                                     const QPainterPath &originalOutline,
                                                     const KisPaintOpSettings::OutlineMode &mode,
+                                                    qreal alignForZoom,
                                                     qreal additionalScale,
                                                     qreal additionalRotation,
                                                     bool tilt, qreal tiltcenterx, qreal tiltcentery) const
@@ -95,6 +103,11 @@ QPainterPath KisCurrentOutlineFetcher::fetchOutline(const KisPaintInformation &i
             d->mirrorOption->resetAllSensors();
         }
 
+        if (d->sharpnessOption) {
+            d->sharpnessOption->readOptionSetting(settings);
+            d->sharpnessOption->resetAllSensors();
+        }
+
         d->isDirty = false;
     }
 
@@ -121,7 +134,7 @@ QPainterPath KisCurrentOutlineFetcher::fetchOutline(const KisPaintInformation &i
         }
         rotation += d->lastRotationApplied;
     } else if (d->rotationOption && tilt) {
-        rotation += settings->getDouble("runtimeCanvasRotation", 0.0) * M_PI / 180.0;
+        rotation += info.canvasRotation() * M_PI / 180.0;
     }
 
     qreal xFlip = 1.0;
@@ -145,7 +158,6 @@ QPainterPath KisCurrentOutlineFetcher::fetchOutline(const KisPaintInformation &i
         }
     }
 
-
     QTransform rot;
     rot.rotateRadians(-rotation);
 
@@ -154,8 +166,20 @@ QPainterPath KisCurrentOutlineFetcher::fetchOutline(const KisPaintInformation &i
         hotSpot.setX(tiltcenterx);
         hotSpot.setY(tiltcentery);
     }
+
+    QPointF pos = info.pos();
+    if (d->sharpnessOption) {
+        qint32 x = 0, y = 0;
+        qreal subPixelX = 0.0, subPixelY = 0.0;
+        d->sharpnessOption->apply(info, pos - hotSpot, x, y, subPixelX, subPixelY);
+        pos = QPointF(x + subPixelX, y + subPixelY) + hotSpot;
+    }
+
+    // align cursor position to widget pixel grid to avoid noise
+    pos = KisAlgebra2D::alignForZoom(pos, alignForZoom);
+
     QTransform T1 = QTransform::fromTranslate(-hotSpot.x(), -hotSpot.y());
-    QTransform T2 = QTransform::fromTranslate(info.pos().x(), info.pos().y());
+    QTransform T2 = QTransform::fromTranslate(pos.x(), pos.y());
     QTransform S  = QTransform::fromScale(xFlip * scale, yFlip * scale);
 
     return (T1 * rot * S * T2).map(originalOutline);
