@@ -36,16 +36,17 @@ struct PrecisionValues {
     qreal sizeFrac;
     qreal subPixel;
     qreal softnessFactor;
+    qreal lightnessStrength;
     qreal ratio;
 };
 
 const qreal eps = 1e-6;
 static const PrecisionValues precisionLevels[] = {
-    {M_PI / 180, 0.05,   1, 0.01, 0.05},
-    {M_PI / 180, 0.01,   1, 0.01, 0.01},
-    {M_PI / 180,    0,   1, 0.01,  eps},
-    {M_PI / 180,    0, 0.5, 0.01,  eps},
-    {       eps,    0, eps,  eps,  eps}
+    {M_PI / 180, 0.05,   1, 0.01, 0.01, 0.05},
+    {M_PI / 180, 0.01,   1, 0.01, 0.01, 0.01},
+    {M_PI / 180,    0,   1, 0.01, 0.01, eps},
+    {M_PI / 180,    0, 0.5, 0.01, 0.01, eps},
+    {       eps,    0, eps,  eps, eps, eps}
 };
 
 struct KisDabCacheBase::SavedDabParameters {
@@ -56,6 +57,7 @@ struct KisDabCacheBase::SavedDabParameters {
     qreal subPixelX;
     qreal subPixelY;
     qreal softnessFactor;
+    qreal lightnessStrength;
     qreal ratio;
     int index;
     MirrorProperties mirrorProperties;
@@ -70,6 +72,7 @@ struct KisDabCacheBase::SavedDabParameters {
                qAbs(subPixelX - rhs.subPixelX) <= prec.subPixel &&
                qAbs(subPixelY - rhs.subPixelY) <= prec.subPixel &&
                qAbs(softnessFactor - rhs.softnessFactor) <= prec.softnessFactor &&
+               qAbs(lightnessStrength - rhs.lightnessStrength) <= prec.lightnessStrength &&
                qAbs(ratio - rhs.ratio) <= prec.ratio &&
                index == rhs.index &&
                mirrorProperties.horizontalMirror == rhs.mirrorProperties.horizontalMirror &&
@@ -128,6 +131,7 @@ KisDabCacheBase::getDabParameters(KisBrushSP brush,
                               const KisPaintInformation& info,
                               double subPixelX, double subPixelY,
                               qreal softnessFactor,
+                              qreal lightnessStrength,
                               MirrorProperties mirrorProperties)
 {
     SavedDabParameters params;
@@ -139,7 +143,8 @@ KisDabCacheBase::getDabParameters(KisBrushSP brush,
     params.subPixelX = subPixelX;
     params.subPixelY = subPixelY;
     params.softnessFactor = softnessFactor;
-    params.index = brush->brushIndex(info);
+    params.lightnessStrength = lightnessStrength;
+    params.index = brush->brushIndex();
     params.mirrorProperties = mirrorProperties;
     params.ratio = shape.ratio();
 
@@ -243,6 +248,7 @@ void KisDabCacheBase::fetchDabGenerationInfo(bool hasDabInCache,
 {
     di->info = request.info;
     di->softnessFactor = request.softnessFactor;
+    di->lightnessStrength = request.lightnessStrength;
 
     if (m_d->mirrorOption) {
         di->mirrorProperties = m_d->mirrorOption->apply(request.info);
@@ -269,6 +275,7 @@ void KisDabCacheBase::fetchDabGenerationInfo(bool hasDabInCache,
                                                     di->subPixel.x(),
                                                     di->subPixel.y(),
                                                     di->softnessFactor,
+                                                    di->lightnessStrength,
                                                     di->mirrorProperties);
 
     int precisionLevel = 4;
@@ -276,7 +283,6 @@ void KisDabCacheBase::fetchDabGenerationInfo(bool hasDabInCache,
         const int effectiveDabSize = qMin(newParams.width, newParams.height);
         precisionLevel = m_d->precisionOption->effectivePrecisionLevel(effectiveDabSize) - 1;
     }
-
     *shouldUseCache = hasDabInCache && di->solidColorFill &&
             newParams.compare(m_d->lastSavedDabParameters, precisionLevel);
 
