@@ -19,17 +19,26 @@
 #ifndef KIS_PARTICLE_PAINTOP_SETTINGS_H_
 #define KIS_PARTICLE_PAINTOP_SETTINGS_H_
 
-#include <brushengine/kis_paintop_settings.h>
+#include <QScopedPointer>
+#include <brushengine/kis_no_size_paintop_settings.h>
 #include <kis_types.h>
 
-class KisParticlePaintOpSettings : public KisPaintOpSettings
+class KisParticlePaintOpSettings : public KisNoSizePaintOpSettings
 {
 
 public:
-    bool paintIncremental();
-    bool isAirbrushing() const;
-    int rate() const;
 
+    KisParticlePaintOpSettings();
+    ~KisParticlePaintOpSettings() override;
+
+    bool lodSizeThresholdSupported() const override;
+    bool paintIncremental() override;
+
+    QList<KisUniformPaintOpPropertySP> uniformProperties(KisPaintOpSettingsSP settings) override;
+
+private:
+    struct Private;
+    const QScopedPointer<Private> m_d;
 };
 
 #endif
