@@ -46,25 +46,21 @@
 #include <cmath>
 #include <ctime>
 
-#include "random_gauss.h"
-
 #include <QtGlobal>
 
 SprayBrush::SprayBrush()
 {
     m_painter = 0;
     m_transfo = 0;
-    m_rand = new RandomGauss(time(0));
 }
 
 SprayBrush::~SprayBrush()
 {
     delete m_painter;
     delete m_transfo;
-    delete m_rand;
 }
 
-void SprayBrush::setProperties(KisSprayProperties * properties,
+void SprayBrush::setProperties(KisSprayOptionProperties * properties,
                                KisColorProperties * colorProperties,
                                KisShapeProperties * shapeProperties,
                                KisShapeDynamicsProperties * shapeDynamicsProperties,
@@ -90,12 +86,18 @@ qreal SprayBrush::rotationAngle(KisRandomSourceSP randomSource)
 
     if (m_shapeDynamicsProperties->randomRotation) {
 
+        qreal randomValue = 0.0;
+
         if (m_properties->gaussian) {
-            rotation = linearInterpolation(rotation , M_PI * 2.0 * qBound<qreal>(0.0, m_rand->nextGaussian(0.0, 0.50) , 1.0), m_shapeDynamicsProperties->randomRotationWeight);
-        }
-        else {
-            rotation = linearInterpolation(rotation, M_PI * 2.0 * randomSource->generateNormalized(), m_shapeDynamicsProperties->randomRotationWeight);
+            randomValue = qBound<qreal>(0.0, randomSource->generateGaussian(0.0, 0.5), 1.0);
+        } else {
+            randomValue = randomSource->generateNormalized();
         }
+
+        rotation =
+            linearInterpolation(rotation ,
+                                M_PI * 2.0 * randomValue,
+                                m_shapeDynamicsProperties->randomRotationWeight);
     }
 
     return rotation;
@@ -131,14 +133,14 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
 
     qreal x = info.pos().x();
     qreal y = info.pos().y();
-    KisRandomAccessorSP accessor = dab->createRandomAccessorNG(qRound(x), qRound(y));
+    KisRandomAccessorSP accessor = dab->createRandomAccessorNG();
 
     Q_ASSERT(color.colorSpace()->pixelSize() == dab->pixelSize());
     m_inkColor = color;
     KisCrossDeviceColorPicker colorPicker(source, m_inkColor);
 
     // apply size sensor
-    m_radius = m_properties->radius * scale * additionalScale;
+    m_radius = m_properties->radius() * scale * additionalScale;
 
     // jitter movement
     if (m_properties->jitterMovement) {
@@ -182,7 +184,7 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
 
         // generate random length
         if (m_properties->gaussian) {
-            length = qBound<qreal>(0.0, m_rand->nextGaussian(0.0, 0.50) , 1.0);
+            length = randomSource->generateGaussian(0.0, 0.5);
         }
         else {
             length = randomSource->generateNormalized();
@@ -312,7 +314,7 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
                     }
                     m_transformed = m_brushQImage.transformed(m, Qt::SmoothTransformation);
                     m_imageDevice->convertFromQImage(m_transformed, 0);
-                    KisRandomAccessorSP ac = m_imageDevice->createRandomAccessorNG(0, 0);
+                    KisRandomAccessorSP ac = m_imageDevice->createRandomAccessorNG();
                     QRect rc = m_transformed.rect();
 
                     if (m_colorProperties->useRandomHSV && m_transfo) {
@@ -336,9 +338,8 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
             // Auto-brush
         }
         else {
-            QPointF hotSpot = m_brush->hotSpot(particleScale * additionalScale,
-                                               particleScale * additionalScale,
-                                               -rotationZ, info);
+            KisDabShape shape(particleScale * additionalScale, 1.0, -rotationZ);
+            QPointF hotSpot = m_brush->hotSpot(shape, info);
             QPointF pos(nx + x, ny + y);
             QPointF pt = pos - hotSpot;
 
@@ -350,12 +351,12 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
             KisPaintOp::splitCoordinate(pt.x(), &ix, &xFraction);
             KisPaintOp::splitCoordinate(pt.y(), &iy, &yFraction);
 
+            m_brush->prepareForSeqNo(info, m_dabSeqNo);
+
             //KisFixedPaintDeviceSP dab;
-            if (m_brush->brushType() == IMAGE ||
-                    m_brush->brushType() == PIPE_IMAGE) {
+            if (m_brush->brushApplication() == IMAGESTAMP) {
                 m_fixedDab = m_brush->paintDevice(m_fixedDab->colorSpace(),
-                                                  particleScale * additionalScale,
-                                                  -rotationZ, info, xFraction, yFraction);
+                          shape, info, xFraction, yFraction);
 
                 if (m_colorProperties->useRandomHSV && m_transfo) {
                     quint8 * dabPointer = m_fixedDab->data();
@@ -365,10 +366,8 @@ void SprayBrush::paint(KisPaintDeviceSP dab, KisPaintDeviceSP source,
 
             }
             else {
-                m_brush->mask(m_fixedDab, m_inkColor,
-                              particleScale * additionalScale,
-                              particleScale * additionalScale,
-                              -rotationZ, info, xFraction, yFraction);
+                m_brush->mask(m_fixedDab, m_inkColor, shape,
+                              info, xFraction, yFraction);
             }
             m_painter->bltFixed(QPoint(ix, iy), m_fixedDab, m_fixedDab->bounds());
         }
@@ -453,7 +452,7 @@ void SprayBrush::paintRectangle(KisPainter* painter, qreal x, qreal y, qreal wid
 void SprayBrush::paintOutline(KisPaintDeviceSP dev , const KoColor &outlineColor, qreal posX, qreal posY, qreal radius)
 {
     QList<QPointF> antiPixels;
-    KisRandomAccessorSP accessor = dev->createRandomAccessorNG(qRound(posX), qRound(posY));
+    KisRandomAccessorSP accessor = dev->createRandomAccessorNG();
 
     for (int y = -radius + posY; y <= radius + posY; y++) {
         for (int x = -radius + posX; x <= radius + posX; x++) {
