@@ -141,7 +141,7 @@ void KisTangentTiltOption::swizzleAssign(qreal const horizontal, qreal const ver
 
 void KisTangentTiltOption::apply(const KisPaintInformation& info,qreal *r,qreal *g,qreal *b)
 {
-    //formula based on http://www.cerebralmeltdown.com/programming_projects/Altitude%20and%20Azimuth%20to%20Vector/index.html
+    //formula based on https://www.cerebralmeltdown.com/programming_projects/Altitude%20and%20Azimuth%20to%20Vector/index.html
 
     /* It doesn't make sense of have higher than 8bit color depth.
      * Instead we make sure in the paintAt function of kis_tangent_normal_paintop to pick an 8bit space of the current
@@ -169,8 +169,6 @@ void KisTangentTiltOption::apply(const KisPaintInformation& info,qreal *r,qreal
         elevation= (info.tiltElevation(info, 60.0, 60.0, true)*90.0);
     }
 
-    const qreal oldDirection = direction;
-
     //subtract/add the rotation of the canvas.
     if (directionType() != 1) {
         direction = normalizeAngleDegrees(direction - info.canvasRotation());
