@@ -36,7 +36,7 @@ void KisMoveBoundsCalculationJob::run()
     QRect handlesRect;
 
     Q_FOREACH (KisNodeSP node, m_nodes) {
-        handlesRect |= KisLayerUtils::recursiveNodeExactBounds(node);
+        handlesRect |= KisLayerUtils::recursiveTightNodeVisibleBounds(node);
     }
 
     if (m_selection) {
@@ -58,3 +58,11 @@ int KisMoveBoundsCalculationJob::levelOfDetail() const
 {
     return 0;
 }
+
+QString KisMoveBoundsCalculationJob::debugName() const
+{
+    QString result;
+    QDebug dbg(&result);
+    dbg << "KisMoveBoundsCalculationJob" << ppVar(m_requestedBy) << m_nodes;
+    return result;
+}
