@@ -192,7 +192,6 @@ void KisToolColorPicker::continuePrimaryAction(KoPointerEvent *event)
     displayPickedColor();
 }
 
-#include "kis_canvas2.h"
 #include "kis_display_color_converter.h"
 
 void KisToolColorPicker::endPrimaryAction(KoPointerEvent *event)
@@ -206,13 +205,14 @@ void KisToolColorPicker::endPrimaryAction(KoPointerEvent *event)
         // We don't ask for a name, too intrusive here
 
         KoColorSet *palette = m_palettes.at(m_optionsWidget->cmbPalette->currentIndex());        
-        palette->add(swatch);
-
-        if (!palette->save()) {
-            QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Cannot write to palette file %1. Maybe it is read-only.", palette->filename()));
-        } 
+        if (palette) {
+            palette->add(swatch);
+            KoResourceServerProvider::instance()->paletteServer()->updateResource(palette);
+            if (!palette->save()) {
+                QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("Cannot write to palette file %1. Maybe it is read-only.", palette->filename()));
+            }
+        }
     }
-
 }
 
 struct PickedChannel {
@@ -419,4 +419,4 @@ void KisToolColorPicker::syncTaggedResourceView() {}
 
 void KisToolColorPicker::syncTagAddition(const QString& /*tag*/) {}
 
-void KisToolColorPicker::syncTagRemoval(const QString& /*tag*/) {}
\ No newline at end of file
+void KisToolColorPicker::syncTagRemoval(const QString& /*tag*/) {}
