@@ -53,7 +53,7 @@
 #include <KisViewManager.h>
 #include <widgets/kis_cmb_composite.h>
 #include <widgets/kis_double_widget.h>
-#include <widgets/kis_slider_spin_box.h>
+#include <kis_slider_spin_box.h>
 #include <kis_cursor.h>
 #include <kis_config.h>
 #include "kis_resources_snapshot.h"
@@ -250,6 +250,8 @@ QWidget* KisToolGradient::createOptionWidget()
     m_cmbShape->addItem(i18nc("the gradient will be drawn in a square around a centre", "Square"));
     m_cmbShape->addItem(i18nc("the gradient will be drawn as an asymmetric cone", "Conical"));
     m_cmbShape->addItem(i18nc("the gradient will be drawn as a symmetric cone", "Conical Symmetric"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn as a spiral", "Spiral"));
+    m_cmbShape->addItem(i18nc("the gradient will be drawn as a reverse spiral", "Reverse Spiral"));
     m_cmbShape->addItem(i18nc("the gradient will be drawn in a selection outline", "Shaped"));
     addOptionWidgetOption(m_cmbShape, m_lbShape);
     connect(m_cmbShape, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetShape(int)));
