@@ -86,10 +86,6 @@ KisToolMove::KisToolMove(KoCanvasBase *canvas)
 
     m_optionsWidget->slotSetTranslate(m_handlesRect.topLeft() + currentOffset());
 
-    connect(m_optionsWidget, SIGNAL(sigSetTranslateX(int)), SLOT(moveBySpinX(int)), Qt::UniqueConnection);
-    connect(m_optionsWidget, SIGNAL(sigSetTranslateY(int)), SLOT(moveBySpinY(int)), Qt::UniqueConnection);
-    connect(m_optionsWidget, SIGNAL(sigRequestCommitOffsetChanges()), this, SLOT(commitChanges()), Qt::UniqueConnection);
-
     connect(this, SIGNAL(moveInNewPosition(QPoint)), m_optionsWidget, SLOT(slotSetTranslate(QPoint)), Qt::UniqueConnection);
 }
 
@@ -100,43 +96,56 @@ KisToolMove::~KisToolMove()
 
 void KisToolMove::resetCursorStyle()
 {
-    KisTool::resetCursorStyle();
-
     if (!isActive()) return;
-    KisImageSP image = this->image();
-    KisResourcesSnapshotSP resources =
-        new KisResourcesSnapshot(image, currentNode(), canvas()->resourceManager());
-    KisSelectionSP selection = resources->activeSelection();
-    KisNodeList nodes = fetchSelectedNodes(moveToolMode(), &m_lastCursorPos, selection);
 
-    if (nodes.isEmpty()) {
-        canvas()->setCursor(Qt::ForbiddenCursor);
-    }
-}
+    bool canMove = true;
 
-KisNodeList KisToolMove::fetchSelectedNodes(MoveToolMode mode, const QPoint *pixelPoint, KisSelectionSP selection)
-{
-    KisNodeList nodes;
+    if (m_strokeId && m_currentlyUsingSelection) {
+        /// noop; whatever the cursor position, we always show move
+        /// cursor, because we don't use 'layer under cursor' mode
+        /// for moving selections
+    } else if (m_strokeId && !m_currentlyUsingSelection) {
+        /// we cannot pick layer's pixel data while the stroke is running,
+        /// because it may run in lodN mode; therefore, we delegate this
+        /// work to the stroke itself
+        if (m_currentMode != MoveSelectedLayer &&
+            (m_handlesRect.isEmpty() ||
+             !m_handlesRect.translated(currentOffset()).contains(m_lastCursorPos))) {
 
-    KisImageSP image = this->image();
-    if (mode != MoveSelectedLayer && pixelPoint) {
-        const bool wholeGroup = !selection &&  mode == MoveGroup;
-        KisNodeSP node = KisToolUtils::findNode(image->root(), *pixelPoint, wholeGroup);
-        if (node) {
-            nodes = {node};
+            image()->addJob(m_strokeId, new MoveStrokeStrategy::PickLayerData(m_lastCursorPos));
+            return;
+        }
+    } else {
+        KisResourcesSnapshotSP resources =
+            new KisResourcesSnapshot(this->image(), currentNode(), canvas()->resourceManager());
+        KisSelectionSP selection = resources->activeSelection();
+
+        KisPaintLayerSP paintLayer =
+            dynamic_cast<KisPaintLayer*>(this->currentNode().data());
+
+        const bool canUseSelectionMode =
+                paintLayer && selection &&
+                !selection->selectedRect().isEmpty() &&
+                !selection->selectedExactRect().isEmpty();
+
+        if (!canUseSelectionMode) {
+            KisNodeSelectionRecipe nodeSelection =
+                    KisNodeSelectionRecipe(
+                        this->selectedNodes(),
+                        (KisNodeSelectionRecipe::SelectionMode)moveToolMode(),
+                        m_lastCursorPos);
+
+            if (nodeSelection.selectNodesToProcess().isEmpty()) {
+                canMove = false;
+            }
         }
     }
 
-    if (nodes.isEmpty()) {
-        nodes = this->selectedNodes();
-
-        KritaUtils::filterContainer<KisNodeList>(nodes,
-                                                 [](KisNodeSP node) {
-                                                     return node->isEditable();
-                                                 });
+    if (canMove) {
+        KisTool::resetCursorStyle();
+    } else {
+       useCursor(Qt::ForbiddenCursor);
     }
-
-    return nodes;
 }
 
 bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
@@ -148,34 +157,43 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         new KisResourcesSnapshot(image, currentNode(), canvas()->resourceManager());
     KisSelectionSP selection = resources->activeSelection();
 
-    KisNodeList nodes = fetchSelectedNodes(mode, pos, selection);
+    KisPaintLayerSP paintLayer =
+        dynamic_cast<KisPaintLayer*>(this->currentNode().data());
 
-    if (nodes.size() == 1) {
-        node = nodes.first();
+    const bool canUseSelectionMode =
+            paintLayer && selection &&
+            !selection->selectedRect().isEmpty() &&
+            !selection->selectedExactRect().isEmpty();
+
+    if (pos) {
+        // finish stroke by clicking outside image bounds
+        if (m_strokeId && !image->bounds().contains(*pos)) {
+            endStroke();
+            return false;
+        }
+
+        // restart stroke when the mode has changed or the user tried to
+        // pick another layer in "layer under cursor" mode.
+        if (m_strokeId &&
+                (m_currentMode != mode ||
+                 m_currentlyUsingSelection != canUseSelectionMode ||
+                 (!m_currentlyUsingSelection &&
+                  mode != MoveSelectedLayer &&
+                  !m_handlesRect.translated(currentOffset()).contains(*pos)))) {
+
+            endStroke();
+        }
     }
 
-    if (nodes.isEmpty()) {
-        return false;
-    }
+    if (m_strokeId) return true;
 
-    /**
-     * If the target node has changed, the stroke should be
-     * restarted. Otherwise just continue processing current node.
-     */
-    if (m_strokeId && !tryEndPreviousStroke(nodes)) {
-        return true;
-    }
+    KisNodeList nodes;
 
     KisStrokeStrategy *strategy;
 
-    KisPaintLayerSP paintLayer = node ?
-        dynamic_cast<KisPaintLayer*>(node.data()) : 0;
-
     bool isMoveSelection = false;
-
-    if (paintLayer && selection &&
-        (!selection->selectedRect().isEmpty() &&
-         !selection->selectedExactRect().isEmpty())) {
+    if (canUseSelectionMode) {
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(selection, false);
 
         MoveSelectionStrokeStrategy *moveStrategy =
             new MoveSelectionStrokeStrategy(paintLayer,
@@ -186,19 +204,38 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         connect(moveStrategy,
                 SIGNAL(sigHandlesRectCalculated(const QRect&)),
                 SLOT(slotHandlesRectCalculated(const QRect&)));
+        connect(moveStrategy,
+                SIGNAL(sigStrokeStartedEmpty()),
+                SLOT(slotStrokeStartedEmpty()));
 
         strategy = moveStrategy;
         isMoveSelection = true;
+        nodes = {paintLayer};
 
     } else {
+        KisNodeSelectionRecipe nodeSelection =
+            pos ?
+                KisNodeSelectionRecipe(
+                        this->selectedNodes(),
+                        (KisNodeSelectionRecipe::SelectionMode)mode,
+                        *pos) :
+                KisNodeSelectionRecipe(this->selectedNodes());
+
 
         MoveStrokeStrategy *moveStrategy =
-            new MoveStrokeStrategy(nodes, image.data(), image.data());
+            new MoveStrokeStrategy(nodeSelection, image.data(), image.data());
         connect(moveStrategy,
                 SIGNAL(sigHandlesRectCalculated(const QRect&)),
                 SLOT(slotHandlesRectCalculated(const QRect&)));
+        connect(moveStrategy,
+                SIGNAL(sigStrokeStartedEmpty()),
+                SLOT(slotStrokeStartedEmpty()));
+        connect(moveStrategy,
+                SIGNAL(sigLayersPicked(const KisNodeList&)),
+                SLOT(slotStrokePickedLayers(const KisNodeList&)));
 
         strategy = moveStrategy;
+        nodes = nodeSelection.selectedNodes;
     }
 
     // disable outline feedback until the stroke calcualtes
@@ -206,6 +243,8 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
     m_handlesRect = QRect();
     m_strokeId = image->startStroke(strategy);
     m_currentlyProcessingNodes = nodes;
+    m_currentlyUsingSelection = isMoveSelection;
+    m_currentMode = mode;
     m_accumulatedOffset = QPoint();
 
     if (!isMoveSelection) {
@@ -281,6 +320,37 @@ void KisToolMove::slotHandlesRectCalculated(const QRect &handlesRect)
     notifyGuiAfterMove(false);
 }
 
+void KisToolMove::slotStrokeStartedEmpty()
+{
+    /**
+     * Notify that move-selection stroke ended unexpectedly
+     */
+    if (m_currentlyUsingSelection) {
+        KisCanvas2 *kisCanvas = static_cast<KisCanvas2*>(canvas());
+        kisCanvas->viewManager()->
+            showFloatingMessage(
+                i18nc("floating message in move tool",
+                      "Selected area has no pixels"),
+                QIcon(), 1000, KisFloatingMessage::High);
+    }
+
+    /**
+     * Since the choice of nodes for the operation happens in the
+     * stroke itself, it may happen that there are no nodes at all.
+     * In such a case, we should just cancel already started stroke.
+     */
+    cancelStroke();
+}
+
+void KisToolMove::slotStrokePickedLayers(const KisNodeList &nodes)
+{
+    if (nodes.isEmpty()) {
+        useCursor(Qt::ForbiddenCursor);
+    } else {
+        KisTool::resetCursorStyle();
+    }
+}
+
 void KisToolMove::moveDiscrete(MoveDirection direction, bool big)
 {
     if (mode() == KisTool::PAINT_MODE) return;  // Don't interact with dragging
@@ -337,6 +407,9 @@ void KisToolMove::activate(ToolActivation toolActivation, const QSet<KoShape*> &
 
     connect(m_showCoordinatesAction, SIGNAL(triggered(bool)), m_optionsWidget, SLOT(setShowCoordinates(bool)), Qt::UniqueConnection);
     connect(m_optionsWidget, SIGNAL(showCoordinatesChanged(bool)), m_showCoordinatesAction, SLOT(setChecked(bool)), Qt::UniqueConnection);
+    connect(m_optionsWidget, SIGNAL(sigSetTranslateX(int)), SLOT(moveBySpinX(int)), Qt::UniqueConnection);
+    connect(m_optionsWidget, SIGNAL(sigSetTranslateY(int)), SLOT(moveBySpinY(int)), Qt::UniqueConnection);
+    connect(m_optionsWidget, SIGNAL(sigRequestCommitOffsetChanges()), this, SLOT(commitChanges()), Qt::UniqueConnection);
 
     connect(&m_changesTracker,
             SIGNAL(sigConfigChanged(KisToolChangesTrackerDataSP)),
@@ -352,7 +425,7 @@ void KisToolMove::paint(QPainter& gc, const KoViewConverter &converter)
 {
     Q_UNUSED(converter);
 
-    if (m_strokeId && !m_handlesRect.isEmpty()) {
+    if (m_strokeId && !m_handlesRect.isEmpty() && !m_currentlyUsingSelection) {
         QPainterPath handles;
         handles.addRect(m_handlesRect.translated(currentOffset()));
 
@@ -444,7 +517,9 @@ void KisToolMove::mouseMoveEvent(KoPointerEvent *event)
     m_lastCursorPos = convertToPixelCoord(event).toPoint();
     KisTool::mouseMoveEvent(event);
 
-    if (moveToolMode() == MoveFirstLayer) {
+    if (moveToolMode() != MoveSelectedLayer ||
+            (m_strokeId && m_currentMode != MoveSelectedLayer)) {
+
         m_updateCursorCompressor.start();
     }
 }
@@ -457,6 +532,13 @@ void KisToolMove::startAction(KoPointerEvent *event, MoveToolMode mode)
 
     if (startStrokeImpl(mode, &pos)) {
         setMode(KisTool::PAINT_MODE);
+
+        if (m_currentlyUsingSelection) {
+            KisImageSP image = currentImage();
+            image->addJob(m_strokeId,
+                          new MoveSelectionStrokeStrategy::ShowSelectionData(false));
+        }
+
     } else {
         event->ignore();
         m_dragPos = QPoint();
@@ -496,6 +578,12 @@ void KisToolMove::endAction(KoPointerEvent *event)
     m_dragPos = QPoint();
     commitChanges();
 
+    if (m_currentlyUsingSelection) {
+        KisImageSP image = currentImage();
+        image->addJob(m_strokeId,
+                      new MoveSelectionStrokeStrategy::ShowSelectionData(true));
+    }
+
     notifyGuiAfterMove();
 
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
@@ -503,7 +591,7 @@ void KisToolMove::endAction(KoPointerEvent *event)
 
 void KisToolMove::drag(const QPoint& newPos)
 {
-    KisImageWSP image = currentImage();
+    KisImageSP image = currentImage();
 
     QPoint offset = m_accumulatedOffset + newPos - m_dragStart;
 
@@ -524,6 +612,8 @@ void KisToolMove::endStroke()
     m_strokeId.clear();
     m_changesTracker.reset();
     m_currentlyProcessingNodes.clear();
+    m_currentlyUsingSelection = false;
+    m_currentMode = MoveSelectedLayer;
     m_accumulatedOffset = QPoint();
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
 }
@@ -594,6 +684,8 @@ void KisToolMove::cancelStroke()
     m_strokeId.clear();
     m_changesTracker.reset();
     m_currentlyProcessingNodes.clear();
+    m_currentlyUsingSelection = false;
+    m_currentMode = MoveSelectedLayer;
     m_accumulatedOffset = QPoint();
     notifyGuiAfterMove();
     qobject_cast<KisCanvas2*>(canvas())->updateCanvas();
@@ -631,14 +723,21 @@ QPoint KisToolMove::applyModifiers(Qt::KeyboardModifiers modifiers, QPoint pos)
 
 void KisToolMove::moveBySpinX(int newX)
 {
-    if (mode() == KisTool::PAINT_MODE) return;  // Don't interact with dragging
-    if (!currentNode()->isEditable()) return; // Don't move invisible nodes
+    if (mode() == KisTool::PAINT_MODE ||    // Don't interact with dragging
+            !currentNode()->isEditable() || // Don't move invisible nodes
+            m_handlesRect.isEmpty()) {
+        return;
+    }
+
+    // starting a new stroke resets m_handlesRect and it gets updated asynchronously,
+    // but in this case no change is expected
+    int handlesRectX = m_handlesRect.x();
 
     if (startStrokeImpl(MoveSelectedLayer, 0)) {
         setMode(KisTool::PAINT_MODE);
     }
 
-    m_accumulatedOffset.rx() =  newX - m_handlesRect.x();
+    m_accumulatedOffset.rx() =  newX - handlesRectX;
 
     image()->addJob(m_strokeId, new MoveStrokeStrategy::Data(m_accumulatedOffset));
 
@@ -648,14 +747,21 @@ void KisToolMove::moveBySpinX(int newX)
 
 void KisToolMove::moveBySpinY(int newY)
 {
-    if (mode() == KisTool::PAINT_MODE) return;  // Don't interact with dragging
-    if (!currentNode()->isEditable()) return; // Don't move invisible nodes
+    if (mode() == KisTool::PAINT_MODE ||    // Don't interact with dragging
+            !currentNode()->isEditable() || // Don't move invisible nodes
+            m_handlesRect.isEmpty()) {
+        return;
+    }
+
+    // starting a new stroke resets m_handlesRect and it gets updated asynchronously,
+    // but in this case no change is expected
+    int handlesRectY = m_handlesRect.y();
 
     if (startStrokeImpl(MoveSelectedLayer, 0)) {
         setMode(KisTool::PAINT_MODE);
     }
 
-    m_accumulatedOffset.ry() =  newY - m_handlesRect.y();
+    m_accumulatedOffset.ry() =  newY - handlesRectY;
 
     image()->addJob(m_strokeId, new MoveStrokeStrategy::Data(m_accumulatedOffset));
 
