@@ -59,6 +59,13 @@ class KisToolPencil : public DelegatedPencilTool
 public:
     KisToolPencil(KoCanvasBase * canvas);
     void mousePressEvent(KoPointerEvent *event) override;
+    // reimplementing KoToolBase's method. It ignores the event, and then
+    // Qt does not send mouseRelease event, which causes the tool to jump.
+    void mouseDoubleClickEvent(KoPointerEvent *event) override;
+
+    void beginPrimaryAction(KoPointerEvent* event) override;
+    void continuePrimaryAction(KoPointerEvent *event) override;
+    void endPrimaryAction(KoPointerEvent *event) override;
 
     QList<QPointer<QWidget> > createOptionWidgets() override;
 
