@@ -94,6 +94,7 @@
 namespace {
 static const QString EditFillGradientFactoryId = "edit_fill_gradient";
 static const QString EditStrokeGradientFactoryId = "edit_stroke_gradient";
+static const QString EditFillMeshGradientFactoryId = "edit_fill_meshgradient";
 
 enum TransformActionType {
     TransformRotate90CW,
@@ -254,6 +255,7 @@ private:
             qreal minDistanceSq = std::numeric_limits<qreal>::max();
 
             KoShapeGradientHandles sh(m_fillVariant, shape);
+            auto handless = sh.handles();
             Q_FOREACH (const KoShapeGradientHandles::Handle &handle, sh.handles()) {
                 const QPointF handlePoint = converter->documentToView(handle.pos);
                 const qreal distanceSq = kisSquareDistance(viewPoint, handlePoint);
@@ -274,6 +276,129 @@ private:
     KoShapeGradientHandles::Handle m_currentHandle;
 };
 
+#include "KoShapeMeshGradientHandles.h"
+#include "ShapeMeshGradientEditStrategy.h"
+
+class DefaultTool::MoveMeshGradientHandleInteractionFactory: public KoInteractionStrategyFactory
+{
+public:
+    MoveMeshGradientHandleInteractionFactory(KoFlake::FillVariant fillVariant,
+                                             int priority,
+                                             const QString& id,
+                                             DefaultTool* _q)
+        : KoInteractionStrategyFactory(priority, id)
+        , m_fillVariant(fillVariant)
+        , q(_q)
+    {
+    }
+
+    KoInteractionStrategy* createStrategy(KoPointerEvent *ev) override
+    {
+        m_currentHandle = handleAt(ev->point);
+        q->m_selectedMeshHandle = m_currentHandle;
+        emit q->meshgradientHandleSelected(m_currentHandle);
+
+
+        if (m_currentHandle.type != KoShapeMeshGradientHandles::Handle::None) {
+            KoShape *shape = onlyEditableShape();
+            KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(shape, 0);
+
+            return new ShapeMeshGradientEditStrategy(q, m_fillVariant, shape, m_currentHandle, ev->point);
+        }
+
+        return nullptr;
+    }
+
+    bool hoverEvent(KoPointerEvent *ev) override
+    {
+        // for custom cursor
+        KoShapeMeshGradientHandles::Handle handle = handleAt(ev->point);
+
+        // refresh
+        if (handle.type != m_currentHandle.type && handle.type == KoShapeMeshGradientHandles::Handle::None) {
+            q->repaintDecorations();
+        }
+
+        m_currentHandle = handle;
+        q->m_hoveredMeshHandle = m_currentHandle;
+
+        // highlight the decoration which is being hovered
+        if (m_currentHandle.type != KoShapeMeshGradientHandles::Handle::None) {
+            q->repaintDecorations();
+        }
+        return false;
+    }
+
+    bool paintOnHover(QPainter &painter, const KoViewConverter &converter) override
+    {
+        Q_UNUSED(painter);
+        Q_UNUSED(converter);
+        return false;
+    }
+
+    bool tryUseCustomCursor() override
+    {
+        if (m_currentHandle.type != KoShapeMeshGradientHandles::Handle::None) {
+            q->useCursor(Qt::OpenHandCursor);
+            return true;
+        }
+
+        return false;
+    }
+
+
+private:
+    KoShape* onlyEditableShape() const {
+        // FIXME: copy of KoShapeGradientHandles
+        KoSelection *selection = q->koSelection();
+        QList<KoShape*> shapes = selection->selectedEditableShapes();
+
+        KoShape *shape = 0;
+        if (shapes.size() == 1) {
+            shape = shapes.first();
+        }
+
+        return shape;
+    }
+
+    KoShapeMeshGradientHandles::Handle handleAt(const QPointF &pos) const
+    {
+        // FIXME: copy of KoShapeGradientHandles. use a template?
+        KoShapeMeshGradientHandles::Handle result;
+
+        KoShape *shape = onlyEditableShape();
+        if (shape) {
+            KoFlake::SelectionHandle globalHandle = q->handleAt(pos);
+            const qreal distanceThresholdSq =
+                globalHandle == KoFlake::NoHandle ?
+                    HANDLE_DISTANCE_SQ : 0.25 * HANDLE_DISTANCE_SQ;
+
+            const KoViewConverter *converter = q->canvas()->viewConverter();
+            const QPointF viewPoint = converter->documentToView(pos);
+            qreal minDistanceSq = std::numeric_limits<qreal>::max();
+
+            KoShapeMeshGradientHandles sh(m_fillVariant, shape);
+
+            for (const auto& handle: sh.handles()) {
+                const QPointF handlePoint = converter->documentToView(handle.pos);
+                const qreal distanceSq = kisSquareDistance(viewPoint, handlePoint);
+
+                if (distanceSq < distanceThresholdSq && distanceSq < minDistanceSq) {
+                    result = handle;
+                    minDistanceSq = distanceSq;
+                }
+            }
+        }
+
+        return result;
+    }
+
+private:
+    KoFlake::FillVariant m_fillVariant;
+    KoShapeMeshGradientHandles::Handle m_currentHandle;
+    DefaultTool *q;
+};
+
 class SelectionHandler : public KoToolSelection
 {
 public:
@@ -300,7 +425,6 @@ DefaultTool::DefaultTool(KoCanvasBase *canvas, bool connectToSelectedShapesProxy
     , m_lastHandle(KoFlake::NoHandle)
     , m_hotPosition(KoFlake::TopLeft)
     , m_mouseWasInsideHandles(false)
-    , m_decorator(0)
     , m_selectionHandler(new SelectionHandler(this))
     , m_tabbedOptionWidget(0)
 {
@@ -383,6 +507,26 @@ void DefaultTool::slotActivateEditStrokeGradient(bool value)
     repaintDecorations();
 }
 
+void DefaultTool::slotActivateEditFillMeshGradient(bool value)
+{
+    if (value) {
+        connect(this, SIGNAL(meshgradientHandleSelected(KoShapeMeshGradientHandles::Handle)),
+                m_tabbedOptionWidget, SLOT(slotMeshGradientHandleSelected(KoShapeMeshGradientHandles::Handle)));
+        addInteractionFactory(
+            new MoveMeshGradientHandleInteractionFactory(KoFlake::Fill, 1,
+                                                         EditFillMeshGradientFactoryId, this));
+    } else {
+        disconnect(this, SIGNAL(meshgradientHandleSelected(KoShapeMeshGradientHandles::Handle)),
+                   m_tabbedOptionWidget, SLOT(slotMeshGradientHandleSelected(KoShapeMeshGradientHandles::Handle)));
+        removeInteractionFactory(EditFillMeshGradientFactoryId);
+    }
+}
+
+void DefaultTool::slotResetMeshGradientState()
+{
+    m_selectedMeshHandle = KoShapeMeshGradientHandles::Handle();
+}
+
 bool DefaultTool::wantsAutoScroll() const
 {
     return true;
@@ -677,7 +821,7 @@ void DefaultTool::paint(QPainter &painter, const KoViewConverter &converter)
 {
     KoSelection *selection = koSelection();
     if (selection) {
-        this->m_decorator = new SelectionDecorator(canvas()->resourceManager());
+        m_decorator.reset(new SelectionDecorator(canvas()->resourceManager()));
 
         {
             /**
@@ -695,6 +839,8 @@ void DefaultTool::paint(QPainter &painter, const KoViewConverter &converter)
         m_decorator->setHandleRadius(handleRadius());
         m_decorator->setShowFillGradientHandles(hasInteractioFactory(EditFillGradientFactoryId));
         m_decorator->setShowStrokeFillGradientHandles(hasInteractioFactory(EditStrokeGradientFactoryId));
+        m_decorator->setShowFillMeshGradientHandles(hasInteractioFactory(EditFillMeshGradientFactoryId));
+        m_decorator->setCurrentMeshGradientHandles(m_selectedMeshHandle, m_hoveredMeshHandle);
         m_decorator->paint(painter, converter);
     }
 
@@ -1460,6 +1606,15 @@ QList<QPointer<QWidget> > DefaultTool::createOptionWidgets()
             SIGNAL(sigSwitchModeEditStrokeGradient(bool)),
             SLOT(slotActivateEditStrokeGradient(bool)));
 
+    connect(m_tabbedOptionWidget,
+            SIGNAL(sigSwitchModeEditFillGradient(bool)),
+            SLOT(slotActivateEditFillMeshGradient(bool)));
+    // TODO: strokes!!
+
+    connect(m_tabbedOptionWidget,
+            SIGNAL(sigMeshGradientResetted()),
+            SLOT(slotResetMeshGradientState()));
+
     return widgets;
 }
 
