@@ -85,6 +85,8 @@ void ToolReferenceImages::setReferenceImageLayer(KisSharedPtr<KisReferenceImages
 {
     m_layer = layer;
     connect(layer.data(), SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
+    connect(layer->shapeManager(), SIGNAL(selectionChanged()), this, SLOT(repaintDecorations()));
+    connect(layer->shapeManager(), SIGNAL(selectionContentChanged()), this, SLOT(repaintDecorations()));
 }
 
 void ToolReferenceImages::addReferenceImage()
@@ -105,8 +107,10 @@ void ToolReferenceImages::addReferenceImage()
     if (!QFileInfo(filename).exists()) return;
 
     auto *reference = KisReferenceImage::fromFile(filename, *kisCanvas->coordinatesConverter(), canvas()->canvasWidget());
-
     if (reference) {
+        if (document()->referenceImagesLayer()) {
+            reference->setZIndex(document()->referenceImagesLayer()->shapes().size());
+        }
         KisDocument *doc = document();
         doc->addCommand(KisReferenceImagesLayer::addReferenceImages(doc, {reference}));
     }
@@ -117,10 +121,17 @@ void ToolReferenceImages::pasteReferenceImage()
     KisCanvas2* kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
     KIS_ASSERT_RECOVER_RETURN(kisCanvas);
 
-            KisReferenceImage* reference = KisReferenceImage::fromClipboard(*kisCanvas->coordinatesConverter());
-    if(reference) {
+    KisReferenceImage* reference = KisReferenceImage::fromClipboard(*kisCanvas->coordinatesConverter());
+    if (reference) {
+        if (document()->referenceImagesLayer()) {
+            reference->setZIndex(document()->referenceImagesLayer()->shapes().size());
+        }
         KisDocument *doc = document();
         doc->addCommand(KisReferenceImagesLayer::addReferenceImages(doc, {reference}));
+    } else {
+        if (canvas()->canvasWidget()) {
+            QMessageBox::critical(canvas()->canvasWidget(), i18nc("@title:window", "Krita"), i18n("Could not load reference image from clipboard"));
+        }
     }
 }
 
@@ -159,10 +170,18 @@ void ToolReferenceImages::loadReferenceImages()
     }
 
     KisReferenceImageCollection collection;
+
+    int currentZIndex = 0;
+    if (document()->referenceImagesLayer()) {
+        currentZIndex = document()->referenceImagesLayer()->shapes().size();
+    }
+
     if (collection.load(&file)) {
         QList<KoShape*> shapes;
         Q_FOREACH(auto *reference, collection.referenceImages()) {
+            reference->setZIndex(currentZIndex);
             shapes.append(reference);
+            currentZIndex += 1;
         }
 
         KisDocument *doc = document();
