@@ -21,6 +21,7 @@
 #include "KarbonCalligraphyTool.h"
 
 #include <KoToolRegistry.h>
+#include <kis_action_registry.h>
 
 #include <KoIcon.h>
 #include <klocalizedstring.h>
@@ -44,3 +45,16 @@ KoToolBase *KarbonCalligraphyToolFactory::createTool(KoCanvasBase *canvas)
 {
     return new KarbonCalligraphyTool(canvas);
 }
+
+QList<QAction *> KarbonCalligraphyToolFactory::createActionsImpl()
+{
+    KisActionRegistry *actionRegistry = KisActionRegistry::instance();
+    QList<QAction *> actions;
+
+    actions << actionRegistry->makeQAction("calligraphy_increase_width");
+    actions << actionRegistry->makeQAction("calligraphy_decrease_width");
+    actions << actionRegistry->makeQAction("calligraphy_increase_angle");
+    actions << actionRegistry->makeQAction("calligraphy_decrease_angle");
+
+    return actions;
+}
