@@ -19,6 +19,7 @@
 #include "kis_transform_utils.h"
 
 #include <cmath>
+#include <QPainterPath>
 #include <QTransform>
 #include <KoUnit.h>
 #include "tool_transform_args.h"
@@ -292,12 +293,21 @@ void KisTransformUtils::transformDevice(const ToolTransformArgs &config,
         worker.prepareTransform();
         worker.setTransformedCage(config.transfPoints());
         worker.run();
-    } else if (config.mode() == ToolTransformArgs::LIQUIFY) {
+    } else if (config.mode() == ToolTransformArgs::LIQUIFY && config.liquifyWorker()) {
         KoUpdaterPtr updater = helper->updater();
         //FIXME:
         Q_UNUSED(updater);
 
         config.liquifyWorker()->run(device);
+    } else if (config.mode() == ToolTransformArgs::MESH) {
+        KoUpdaterPtr updater = helper->updater();
+        //FIXME:
+        Q_UNUSED(updater);
+
+        KisPaintDeviceSP srcDevice = new KisPaintDevice(*device);
+        device->clear();
+        config.meshTransform()->transformMesh(srcDevice, device);
+
     } else {
         QVector3D transformedCenter;
         KoUpdaterPtr updater1 = helper->updater();
@@ -355,6 +365,8 @@ QRect KisTransformUtils::needRect(const ToolTransformArgs &config,
     } else if (config.mode() == ToolTransformArgs::LIQUIFY) {
         result = config.liquifyWorker() ?
             config.liquifyWorker()->approxNeedRect(rc, srcBounds) : rc;
+    } else if (config.mode() == ToolTransformArgs::MESH) {
+        result = config.meshTransform()->approxNeedRect(rc);
     } else {
         KIS_ASSERT_RECOVER_NOOP(0 && "this works for non-affine transformations only!");
     }
@@ -388,6 +400,9 @@ QRect KisTransformUtils::changeRect(const ToolTransformArgs &config,
     } else if (config.mode() == ToolTransformArgs::LIQUIFY) {
         result = config.liquifyWorker() ?
             config.liquifyWorker()->approxChangeRect(rc) : rc;
+    } else if (config.mode() == ToolTransformArgs::MESH) {
+        result = config.meshTransform()->approxChangeRect(rc);
+
     } else {
         KIS_ASSERT_RECOVER_NOOP(0 && "this works for non-affine transformations only!");
     }
@@ -480,7 +495,13 @@ ToolTransformArgs KisTransformUtils::resetArgsForMode(ToolTransformArgs::Transfo
         args.setMode(ToolTransformArgs::LIQUIFY);
         const QRect srcRect = transaction.originalRect().toAlignedRect();
         if (!srcRect.isEmpty()) {
-            args.initLiquifyTransformMode(transaction.originalRect().toAlignedRect());
+            args.initLiquifyTransformMode(srcRect);
+        }
+    } else if (mode == ToolTransformArgs::MESH) {
+        args.setMode(ToolTransformArgs::MESH);
+        const QRect srcRect = transaction.originalRect().toAlignedRect();
+        if (!srcRect.isEmpty()) {
+            *args.meshTransform() = KisBezierTransformMesh(QRectF(srcRect));
         }
     } else if (mode == ToolTransformArgs::PERSPECTIVE_4POINT) {
         args.setMode(ToolTransformArgs::PERSPECTIVE_4POINT);
