@@ -39,6 +39,7 @@
 #include "kis_projection_leaf.h"
 #include "kis_modify_transform_mask_command.h"
 
+#include "kis_image_animation_interface.h"
 #include "kis_sequential_iterator.h"
 #include "kis_selection_mask.h"
 #include "kis_image_config.h"
@@ -80,6 +81,27 @@ TransformStrokeStrategy::~TransformStrokeStrategy()
 {
 }
 
+bool TransformStrokeStrategy::shouldRestartStrokeOnModeChange(ToolTransformArgs::TransformMode oldMode, ToolTransformArgs::TransformMode newMode, KisNodeList processedNodes)
+{
+    bool hasExternalLayers = false;
+    Q_FOREACH (KisNodeSP node, processedNodes) {
+        if (node->inherits("KisShapeLayer")) {
+            hasExternalLayers = true;
+            break;
+        }
+    }
+
+    bool result = false;
+
+    if (hasExternalLayers) {
+        result =
+            (oldMode == ToolTransformArgs::FREE_TRANSFORM) !=
+            (newMode == ToolTransformArgs::FREE_TRANSFORM);
+    }
+
+    return result;
+}
+
 KisPaintDeviceSP TransformStrokeStrategy::createDeviceCache(KisPaintDeviceSP dev)
 {
     KisPaintDeviceSP cache;
@@ -158,6 +180,7 @@ void TransformStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
 
             } else if (KisGroupLayer *group = dynamic_cast<KisGroupLayer*>(rootNode.data())) {
                 const QRect bounds = group->image()->bounds();
+                const int desiredAnimTime = group->image()->animationInterface()->currentTime();
 
                 KisImageSP clonedImage = new KisImage(0,
                                                       bounds.width(),
@@ -165,6 +188,11 @@ void TransformStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
                                                       group->colorSpace(),
                                                       "transformed_image");
 
+                // BUG: 413968
+                // Workaround: Group layers wouldn't properly render the right frame
+                // since `clonedImage` would always have a time value of 0.
+                clonedImage->animationInterface()->explicitlySetCurrentTime(desiredAnimTime);
+
                 KisGroupLayerSP clonedGroup = dynamic_cast<KisGroupLayer*>(group->clone().data());
 
                 // In case the group is pass-through, it needs to be disabled for the preview,
