@@ -27,16 +27,28 @@
 #include "tool_transform_args.h"
 #include <kis_processing_visitor.h>
 #include <kritatooltransform_export.h>
-
+#include <boost/optional.hpp>
 
 
 class KisPostExecutionUndoAdapter;
+class TransformTransactionProperties;
+class KisUpdatesFacade;
+class KisDecoratedNodeInterface;
 
 
 class TransformStrokeStrategy : public QObject, public KisStrokeStrategyUndoCommandBased
 {
     Q_OBJECT
 public:
+    struct TransformAllData : public KisStrokeJobData {
+        TransformAllData(const ToolTransformArgs &_config)
+            : KisStrokeJobData(SEQUENTIAL, NORMAL),
+              config(_config) {}
+
+        ToolTransformArgs config;
+    };
+
+
     class TransformData : public KisStrokeJobData {
     public:
         enum Destination {
@@ -77,12 +89,20 @@ public:
     };
 
 public:
-    TransformStrokeStrategy(KisNodeSP rootNode, KisNodeList processedNodes,
+    TransformStrokeStrategy(ToolTransformArgs::TransformMode mode,
+                            bool workRecursively,
+                            const QString &filterId,
+                            bool forceReset,
+                            KisNodeSP rootNode,
                             KisSelectionSP selection,
-                            KisStrokeUndoFacade *undoFacade);
+                            KisStrokeUndoFacade *undoFacade, KisUpdatesFacade *updatesFacade);
 
     ~TransformStrokeStrategy() override;
 
+    static bool shouldRestartStrokeOnModeChange(ToolTransformArgs::TransformMode oldMode,
+                                                ToolTransformArgs::TransformMode newMode,
+                                                KisNodeList processedNodes);
+
     void initStrokeCallback() override;
     void finishStrokeCallback() override;
     void cancelStrokeCallback() override;
@@ -91,6 +111,7 @@ public:
     static bool fetchArgsFromCommand(const KUndo2Command *command, ToolTransformArgs *args, KisNodeSP *rootNode, KisNodeList *transformedNodes);
 
 Q_SIGNALS:
+    void sigTransactionGenerated(TransformTransactionProperties transaction, ToolTransformArgs args, void *cookie);
     void sigPreviewDeviceReady(KisPaintDeviceSP device);
 
 protected:
@@ -118,7 +139,26 @@ private:
     void putDeviceCache(KisPaintDeviceSP src, KisPaintDeviceSP cache);
     KisPaintDeviceSP getDeviceCache(KisPaintDeviceSP src);
 
+    QList<KisNodeSP> fetchNodesList(ToolTransformArgs::TransformMode mode, KisNodeSP root, bool recursive);
+    ToolTransformArgs resetArgsForMode(ToolTransformArgs::TransformMode mode,
+                                       const QString &filterId,
+                                       const TransformTransactionProperties &transaction);
+    bool tryInitArgsFromNode(KisNodeSP node, ToolTransformArgs *args);
+    bool tryFetchArgsFromCommandAndUndo(ToolTransformArgs *args,
+                                        ToolTransformArgs::TransformMode mode,
+                                        KisNodeSP currentNode,
+                                        KisNodeList selectedNodes, QVector<KisStrokeJobData *> *undoJobs);
+
+    void finishStrokeImpl(bool applyTransform,
+                          const ToolTransformArgs &args);
+
 private:
+    KisUpdatesFacade *m_updatesFacade;
+    ToolTransformArgs::TransformMode m_mode;
+    bool m_workRecursively;
+    QString m_filterId;
+    bool m_forceReset;
+
     KisSelectionSP m_selection;
 
     QMutex m_devicesCacheMutex;
@@ -126,11 +166,19 @@ private:
 
     KisTransformMaskSP writeToTransformMask;
 
-    ToolTransformArgs m_savedTransformArgs;
-    KisNodeSP m_savedRootNode;
-    KisNodeList m_savedProcessedNodes;
+    ToolTransformArgs m_initialTransformArgs;
+    boost::optional<ToolTransformArgs> m_savedTransformArgs;
+    KisNodeSP m_rootNode;
+    KisNodeList m_processedNodes;
     QList<KisSelectionSP> m_deactivatedSelections;
     QList<KisNodeSP> m_hiddenProjectionLeaves;
+    KisSelectionMaskSP m_deactivatedOverlaySelectionMask;
+    QVector<KisDecoratedNodeInterface*> m_disabledDecoratedNodes;
+
+    const KisSavedMacroCommand *m_overriddenCommand = 0;
+    QVector<const KUndo2Command*> m_skippedWhileMergeCommands;
+
+    bool m_finalizingActionsStarted = false;
 };
 
 #endif /* __TRANSFORM_STROKE_STRATEGY_H */
