@@ -37,6 +37,8 @@ ToolTransformArgs::ToolTransformArgs()
     QString savedFilterId = configGroup.readEntry("filterId", "Bicubic");
     setFilterId(savedFilterId);
     m_transformAroundRotationCenter = configGroup.readEntry("transformAroundRotationCenter", "0").toInt();
+    m_meshShowHandles = configGroup.readEntry("meshShowHandles", true);
+    m_meshSymmetricalHandles = configGroup.readEntry("meshSymmetricalHandles", true);
 }
 
 void ToolTransformArgs::setFilterId(const QString &id) {
@@ -87,6 +89,10 @@ void ToolTransformArgs::init(const ToolTransformArgs& args)
         m_liquifyWorker.reset(new KisLiquifyTransformWorker(*args.m_liquifyWorker.data()));
     }
 
+    m_meshTransform = args.m_meshTransform;
+    m_meshShowHandles = args.m_meshShowHandles;
+    m_meshSymmetricalHandles = args.m_meshSymmetricalHandles;
+
     m_continuedTransformation.reset(args.m_continuedTransformation ? new ToolTransformArgs(*args.m_continuedTransformation) : 0);
 }
 
@@ -94,6 +100,7 @@ void ToolTransformArgs::clear()
 {
     m_origPoints.clear();
     m_transfPoints.clear();
+    m_meshTransform = KisBezierTransformMesh();
 }
 
 ToolTransformArgs::ToolTransformArgs(const ToolTransformArgs& args)
@@ -145,6 +152,7 @@ bool ToolTransformArgs::operator==(const ToolTransformArgs& other) const
         m_editTransformPoints == other.m_editTransformPoints &&
         (m_liquifyProperties == other.m_liquifyProperties ||
          *m_liquifyProperties == *other.m_liquifyProperties) &&
+        m_meshTransform == other.m_meshTransform &&
 
         // pointer types
 
@@ -199,6 +207,8 @@ bool ToolTransformArgs::isSameMode(const ToolTransformArgs& other) const
              *m_liquifyWorker == *other.m_liquifyWorker)
             || m_liquifyWorker == other.m_liquifyWorker;
 
+    } else if (m_mode == MESH) {
+        result &= m_meshTransform == other.m_meshTransform;
     } else {
         KIS_SAFE_ASSERT_RECOVER_NOOP(0 && "unknown transform mode");
     }
@@ -266,6 +276,8 @@ void ToolTransformArgs::translate(const QPointF &offset)
     } else if (m_mode == LIQUIFY) {
         KIS_ASSERT_RECOVER_RETURN(m_liquifyWorker);
         m_liquifyWorker->translate(offset);
+    } else if (m_mode == MESH) {
+        m_meshTransform.transformSrcAndDst(QTransform::fromTranslate(offset.x(), offset.y()));
     } else {
         KIS_ASSERT_RECOVER_NOOP(0 && "unknown transform mode");
     }
@@ -288,8 +300,9 @@ bool ToolTransformArgs::isIdentity() const
 
         return true;
     } else if (m_mode == LIQUIFY) {
-        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(m_liquifyWorker, false);
-        return m_liquifyWorker->isIdentity();
+        return !m_liquifyWorker || m_liquifyWorker->isIdentity();
+    } else if (m_mode == MESH) {
+        return m_meshTransform.isIdentity();
     } else {
         KIS_ASSERT_RECOVER_NOOP(0 && "unknown transform mode");
         return true;
@@ -362,6 +375,11 @@ void ToolTransformArgs::toXML(QDomElement *e) const
 
         m_liquifyProperties->toXML(&liqEl);
         m_liquifyWorker->toXML(&liqEl);
+    } else if (m_mode == MESH) {
+        QDomElement meshEl = doc.createElement("mesh_transform");
+        e->appendChild(meshEl);
+
+        KisDomUtils::saveValue(&meshEl, "mesh", m_meshTransform);
     } else {
         KIS_ASSERT_RECOVER_RETURN(0 && "Unknown transform mode");
     }
@@ -462,6 +480,14 @@ ToolTransformArgs ToolTransformArgs::fromXML(const QDomElement &e)
 
         *args.m_liquifyProperties = KisLiquifyProperties::fromXML(e);
         args.m_liquifyWorker.reset(KisLiquifyTransformWorker::fromXML(e));
+    } else if (args.m_mode == MESH) {
+        QDomElement meshEl;
+
+        result =
+            KisDomUtils::findOnlyElement(e, "mesh_transform", &meshEl);
+
+        result &= KisDomUtils::loadValue(meshEl, "mesh", &args.m_meshTransform);
+
     } else {
         KIS_ASSERT_RECOVER_NOOP(0 && "Unknown transform mode");
     }
@@ -492,3 +518,39 @@ const ToolTransformArgs* ToolTransformArgs::continuedTransform() const
 {
     return m_continuedTransformation.data();
 }
+
+const KisBezierTransformMesh *ToolTransformArgs::meshTransform() const
+{
+    return &m_meshTransform;
+}
+
+KisBezierTransformMesh *ToolTransformArgs::meshTransform()
+{
+    return &m_meshTransform;
+}
+
+bool ToolTransformArgs::meshShowHandles() const
+{
+    return m_meshShowHandles;
+}
+
+void ToolTransformArgs::setMeshShowHandles(bool value)
+{
+    m_meshShowHandles = value;
+
+    KConfigGroup configGroup =  KSharedConfig::openConfig()->group("KisToolTransform");
+    configGroup.writeEntry("meshShowHandles", value);
+}
+
+bool ToolTransformArgs::meshSymmetricalHandles() const
+{
+    return m_meshSymmetricalHandles;
+}
+
+void ToolTransformArgs::setMeshSymmetricalHandles(bool value)
+{
+    m_meshSymmetricalHandles = value;
+
+    KConfigGroup configGroup =  KSharedConfig::openConfig()->group("KisToolTransform");
+    configGroup.writeEntry("meshSymmetricalHandles", value);
+}
